// FILE: ./assets/js/tcoAuthenticator/TcoAuthFunctions.js
var TcoAuthFunctions = function(ScreenService, MessageService) {

    /**
     * Check if there is an exception that
     * come from TcoAuthentication Backend.
     * 
     * @param {String} responseData 
     * @param {String} url 
     */
    var tcoAuthExceptionHandler = function(responseData, url) {
        if (url) {
            try {
                var responseObj = JSON.parse(responseData);
                var errorCode = responseObj.error;

                if (errorCode && errorCode.indexOf('TCOAUTH_') > -1) {
                    MessageService.showMessageByCode(errorCode)
                        .then(() => {
                            if (typeof exceptionHandlerFunctions[errorCode] == 'function') {
                                exceptionHandlerFunctions[errorCode]();
                            }
                        });
                }
            } catch (e) {
                return false;
            }
        }
    };

    /**
     * Redirect user for dashboard if is
     * a invalid system user.
     */
    var invalidUserExceptionHandler = function() {
        var productInitUrl = window.metadataUrls.filter(urlObj => urlObj.package == 'product')[0].baseUrl;
        productInitUrl = productInitUrl.replace('assets/', '');

        window.location.replace(productInitUrl);
    };

    /**
     * Object that points functions to be
     * called after exception handler message,
     * according to the error code.
     */
    var exceptionHandlerFunctions = {
        'TCOAUTH_INVALID_USER': invalidUserExceptionHandler
    };

    /**
     * Object that points functions to be
     * called default.
     */
    this.defaultFunctions = [
        tcoAuthExceptionHandler
    ];

    /**
     * Object that points a function to
     * be executed when a backend route
     * is called.
     */
    this.functionByRoute = {};
};

// FILE: ./assets/js/tcoAuthenticator/TcoAuthListener.js
var TcoAuthListener = function(AuthRunner) {

    /**
     * Overwrite XMLHttpRequest.prototype.open
     * and add a listener on it.
     */
    this.initRoutesListener = function() {
        (function(open) {
            XMLHttpRequest.prototype.open = function(method, url, async, user, pass) {
                this.addEventListener("readystatechange", function() {
                    callFunction(this);
                }, false);

                open.call(this, method, url, async, user, pass);
            };
        })(XMLHttpRequest.prototype.open);
    };

    /**
     * Call AuthFunctions user-made function
     * to validate tco routes response.
     * 
     * @param {Response} response 
     */
    var callFunction = function(response) {
        AuthRunner.run(response);
    };
};

// FILE: ./assets/js/tcoAuthenticator/TcoAuthRunner.js
var TcoAuthRunner = function(tcoAuthFunctions) {
    /**
     * Run functions defined at TcoAuthFunctions
     * after run this.defaultFunctions. Pass responseUrl
     * and reponse data as params.
     * 
     * @param {Response} response Response data
     * passed by from TcoAuthListener.
     */
    this.run = function(response) {
        var responseURL = getReducedValidURL(response.responseURL);
        var responseData = response.response;
        var functionPerRoute = tcoAuthFunctions.functionByRoute[responseURL] || null;

        tcoAuthFunctions.defaultFunctions.forEach(functionToCall => {
            functionToCall(responseData, responseURL);
        });

        if (functionPerRoute) {
            functionPerRoute(responseData, responseURL);
        }
    };

    /**
     * Return a parsed chopped URL if it comes
     * from tco/index.php, or null if not.
     * 
     * @param {String} url
     */
    var getReducedValidURL = function(url) {
        var returnUrl = null;

        if (url.indexOf('index.php/') > -1) {
            returnUrl = url.slice(url.indexOf('index.php/') + 9);
        }

        return returnUrl;
    };
};

// FILE: ./assets/js/tcoAuthenticator/TcoAuthenticator.js
var TcoAuthenticator = function(ScreenService, MessageService) {
    var tcoAuthFunctions = new TcoAuthFunctions(ScreenService, MessageService);
    var tcoAuthRunner = new TcoAuthRunner(tcoAuthFunctions);
    var tcoAuthListener = new TcoAuthListener(tcoAuthRunner);

    tcoAuthListener.initRoutesListener();
};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('TcoAuthenticator', TcoAuthenticator);
});

// FILE: ./assets/js/utilites/Tco01200_ActionsValidator.js
/*global Util, Query, angular, Configuration*/
var Tco01200_ActionsValidator = function(ScreenService, General, ValidationEngine, eventAggregator, ZHPromise, ZhPreferences, ZhFilterPreferences, maskEngine, templateManager, ZhKeyboardService, MessageService) {

    /**
     * Verifica se o usuário pode usar a ação selecionada, olhando 
     * os parametros de situação e motivo.
     * 
     * @params {String} action - nome da ação acionada.
     * @params {Object} currentRow - objeto currentRow.
     * 
     * @return {Boolean} verdadeiro ou falso.
     **/

    this.verifyPermission = function(action, currentRow) {
        let allowed;
        let reason = currentRow.IDMOTIVO;
        let situation = currentRow.IDSERVICOPROPCOM;
        let params = getParamsByReason(reason);
        let situations = params.SITUATIONS;
        let msgError = params.MSGERROR;

        allowed = situations[situation].find(act => {
            return act === action;
        });

        if (allowed !== undefined) {
            return true;
        } else {
            MessageService.showMessageByCode(msgError);
            return false;
        }
    };

    /**
     * Decide qual objeto será retornado de acordo com o motivo filtrado.
     * 
     * @params {String} reason - nome do motivo filtrado.
     * 
     * @return {Object} objeto com os parâmetros que contem as ações permitidas
     * para cada situação e a respectiva mensagem de erro em caso de bloqueio.
     **/

    let getParamsByReason = function(reason) {
        if (reason === "SEMCARDBASE") {
            return {
                "SITUATIONS": {
                    NAOSOLICITADO: ["DETAILSERVICE", "DOWNLOAD", "SCOPE"],
                    AGUARDANDOPLANEJAMENTO: ["DETAILSERVICE", "FINISH", "RECUSE", "REQUEST", "RESPONSIBLE", "DOWNLOAD", "SCOPE"],
                    EMANDAMENTO: ["DETAILSERVICE", "FINISH", "RECUSE", "REQUEST", "DOWNLOAD", "RESPONSIBLE", "SCOPE"],
                    RECUSADO: ["DETAILSERVICE", "DOWNLOAD", "SCOPE"],
                    AGUARDANDOGN: ["DETAILSERVICE", "DOWNLOAD", "SCOPE"],
                    EMANDAMENTO_RETORNOGN: ["DETAILSERVICE", "FINISH", "RECUSE", "REQUEST", "DOWNLOAD", "RESPONSIBLE", "SCOPE"],
                    FINALIZADOPLANEJAMENTO: ["DETAILSERVICE", "DOWNLOAD", "SCOPE"],
                    CONCLUIDO: ["DETAILSERVICE", "DOWNLOAD", "SCOPE"],
                    APROVADO: ["DETAILSERVICE", "DOWNLOAD", "SCOPE"],
                    EFETIVADO: ["DETAILSERVICE", "DOWNLOAD", "SCOPE"]
                },
                "MSGERROR": "TCO01200_SEM_CARD_BASE"
            };
        } else {
            return {
                "SITUATIONS": {
                    NAOSOLICITADO: ["DETAILSERVICE", "CARDAPIO", "CONFIG", "ANALISE", "DOWNLOAD", "SCOPE"],
                    AGUARDANDOPLANEJAMENTO: ["DETAILSERVICE", "CARDAPIO", "FINISH", "RECUSE", "REQUEST", "RESPONSIBLE", "IMPORT", "CONFIG", "ANALISE", "DOWNLOAD", "SELECT_BRAND", "SCOPE", "COPIA_CARDAPIO_BASE"],
                    EMANDAMENTO: ["DETAILSERVICE", "CARDAPIO", "FINISH", "RECUSE", "REQUEST", "RESPONSIBLE", "IMPORT", "CONFIG", "ANALISE", "DOWNLOAD", "SELECT_BRAND", "SCOPE"],
                    RECUSADO: ["DETAILSERVICE", "CARDAPIO", "CONFIG", "ANALISE", "DOWNLOAD", "SCOPE"],
                    AGUARDANDOGN: ["DETAILSERVICE", "CARDAPIO", "CONFIG", "ANALISE", "DOWNLOAD", "SCOPE"],
                    EMANDAMENTO_RETORNOGN: ["DETAILSERVICE", "CARDAPIO", "FINISH", "RECUSE", "REQUEST", "RESPONSIBLE", "IMPORT", "CONFIG", "ANALISE", "DOWNLOAD", "SELECT_BRAND", "SCOPE"],
                    FINALIZADOPLANEJAMENTO: ["DETAILSERVICE", "CARDAPIO", "CONFIG", "ANALISE", "DOWNLOAD", "SCOPE"],
                    CONCLUIDO: ["DETAILSERVICE", "CARDAPIO", "CONFIG", "ANALISE", "DOWNLOAD", "SCOPE"],
                    APROVADO: ["DETAILSERVICE", "EFFECT", "CARDAPIO", "CONFIG", "ANALISE", "DOWNLOAD", "SCOPE"],
                    EFETIVADO: ["DETAILSERVICE", "CARDAPIO", "CONFIG", "ANALISE", "DOWNLOAD", "SCOPE"]
                },
                "MSGERROR": "TCO01200_ACTION_BLOCKED_BY_SITUATION"
            };
        }
    };

    /**
     * Verifica apenas as ações referentes ao cardápio.
     * 
     * @params {Object} filterRow - objeto filterRow.
     * 
     * @return {Boolean} verdadeiro ou falso.
     **/

    this.blockActionsCard = function(row) {
        let situation = row.IDSERVICOPROPCOM;
        let reason = row.IDMOTIVO;
        if (reason === "SEMCARDBASE") {
            MessageService.showMessageByCode("TCO01200_SEM_CARD_BASE");
            return false;
        } else {
            let allowedSituations = ["AGUARDANDOPLANEJAMENTO", "EMANDAMENTO_RETORNOGN", "EMANDAMENTO"];
            let allowed = allowedSituations.find(sit => {
                return sit === situation;
            });

            if (allowed !== undefined) {
                return true;
            } else {
                MessageService.showMessageByCode("TCO01200_ACTION_BLOCKED_BY_SITUATION");
                return false;
            }
        }
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01200_ActionsValidator', Tco01200_ActionsValidator);
});

// FILE: ./assets/js/utilites/TcoUtil.js
/*global Util, Query, angular, Configuration*/
var TcoUtil = function(ScreenService, General, ValidationEngine, eventAggregator, ZHPromise, ZhPreferences, ZhFilterPreferences, maskEngine, templateManager, ZhKeyboardService, metaDataFactory, MessageService) {

    this.validations = function() {

        var codeValidation = function(value, validationAttrs) {
            var retorno;

            retorno = {
                "valid": (!isNaN(value)),
                "message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n("Código inválido")
            };

            return retorno;
        };

        ValidationEngine.registerValidation('isValidCodeNumber', codeValidation);
    };

    eventAggregator.subscribe('authSuccess', function(session) {
        this.validations();
    }.bind(this));

    this.delimitCaracter = function(args, limit) {
        if (!Util.isEmptyOrBlank(args.owner.field.value())) {
            var ano = args.owner.field.value().toString();
            if (ano.length > limit) {
                args.owner.field.setValue(parseInt(ano.substr(0, limit), 10));
            }
        }
    };

    this.startFilter = function(widget, currentRow) {
        return ZhPreferences.getDefaultWidgetPreferences(widget.id).then(function(result) {
            if (result && result.DEFAULT_FILTER) {
                ZhPreferences.getPreferences('filter', Query.build().where('WIDGET_ID').equals(widget.id)).then(function(filters) {
                    var defaultFilter = filters.filter(function(filter) {
                        return filter.ID == result.DEFAULT_FILTER;
                    })[0];
                    var filter = {
                        ID: defaultFilter.LABEL_TO_PREFERENCE || 'EDITED',
                        FILTER_ROW: JSON.parse(defaultFilter.FILTER_ROW)
                    };

                    currentRow = filter.FILTER_ROW;

                    ZhFilterPreferences.applyFilter(widget, filter).then(function() {
                        widget.appliedFilter = widget.appliedFilter || filter;
                        widget.appliedFilter.ID = result.DEFAULT_FILTER;
                        if (defaultFilter.LABEL_TO_PREFERENCE) {
                            widget._filterInfo[0].value = defaultFilter.LABEL_TO_PREFERENCE;
                        }
                    });
                });
            } else {
                if (Util.isUndefinedOrNull(currentRow) || Util.isEmptyObject(currentRow)) {
                    currentRow = buildFilterCurrentRow(widget);
                }
            }
            ZhFilterPreferences.openFilterWidget(widget, currentRow);
        });
    };

    var buildFilterCurrentRow = function(widget) {
        // let filterFields = getFilterFields(widget);
        return getFilterRowWithDefaultValues(widget.fields);
    };

    var getFilterRowWithDefaultValues = function(filterFields) {
        let filterRow = {};
        filterFields.forEach(field => {
            let defaultValue = Util.getNestedProperty(field, ["filterProperties", "defaultValue"]) || field.defaultValue;
            if (!Util.isUndefinedOrNull(defaultValue)) {
                filterRow[field.name] = Util.evaluate(defaultValue);
            }
        });
        return filterRow;
    };

    this.justInteger = function(selector, retryCount) {
        if (!retryCount || retryCount > 5)
            retryCount = 5;
        if (!selector) {
            throw {
                "message": "Selector is not defined!"
            };
        }
        setTimeout(function() {
            var elem = $(selector);
            if (elem[0]) {
                var length = selector.split(',').length;
                for (var i = 0; i < length; i++) {
                    if (elem.hasOwnProperty(i)) {
                        elem[i].onkeypress = avoidNonInteger;
                        elem[i].onpaste = preventEvent;
                    }
                }
            } else {
                if (!retryCount) {
                    console.error("Selector not found!");
                } else {
                    retryCount--;
                    this.justInteger(selector, retryCount);
                }
            }
        }.bind(this), 500);
    };

    function avoidNonInteger(e) {
        var integerNumbers = [44, 46, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57];
        var numberCode = e.which;
        if (integerNumbers.indexOf(numberCode) < 0) {
            preventEvent(e);
        }
    }

    function preventEvent(e) {
        e.preventDefault();
    }

    /**
     * Função responsável por chamar a maskCurrency sempre que o 
     * usuário desfocar do campo onde ela é aplicada.
     * Função deve ser usada no evento FieldOnBlur, junto com a 
     * removeMask, essa no FieldOnFocus.
     * 
     * @param {Field} field - field.
     * @param {Char} decimal - Caráctere que será usado para a casa decimal.
     * @param {Char} thousand - Caráctere que será usado para as unidades de milhar.
     * @param {Number} precision - Quantidade de casas decimais.
     */

    this.callMaskCurrency = function(field, decimal, thousands, precision = 2) {
        var value = field.value();
        var params = {
            "decimal": decimal,
            "thousands": thousands,
            "precision": precision
        };
        // field.modelValue();
        // field.widget.redraw();
        // templateManager.updateTemplate()
        field.value(this.maskCurrency(value, params));
    };

    var maskFunction = function(element, viewValue, params, callBack) {
        // do something
    };

    /**
     * Função responsável por aplicar uma máscara do type currency
     * em algum field.
     * 
     * @param {Value} value - value do field.
     * @param {Object} params - Objeto param da máscara com {decimal, thousand, precision}.
     * 
     */

    this.maskCurrency = function(value, params) {
        var thousands = Util.isEmptyOrBlank(params.thousands) ? params.thousand : params.thousands;
        if (!Util.isEmptyOrBlank(value)) {

            if (value[0] === '.') {
                value = '0' + '.' + value.split('.').join('');
            } else if (value[0] === ',') {
                value = '0' + ',' + value.split(',').join('');
            }

            if (params.decimal === ',')
                value = numberFormatBr(value);
            var newValue = "";
            value = value.split('.').join('');
            var dec = value.split(',')[1];
            var mil = [];
            var int = value.split(',')[0];

            if (dec === undefined) {
                dec = "";
                dec = dec.padEnd(params.precision, "0");
                int = value;
            } else if (dec.length < params.precision) {
                dec = dec.padEnd(params.precision, "0");
            } else if (dec.length > params.precision) {
                dec = parseFloat('1.' + dec).toFixed(params.precision).toString().split('.')[1];
            }

            int = parseInt(int).toString();
            var counter = angular.copy(int.length / 3);
            for (var i = 1; i <= counter; i++) {
                if (i != counter)
                    mil.push(thousands + int.substr(-3));
                else
                    mil.push(int.substr(-3));
                int = int.substr(0, (int.length - 3));
            }

            if (int != "") mil.push(int);

            mil = mil.reverse().join('');

            newValue = mil + params.decimal + dec;
            var regex = '^(^([0-9]{1,3}\\' + thousands + ')((\\d{3}\\' + thousands + '{1})|(\\d{3}\\' + params.decimal + '\\d{2,}))*$)|^(\\d{0,3}\\' + params.decimal + '\\d{2,})*$';
            regex = new RegExp(regex, 'g');
            if (regex.test(newValue))
                return newValue;
            else
                return 0;
        }
    };

    maskEngine.registerMask('maskCurrencyTCO', maskFunction, this.maskCurrency);

    /**
     * Função responsável por retirar os pontos da máscara no
     * field para o usuário digitar.
     * 
     * @param {Args} args - Args do field.
     */

    this.removeMask = function(args) {
        var field = args.owner.field;
        var value = field.value();
        if (/^(.*)(\.+).*(,+)(.*)$/gm.test(field.value())) {
            field.value(value.split('.').join(''));
        }
    };

    /**
     * Função que retira os '.' das casas de milhar dos números
     * e troca as ',' dos decimais por '.'
     * Dessa forma o campo fica compativel com o BD do oracle.
     * 
     * @param {String} value - Valor numérico.
     */

    this.parseFloat = function(value) {
        if (/^(.*)(\.+).*(,+)(.*)$/gm.test(value)) {
            var loop = value.toString().split('.');
            for (var i = 0; i < loop.length - 1; i++) {
                value = value.toString().replace('.', '');
            }
        }
        if (/\,/g.test(value))
            value = value.toString().replace(',', '.');
        return parseFloat(value);
    };

    /**
     * Função que redireciona para outra tela fazendo o uso do window history.
     * 
     * @param {String} urlString - URL destino.
     * @param {Object} params - Parametros que podem ser capturados via window.history.
     * @param {String} title - Define um título para o histórico no navegador.
     */

    this.openWindowByHistory = function(urlString, params, title) {
        title = title === null ? "TecFood" : title;
        window.history.pushState(params, title, urlString);
    };

    /**
     * Função responsável por definir os 
     * parâmetros paramsNewCode e __is_new de um 
     * registro antes de salvar.
     * 
     * @param {Widget} widget Widget contendo o registro.
     */
    this.setNewCodeParametersInRow = function(widget) {

        widget.currentRow.__using_unit = widget.paramsNewCode.__using_unit || false;
        widget.currentRow.__new_code_pk = widget.paramsNewCode.__new_code_pk;
        widget.currentRow.__table_pk = widget.paramsNewCode.__table_pk;
        widget.currentRow.__new_code_length = widget.paramsNewCode.__new_code_length || 5;
        widget.currentRow.__new_code_column_size = widget.paramsNewCode.__new_code_column_size || 10;
        widget.currentRow.__cast_number = widget.paramsNewCode.__cast_number || false;

        if (!widget.currentRow.__columns_to_return && widget.paramsNewCode && widget.paramsNewCode.__columns_to_return)
            widget.currentRow.__columns_to_return = widget.paramsNewCode.__columns_to_return;

    };

    /**
     * Preenche a string passada por parâmetro
     * com '0' a esquerda até atingir o tamanho 
     * indicado pelo segundo parâmetro da função.
     * 
     * @param {String} value Valor a ser preenchido
     *  com zerios.
     * @param {Number} length Tamanho total final 
     *  da string.
     * 
     * @return {String} String formatada com '0'.
     */
    this.padLeft = function(value, length) {
        return (value.toString().length < length) ? this.padLeft("0" + value, length) : value;
    };

    /**
     * Preenche a string passada por parâmetro
     * com '0' a esquerda até atingir o tamanho 
     * indicado pelo segundo parâmetro da função.
     * 
     * @param {String} value Valor a ser preenchido
     *  com zerios.
     * @param {Number} length Tamanho total final 
     *  da string.
     * 
     * @return {String} String formatada com '0'.
     */
    this.padRight = function(value, length) {
        return (value.toString().length < length) ? this.padRight(value + "0", length) : value;
    };

    this.buildRegExp = function(value) {
        var filterValue = "^" + value + "$";
        var rgx = /\%/;
        while (rgx.test(filterValue)) {
            filterValue = filterValue.replace(rgx, '(.*?)');
        }
        return new RegExp(filterValue, 'i');
    };

    this.concatenaSemanaEDias = function(args, widget = null) {
        var widgetMain = widget ? widget : args.owner;
        widgetMain.dataSource.data.forEach(function(row, count) {
            if (row.NRSEMANA !== 0)
                widgetMain.dataSource.data[count].LABELDIA = row.NRSEMANA + "° " + ScreenService.i18n(row.LABELDIA);
        });
    };

    this.buildMessagesToShow = function(message) {
        if (typeof message === "string") {

            return MessageService.getMessageByCode(message);

        } else if (typeof message === "object") {
            var mensagens = [];
            var msgParams = [];

            message.forEach(function(msg) {
                msgParams.push(msg.PARAMS);
                mensagens.push(msg.MSGCODE);
            });

            return MessageService.getMessagesByCode(mensagens).then(function(mensagensTraduzidas) {
                for (let i = 0; i < mensagensTraduzidas.length; i++) {
                    mensagensTraduzidas[i] = ScreenService.i18n(mensagensTraduzidas[i], msgParams[i]);
                }
                return mensagensTraduzidas;
            });
        }
    };

    this.buildRegExp = function(value) {
        var filterValue = "^" + value + "$";
        var rgx = /\%/;
        while (rgx.test(filterValue)) {
            filterValue = filterValue.replace(rgx, '(.*?)');
        }
        return new RegExp(filterValue, 'i');
    };

    var numberFormatBr = function(value) {
        if (/^(.*)(\.+).*(\,+)(.*)$/gm.test(value)) {
            let loop = value.toString().split('.');
            for (let i = 0; i < loop.length - 1; i++) {
                value = value.toString().replace('.', '');
            }
        } else if (/^(.*)(\,+).*(\.+)(.*)$/gm.test(value)) {
            let loop = value.toString().split(',');
            for (let i = 0; i < loop.length - 1; i++) {
                value = value.toString().replace(',', '');
            }
        }
        if (/^\d{1,}\.{1}\d*$/g.test(value))
            value = value.toString().replace('.', ',');
        return value.toString();
    };

    /**
     * Função responsável buscar o nivel maximo do Prato.
     * 
     * @param {nameWid} nameWid - Nome da widget que contem o field CDPRATO.
     */

    //CDPICTPRAT
    this.maxLevelPrato = function(nameWid) {
        var widget = templateManager.container.getWidget(nameWid);
        this.getMaxLevelPrato().then(function(response) {
            widget.maxNivelPrato = response.dataset.getMaxLevelPrato.MAXLEVELPRATO;
        });
    };

    this.getMaxLevelPrato = function() {
        return General.callRestEngine('tco#/getMaxLevelPrato', "DataSet", []);
    };

    /**
     * Função responsável buscar o nivel maximo do Produto.
     * 
     * @param {nameWid} nameWid - Nome da widget que contem o field CDARVPROD.
     */

    //CDPICTPROD
    this.maxLevelProduto = function(nameWid) {
        var widget = templateManager.container.getWidget(nameWid);
        this.getMaxLevelProduto().then(function(response) {
            widget.maxNivelProd = response.dataset.getMaxLevelProduto.MAXLEVELPROD;
        });
    };

    this.getMaxLevelProduto = function() {
        return General.callRestEngine('tco#/getMaxLevelProduto', "DataSet", []);
    };

    /**
     * Função responsável buscar a máscara de Produto.
     * 
     * @param {nameWid} nameWid - Nome da widget que contem o field CDARVPROD.
     */

    //CDPICTPROD
    this.maskProduto = function(nameWid) {
        var widget = templateManager.container.getWidget(nameWid);
        this.getMaskProduto().then(function(response) {
            widget.fields.forEach(function(field) {
                if (field.name === "CDARVPROD") {
                    applyMaskProd(field, response);
                }
                if (field.fields) {
                    field.fields.forEach(function(field) {
                        if (field.name === "CDARVPROD") {
                            applyMaskProd(field, response);
                        }
                    });
                }
            });
        });
    };

    this.getMaskProduto = function() {
        return General.callRestEngine('tco#/getMaskProduto', "DataSet", []);
    };

    /**
     * Função responsável buscar a máscara de Prato.
     * 
     * @param {nameWid} nameWid - Nome da widget que contem o field CDPRATO.
     */

    //CDPICTPRAT
    this.maskPrato = function(nameWid) {
        var widget = templateManager.container.getWidget(nameWid);
        this.getMaskPrato().then(function(response) {
            widget.fields.forEach(function(field) {
                if (field.name === "CDPRATO") {
                    applyMaskPrato(field, response);
                }
                if (field.fields) {
                    field.fields.forEach(function(field) {
                        if (field.name === "CDPRATO") {
                            applyMaskPrato(field, response);
                        }
                    });
                }
            });
        });
    };

    this.getMaskPrato = function() {
        return General.callRestEngine('tco#/getMaskPrato', "DataSet", []);
    };

    var applyMaskProd = function(field, response) {
        field.mask = {
            "type": "fix",
            "params": {
                "mask": response.dataset.getMaskProduto.CDPICTPROD.replace(/[a,9]/g, '*')
            }
        };
    };

    var applyMaskPrato = function(field, response) {
        field.mask = {
            "type": "fix",
            "params": {
                "mask": response.dataset.getMaskPrato.CDPICTPRAT.replace(/[a,9]/g, '*')
            }
        };
    };

    /**
     * Função responsável por alterar o estado required 
     * do field passado por parâmetro, corrigindo o 
     * erro de 'required':false do framework.
     * 
     * @params {Field} field - Field a ser alterado.
     * @params {Boolean} state - Estado de required.
     */
    this.changeRequiredField = function(field, state) {
        if (typeof field.validations == 'undefined') {
            field.validations = {};
        }

        if (state) {
            field.validations.required = true;
        } else if (typeof field.validations.required != 'undefined') {
            delete field.validations.required;
        }
    };

    var trataNumerico = function(value, decimalSeparator) {
        if (typeof value == "number") {
            return value.toString().split(".").join(decimalSeparator);
        }
        return (value) ? value.toString() : value + '';
    };

    /*
     * Usar no FieldOnKeydown
     *
     */

    this.lengthCurrency = function(args) {
        var value;
        var fieldValue;
        var field = args.owner.field;
        // var allowedKeys = [8, 9, 37, 38, 39, 40, 46, 44, 108, 188, 194];
        var allowedKeys = ['Tab', 'ArrowLeft', 'ArrowRight', 'ArrowUp', 'ArrowDown', 'End', 'Home', 'Shift', 'Control', 'Delete', 'Backspace', 'Alt'];

        let decimalSeparator = ',';
        let thousandSeparator = '.';

        if (field.mask && field.mask.params && field.mask.params.decimal) {
            decimalSeparator = field.mask.params.decimal;
        }

        if (field.inlineEditable && (field.widget.mode === "list" || field.widget.mode === "inline_edit")) {
            fieldValue = trataNumerico(field.widget.currentRow[field.name + '_inline'], decimalSeparator);
            fieldValue = fieldValue.toString();
            if (event.target.selectionStart !== event.target.selectionEnd) {
                fieldValue = fieldValue.substring(0, event.target.selectionStart) || fieldValue.substring(event.target.selectionEnd);
            }
            value = Util.isEmptyOrBlank(fieldValue) ? "" : fieldValue;
        } else {
            fieldValue = trataNumerico(field.value(), decimalSeparator);
            fieldValue = fieldValue.toString();
            if (event.target.selectionStart !== event.target.selectionEnd) {
                fieldValue = fieldValue.substring(0, event.target.selectionStart) || fieldValue.substring(event.target.selectionEnd);
            }
            value = Util.isEmptyOrBlank(fieldValue) ? "" : fieldValue;
        }

        fieldValue = fieldValue.toString();
        if (event.target.selectionStart !== event.target.selectionEnd) {
            fieldValue = fieldValue.substring(0, event.target.selectionStart) || fieldValue.substring(event.target.selectionEnd);
        }

        let posInsertChar = event.target.selectionStart;

        var cleanNumber = value.split('-').join('').split('.').join('').split(',').join('');
        var realLength = value.split('-').join('').split('.').join('').split(',')[0].length;
        var last = Math.max(...[value.lastIndexOf(','), value.lastIndexOf('.')]);
        var separador;
        var decimalLength = value.split('-').join('').split('.').join('').split(',')[1] ? value.split('-').join('').split('.').join('').split(',')[1].length : 0;
        var precision = "d{0,5}";
        var precisionLength = 0;
        var regex = "^(\\d{1," + field.maxLengthCurrency + "})$";

        if (field.mask && field.mask.params && field.mask.params.precision) {
            precision = "d{1," + field.mask.params.precision + "}";
            regex = "^(\\d{1," + field.maxLengthCurrency + "})(\\,|\\.){1}(\\" + precision + ")$";
            precisionLength = field.mask.params.precision;
        }
        var validation = new RegExp(regex, "gm");

        if (last > -1) {
            separador = value.substring(0, last).length >= field.maxLengthCurrency - precisionLength ? ',' : value[last];
            value = value.substr(0, last).concat(separador + value.substr(last + 1, value.length));
            value = value.split('.').join('');
            last = value.lastIndexOf(',');
            realLength = value.substr(0, last).length;
            decimalLength = value.substr(last + 1, value.length).length;
        }

        let isDecimalSeparator = (args.e.key === decimalSeparator);
        let allowDecimalSeparator = ((precisionLength > 0) && (!value.includes(decimalSeparator)));
        let posDecimalseparator = fieldValue.indexOf(decimalSeparator);
        if (isDecimalSeparator && !allowDecimalSeparator) {
            args.e.preventDefault();
        }

        if (field.mask && field.mask.params && field.mask.params.thousands) {
            thousandSeparator = field.mask.params.thousands;
        }
        let isThousandSeparator = (args.e.key === thousandSeparator);

        if (isThousandSeparator) {
            args.e.preventDefault();
        }

        let allowNegative = true;
        if (field.mask && field.mask.params) {
            allowNegative = field.mask.params.allowNegative;
        }
        allowNegative = (allowNegative && !value.includes('-'));
        isNegativeSimbol = (args.e.key === '-');

        if (isNegativeSimbol && (!allowNegative || (event.target.selectionStart > 0))) {
            args.e.preventDefault();
        }
        if (!allowedKeys.includes(args.e.key) && !isNegativeSimbol && !isDecimalSeparator && !isThousandSeparator) {
            if (realLength >= field.maxLengthCurrency) {
                if ((posDecimalseparator < 0) || (posInsertChar <= posDecimalseparator))
                    args.e.preventDefault();
            }

            if ((posDecimalseparator >= 0) && (posInsertChar > posDecimalseparator) && (precisionLength == decimalLength)) {
                args.e.preventDefault();
            }
        }
    };

    //Crosstab navigation
    var _isKeysBinded = false;
    var kbleft;
    var kbup;
    var kbright;
    var kbdown;
    var kbenter;
    var kbctrlc;
    var kbctrlv;
    var kbctrldel;
    var _gridElement;
    var _selectedCellElement;
    var _selectedRowElement;
    var _copiedCellElement;
    var _copiedRowElement;
    var _rowHeight = 36;
    var crosstabWidget;
    var pasteFunction;
    var deleteFunction;

    this.bindKeyboardEvents = function(funcToPaste, funcToDelete) {
        this.unbindKeyboardEvents();

        kbleft = ZhKeyboardService.bind("left", navigateLeft);
        kbup = ZhKeyboardService.bind("up", navigateUp);
        kbright = ZhKeyboardService.bind("right", navigateRight);
        kbdown = ZhKeyboardService.bind("down", navigateDown);
        kbenter = ZhKeyboardService.bind("enter", openCell);

        if (funcToPaste && typeof funcToPaste === "function") {
            kbctrlc = ZhKeyboardService.bind("mod+c", copyCell);
            kbctrlv = ZhKeyboardService.bind("mod+v", pasteCell);
            pasteFunction = funcToPaste;
        }
        if (funcToDelete && typeof funcToDelete === "function") {
            kbctrldel = ZhKeyboardService.bind("del", deleteCell);
            deleteFunction = funcToDelete;
        }
    };

    this.unbindKeyboardEvents = function() {
        if (kbleft)
            ZhKeyboardService.unbind(kbleft);
        if (kbup)
            ZhKeyboardService.unbind(kbup);
        if (kbright)
            ZhKeyboardService.unbind(kbright);
        if (kbdown)
            ZhKeyboardService.unbind(kbdown);
        if (kbenter)
            ZhKeyboardService.unbind(kbenter);
        if (kbctrlc)
            ZhKeyboardService.unbind(kbctrlc);
        if (kbctrlv)
            ZhKeyboardService.unbind(kbctrlv);
        if (kbctrldel)
            ZhKeyboardService.unbind(kbctrldel);
    };

    function initCrosstabNavigation() {
        if (canMoveCell()) {
            var firstRow = getFirstRowElement(_gridElement);
            var firstCell = getFirstCellElement(firstRow);
            firstCell.classList.add('selected-cell');
            _selectedCellElement = firstCell;
            _selectedRowElement = firstRow;
            _selectedRowElement._childIndex = getElementChildIndex(_selectedRowElement);
        }
    }

    function resetCrosstabNavigation() {
        if (_copiedCellElement && _copiedCellElement.classList) {
            _copiedCellElement.classList.remove('copied-cell');
        }
        _gridElement = false;
        _copiedCellElement = false;
        _copiedRowElement = false;
    }

    function restoreCrosstabNavigation() {
        _gridElement = _gridElement || document.getElementsByClassName('crosstab-body-content')[0];
        if (_gridElement && _selectedRowElement && _selectedCellElement) {
            var rowElementIndex = _selectedRowElement._childIndex;
            var cellElementIndex = getElementChildIndex(_selectedCellElement);
            if (_gridElement.children && _gridElement.children[rowElementIndex]) {
                _selectedRowElement = _gridElement.children[rowElementIndex];
                _selectedRowElement._childIndex = getElementChildIndex(_selectedRowElement);
                if (_selectedRowElement.children && _selectedRowElement.children[cellElementIndex]) {
                    _selectedCellElement = _selectedRowElement.children[cellElementIndex];
                    _selectedCellElement.classList.add('selected-cell');
                    restoreScrollPosition();
                }
            }
        }
    }

    function restoreScrollPosition() {
        var crosstabElement = document.getElementById('grid-' + crosstabWidget.id);
        var XScrollControl = crosstabElement.getElementsByClassName('body-scroll-control')[0];
        if (XScrollControl && _selectedCellElement) {
            var lastPosition = XScrollControl.scrollLeft;
            setTimeout(() => {
                XScrollControl.scrollLeft = lastPosition;
            }, 200);
        }
    }

    function deleteCell() {
        if (_selectedRowElement && _selectedCellElement && canDeleteCell(_selectedCellElement)) {
            ScreenService.confirmMessage('Confirmar exclusão?', 'question', function() {
                var cellIndex = getElementChildIndex(_selectedCellElement);
                var rowCelula = getDataRowByIndex(crosstabWidget.crosstabData, _selectedRowElement._childIndex, cellIndex, crosstabWidget.crosstabColumns);
                deleteFunction(rowCelula);
            });
        }
    }

    function canCopyCell(cell) {
        if (canMoveCell()) // crosstab widget activated
            if (!Util.isEmptyOrBlank(cell.innerText)) // has content on cell 
                return true;

        return false;
    }

    function canDeleteCell(cell) {
        if (canMoveCell()) // crosstab widget activated
            if (getElementChildIndex(cell) > 0)
                if (!Util.isEmptyOrBlank(cell.innerText)) // has content on cell
                    return true;

        return false;
    }

    function canPasteCell(cell) {
        if (canMoveCell()) // crosstab widget activated
            if (getElementChildIndex(cell) > 0)
                return true;

        return true;
    }

    function copyCell() {
        if (_selectedRowElement && _selectedCellElement && canCopyCell(_selectedCellElement)) {
            if (_copiedCellElement) {
                _copiedCellElement.classList.remove('copied-cell');
            }
            _copiedRowElement = _selectedRowElement;
            _copiedCellElement = _selectedCellElement;
            _copiedCellElement.classList.add('copied-cell');
            ScreenService.notificationMessage("Prato copiado.", "success");
        }
    }

    function pasteCell() {
        if (_copiedRowElement && _copiedCellElement && canPasteCell(_selectedCellElement)) {
            var origCellIndex = getElementChildIndex(_copiedCellElement);
            var destCellIndex = getElementChildIndex(_selectedCellElement);
            var origRow = getDataRowByIndex(crosstabWidget.crosstabData, _copiedRowElement._childIndex, origCellIndex, crosstabWidget.crosstabColumns);
            var destRow = getDataRowByIndex(crosstabWidget.crosstabData, _selectedRowElement._childIndex, destCellIndex, crosstabWidget.crosstabColumns);
            pasteFunction(origRow, destRow);
            _copiedCellElement.classList.remove('copied-cell');
        }
    }

    function getDataRowByIndex(data, rowIndex, cellIndex, columns) {
        var row = false;
        if (data[rowIndex]) {
            for (let i = 0; i < Object.keys(data[rowIndex]).length; i++) {
                if (i === cellIndex - 1) {
                    row = data[rowIndex][columns[cellIndex].key].__row;
                }
            }
        }
        return row;
    }

    function canMoveCell() {
        return templateManager.container.getActiveWidget().name == crosstabWidget.name && !ScreenService.isPopupOpen();
    }

    function navigateLeft() {
        _gridElement = _gridElement || document.getElementsByClassName('crosstab-body-content')[0];
        if (!_selectedCellElement) {
            initCrosstabNavigation();
        } else {
            if (canMoveCell()) {
                if (_selectedCellElement.previousElementSibling && !_selectedCellElement.previousElementSibling.classList.contains('fixed')) {
                    _selectedCellElement.classList.remove('selected-cell');
                    _selectedCellElement = _selectedCellElement.previousElementSibling;
                    _selectedCellElement.classList.add('selected-cell');
                    scrollLeft();
                }
            }
        }
    }

    function navigateUp() {
        _gridElement = _gridElement || document.getElementsByClassName('crosstab-body-content')[0];
        if (!_selectedCellElement) {
            initCrosstabNavigation();
        } else {
            if (canMoveCell()) {
                if (_selectedRowElement.previousElementSibling) {
                    var childIndex = getElementChildIndex(_selectedCellElement);
                    _selectedRowElement = _selectedRowElement.previousElementSibling;
                    _selectedCellElement.classList.remove('selected-cell');
                    _selectedRowElement._childIndex = getElementChildIndex(_selectedRowElement);
                    _selectedCellElement = _selectedRowElement.children[childIndex];
                    _selectedCellElement.classList.add('selected-cell');
                    scrollUp();
                }
            }
        }
    }

    function navigateRight() {
        _gridElement = _gridElement || document.getElementsByClassName('crosstab-body-content')[0];
        if (!_selectedCellElement) {
            initCrosstabNavigation();
        } else {
            if (canMoveCell()) {
                if (_selectedCellElement.nextElementSibling) {
                    _selectedCellElement.classList.remove('selected-cell');
                    _selectedCellElement = _selectedCellElement.nextElementSibling;
                    _selectedCellElement.classList.add('selected-cell');
                    scrollRight();
                }
            }
        }
    }

    function navigateDown() {
        _gridElement = _gridElement || document.getElementsByClassName('crosstab-body-content')[0];
        if (!_selectedCellElement) {
            initCrosstabNavigation();
        } else {
            if (canMoveCell()) {
                if (_selectedRowElement.nextElementSibling) {
                    var childIndex = getElementChildIndex(_selectedCellElement);
                    _selectedRowElement = _selectedRowElement.nextElementSibling;
                    _selectedRowElement._childIndex = getElementChildIndex(_selectedRowElement);
                    _selectedCellElement.classList.remove('selected-cell');
                    _selectedCellElement = _selectedRowElement.children[childIndex];
                    _selectedCellElement.classList.add('selected-cell');
                    scrollDown();
                }
            }
        }
    }

    function scrollLeft() {
        var crosstabElement = document.getElementById('grid-' + crosstabWidget.id);
        if (crosstabElement) {
            var XScrollControl = crosstabElement.getElementsByClassName('body-scroll-control')[0];
            if (XScrollControl) {
                if (_selectedCellElement.offsetLeft < XScrollControl.offsetLeft) {
                    XScrollControl.scrollLeft -= XScrollControl.offsetLeft - _selectedCellElement.offsetLeft;
                }
            }
        }
    }

    function scrollRight() {
        var crosstabElement = document.getElementById('grid-' + crosstabWidget.id);
        if (crosstabElement) {
            var XScrollControl = crosstabElement.getElementsByClassName('body-scroll-control')[0];
            if (XScrollControl) {
                if (_selectedCellElement.offsetLeft + _selectedCellElement.offsetWidth > XScrollControl.offsetWidth) {
                    XScrollControl.scrollLeft += _selectedCellElement.offsetWidth;
                }
            }
        }
    }

    function scrollDown() {
        var crosstabElement = document.getElementById('grid-' + crosstabWidget.id);
        if (crosstabElement) {
            var YScrollControl = crosstabElement.getElementsByClassName('body')[0];
            if (YScrollControl) {
                if (getElementChildIndex(_selectedRowElement) > Math.round(YScrollControl.offsetHeight / _rowHeight - 1)) {
                    YScrollControl.scrollTop += _rowHeight;
                }
            }
        }
    }

    function scrollUp() {
        var crosstabElement = document.getElementById('grid-' + crosstabWidget.id);
        if (crosstabElement) {
            var YScrollControl = crosstabElement.getElementsByClassName('body')[0];
            if (YScrollControl) {
                if (getElementChildIndex(_selectedRowElement) < Math.round(YScrollControl.scrollTop / _rowHeight)) {
                    YScrollControl.scrollTop -= _rowHeight;
                }
            }
        }
    }

    function getElementChildIndex(element) {
        if (element) {
            for (var i = 0;
                (element = element.previousElementSibling); i++);
            return i;
        } else {
            return false;
        }
    }

    function openCell() {
        _gridElement = _gridElement || document.getElementsByClassName('crosstab-body-content')[0];
        if (_selectedCellElement && canMoveCell()) {
            _selectedCellElement.click();
        }
    }

    function getFirstRowElement(gridElement) {
        var firstRowElement = false;
        var rowElements = gridElement.children;
        if (rowElements.length) {
            firstRowElement = rowElements[0];
        }
        return firstRowElement;
    }

    function getFirstCellElement(rowElement) {
        var firstCellElement = false;
        var cellElements = rowElement.children;
        if (cellElements.length) {
            var currentElement = cellElements[0];
            while (!firstCellElement && currentElement) {
                if (!currentElement.classList.contains('fixed')) {
                    firstCellElement = currentElement;
                } else {
                    currentElement = currentElement.nextElementSibling;
                }
            }
        }
        return firstCellElement;
    }

    this.updateCustomElements = function(widget) {
        resetCrosstabNavigation();
        if (widget && !widget.defaultReload) {
            widget.defaultReload = widget.reload;
            widget.reload = crosstabCustomReload;
        }
        crosstabWidget = widget;
    };

    function crosstabCustomReload(additionalFilters, shouldRefreshCurrentRow, keepCopiedCell) {
        ScreenService.changeLoadingMessage(ScreenService.i18n("Carregando cardápio..."));
        ScreenService.showLoader();
        return crosstabWidget.defaultReload(additionalFilters, shouldRefreshCurrentRow).then(function() {
            if (!keepCopiedCell) {
                _copiedCellElement = false;
                _copiedRowElement = false;
            }
            setTimeout(function() {
                restoreCrosstabNavigation();
                crosstabWidget.activate();
                ScreenService.hideLoader();
                ScreenService.restoreDefaultLoadingMessage();
            }, 200);
        });
    }

    this.retornaRowSelecionada = function() {
        var cellIndex = getElementChildIndex(_selectedCellElement);
        if (cellIndex) {
            var rowCelula = getDataRowByIndex(crosstabWidget.crosstabData, _selectedRowElement._childIndex, cellIndex, crosstabWidget.crosstabColumns);
            return rowCelula;
        } else {
            return false;
        }
    };

    this.isClickOnFocusedCell = function() {
        var targetElement = window.event.target.closest('.td.zh-standard-column');
        return targetElement && targetElement == _selectedCellElement;
    };

    this.setCurrentNavigationElement = function() {
        if (_selectedCellElement) {
            _selectedCellElement.classList.remove('selected-cell');
        }
        var target = window.event.target;
        var cell = target.closest('.td.zh-standard-column');
        if (cell) {
            var row = cell.closest('.tr');
            if (row) {
                _selectedRowElement = row;
                _selectedRowElement._childIndex = getElementChildIndex(_selectedRowElement);
                _selectedCellElement = cell;
                _selectedCellElement.classList.add('selected-cell');
            }
        }
    };

    var reportWidget = {
        "id": 0,
        "name": "reportWidget",
        "label": "Tipo de Relatório",
        "template": "widget/form.html",
        "container": {},
        "isVisible": true,
        "popupNoBlock": false,
        "fields": [],
        "actions": [],
        "events": []
    };
    var reportFields = [
        {
            "id": 99999999,
            "name": "BIRTTYPE",
            "label": "",
            "class": 12,
            "readOnly": false,
            "isVisible": true,
            "isVisibleForm": true,
            "isVisibleGrid": false,
            "template": "field/radio-group.html",
            "events": [],
            "valueField": "value",
            "descriptionField": "name",
            "defaultOption": 0,
            "dataSource": {
                "data": [
                    {
                        "name": "pdf",
                        "value": "pdf"
                    },
                    {
                        "name": "xlsx",
                        "value": "xlsx"
                    }
                ]
            }
        }
    ];
    var reportActions = [
        {
            "name": "cancelar",
            "label": "Cancelar",
            "idVisible": true,
            "hideIcon": true,
            "showAsAction": "back",
            "events": [
                {
                    "name": "ActionEvent",
                    "code": "ScreenService.closePopup();"
                }
            ]
        },
        {
            "name": "confirmar",
            "label": "Confirmar",
            "idVisible": true,
            "hideIcon": true,
            "showAsAction": "never",
            "events": [
                {
                    "name": "ActionEvent",
                    "code": "TcoUtil.generateReport(args.owner.widget, args);"
                }
            ]
        }
    ];

    var reportCallback;
    var reportParameters;
    var reportWidgetParentName;

    this.openReportPopup = function(widget, callback, parameters) {

        reportCallback = callback;
        reportParameters = parameters || [];
        reportWidgetParentName = widget.name;
        reportWidget.container = widget.container;
        reportWidget.fields = reportFields;
        reportWidget.actions = reportActions;

        var reportWidgetInitialized = metaDataFactory.widgetFactory(reportWidget);

        ScreenService.openPopup(reportWidgetInitialized);
    };

    this.generateReport = function(widget, args) {

        var reportWidgetParentCurrentRow = widget.container.getWidget(reportWidgetParentName).currentRow;

        reportWidgetParentCurrentRow.BIRTTYPE = widget.currentRow.BIRTTYPE;

        args.row = reportWidgetParentCurrentRow;

        if (typeof reportCallback == 'function') {

            reportCallback.apply(null, reportParameters);
        }

        ScreenService.closePopup();

    };

    /*
     * Formata o texto usando máscara separada por pontos e retorna texto formatado
     *
     */

    this.formatString = function(text, mask, removeEmptyPlaceholders = true) {
        if (!text || !mask) return text;
        let newText = text;
        let regex = '';
        let toReplace = '';
        mask.match(/[^\.]+/g).forEach((math, index) => {
            regex += index === 0 ? `(.{${math.length}})` : `(.{0,${math.length}})`;
            toReplace += index !== 0 ? '.' : toReplace;
            toReplace += '$' + (index + 1);
        });
        regex = new RegExp(regex);
        newText = newText.replace(regex, toReplace);
        if (removeEmptyPlaceholders)
            newText = newText.replace(/[.]+$/, '');
        return newText;
    };

    this.clearFilterReport = function(args) {
        args.e.metadata.filter = [];
    };

    function setDefaultValuesConfig(stripeWidget) {
        if (!stripeWidget || stripeWidget == 'DEFAULT') {
            stripeWidget = templateManager.container.widgets.filter(widget =>
                widget.template && widget.template.toLowerCase().indexOf('stripe') > -1
            )[0];
        } else {
            stripeWidget = typeof stripeWidget === 'string' ? templateManager.container.getWidget(stripeWidget) : stripeWidget;
        }
        stripeWidget.floatingControlConfig = stripeWidget.floatingControlConfig || {};
        stripeWidget.floatingControlConfig.dataset = stripeWidget.floatingControlConfig.dataset || [];
        stripeWidget.floatingControlConfig.rangeFields = stripeWidget.floatingControlConfig.rangeFields || [];
        stripeWidget.floatingControlConfig.ignoreFieldsStripe = stripeWidget.floatingControlConfig.hasOwnProperty('ignoreFieldsStripe') ? stripeWidget.floatingControlConfig.ignoreFieldsStripe : false;
        return stripeWidget;
    }

    this.setFilterReport = function(args, stripeWidget) {
        stripeWidget = setDefaultValuesConfig(stripeWidget);
        args.e.metadata.filter = stripeWidget.floatingControlConfig.dataset;
    };

    function treatLabel(label) {
        return ScreenService.i18n(label).split(':').join('');
    }
    /*
        Inserir essa função apos aplicar o filtro, seja no FilterAfterApply ou em um OnChange caso seja uma 
        field que filtra a tela ao inves de um popup de filtro
        
        @params
        stripeWidget => nome da widget de stripe ou o objecto em si
        row (opcional) => row contendo os dados que vão para o filtro do relatorio
        fieldsToUse (opcional) => array com nome de fields a serem exibidas no filtro do relatorio
        ['NMFILIAL']
        default:
            ele exibe todas as fields visiveis do stripe no filtro do relatorio
        Issue:
            stripeWidget por default procurar por uma widget com template widget/stripe.html
    */
    this.newFilterReport = function(stripeWidget, row, fieldsToUse) {
        stripeWidget = setDefaultValuesConfig(stripeWidget);
        let config = stripeWidget.floatingControlConfig;
        let dataSet = [];
        row = row || stripeWidget.currentRow;
        let arrayKeysRangeFields = Object.keys(config.rangeFields);
        let allFieldsRange = arrayKeysRangeFields.join('_');
        if (Array.isArray(arrayKeysRangeFields) && arrayKeysRangeFields.length) {
            arrayKeysRangeFields.forEach(rangeField => {
                let dataSets = {
                    $I: {
                        propertyToGet: 'INDEX',
                        value: config.rangeFields[rangeField].itemsToReplace,
                    },
                    $V: {
                        propertyToGet: 'NAMEFIELD',
                        value: row
                    }
                };
                let fieldsRange = rangeField.split('_');
                let treatStr = config.rangeFields[rangeField].format.replace(/\$[A-Z]+\d+/g, function(currentMatch) {
                    let cKey = currentMatch.match(/(\$[A-Za-z]+)/g)[0];
                    let arrayValues = dataSets[cKey];
                    let index = currentMatch.replace(cKey, '');
                    let currentItem = arrayValues.propertyToGet == 'INDEX' ? arrayValues.value[index] : arrayValues.value[fieldsRange[index]];
                    return ScreenService.i18n(currentItem);
                });
                dataSet.push({
                    label: treatLabel(config.rangeFields[rangeField].label),
                    value: treatStr
                });
            });
        }
        if (Array.isArray(fieldsToUse) && fieldsToUse.length) {
            fieldsToUse.forEach(fieldName => {
                let rangeContainsCurrentField = new RegExp(`(?<!([A-Z]+))(${fieldName})(?!([A-Z]+))`, 'gm').test(allFieldsRange);
                if (!rangeContainsCurrentField) {
                    let field = stripeWidget.getField(fieldName);
                    dataSet.push({
                        label: treatLabel(field.label),
                        value: row[field.name]
                    });
                }
            });
        }
        if (!config.ignoreFieldsStripe) {
            stripeWidget.fields.forEach(field => {
                if (field.isVisible) {
                    dataSet.push({
                        label: treatLabel(field.label),
                        value: row[field.name]
                    });
                }
            });
        }
        config.dataset = dataSet;
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('TcoUtil', TcoUtil);
});

// FILE: ./assets/js/services/Tco00100_ramo_atividade_segmento_service.js
var Tco00100_ramo_atividade_segmento_service = function(ScreenService, General, restEngine) {

    this.findParamgeral = function(dataSet) {
        return restEngine.post('tco#/crud/paramgeral', {
            "requestType": "FilterData",
            "filter": null
        });
    };

    this.salvaNovosRamosAtividade = function(dataSet) {
        return restEngine.post('tco#/ativclisegmer_ativcliente/save', {
            "requestType": "DataSet",
            "dataset": dataSet
        });
    };

    this.removeRamosAtividade = function(dataSet) {
        return restEngine.post('tco#/ativclisegmer_ativcliente/remove', {
            "requestType": "DataSet",
            "dataset": dataSet
        });
    };

    this.mudarEstado = function(dataSet) {
        return restEngine.post('tco#/ativclisegmer_ativcliente/mudarEstado', {
            "requestType": "DataSet",
            "dataset": dataSet
        });
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco00100_ramo_atividade_segmento_service', Tco00100_ramo_atividade_segmento_service);
});

// FILE: ./assets/js/services/Tco00200_porcionamento_service.js
var Tco00200_porcionamento_service = function(ScreenService, General, restEngine) {

    this.salvaPorcionamento = function(dataSet) {
        return restEngine.post('tco#/porcionamento/save', {
            "requestType": "DataSet",
            "dataset": dataSet
        });
    };

    this.apagaPorcionamento = function(dataSet) {
        return restEngine.post('tco#/porcionamento/delete', {
            "requestType": "DataSet",
            "dataset": dataSet
        });
    };

    this.adicionarRelPorcioGrTipoPr = function(dataSet) {
        return restEngine.post('tco#/relPorcioGrTipoPr/save', {
            "requestType": "DataSet",
            "dataset": dataSet
        });
    };

    this.apagarRelPorcioGrTipoPr = function(dataSet) {
        return restEngine.post('tco#/relPorcioGrTipoPr/delete', {
            "requestType": "DataSet",
            "dataset": dataSet
        });
    };
};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco00200_porcionamento_service', Tco00200_porcionamento_service);
});

// FILE: ./assets/js/services/Tco00300_servico_proposta_service.js
var Tco00300_servico_proposta_service = function(ScreenService, General, restEngine) {

    this.salvaNovoServicoProposta = function(row) {
        return restEngine.post('tco#/servicoproposta/save', {
            "requestType": "Row",
            "row": row
        });
    };

    this.removeServicoProposta = function(row) {
        return restEngine.post('tco#/servicoproposta/remove', {
            "requestType": "Row",
            "row": row
        });
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco00300_servico_proposta_service', Tco00300_servico_proposta_service);
});

// FILE: ./assets/js/services/Tco00400_tipo_contrato_service.js
var Tco00400_tipo_contrato_service = function(ScreenService, General, restEngine) {

    this.salvaTipoContrato = function(row) {
        return restEngine.post('tco#/tipocontrato/save', {
            "requestType": "Row",
            "row": row
        });
    };

    this.removeTipoContrato = function(dataSet) {
        return restEngine.post('tco#/tipocontrato/remove', {
            "requestType": "DataSet",
            "dataset": dataSet
        });
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco00400_tipo_contrato_service', Tco00400_tipo_contrato_service);
});

// FILE: ./assets/js/services/Tco00500_padrao_qualidade_service.js
var Tco00500_padrao_qualidade_service = function(ScreenService, General, restEngine) {

    this.salvaNovoPadraoQualidade = function(row) {
        return restEngine.post('tco#/padraoqualidade/save', {
            "requestType": "Row",
            "row": row
        });
    };

    this.removePadraoQualidade = function(row) {
        return restEngine.post('tco#/padraoqualidade/remove', {
            "requestType": "Row",
            "row": row
        });
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco00500_padrao_qualidade_service', Tco00500_padrao_qualidade_service);
});

// FILE: ./assets/js/services/Tco00600_gerente_negocio_service.js
var Tco00600_gerente_negocio_service = function(ScreenService, General, restEngine) {

    /**
     * Microservice para adicionar gerentes.
     * 
     * @param {Array[Object]} dataSet Array de objetos
     *  contendo dados para adição.
     */
    this.saveManager = function(dataSet) {
        return restEngine.post('tco#/tco00600_gerente_negocio/saveManager', {
            "requestType": "DataSet",
            "dataset": dataSet
        });
    };

    /**
     * Microservice para remover gerentes.
     * 
     * @param {Object} row Objeto contendo 
     *  dados para remoção.
     */
    this.removeManager = function(row) {
        return restEngine.post('tco#/tco00600_gerente_negocio/removeManager', {
            "requestType": "Row",
            "row": row
        });
    };

    /**
     * Microservice para adicionar gerentes à 
     * liderança de outro.
     * 
     * @param {Object} row Objeto contendo 
     *  dados para adiçao.
     */
    this.addToManagerGrid = function(row) {
        return restEngine.post('tco#/tco00600_gerente_negocio/addToManagerGrid', {
            "requestType": "Row",
            "row": row
        });
    };

    /**
     * Microservice para remover gerentes da
     * liderança de outro.
     * 
     * @param {Object} row Objeto contendo 
     *  dados para remoção.
     */
    this.removeFromManagerGrid = function(row) {
        return restEngine.post('tco#/tco00600_gerente_negocio/removeFromManagerGrid', {
            "requestType": "Row",
            "row": row
        });
    };

    /**
     * Microservice para alterar a liderança dos
     * colaboradores.
     * 
     * @param {Object} row Objeto contendo 
     * dados para a alteração.
     */
    this.changeManager = function(row) {
        return restEngine.post('tco#/tco00600_gerente_negocio/changeManager', {
            "requestType": "Row",
            "row": row
        });
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco00600_gerente_negocio_service', Tco00600_gerente_negocio_service);
});

// FILE: ./assets/js/services/Tco00800_prospect_service.js
function Tco00800_prospect_service(restEngine) {

    /**
     * Microservice para salvar alterações/
     * inserções de gerentes.
     * 
     * @param {Object} row Objeto contendo 
     *  dados para remoção.
     */
    this.saveProspect = function(row) {
        return restEngine.post('tco#/tco00800Prospect/save', {
            "requestType": "DataSet",
            "dataset": row
        });
    };

    /**
     * Microservice para remover prospects.
     * 
     * @param {Object} row Objeto contendo 
     *  dados para remoção.
     */
    this.removeProspect = function(row) {
        return restEngine.post('tco#/tco00800Prospect/remove', {
            "requestType": "Row",
            "row": row
        });
    };

    /**
     * Microservice para salvar alterações/
     * inserções de endereços.
     * 
     * @param {Object} row Objeto contendo 
     *  dados para remoção.
     */
    this.saveAddress = function(row) {
        return restEngine.post('tco#/tco00800Prospect/saveAddress', {
            "requestType": "Row",
            "row": row
        });
    };

    /**
     * Microservice para remover endereço 
     * de prospects.
     * 
     * @param {Object} row Objeto contendo 
     *  dados para remoção.
     */
    this.removeAddress = function(row) {
        return restEngine.post('tco#/tco00800Prospect/removeAddress', {
            "requestType": "Row",
            "row": row
        });
    };

}

Configuration(function(ContextRegister) {
    ContextRegister.register("Tco00800_prospect_service", Tco00800_prospect_service);
});

// FILE: ./assets/js/services/Tco00900ParametrosComerciaisService.js
var Tco00900ParametrosComerciaisService = function(ScreenService, General, restEngine) {

    this.findGeral = function() {
        return General.callRestEngine('tco#/crud/tco00900_paramtco', "FilterData", []);
    };

    this.saveGeral = function(dataSet) {
        return General.callRestEngine('tco#/tco00900_paramtco/save', "DataSet", [dataSet]);
    };

    this.findLimitador = function() {
        return General.callRestEngine('tco#/crud/tco00900_findLimitador', "FilterData", []);
    };

    this.saveLimitadorCustos = function(dataSet) {
        return General.callRestEngine('tco#/tco00900_findLimitador/save', "DataSet", [dataSet]);
    };

    this.saveQtdPorRefeicao = function(dataSet) {
        return General.callRestEngine('tco#/tco00900_qtdPorRefeicao/save', "DataSet", [dataSet]);
    };

    this.removeQtdPorRefeicao = function(dataSet) {
        return General.callRestEngine('tco#/tco00900_qtdPorRefeicao/remove', "DataSet", [dataSet]);
    };
};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco00900ParametrosComerciaisService', Tco00900ParametrosComerciaisService);
});

// FILE: ./assets/js/services/Tco01000_cardapio_base_service.js
var Tco01000_cardapio_base_service = function(ScreenService, General, restEngine) {

    this.salvaNovoCardapioBase = function(row) {
        return restEngine.post('tco#/cardapiobase/save', {
            "requestType": "Row",
            "row": row
        });
    };

    this.removeCardapioBase = function(row) {
        return restEngine.post('tco#/cardapiobase/remove', {
            "requestType": "Row",
            "row": row
        });
    };

    this.verificaComposicaoCardapioBaseExistente = function(row) {
        return restEngine.post('tco#/cardapiobase/verifica', {
            "requestType": "Row",
            "row": row
        });
    };

    this.addOrUpdatePratoTrab = function(dataset) {
        return restEngine.post('tco#/addOrUpdatePratoTrab', {
            "requestType": "DataSet",
            "dataset": dataset
        });
    };
};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01000_cardapio_base_service', Tco01000_cardapio_base_service);
});

// FILE: ./assets/js/services/Tco01001_elaboracao_cardapio_service.js
var Tco01001_elaboracao_cardapio_service = function(ScreenService, General, restEngine) {

    this.alteraCardapioBase = function(row) {
        return restEngine.post('tco#/cardapiobaseelaboracao/save', {
            "requestType": "Row",
            "row": row
        });
    };

    this.importarCardapioBase = function(row) {
        return restEngine.post('tco#/cardapiobaseelaboracao/importproc', {
            "requestType": "Row",
            "row": row
        });
    };

    this.excluirPratoCardapioBase = function(row) {
        return restEngine.post('tco#/cardapiobaseelaboracao/removeproc', {
            "requestType": "Row",
            "row": row
        });
    };

    this.copiaPratos = function(row) {
        return restEngine.post('tco#/cardapiobaseelaboracao/copiaPratos', {
            "requestType": "Row",
            "row": row
        });
    };

    this.replicaPratos = function(row) {
        return restEngine.post('tco#/cardapiobaseelaboracao/replicaPratos', {
            "requestType": "Row",
            "row": row
        });
    };

    this.alteraDiasElaboracaoCardapio = function(row) {
        return restEngine.post('tco#/cardapiobaseelaboracao/alteradiaselaboracaocardapio', {
            "requestType": "Row",
            "row": row
        });
    };

    this.gerarRelatorioCardapioBase = function(row) {
        return restEngine.post('tco#/tco01001elaboracaocardapio/relatoriocardapiobase', {
            "requestType": "Row",
            "row": row
        });
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01001_elaboracao_cardapio_service', Tco01001_elaboracao_cardapio_service);
});

// FILE: ./assets/js/services/Tco01002_configuracao_cardapio_service.js
var Tco01002_configuracao_cardapio_service = function(ScreenService, General, restEngine) {

    this.salvaConfiguracaoCardapioBase = function(row) {
        return restEngine.post('tco#/configcardbase/save', {
            "requestType": "Row",
            "row": row
        });
    };

    this.removeConfigCardapioBase = function(dataset) {
        return restEngine.post('tco#/configcardbase/remove', {
            "requestType": "DataSet",
            "dataset": dataset
        });
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01002_configuracao_cardapio_service', Tco01002_configuracao_cardapio_service);
});

// FILE: ./assets/js/services/Tco01003_dados_do_prato_service.js
var Tco01003_dados_do_prato_service = function(ScreenService, General, restEngine) {

    this.removePratCompCardBase = function(row) {
        return restEngine.post('tco#/pratcompcardbase/remove', {
            "requestType": "Row",
            "row": row
        });
    };

    this.definirReferenciaPrato = function(row) {
        return restEngine.post('tco#/pratcompcardbase/save', {
            "requestType": "Row",
            "row": row
        });
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01003_dados_do_prato_service', Tco01003_dados_do_prato_service);
});

// FILE: ./assets/js/services/Tco01004_novo_prato_elaboracao_cardapio_base_service.js
var Tco01004_novo_prato_elaboracao_cardapio_base_service = function(ScreenService, General, restEngine) {

    this.retornaReceitas = function(row) {
        return restEngine.post('tco#/retornaReceitas', {
            "requestType": "Row",
            "row": row
        });
    };

    this.retornaMascaraNivelPrato = function() {
        return restEngine.post('tco#/retornaMascaraNivelPrato', {
            "requestType": "Row",
            "row": {}
        });
    };

    this.salvaNovoPratoComp = function(row) {
        return restEngine.post('tco#/salvaNovoPratoComp', {
            "requestType": "Row",
            "row": row
        });
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01004_novo_prato_elaboracao_cardapio_base_service', Tco01004_novo_prato_elaboracao_cardapio_base_service);
});

// FILE: ./assets/js/services/Tco01005_regioes_prato_elaboracao_cardapio_base_service.js
var Tco01005_regioes_prato_elaboracao_cardapio_base_service = function(ScreenService, General, restEngine) {

    this.removeRegiaoPrato = function(rows) {
        return restEngine.post('tco#/regiaopratcardbase/remove', {
            "requestType": "DataSet",
            "dataset": rows
        });
    };

    this.salvaRegiaoPratoSelecionada = function(dataset) {
        return restEngine.post('tco#/regiaopratcardbase/save', {
            "requestType": "DataSet",
            "dataset": dataset
        });
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01005_regioes_prato_elaboracao_cardapio_base_service', Tco01005_regioes_prato_elaboracao_cardapio_base_service);
});

// FILE: ./assets/js/services/Tco01006_regras_cardapio_service.js
/*global Configuration*/
var Tco01006_regras_cardapio_service = function(ScreenService, General, restEngine) {

    this.salvaRegraAdesao = function(row) {
        return restEngine.post('tco#/regraAdesao/save', {
            "requestType": "Row",
            "row": row
        }).then(function(callback) {
            return callback.dataset.chaves;
        });
    };

    this.apagaRegraAdesao = function(dataSet) {
        return restEngine.post('tco#/regraAdesao/delete', {
            "requestType": "DataSet",
            "dataset": dataSet
        });
    };

    this.retornaCamposDinamicosAdesao = function(row) {
        return restEngine.post('tco#/regraAdesao/retornaCamposDinamicosAdesao', {
            "requestType": "Row",
            "row": row
        }).then(function(callback) {
            return callback.dataset;
        });
    };

    this.salvaRegraGramatura = function(row) {
        return restEngine.post('tco#/regraGramatura/save', {
            "requestType": "Row",
            "row": row
        }).then(function(callback) {
            return callback.dataset.chaves;
        });
    };

    this.apagaRegraGramatura = function(row) {
        return restEngine.post('tco#/regraGramatura/delete', {
            "requestType": "Row",
            "row": row
        });
    };

    this.salvaGramaturaAgrupamento = function(row) {
        return restEngine.post('tco#/regraGramatura/salvaGramaturaAgrupamento', {
            "requestType": "Row",
            "row": row
        });
    };

    this.retornaCamposDinamicosGramatura = function(row) {
        return restEngine.post('tco#/regraGramatura/retornaCamposDinamicosGramatura', {
            "requestType": "Row",
            "row": row
        }).then(function(callback) {
            return callback.dataset;
        });
    };

    this.salvaRegraParticularidade = function(row) {
        return restEngine.post('tco#/regraParticularidade/save', {
            "requestType": "Row",
            "row": row
        }).then(function(callback) {
            return callback.dataset;
        });
    };

    this.apagaRegraParticularidade = function(row) {
        return restEngine.post('tco#/regraParticularidade/delete', {
            "requestType": "Row",
            "row": row
        });
    };

    this.salvaTipoPratoPart = function(dataSet) {
        return restEngine.post('tco#/regraParticularidade/salvaTipoPratoPart', {
            "requestType": "DataSet",
            "dataset": dataSet
        });
    };

    this.getDescartaveisData = function(filterRow) {
        return restEngine.post('tco#/regraDescartaveis/getData', {
            "requestType": "Row",
            "row": filterRow
        });
    };

    this.saveChangesAmostra = function(row) {
        return restEngine.post('tco#/regraDescartaveis/saveAmostra', {
            "requestType": "Row",
            "row": row
        });
    };

    this.changeFormaCalculo = function(row) {
        return restEngine.post('tco#/regraDescartaveis/changeFormaCalculo', {
            "requestType": "Row",
            "row": row
        });
    };

    this.saveLimpeza = function(dataSet) {
        return restEngine.post('tco#/regraDescartaveis/saveLimpeza', {
            "requestType": "DataSet",
            "dataset": dataSet
        });
    };

    this.removeLimpezaCardBase = function(dataSet) {
        return restEngine.post('tco#/regraDescartaveis/removeLimpezaCardBase', {
            "requestType": "DataSet",
            "dataset": dataSet
        });
    };

    this.findParamTco = function() {
        return General.callRestEngine('tco#/crud/paramtco', "FilterData", []).then(function(callback) {
            return callback.dataset.paramtco;
        });
    };

    this.duplicaRegraAdesao = function(row) {
        return restEngine.post('tco#/regraAdesao/duplicaRegraAdesao', {
            "requestType": "Row",
            "row": row
        }).then(function(callback) {
            return callback.dataset.chaves;
        });
    };

    this.importaItemGramatura = function(row) {
        return restEngine.post('tco#/regraGramatura/importaitemgramatura', {
            "requestType": "Row",
            "row": row
        });
    };
    this.saveAddPorcGram = function(row) {
        return restEngine.post('tco#/Tco01006_regras_cardapio_service/saveAddPorcGram', {
            "requestType": "Row",
            "row": row
        });
    };
};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01006_regras_cardapio_service', Tco01006_regras_cardapio_service);
});

// FILE: ./assets/js/services/Tco01007_utiliza_substitui_pratos_cardapio_base_service.js
var Tco01007_utiliza_substitui_pratos_cardapio_base_service = function(ScreenService, General, restEngine) {

    this.substituirPratoCardapioBase = function(dataset) {
        return restEngine.post('tco#/pratcompcardbase/substitui', {
            "requestType": "DataSet",
            "dataset": dataset
        });
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01007_utiliza_substitui_pratos_cardapio_base_service', Tco01007_utiliza_substitui_pratos_cardapio_base_service);
});

// FILE: ./assets/js/services/Tco01008_cardapios_complementares_service.js
var Tco01008_cardapios_complementares_service = function(ScreenService, General, restEngine) {

    this.salvaCnfCComplCardapioBase = function(dataset) {
        return restEngine.post('tco#/cnfccomplcardbase/save', {
            "requestType": "DataSet",
            "dataset": dataset
        });
    };

    this.salvaCardapiosComplementados = function(dataset) {
        return restEngine.post('tco#/salvacardapioscomplementados', {
            "requestType": "DataSet",
            "dataset": dataset
        });
    };

    this.removeCardapiosComplementados = function(dataset) {
        return restEngine.post('tco#/removecardapioscomplementados', {
            "requestType": "DataSet",
            "dataset": dataset
        });
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01008_cardapios_complementares_service', Tco01008_cardapios_complementares_service);
});

// FILE: ./assets/js/services/Tco01009_analise_custos_service.js
var Tco01009_analise_custos_service = function(ScreenService, General, restEngine) {

    this.findParamgeral = function() {
        return restEngine.post('tco#/crud/paramgeral', {
            "requestType": "FilterData",
            "filter": null
        });
    };

    this.executaProcedureAnaliseCusto = function(row) {
        return restEngine.post('tco#/procedureAnaliseCusto', {
            "requestType": "Row",
            "row": row
        }).then(function(callback) {
            return callback.dataset;
        });
    };

    this.criaRelatorioAnaliseCusto = function(row) {
        return restEngine.post('tco#/criaRelatorioAnaliseCusto', {
            "requestType": "Row",
            "row": row
        });
    };

    this.apagaCardapioTrab = function(row) {
        return restEngine.post('tco#/apagaCardapioTrab', {
            "requestType": "Row",
            "row": row
        }).then(function(callback) {
            return callback.dataset;
        });
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01009_analise_custos_service', Tco01009_analise_custos_service);
});

// FILE: ./assets/js/services/Tco01010_revisar_receita_card_base_service.js
var Tco01010_revisar_receita_card_base_service = function(ScreenService, General, restEngine) {

    this.saveProduct = function(params) {
        return restEngine.post('tco#/tco01010_saveProduct', {
            "requestType": "DataSet",
            "dataset": [params]
        });
    };

    this.getValuesStripe = function(row) {
        return restEngine.post('tco#/tco01010_getValuesStripe', {
            "requestType": "Row",
            "row": row
        });
    };

    this.replicaReceitas = function(row) {
        return restEngine.post('tco#/replicaReceitas', {
            "requestType": "Row",
            "row": row
        });
    };

    this.findParamgeral = function() {
        return restEngine.post('tco#/crud/paramgeral', {
            "requestType": "FilterData",
            "filter": null
        });
    };

    this.importarReceitaTCO = function(row) {
        return restEngine.post('tco#/importarReceitaTCO', {
            "requestType": "Row",
            "row": row
        });
    };
};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01010_revisar_receita_card_base_service', Tco01010_revisar_receita_card_base_service);
});

// FILE: ./assets/js/services/Tco01011_dados_prato_service.js
var Tco01011_dados_prato_service = function(ScreenService, General, restEngine) {

    this.retornaReceitas = function(row) {
        return restEngine.post('tco#/retornaReceitasSimulacao', {
            "requestType": "Row",
            "row": row
        });
    };
};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01011_dados_prato_service', Tco01011_dados_prato_service);
});

// FILE: ./assets/js/services/Tco01100_document_service.js
function Tco01100_document_service(restEngine) {

    /**
     * Realiza upload do arquivo para o 
     * backend.
     * 
     * @param {Array[Object]} fileRow Row contendo
     *  objeto do arquivo.
     */
    this.uploadFile = function(fileRow) {
        return restEngine.post('tco#/tco01100_document/uploadFile', {
            "requestType": "Row",
            "row": fileRow
        });
    };

    /**
     * Realiza download do arquivo.
     * 
     * @param {Array[Object]} fileRow Row contendo
     *  objeto do arquivo.
     */
    this.downloadFile = function(fileRow) {
        var response = restEngine.post('tco#/tco01100_document/downloadFile', {
            "requestType": "Row",
            "row": fileRow
        });

        response.then((success) => {
            let file = success.dataset ? success.dataset.fileToDownload : null;

            if (file) {
                saveDownloadedFile(file);
            }
        });

        return response;
    };

    /**
     * Salva arquivo no dispositivo do usuário.
     * 
     * @example Exemplo de arquivo:
     * {
     *     FILE_NAME : 'text.txt',
     *     FILE_TYPE : 'plain/text',
     *     FILE_B64 : 'SASD[...]as/S=='
     * }
     * 
     * @param {Object} file Objeto contendo
     *  parametros do arquivo.
     */
    var saveDownloadedFile = function(file) {
        let url = 'data:' + file.FILE_TYPE + ';base64,';
        url += encodeURI(file.FILE_B64);

        let a = document.createElement("a");
        a.style = "display: none";
        a.href = url;
        a.download = file.FILE_NAME;
        a.click();
        URL.revokeObjectURL(url);
    };

    /**
     * Remove o arquivo da lista de arquivos
     * do documento.
     * 
     * @param {Array[Object]} fileObject lista
     * de arquivos a serem removidos.
     */
    this.removeFile = function(fileObject) {
        return restEngine.post('tco#/tco01100_document/removeFile', {
            "requestType": "Row",
            "row": fileObject
        });
    };

}

Configuration(function(ContextRegister) {
    ContextRegister.register("Tco01100_document_service", Tco01100_document_service);
});

// FILE: ./assets/js/services/Tco01100_proposta_comercial_service.js
function Tco01100_proposta_comercial_service(restEngine) {

    /**
     * Retorna os dados da tabela paramgeral.
     */
    this.findParamgeral = function() {
        return restEngine.post('tco#/crud/paramgeral', {
            "requestType": "FilterData",
            "filter": null
        });
    };

    /**
     * Retorna os dados da tabela paramtco.
     */
    this.findParamTCO = function() {
        return restEngine.post('tco#/crud/paramtco', {
            "requestType": "FilterData",
            "filter": null
        });
    };

    /**
     * Retorna os dados de um cliente para preenchimento de formulário
     * 
     * @param {Filter} filter Array 
     */
    this.findDadosFormCliente = function(filter) {
        return restEngine.post('tco#/crud/dadosformcliente', {
            "requestType": "FilterData",
            "filter": filter
        });
    };

    /**
     * Retorna os dados de um prospect para preenchimento de formulário
     * 
     * @param {Filter} filter Array 
     */
    this.findDadosFormProspect = function(filter) {
        return restEngine.post('tco#/crud/dadosformprospect', {
            "requestType": "FilterData",
            "filter": filter
        });
    };

    /**
     * Salva inserção/alteração registro de 
     * proposta comercial.
     * 
     * @param {Row} row Objeto row contendo
     *  dados para registro.
     */
    this.saveProposta = function(row) {
        return restEngine.post('tco#/tco01100PropostaComercial/saveProposta', {
            "requestType": "Row",
            "row": row
        });
    };

    /**
     * Salva inserção registro de 
     * serviço de proposta comercial.
     * 
     * @param {Row} row Objeto row contendo
     *  dados para registro.
     */
    this.insertServicoPropCom = function(row) {
        return restEngine.post('tco#/tco01100PropostaComercial/insertServicoPropCom', {
            "requestType": "Row",
            "row": row
        });
    };

    /**
     * Remove registro de versão de
     * proposta comercial.
     * 
     * @param {Row} row Objeto row contendo
     *  dados para registro.
     */
    this.removeVersion = function(dataSet) {
        return restEngine.post('tco#/tco01100PropostaComercial/removeVersion', {
            "requestType": "DataSet",
            "dataset": dataSet
        });
    };

    /**
     * Remove registro de proposta comercial.
     * 
     * @param {Row} row Objeto row contendo
     *  dados para registro.
     */
    this.removeProposta = function(row) {
        return restEngine.post('tco#/tco01100PropostaComercial/removeProposta', {
            "requestType": "Row",
            "row": row
        });
    };

    /**
     * Remove registro de serviço de
     * proposta comercial.
     * 
     * @param {Row} row Objeto row contendo
     *  dados para registro.
     */
    this.removeServicoProposta = function(dataSet) {
        return restEngine.post('tco#/tco01100PropostaComercial/removeServicoProposta', {
            "requestType": "DataSet",
            "dataset": dataSet
        });
    };

    /**
     * Função responsável por retornar a 
     * região.
     * 
     * @param {Array[Object]} filter Filtro
     *  a ser aplicado.
     */
    this.getRegiao = function(filter) {
        return restEngine.post('tco#/crud/regiao', {
            "requestType": "FilterData",
            "filter": filter
        });
    };

    /**
     * Função responsável por retornar a 
     * Servicos da Proposta Comercial.
     * 
     * @param {Array[Object]} filter Filtro
     *  a ser aplicado.
     */
    this.getServicoPropostaComercial = function(filter) {
        return restEngine.post('tco#/crud/servicoPropostaComercial', {
            "requestType": "FilterData",
            "filter": filter
        });
    };

    /**
     * Função responsável por adicionar nova
     * versão à proposta comercial selecionada.
     * 
     * @param {Row} row Objeto row contendo
     *  dados para registro.
     */
    this.newVersion = function(row) {
        return restEngine.post('tco#/tco01100PropostaComercial/newVersion', {
            "requestType": "Row",
            "row": row
        });
    };

    /**
     * Função responsável por realizar rotina
     * de abertura de detalhe do serviço
     * 
     * @param {Row} row Objeto row contendo
     *  dados do registro.
     */
    this.openServPropComDetail = function(row) {
        return restEngine.post('tco#/tco01100PropostaComercial/openServPropComDetail', {
            "requestType": "Row",
            "row": row
        });
    };

    /**
     * Função responsável por alterar
     * responsavel pela proposta
     * comercial.
     * 
     * @param {Row} row Objeto row contendo
     *  dados do registro.
     */
    this.changeResponsavelPropCom = function(row) {
        return restEngine.post('tco#/tco01100PropostaComercial/changeResponsavelPropCom', {
            "requestType": "Row",
            "row": row
        });
    };

    /**
     * Rotina de notificação do planejamento.
     * 
     * @param {Row} row Objeto row contendo
     *  dados para registro.
     */

    this.notificarPlan = function(row) {
        return restEngine.post('tco#/tco01100PropostaComercial/notificarPlan', {
            "requestType": "Row",
            "row": row
        });
    };

    /**
     * Conclui versão selecionada
     * na tela de versão. 
     * @param {Row} row Objeto row contendo
     *  dados para registro.
     */
    this.concludeVersion = function(row) {
        return restEngine.post('tco#/tco01100PropostaComercial/concludeVersion', {
            "requestType": "Row",
            "row": row
        });
    };

    /**
     * Cancela conclusão versão selecionada
     * na tela de versão.
     * @param {Row} row Objeto row contendo
     *  dados para registro.
     */
    this.cancelConcludeVersion = function(row) {
        return restEngine.post('tco#/tco01100PropostaComercial/cancelConcludeVersion', {
            "requestType": "Row",
            "row": row
        });
    };

    /**
     * Aprova versão selecionada
     * na tela de versão.
     * @param {Row} row Objeto row contendo
     *  dados para registro.
     */
    this.aproveVersion = function(row) {
        return restEngine.post('tco#/tco01100PropostaComercial/aproveVersion', {
            "requestType": "Row",
            "row": row
        });
    };

    /**
     * Cancela aprovação versão selecionada
     * na tela de versão.
     * @param {Row} row Objeto row contendo
     *  dados para registro.
     */
    this.cancelAproveVersion = function(row) {
        return restEngine.post('tco#/tco01100PropostaComercial/cancelAproveVersion', {
            "requestType": "Row",
            "row": row
        });
    };

    /**
     * Verifica se pode ser aberta widget
     * de aprovação.
     * 
     * @param {Row} row Objeto row contendo
     *  dados para registro.
     */
    this.openAproveWidget = function(row) {
        return restEngine.post('tco#/tco01100PropostaComercial/openAproveWidget', {
            "requestType": "Row",
            "row": row
        });
    };

    /**
     * Duplica versão da proposta comercial.
     * 
     * @param {Row} row Objeto row contendo
     *  dados para registro.
     */
    this.duplicateVersion = function(dataSet) {
        return restEngine.post('tco#/tco01100PropostaComercial/duplicateVersion', {
            "requestType": "DataSet",
            "dataset": dataSet
        });
    };

    /**
     * 
     */
    this.chamadaRelatorio = function(row) {
        return restEngine.post('tco#/tco01100PropostaComercial/openreport', {
            "requestType": "Row",
            "row": row
        });
    };

    /**
     * Chama a rota para importação dos Serviços.
     * 
     * @param {Row} row Objeto row de dados necessários para a importação.
     */
    this.importServicos = function(row) {
        return restEngine.post('tco#/tco01100PropostaComercial/importServicos', {
            "requestType": "Row",
            "row": row
        });
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register("Tco01100_proposta_comercial_service", Tco01100_proposta_comercial_service);
});

// FILE: ./assets/js/services/Tco01101_detalhes_service.js
var Tco01101_detalhes_service = function(restEngine) {

    /**
     * Salva alteração no registro de 
     * serviço de proposta comercial.
     * 
     * @param {Row} row Objeto row contendo
     *  dados para registro.
     */
    this.saveEditServicoProposta = function(row) {
        return restEngine.post('tco#/tco01101DetalhesServico/saveEditServicoProposta', {
            "requestType": "Row",
            "row": row
        });
    };

    /**
     * Executa a rotina de calculo do custo do
     * serviço.
     * 
     * @param {Row} row Objeto row contendo
     *  dados para registro.
     */
    this.calcularCustServ = function(row) {
        return restEngine.post('tco#/tco01101DetalhesServico/calcularCustServ', {
            "requestType": "Row",
            "row": row
        });
    };

    /**
     * Salva alteração nos registros de 
     * tipo de prato do serviço da proposta.
     * 
     */
    this.saveEditTipoPrat = function(dataSet) {
        return restEngine.post('tco#/tco01101DetalhesServico/saveEditTipoPrat', {
            "requestType": "DataSet",
            "dataset": dataSet
        });
    };

    /**
     * Salva alterações de porcionamento e gramatura
     * para o grupo de tipo de prato relacionado.
     * 
     * @param {Row} row Objeto row contendo
     *  dados para registro.
     */
    this.saveAddPorcGram = function(row) {
        return restEngine.post('tco#/tco01101DetalhesServico/saveAddPorcGram', {
            "requestType": "Row",
            "row": row
        });
    };

    /**
     * Confirma envio de solicitação 
     * de customização.
     * 
     * @param {Row} row Objeto row contendo
     *  dados para registro.
     */
    this.confirmCustomize = function(row) {
        return restEngine.post('tco#/tco01101DetalhesServico/confirmCustomize', {
            "requestType": "Row",
            "row": row
        });
    };

    /**
     * Envia resposta ao operador do planejamento
     * configurado.
     * 
     * @param {Row} row Objeto row contendo
     *  dados para registro.
     */
    this.enviaResposta = function(row) {
        return restEngine.post('tco#/tco01101DetalhesServico/enviaResposta', {
            "requestType": "Row",
            "row": row
        });
    };

    /**
     * Altera marca do produto.
     * 
     * @param {Row} row Objeto row contendo
     *  dados para registro.
     */
    this.selectMarca = function(row) {
        return restEngine.post('tco#/tco01101DetalhesServico/selectMarca', {
            "requestType": "Row",
            "row": row
        });
    };

    /**
     * Altera marca do produto.
     * 
     * @param {Row} row Objeto row contendo
     *  dados para registro.
     */
    this.clearMarcaProduto = function(row) {
        return restEngine.post('tco#/tco01101DetalhesServico/clearMarcaProduto', {
            "requestType": "Row",
            "row": row
        });
    };

    /**
     * Rotina de notificação do planejamento.
     * 
     * @param {Row} row Objeto row contendo
     *  dados para registro.
     */
    this.notificarPlan = function(row) {
        return restEngine.post('tco#/tco01101DetalhesServico/notificarPlan', {
            "requestType": "Row",
            "row": row
        });
    };

    /**
     * Salva incidencias alteradas.
     * 
     * @param {dataSet} dataSet
     */
    this.saveIncidencias = function(dataSet) {
        return restEngine.post('tco#/tco01101DetalhesServico/saveIncidencias', {
            "requestType": "DataSet",
            "dataset": dataSet
        });
    };

    /**
     * Abre widget de serviços complementares.
     * 
     * @param {Object} row
     */
    this.openServCompl = function(row) {
        return restEngine.post('tco#/tco01101DetalhesServico/openServCompl', {
            "requestType": "Row",
            "row": row
        });
    };

    /**
     * Selecionar serviços complementares
     * para integrar custo do serviço.
     * 
     * @param {Object} row
     */
    this.selectServCompl = function(row) {
        return restEngine.post('tco#/tco01101DetalhesServico/selectServCompl', {
            "requestType": "Row",
            "row": row
        });
    };

    /**
     * Obter última solicitação feita planejamento ou GN
     * 
     * @param {Object} row
     */
    this.getUltimaSolicitacao = function(row) {
        return restEngine.post('tco#/tco01101DetalhesServico/getUltimaSolicitacao', {
            "requestType": "Row",
            "row": row
        });
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("Tco01101_detalhes_service", Tco01101_detalhes_service);
});

// FILE: ./assets/js/services/Tco01200CardapioDaPropostaService.js
var Tco01200CardapioDaPropostaService = function(ScreenService, General, restEngine) {

    this.findObservacoesServComercial = function(filter) {
        return General.callRestEngine('tco#/crud/observacoesServComercial', "FilterData", filter);
    };

    this.saveConfigTipoPrato = function(data) {
        return General.callRestEngine('tco#/saveConfigTipoPrato', "DataSet", [data]);
    };

    this.deleteConfigTipoPrato = function(data) {
        return General.callRestEngine('tco#/deleteConfigTipoPrato', "DataSet", [data]);
    };

    this.getDadosPrato = function(row) {
        return General.callRestEngine('tco#/getDadosPrato', "Row", row);
    };

    this.addOrUpdatePrato = function(data) {
        return General.callRestEngine('tco#/addOrUpdatePrato', "DataSet", [data]);
    };

    this.removePrato = function(data) {
        return General.callRestEngine('tco#/removePrato', "DataSet", [data]);
    };

    this.confirmReview = function(data) {
        return General.callRestEngine('tco#/confirmReview', "DataSet", data);
    };

    this.getAnaliseCusto = function(filter) {
        return General.callRestEngine('tco#/getAnaliseCusto', "FilterData", filter);
    };

    this.confirmResponsible = function(data) {
        return General.callRestEngine('tco#/confirmResponsible', "DataSet", [data]);
    };

    this.finish = function(data) {
        return General.callRestEngine('tco#/finish', "DataSet", [data]);
    };

    this.requireInfoOrRecuse = function(data) {
        return General.callRestEngine('tco#/requireInfoOrRecuse', "DataSet", [data]);
    };

    this.importCardTrab = function(data) {
        return General.callRestEngine('tco#/importCardTrab', "DataSet", [data]);
    };

    this.getListaError = function(data) {
        return General.callRestEngine('tco#/getListaError', "DataSet", data);
    };

    this.retornaReceitas = function(row) {
        return restEngine.post('tco#/retornaReceitasSimulacao', {
            "requestType": "Row",
            "row": row
        });
    };

    this.confirmSelectedBrand = function(row) {
        return restEngine.post('tco#/confirmSelectedBrand', {
            "requestType": "Row",
            "row": row
        });
    };

    this.effect = function(row) {
        return restEngine.post('tco#/effect', {
            "requestType": "Row",
            "row": row
        });
    };

    this.getServCompl = function(row) {
        return restEngine.post('tco#/getServCompl', {
            "requestType": "Row",
            "row": row
        });
    };

    this.getErrorServComp = function(row) {
        return restEngine.post('tco#/getErrorServComp', {
            "requestType": "Row",
            "row": row
        });
    };

    this.getInfoADestacar = function(row) {
        return restEngine.post('tco#/tco01202_getInfoADestacar', {
            "requestType": "Row",
            "row": row
        });
    };

    this.replicaPratos = function(row) {
        return restEngine.post('tco#/cardapioproposta/replicaPratos', {
            "requestType": "Row",
            "row": row
        });
    };

    this.getPratos = function(row) {
        return restEngine.post('tco#/cardapioproposta/getPratos', {
            "requestType": "Row",
            "row": row
        });
    };

    this.getReceita = function(row) {
        return restEngine.post('tco#/cardapioproposta/getReceita', {
            "requestType": "Row",
            "row": row
        });
    };

    this.substituiProduto = function(row) {
        return restEngine.post('tco#/substituiProduto', {
            "requestType": "Row",
            "row": row
        });
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01200CardapioDaPropostaService', Tco01200CardapioDaPropostaService);
});

// FILE: ./assets/js/services/Tco01300_lista_precos_service.js
var Tco01300_lista_precos_service = function(ScreenService, General, restEngine) {

    this.atualizaListaPrecos = function(row) {
        return restEngine.post('tco#/atualizalistaprecos', {
            "requestType": "Row",
            "row": row
        });
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01300_lista_precos_service', Tco01300_lista_precos_service);
});

// FILE: ./assets/js/services/Tco01400_custo_pela_media_service.js
var Tco01400_custo_pela_media_service = function(ScreenService, General, restEngine) {

    this.adicionaNovoProduto = function(row) {
        return restEngine.post('tco#/tco01400custopelamedia/adicionaNovoProduto', {
            "requestType": "Row",
            "row": row
        });
    };

    this.excluiProdutos = function(dataset) {
        return restEngine.post('tco#/tco01400custopelamedia/excluiProdutosSelecionados', {
            "requestType": "DataSet",
            "dataset": dataset
        });
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01400_custo_pela_media_service', Tco01400_custo_pela_media_service);
});

// FILE: ./assets/js/services/Tco01500_fornecedor_prioritario_produto_service.js
var Tco01500_fornecedor_prioritario_produto_service = function(ScreenService, General, restEngine) {

    this.salvaProdutoFornecedor = function(row) {
        return restEngine.post('tco#/Tco01500FornecedorPrioritarioProduto/salvaProdutoFornecedor', {
            "requestType": "Row",
            "row": row
        });
    };

    this.excluiProdutosFornecedor = function(dataset) {
        return restEngine.post('tco#/Tco01500FornecedorPrioritarioProduto/excluiProdutosFornecedor', {
            "requestType": "DataSet",
            "dataset": dataset
        });
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01500_fornecedor_prioritario_produto_service', Tco01500_fornecedor_prioritario_produto_service);
});

// FILE: ./assets/js/services/Tco01600_previsao_consumo_service.js
var Tco01600_previsao_consumo_service = function(ScreenService, General, restEngine) {

    this.relatorioPrevisaoConsumo = function(row) {
        return restEngine.post('tco#/tco01600previsaoconsumo/relatorioprevisaoconsumo', {
            "requestType": "Row",
            "row": row
        });
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01600_previsao_consumo_service', Tco01600_previsao_consumo_service);
});

// FILE: ./assets/js/services/Tco01700_tipo_preparo_service.js
var Tco01700_tipo_preparo_service = function(ScreenService, General, restEngine) {

    this.salvarTipoPreparo = function(row) {
        return restEngine.post('tco#/tipopreparotco/save', {
            "requestType": "Row",
            "row": row
        });
    };

    this.deleteTipoPreparo = function(row) {
        return restEngine.post('tco#/tipopreparotco/delete', {
            "requestType": "Row",
            "row": row
        });
    };

    this.vinculaTipoPreparo = function(row) {
        return restEngine.post('tco#/tipopreparotco/vinculaTipoPreparoPrato', {
            "requestType": "Row",
            "row": row
        });
    };

    this.desvinculaTipoPreparo = function(dataset) {
        return restEngine.post('tco#/tipopreparotco/desvinculaTipoPreparoPrato', {
            "requestType": "DataSet",
            "dataset": dataset
        });
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01700_tipo_preparo_service', Tco01700_tipo_preparo_service);
});

// FILE: ./assets/js/services/Tco01800_fator_correcao_service.js
var Tco01800_fator_correcao_service = function(ScreenService, General, restEngine) {

    this.salvaNovaTabFatorCorrecao = function(row) {
        return restEngine.post('tco#/Tco01800_fator_correcao/save', {
            "requestType": "Row",
            "row": row
        });
    };

    this.salvaEdicaoTabFatorCorrecao = function(data) {
        return restEngine.post('tco#/Tco01800_fator_correcao/saveedit', {
            "requestType": "DataSet",
            "dataset": data
        });
    };

    this.excluiTabFatorCorrecao = function(data) {
        return restEngine.post('tco#/Tco01800_fator_correcao/delete', {
            "requestType": "DataSet",
            "dataset": data
        });
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01800_fator_correcao_service', Tco01800_fator_correcao_service);
});

// FILE: ./assets/js/services/TcoEmailService.js
var TcoEmailService = function(ScreenService, General, restEngine) {

    /**
     * Service para enviar email de 
     * notificaçõe
     * 
     * @param {Object} row Objeto contendo 
     *  dados para envio.
     * 
     * Motivos:
     *   DUVIDA
     *   RECUSADA
     *   FINALIZACAO
     *   FALHA
     *   CUSTOMIZACAO
     *   CORRECAO
     *   APROVACAO
     *   NAO_ENCONTRADA
     * 
     *   Exemplo:
     * {
     *      'NRDETALHESERVPROPCOM': '1',
     *      'MOTIVO': 'FALHA',
     * }
     */

    this.enviaEmail = function(row) {
        return restEngine.post('tco#/enviaEmail', {
            "requestType": "Row",
            "row": row
        });
    };
};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('TcoEmailService', TcoEmailService);
});

// FILE: ./assets/js/services/tco00700_tipo_de_publico_service.js
var tco00700_tipo_de_publico_service = function(ScreenService, General, restEngine) {

    this.salvaTipoPublico = function(dataSet) {
        return restEngine.post('tco#/tipopublico/save', {
            "requestType": "DataSet",
            "dataset": dataSet
        });
    };

    this.apagaTipoPublico = function(dataSet) {
        return restEngine.post('tco#/tipopublico/delete', {
            "requestType": "DataSet",
            "dataset": dataSet
        });
    };
};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('tco00700_tipo_de_publico_service', tco00700_tipo_de_publico_service);
});

// FILE: ./assets/js/controllers/Sample_controller.js
var Sample_controller = function(ScreenService, General) {

    this.message = function(widget) {
        ScreenService.showMessage(widget.currentRow.NMRAZSOCCLIE);
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Sample_controller', Sample_controller);
});

// FILE: ./assets/js/controllers/Tco00100_ramo_atividade_segmento_controller.js
/*global Configuration, Util*/
var Tco00100_ramo_atividade_segmento_controller = function(ScreenService, General, Tco00100_ramo_atividade_segmento_service, MessageService) {

    this.carregaParametroMascara = function(args) {
        Tco00100_ramo_atividade_segmento_service.findParamgeral().then(function(success) {
            if (typeof success.dataset.paramgeral !== "undefined" && success.dataset.paramgeral.length > 0) {
                args.owner.CDPICTATIVCL = success.dataset.paramgeral[0].CDPICTATIVCL;
            }
        });
    };

    this.aplicaMascaraArvoreRamoAtividade = function(args) {
        var fields = args.owner.fields;

        fields.forEach(function(field) {
            if (field.name === "CDARVATIVCLI") {
                field.mask = {
                    "type": "fix",
                    "params": {
                        "mask": args.owner.container.CDPICTATIVCL
                    }
                };
            }
        });
    };

    this.backDetail = function(args) {
        var widgetRamoAtividade = args.owner.widget.container.getWidget('widgetRamoAtividade');
        widgetRamoAtividade.dataSource.clearCheckedRows();
        ScreenService.closeSwipe();
    };

    this.filtraRamosAtividade = function(args) {
        var currentRow = args.owner.currentRow;
        var widgetRamoAtividade = args.owner.container.getWidget('widgetRamoAtividade');
        widgetRamoAtividade.dataSource.clearCheckedRows();
        widgetRamoAtividade.dataSourceFilter = buildFilterDependent(['CDSEGMER'], currentRow);
        widgetRamoAtividade.dataSourceFilter.push({
            "operator": "ORDER_BY",
            "name": [
                {
                    "name": "CDARVATIVCLI",
                    "direction": "ASC"
                }
            ]
        });
        widgetRamoAtividade.reload();
    };

    this.adicionarRamoAtividade = function(args) {
        var popupRamoAtividade = args.owner.widget.container.getWidget('popupRamoAtividade');
        var widgetRamoAtividade = args.owner.widget.container.getWidget('widgetRamoAtividade');

        var ramosAtividadeAssociados = [];

        if (typeof widgetRamoAtividade.dataSource.data !== 'undefined' && widgetRamoAtividade.dataSource.data.length > 0) {
            widgetRamoAtividade.dataSource.data.forEach(function(ramo) {
                ramosAtividadeAssociados.push(ramo.CDATIVCLI);
            });
            popupRamoAtividade.dataSourceFilter = [
                {
                    "name": "CDATIVCLI",
                    "operator": "NOT_IN",
                    "value": ramosAtividadeAssociados
                },
                {
                    "operator": "ORDER_BY",
                    "name": [
                        {
                            "name": "CDARVATIVCLI",
                            "direction": "ASC"
                        }
                    ]
                }
            ];
        } else {
            popupRamoAtividade.dataSourceFilter = [
                {
                    "operator": "ORDER_BY",
                    "name": [
                        {
                            "name": "CDARVATIVCLI",
                            "direction": "ASC"
                        }
                    ]
                }
            ];
        }
        popupRamoAtividade.reload();

        ScreenService.openPopup(popupRamoAtividade);
    };

    this.removeRamosAtividade = function(args) {
        var widgetRamoAtividade = args.owner.widget;
        var ramosAtividadeSelecionados = args.owner.widget.getCheckedRows();
        ScreenService.confirmMessage(ScreenService.i18n('Remover Ramo(s) de Atividade?'), 'question', function(success) {
            Tco00100_ramo_atividade_segmento_service.removeRamosAtividade(ramosAtividadeSelecionados).then(function(success) {
                if (success.dataset.ramosAtividadeSegmentoUtilizados.length > 0) {

                    buildMessages(success.dataset.ramosAtividadeSegmentoUtilizados).then(function(mensagens) {
                        General.showErrorMessages(widgetRamoAtividade, mensagens, ScreenService.i18n("Conflitos ao excluir Ramos de Atividade."));
                    });

                } else {
                    ScreenService.showMessage("Ramo(s) de Atividade excluído(s) com sucesso.", null, "checked");
                    widgetRamoAtividade.reload();
                    widgetRamoAtividade.dataSource.clearCheckedRows();
                    ScreenService.closePopup();
                }
            });
        });
    };

    this.cancelPopup = function(args) {
        args.owner.widget.dataSource.clearCheckedRows();
        ScreenService.closePopup();
    };

    this.salvaRamosAtividade = function(args) {

        var ramosAtividadeSelecionados = args.owner.widget.getCheckedRows();
        var dataRamosAtividade = [];
        var widgetMain = args.owner.widget.container.getWidget('widgetMain');
        var widgetRamoAtividade = args.owner.widget.container.getWidget('widgetRamoAtividade');

        ramosAtividadeSelecionados.forEach(function(ramo) {
            ramo.CDSEGMER = widgetMain.currentRow.CDSEGMER;
            ramo.__is_new = true;
            ramo.changed = true;
            ramo.__using_unit = widgetRamoAtividade.paramsNewCode.__using_unit || false;
            ramo.__new_code_pk = widgetRamoAtividade.paramsNewCode.__new_code_pk;
            ramo.__table_pk = widgetRamoAtividade.paramsNewCode.__table_pk;
            ramo.__new_code_length = widgetRamoAtividade.paramsNewCode.__new_code_length || 1;
            ramo.__cast_number = widgetRamoAtividade.paramsNewCode.__cast_number || false;

            dataRamosAtividade.push(ramo);
        });

        Tco00100_ramo_atividade_segmento_service.salvaNovosRamosAtividade(dataRamosAtividade).then(function(success) {
            if (success.dataset.ramosAtividade.length > 0) {
                buildMessages(success.dataset.ramosAtividade).then(function(ramosAtividadeUtilizados) {
                    General.showErrorMessages(widgetRamoAtividade, ramosAtividadeUtilizados, ScreenService.i18n("Ramos de Atividade já Existentes."));
                });
            } else {
                ScreenService.notificationMessage(ScreenService.i18n("Ramo(s) de Atividade salvo(s) com sucesso."), 'success', 3000);
                widgetRamoAtividade.reload();
                ScreenService.closePopup();
            }
        });

    };

    var buildMessages = function(data) {
        var mensagens = [];
        var msgParams = [];
        data.forEach(function(row) {
            //mensagens.push(ScreenService.i18n(row.MESSAGECODE, row));
            msgParams.push(row);
            mensagens.push(row.MESSAGECODE);
        });

        return MessageService.getMessagesByCode(mensagens).then(function(mensagensTraduzidas) {
            for (let i = 0; i < mensagensTraduzidas.length; i++) {
                mensagensTraduzidas[i] = ScreenService.i18n(mensagensTraduzidas[i], msgParams[i]);
            }
            return mensagensTraduzidas;
        });
    };

    var buildFilterDependent = function(columns, values) {
        var filter = [];
        columns.forEach(function(column) {
            filter.push({
                name: column,
                operator: "=",
                value: values[column]
            });
        });
        return filter;
    };

    this.changeEnabled = function(args) {
        var widget = args.owner.widget;

        var MudaEstado = function(estado) {
            var dadosParaMudanca = widget.getCheckedRows();
            dadosParaMudanca.forEach(function(row) {
                row.IDHABILITADOTCO = estado;
            });

            Tco00100_ramo_atividade_segmento_service.mudarEstado(dadosParaMudanca).then(function() {
                ScreenService.successNotification('Atualizado com sucesso!');
                widget.unSelectAll();
                widget.reload();
                ScreenService.closeSwipe();
            }, function(error) {
                ScreenService.errorNotification(error.data.error);
            });
        };

        ScreenService.showCustomDialog(null, null, null, [{
            label: ScreenService.i18n("Habilitar"),
            code: function(e, deferred) {
                if (Util.isFunction(MudaEstado)) {
                    MudaEstado('S');
                }
                deferred.resolve(e);
            }
        }, {
            label: ScreenService.i18n("Desabilitar"),
            code: function(e, deferred) {
                if (Util.isFunction(MudaEstado)) {
                    MudaEstado('N');
                }
                deferred.reject(e);
            }
        }, {
            label: ScreenService.i18n("Cancelar")
        }], '');
    };
};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco00100_ramo_atividade_segmento_controller', Tco00100_ramo_atividade_segmento_controller);
});

// FILE: ./assets/js/controllers/Tco00200_porcionamento_controller.js
/* global angular, Configuration*/
var Tco00200_porcionamento_controller = function(ScreenService, General, Tco00200_porcionamento_service, templateManager, MessageService) {

    this.cancel = function(args) {
        var widget = args.owner.widget;
        if (widget.currentRow.__is_new) {
            ScreenService.closeSwipe();
        } else {
            General.backOrCancel(args);
        }
        widget.reload();
        this.habilitarRelPorcioGRTipoPR(true);
    };

    this.backDetail = function(args) {
        ScreenService.closeSwipe();
    };

    this.editarPorcionamento = function(args) {
        var widget = args.owner.widget;
        widget.edit();
        this.habilitarRelPorcioGRTipoPR(false);
    };

    this.adicionarPorcionamento = function(args) {
        var widget = args.owner.widget;
        widget.newRow();
        this.habilitarRelPorcioGRTipoPR(false);
        widget.applyDefaultValue();
    };

    this.formataCampo = function(args) {
        var field = args.owner.field;
        var valor = field.value();
        if (valor) {
            field.setValue(valor.padStart(5, 0));
        }
    };

    this.salvaPorcionamento = function(args) {
        var widget = args.owner.widget;
        if (widget.isValid()) {
            var row = angular.copy(widget.currentRow);
            Tco00200_porcionamento_service.salvaPorcionamento([row]).then(function(callback) {
                if (callback.dataset.result.success) {
                    widget.view();
                    widget.currentRow.__is_new = false;
                    widget.currentRow.NRPORCIONAMENTO = callback.dataset.result.code;
                    widget.reload();
                    this.habilitarRelPorcioGRTipoPR(true);
                } else if (callback.dataset.result.error) {
                    ScreenService.showMessage(callback.dataset.result.error);
                }
            }.bind(this), function(error) {
                ScreenService.showMessage(error.data.error, 'error');
            });
        }
    };

    this.apagaPorcionamento = function(args) {
        var widget = args.owner.widget;
        var row = angular.copy(widget.currentRow);
        ScreenService.confirmMessage(ScreenService.i18n('Tem certeza que deseja excluir o porcionamento?'), 'question', function() {
            Tco00200_porcionamento_service.apagaPorcionamento([row]).then(function(callback) {
                if (callback.dataset.porcionamentosUtilizados.length > 0) {
                    buildMessages(callback.dataset.porcionamentosUtilizados).then(function(mensagens) {
                        General.showErrorMessages(widget, mensagens, ScreenService.i18n("Conflitos ao excluir Porcionamento."));
                    });
                } else {
                    ScreenService.closeSwipe();
                    widget.reload();
                }
            }, function(error) {
                ScreenService.showMessage(error.data.error, 'error');
            });
        });
    };

    var buildMessages = function(data) {

        var mensagens = [];
        var msgParams = [];
        data.forEach(function(row) {
            //mensagens.push(ScreenService.i18n(row.MESSAGECODE, row));
            msgParams.push(row);
            mensagens.push(row.MESSAGECODE);
        });

        return MessageService.getMessagesByCode(mensagens).then(function(mensagensTraduzidas) {
            for (let i = 0; i < mensagensTraduzidas.length; i++) {
                mensagensTraduzidas[i] = ScreenService.i18n(mensagensTraduzidas[i], msgParams[i]);
            }
            return mensagensTraduzidas;
        });

    };

    this.atualizarRelPorcioGrTipoPr = function(args) {
        var widgetMain = templateManager.container.getWidget('widgetMain');
        var widgetRelPorcioGrTipoPr = templateManager.container.getWidget('widgetRelPorcioGrTipoPr');
        let widgetGrTipoPr = templateManager.container.getWidget('widgetGrTipoPr');

        let filter = [
            {
                "name": "P_NRPORCIONAMENTO",
                "operator": "=",
                "value": widgetMain.currentRow.NRPORCIONAMENTO
            }
        ];

        widgetRelPorcioGrTipoPr.dataSourceFilter = filter;
        widgetGrTipoPr.dataSourceFilter = filter;
        widgetRelPorcioGrTipoPr.reload();
        widgetRelPorcioGrTipoPr.unSelectAll();
    };

    this.abreAdicionarRelPorcioGrTipoPr = function(args) {
        let widgetGrTipoPr = templateManager.container.getWidget('widgetGrTipoPr');
        widgetGrTipoPr.reload().then(function() {
            widgetGrTipoPr.unSelectAll();
            ScreenService.openPopup(widgetGrTipoPr);
        });
    };

    this.adicionarRelPorcioGrTipoPr = function(args) {
        let widgetMain = templateManager.container.getWidget('widgetMain');
        let widgetGrTipoPr = templateManager.container.getWidget('widgetGrTipoPr');
        let checkedRows = widgetGrTipoPr.getCheckedRows();
        checkedRows.forEach(function(row) {
            row.NRPORCIONAMENTO = widgetMain.currentRow.NRPORCIONAMENTO;
        });
        Tco00200_porcionamento_service.adicionarRelPorcioGrTipoPr(checkedRows).then(function() {
            ScreenService.closePopup();
            var widgetRelPorcioGrTipoPr = templateManager.container.getWidget('widgetRelPorcioGrTipoPr');
            widgetRelPorcioGrTipoPr.activate();
            widgetRelPorcioGrTipoPr.reload();
            MessageService.successNotificationByCode("TCO00200_SUCESSO_INC_REL", 3000);
        });
    };

    this.apagarRelPorcioGrTipoPr = function(args) {
        let widgetRelPorcioGrTipoPr = templateManager.container.getWidget('widgetRelPorcioGrTipoPr');
        let checkedRows = widgetRelPorcioGrTipoPr.getCheckedRows();
        MessageService.confirmMessageByCode('TCO00200_CONFIRM_EXCL_RELPORCIOGR').then(function() {
            Tco00200_porcionamento_service.apagarRelPorcioGrTipoPr(checkedRows).then(function() {
                ScreenService.closePopup();
                var widgetRelPorcioGrTipoPr = templateManager.container.getWidget('widgetRelPorcioGrTipoPr');
                widgetRelPorcioGrTipoPr.activate();
                widgetRelPorcioGrTipoPr.reload();
                widgetRelPorcioGrTipoPr.unSelectAll();
                MessageService.successNotificationByCode("TCO00200_SUCESSO_EXCL_REL", 3000);
            });
        });
    };

    this.cancelAdicionarRelPorcioGrTipoPr = function(args) {
        ScreenService.closePopup();
        var widgetRelPorcioGrTipoPr = templateManager.container.getWidget('widgetRelPorcioGrTipoPr');
        widgetRelPorcioGrTipoPr.activate();
    };

    this.habilitarRelPorcioGRTipoPR = function(habilitado) {
        let widgetRelPorcioGrTipoPr = templateManager.container.getWidget('widgetRelPorcioGrTipoPr');
        widgetRelPorcioGrTipoPr.setEnabled(habilitado);
    };
};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco00200_porcionamento_controller', Tco00200_porcionamento_controller);
});

// FILE: ./assets/js/controllers/Tco00300_servico_proposta_controller.js
var Tco00300_servico_proposta_controller = function(ScreenService, General, Tco00300_servico_proposta_service, MessageService) {

    this.backDetail = function() {
        ScreenService.closeSwipe();
    };

    this.cancelaEdit = function(args) {
        var widgetMain = args.owner.widget;
        widgetMain.currentRow = widgetMain.backUpRow;
        widgetMain.reload();
        General.backOrCancel(args);
    };

    this.novoServicoProposta = function(args) {
        var widgetMain = args.owner.widget;
        widgetMain.newRow();
        widgetMain.currentRow.IDATIVO = "S";
    };

    this.salvaServicoProposta = function(args) {
        var widgetMain = args.owner.widget;
        var currentRow = args.owner.widget.currentRow;
        if (widgetMain.isValid()) {
            currentRow.__using_unit = widgetMain.paramsNewCode.__using_unit || false;
            currentRow.__new_code_pk = widgetMain.paramsNewCode.__new_code_pk;
            currentRow.__table_pk = widgetMain.paramsNewCode.__table_pk;
            currentRow.__new_code_length = widgetMain.paramsNewCode.__new_code_length || 1;
            currentRow.__cast_number = widgetMain.paramsNewCode.__cast_number || false;

            Tco00300_servico_proposta_service.salvaNovoServicoProposta(currentRow).then(function(success) {
                if (typeof success.messages !== 'undefined' && success.messages.length > 0) {
                    MessageService.showMessageByCode(success.messages[0].message);
                } else if (typeof success.dataset.servicoProposta !== "undefined" && success.dataset.servicoProposta.length > 0) {
                    MessageService.showMessageByCode(success.dataset.servicoProposta[0].MESSAGECODE, success.dataset.servicoProposta[0]);
                } else {
                    if (typeof success.dataset.columnsToReturn !== "undefined" && success.dataset.columnsToReturn.length > 0) {
                        widgetMain.currentRow = success.dataset.columnsToReturn[0];
                    }
                    ScreenService.notificationMessage(ScreenService.i18n("Serviço para Proposta salvo com sucesso."), 'success', 3000);
                    widgetMain.reload();
                    widgetMain.view();
                }
            });
        }
    };

    this.vinculaServicoProposta = function(args) {
        var popupServicoPlanejamento = args.owner.widget.container.getWidget('popupServicoPlanejamento');
        var widgetMain = args.owner.widget.container.getWidget('widgetMain');

        if (typeof widgetMain.currentRow.CDSERVFILI !== 'undefined' && widgetMain.currentRow.CDSERVFILI !== null && widgetMain.currentRow.CDSERVFILI !== "") {

            popupServicoPlanejamento.dataSourceFilter = [
                {
                    "name": "CDSERVFILI",
                    "operator": "NOT_IN",
                    "value": widgetMain.currentRow.CDSERVFILI
                },
                {
                    "operator": "ORDER_BY",
                    "name": [
                        {
                            "name": "CDSERVFILI",
                            "direction": "ASC"
                        }
                    ]
                }
            ];
        } else {
            popupServicoPlanejamento.dataSourceFilter = [
                {
                    "operator": "ORDER_BY",
                    "name": [
                        {
                            "name": "CDSERVFILI",
                            "direction": "ASC"
                        }
                    ]
                }
            ];
        }
        popupServicoPlanejamento.reload();

        ScreenService.openPopup(popupServicoPlanejamento);
    };

    this.salvaVinculoServicoProposta = function(args) {
        var widgetMain = args.owner.container.getWidget('widgetMain');
        var popupServicoPlanejamento = args.owner.container.getWidget('popupServicoPlanejamento');
        widgetMain.currentRow.CDFILIAL = popupServicoPlanejamento.currentRow.CDFILIAL;
        widgetMain.currentRow.CDSERVFILI = popupServicoPlanejamento.currentRow.CDSERVFILI;
        widgetMain.currentRow.NMSERVICO = popupServicoPlanejamento.currentRow.CDNMSERVICO;

        Tco00300_servico_proposta_service.salvaNovoServicoProposta(widgetMain.currentRow).then(function(success) {
            if (typeof success.messages !== 'undefined' && success.messages.length > 0) {
                MessageService.showMessageByCode(success.messages[0].message);
            } else if (typeof success.dataset.servicoProposta !== "undefined" && success.dataset.servicoProposta.length > 0) {
                MessageService.showMessageByCode(success.dataset.servicoProposta[0].MESSAGECODE, success.dataset.servicoProposta[0]);
            } else {
                ScreenService.notificationMessage(ScreenService.i18n("Vinculo Serviço para Proposta salvo com sucesso."), 'success', 3000);
                widgetMain.reload().then(function() {
                    if (typeof success.dataset.columnsToReturn !== "undefined" && success.dataset.columnsToReturn.length > 0) {
                        widgetMain.currentRow = success.dataset.columnsToReturn[0];
                    }
                    widgetMain.dataSource.data.forEach(function(row) {
                        if (row.CDSERVICOPROPOSTA === widgetMain.currentRow.CDSERVICOPROPOSTA) {
                            row.NMSERVICO = widgetMain.currentRow.NMSERVICO;
                        }
                    });
                    ScreenService.closePopup();
                });
            }
        });
    };

    this.editServicoProposta = function(args) {
        args.owner.widget.backUpRow = angular.copy(args.owner.widget.currentRow);
        args.owner.widget.edit();
    };

    this.removeServicoProposta = function(args) {
        var widgetMain = args.owner.widget;
        var currentRow = args.owner.widget.currentRow;
        ScreenService.confirmMessage(ScreenService.i18n('Remover Serviço para Proposta?'), 'question', function(success) {
            Tco00300_servico_proposta_service.removeServicoProposta(currentRow).then(function(success) {
                if (success.dataset.servicoPropostaEmUso.length > 0) {
                    buildMessages(success.dataset.servicoPropostaEmUso).then(function(ServicoPropostaEmUso) {
                        General.showErrorMessages(widgetMain, ServicoPropostaEmUso, ScreenService.i18n("Serviço para Proposta em uso."));
                    });
                } else {
                    ScreenService.showMessage(ScreenService.i18n("Serviço para Proposta excluído com sucesso."), null, "checked");
                    ScreenService.closeSwipe().then(function() {
                        widgetMain.reload();
                    });
                }
            });
        });
    };

    this.cancelPopup = function(args) {
        ScreenService.closePopup();
    };

    var buildMessages = function(data) {
        var mensagens = [];
        var msgParams = [];
        data.forEach(function(row) {
            //mensagens.push(ScreenService.i18n(row.MESSAGECODE, row));
            msgParams.push(row);
            mensagens.push(row.MESSAGECODE);
        });

        return MessageService.getMessagesByCode(mensagens).then(function(mensagensTraduzidas) {
            for (let i = 0; i < mensagensTraduzidas.length; i++) {
                mensagensTraduzidas[i] = ScreenService.i18n(mensagensTraduzidas[i], msgParams[i]);
            }
            return mensagensTraduzidas;
        });
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco00300_servico_proposta_controller', Tco00300_servico_proposta_controller);
});

// FILE: ./assets/js/controllers/Tco00400_tipo_contrato_controller.js
var Tco00400_tipo_contrato_controller = function(ScreenService, General, Tco00400_tipo_contrato_service, MessageService) {

    this.salvaTipoContrato = function(args) {
        var widget = args.owner.widget;
        if (widget.isValid()) {
            widget.currentRow.CDTIPOCONTRATO = widget.currentRow.CDTIPOCONTRATO.padStart(5, 0);

            var paramsNewCode = widget.paramsNewCode;
            if (paramsNewCode && paramsNewCode.__is_new_code) {
                widget.currentRow.__using_unit = paramsNewCode.__using_unit || false;
                widget.currentRow.__new_code_pk = paramsNewCode.__new_code_pk;
                widget.currentRow.__table_pk = paramsNewCode.__table_pk;
                widget.currentRow.__new_code_length = paramsNewCode.__new_code_length || 5;
                widget.currentRow.__cast_number = paramsNewCode.__cast_number || false;
            }

            Tco00400_tipo_contrato_service.salvaTipoContrato(widget.currentRow).then(function(response) {

                if (response.dataset !== undefined) {
                    ScreenService.successNotification("Registro salvo com sucesso");
                    widget.view();
                    widget.reload().then(function() {
                        if (widget.currentRow.__is_new)
                            widget.currentRow = response.dataset.columnsToReturn[0];
                    });
                } else if (response.messages !== undefined) {
                    var msg = response.messages.map(reg => {
                        return ScreenService.i18n(reg.message);
                    });

                    ScreenService.showMessage('Erro ao salvar registro.').then(() => {
                        General.showErrorMessages(widget, msg, ScreenService.i18n("Erro ao salvar registro."));
                    });
                }

            }, function(erro) {
                if (erro.data.error.indexOf("ORA-00001") != -1) {
                    ScreenService.showMessage("Impossível adicionar. Registro já existe");
                }
            });
        }
    };

    var buildMessages = function(data) {
        var mensagens = [];
        var msgParams = [];
        data.forEach(function(row) {
            //mensagens.push(ScreenService.i18n(row.MESSAGECODE, row));
            msgParams.push(row);
            mensagens.push(row.MESSAGECODE);
        });

        return MessageService.getMessagesByCode(mensagens).then(function(mensagensTraduzidas) {
            for (let i = 0; i < mensagensTraduzidas.length; i++) {
                mensagensTraduzidas[i] = ScreenService.i18n(mensagensTraduzidas[i], msgParams[i]);
            }
            return mensagensTraduzidas;
        });
    };

    this.removeTipoContrato = function(args) {
        var widgetTipoContrato = args.owner.widget;
        ScreenService.confirmMessage(ScreenService.i18n('Remover Tipo de Contrato?'), 'question', function(success) {
            Tco00400_tipo_contrato_service.removeTipoContrato([widgetTipoContrato.currentRow]).then(function(success) {
                if (success.dataset.tipoContratoVinculados.length > 0) {

                    buildMessages(success.dataset.tipoContratoVinculados).then(function(mensagens) {
                        General.showErrorMessages(widgetTipoContrato, mensagens, ScreenService.i18n("Conflitos ao excluir Tipo de Contrato."));
                    });

                } else {

                    ScreenService.showMessage('Excluído com Sucesso.').then(() => {
                        ScreenService.closeSwipe().then(function() {
                            widgetTipoContrato.reload();
                        });
                    });

                }
            });
        });
    };

    this.voltarTipoContrato = function(args) {
        var widgetTipoContrato = args.owner.widget;
        ScreenService.closeSwipe();
        widgetTipoContrato.reload().then(function() {
            widgetTipoContrato.activate();
        });
    };
};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco00400_tipo_contrato_controller', Tco00400_tipo_contrato_controller);
});

// FILE: ./assets/js/controllers/Tco00500_padrao_qualidade_controller.js
var Tco00500_padrao_qualidade_controller = function(ScreenService, General, Tco00500_padrao_qualidade_service, MessageService) {

    this.backDetail = function() {
        ScreenService.closeSwipe();
    };

    this.cancelaEdit = function(args) {
        var widgetMain = args.owner.widget;
        widgetMain.currentRow = widgetMain.backUpRow;
        widgetMain.reload();
        General.backOrCancel(args);
    };

    this.novoPadraoQualidade = function(args) {
        var widgetMain = args.owner.widget;
        widgetMain.newRow();
        widgetMain.currentRow.IDATIVO = "S";
    };

    this.salvaPadraoQualidade = function(args) {
        var widgetMain = args.owner.widget;
        var currentRow = args.owner.widget.currentRow;
        if (widgetMain.isValid()) {
            currentRow.__using_unit = widgetMain.paramsNewCode.__using_unit || false;
            currentRow.__new_code_pk = widgetMain.paramsNewCode.__new_code_pk;
            currentRow.__table_pk = widgetMain.paramsNewCode.__table_pk;
            currentRow.__new_code_length = widgetMain.paramsNewCode.__new_code_length || 1;
            currentRow.__cast_number = widgetMain.paramsNewCode.__cast_number || false;

            Tco00500_padrao_qualidade_service.salvaNovoPadraoQualidade(currentRow).then(function(success) {
                if (typeof success.messages !== 'undefined' && success.messages.length > 0) {
                    MessageService.showMessageByCode(success.messages[0].message);
                } else if (typeof success.dataset.padraoQualidade !== "undefined" && success.dataset.padraoQualidade.length > 0) {
                    MessageService.showMessageByCode(success.dataset.padraoQualidade[0].MESSAGECODE, success.dataset.padraoQualidade[0]);
                } else {
                    if (typeof success.dataset.columnsToReturn !== "undefined" && success.dataset.columnsToReturn.length > 0) {
                        widgetMain.currentRow = success.dataset.columnsToReturn[0];
                    }
                    ScreenService.notificationMessage(ScreenService.i18n("Padrão de Qualidade salvo com sucesso."), 'success', 3000);
                    widgetMain.reload();
                    widgetMain.view();
                }
            });
        }
    };

    this.editPadraoQualidade = function(args) {
        args.owner.widget.backUpRow = angular.copy(args.owner.widget.currentRow);
        args.owner.widget.edit();

    };

    this.removePadraoQualidade = function(args) {
        var widgetMain = args.owner.widget;
        var currentRow = args.owner.widget.currentRow;

        ScreenService.confirmMessage(ScreenService.i18n('Remover Padrão de Qualidade?'), 'question', function(success) {
            Tco00500_padrao_qualidade_service.removePadraoQualidade(currentRow).then(function(success) {
                if (success.dataset.padraoQualidadeEmUso.length > 0) {
                    buildMessages(success.dataset.padraoQualidadeEmUso).then(function(padraoQualidadeEmUso) {
                        General.showErrorMessages(widgetMain, padraoQualidadeEmUso, ScreenService.i18n("Padrões de Qualidade em uso."));
                    });
                } else {
                    ScreenService.showMessage(ScreenService.i18n("Padrão de Qualidade excluído com sucesso."), null, "checked");
                    ScreenService.closeSwipe().then(function() {
                        widgetMain.reload();
                    });
                }
            });
        });
    };

    var buildMessages = function(data) {
        var mensagens = [];
        var msgParams = [];
        data.forEach(function(row) {
            //mensagens.push(ScreenService.i18n(row.MESSAGECODE, row));
            msgParams.push(row);
            mensagens.push(row.MESSAGECODE);
        });

        return MessageService.getMessagesByCode(mensagens).then(function(mensagensTraduzidas) {
            for (let i = 0; i < mensagensTraduzidas.length; i++) {
                mensagensTraduzidas[i] = ScreenService.i18n(mensagensTraduzidas[i], msgParams[i]);
            }
            return mensagensTraduzidas;
        });
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco00500_padrao_qualidade_controller', Tco00500_padrao_qualidade_controller);
});

// FILE: ./assets/js/controllers/Tco00600_gerente_negocio_controller.js
var Tco00600_gerente_negocio_controller = function(General, ScreenService, Tco00600_gerente_negocio_service, MessageService) {

    /**
     * Função responsável por definir filtro
     * para campo de operador ao adicionar novo
     * registro.
     * 
     * @param {Args} args Objeto args.
     */
    this.newRow = function(args) {
        let mainWidget = args.owner.widget;

        mainWidget.getField('IDATIVO').showOnForm = false;
        mainWidget.getField('OPERADOR').showOnForm = true;
        mainWidget.getField('CDOPERADOR').showOnForm = false;
        mainWidget.getField('NMOPERADOR').showOnForm = false;
        mainWidget.getField('NMOPERADORLIDER').showOnForm = false;

        this.setNewGerenteFilter(mainWidget);
        General.widgetNewRow(mainWidget);
    };

    /**
     * Função responsável por colocar a widget
     * principal em modo de edição
     * 
     * @param {Args} args Objeto args.
     */
    this.editRow = function(args) {
        let mainWidget = args.owner.widget;

        mainWidget.backupRow = {
            IDATIVO: mainWidget.currentRow.IDATIVO
        };

        General.changeWidgetAcessibility(mainWidget.widgets, false);
        mainWidget.edit();
    };

    /**
     * Função responsável por cancelar edição
     * na widget principal.
     * 
     * @param {Args} args Objeto args.
     */
    this.cancelEdit = function(args) {
        let mainWidget = args.owner.widget;

        mainWidget.getField('IDATIVO').showOnForm = true;
        mainWidget.getField('OPERADOR').showOnForm = false;
        mainWidget.getField('CDOPERADOR').showOnForm = true;
        mainWidget.getField('NMOPERADOR').showOnForm = true;
        mainWidget.getField('NMOPERADORLIDER').showOnForm = true;

        General.widgetCancelForm(mainWidget);
    };

    /**
     * Função responsável por cancelar um popup
     * aberto.
     * 
     */
    this.closeWidgetPopup = function() {
        ScreenService.closePopup();
    };

    /**
     * Define filtro para a adição de novos
     * gerentes, evitando a exibição de operadores
     * já adicionados. 
     * 
     * @param {Widget} mainWidget Widget principal.
     */
    this.setNewGerenteFilter = function(mainWidget) {
        let operadorField = mainWidget.getField('OPERADOR');
        let arrayOperador = mainWidget.dataSource.data.map(reg => reg.CDOPERADOR);

        operadorField.dataSourceFilter = [
            {
                operator: "ORDER_BY",
                name: [
                    {
                        name: "CDOPERADOR",
                        direction: "ASC"
                    }
                ]
            }
        ];

        if (arrayOperador.length) {
            operadorField.dataSourceFilter.push({
                name: "CDOPERADOR",
                operator: "NOT_IN",
                value: arrayOperador
            });
        }

        return operadorField.reload();
    };

    /**
     * Função responsável por salvar novo registro
     * ou alteração em registro existente, e definir o 
     * NRGERENTENEGOCIO na currentRow após salvar.
     * 
     * @param {Args} args Objeto args.
     */
    this.saveManager = function(args) {
        let widget = args.owner.widget;
        let cRow = widget.currentRow;

        if (!cRow.__is_new) {
            delete widget.getField('OPERADOR').validations.required;
        }

        if (widget.isValid()) {
            this.setParametersInRow(widget);

            cRow.__columns_to_return = ["NRGERENTENEGOCIO"];
            if (!cRow.__is_new) {
                delete cRow.CDOPERADORLIDER;
            } else {
                cRow.CDOPERADORLIDER = "";
            }

            Tco00600_gerente_negocio_service.saveManager([cRow]).then((response) => {

                let msg = cRow.__is_new ? 'Adicionado com sucesso.' : 'Registro salvo com sucesso.';
                ScreenService.successNotification(msg);

                General.changeWidgetAcessibility(widget.widgets, true);
                widget.view();
                widget.reload();
                cRow.NRGERENTENEGOCIO = response.dataset.columnsToReturn[0].NRGERENTENEGOCIO;
                cRow.__is_new = false;

                widget.getField('IDATIVO').showOnForm = true;
                widget.getField('OPERADOR').showOnForm = false;
                widget.getField('CDOPERADOR').showOnForm = true;
                widget.getField('NMOPERADOR').showOnForm = true;
                widget.getField('NMOPERADORLIDER').showOnForm = true;
                widget.getField('OPERADOR').validations.required = true;

            }).catch((error) => {
                if (error.error) {
                    MessageService.showMessageByCode(error.error, cRow);
                }
            });
        }
    };

    /**
     * Função responsável por definir os 
     * parâmetros paramsNewCode e __is_new de um 
     * registro antes de salvar.
     * 
     * @param {Widget} widget Widget contendo o registro.
     */
    this.setParametersInRow = function(widget) {

        widget.currentRow.__using_unit = widget.paramsNewCode.__using_unit || false;
        widget.currentRow.__new_code_pk = widget.paramsNewCode.__new_code_pk;
        widget.currentRow.__table_pk = widget.paramsNewCode.__table_pk;
        widget.currentRow.__new_code_length = widget.paramsNewCode.__new_code_length || 5;
        widget.currentRow.__new_code_column_size = widget.paramsNewCode.__new_code_column_size || 10;
        widget.currentRow.__cast_number = widget.paramsNewCode.__cast_number || false;

        if (!widget.currentRow.__columns_to_return && widget.paramsNewCode && widget.paramsNewCode.__columns_to_return)
            widget.currentRow.__columns_to_return = widget.paramsNewCode.__columns_to_return;

    };

    /**
     * Função responsável por abrir pop-up de adição
     * de operadores à liderança do operador selecionado.
     * 
     * @param {Args} args Objeto args.
     */
    this.openAddToManager = function(args) {
        let insideGrid = args.owner.widget;
        let managerLeadAddGrid = insideGrid.container.getWidget('managerLeadAddGrid');

        this.setManagerLeadAddGridFilter(managerLeadAddGrid).then(() => {
            ScreenService.openPopup(managerLeadAddGrid);
            managerLeadAddGrid.edit();
            managerLeadAddGrid.dataSource.clearCheckedRows();
        });
    };

    /**
     * Função responsável por definir o filtro na widget
     * de adição de liderados à um gerente.
     * 
     * @param {Widget} widget Widget de adição de novos
     *  liderados.
     * 
     * @return {Promise} Promise para recarregar dataSource.
     */
    this.setManagerLeadAddGridFilter = function(widget) {
        let cdOperadorNotIn = widget.getParent().dataSource.data.map(reg => reg.CDOPERADOR);
        cdOperadorNotIn.push(widget.getParent().getParent().currentRow.CDOPERADOR);

        widget.dataSourceFilter = [
            {
                name: "CDOPERADOR",
                operator: "NOT_IN",
                value: cdOperadorNotIn
            },
            {
                operator: "ORDER_BY",
                name: [
                    {
                        name: "CDOPERADOR",
                        direction: "ASC"
                    }
                ]
            }
        ];

        return widget.reload();
    };

    /**
     * Função responsável por fechar o popup
     * de adição à Liderança.
     * 
     * @param {Args} args Objeto args.
     */
    this.closeManagerLeadAddGrid = function(args) {
        let managerLeadAddGrid = args.owner.widget;

        ScreenService.closePopup(managerLeadAddGrid);
        managerLeadAddGrid.dataSource.clearCheckedRows();
    };

    /**
     * Função responsável por remover um gerente.
     * 
     * @param {Args} args Objeto args.
     */
    this.removeManager = function(args) {
        let widget = args.owner.widget;

        ScreenService.confirmMessage(ScreenService.i18n('Confirmar exclusão?'), 'question', (success) => {
            Tco00600_gerente_negocio_service.removeManager(widget.currentRow).then((success) => {

                if (success.notifications) {

                    ScreenService.closeSwipe().then(() => {
                        widget.activate();
                        widget.reload();
                        ScreenService.showMessage('Excluído com Sucesso!');
                    });

                } else if (success.dataset && success.dataset.invalidManagerRemove) {

                    let data = success.dataset.invalidManagerRemove;
                    MessageService.showMessageByCode('TCO00600_ERRO_EXCLUIR').then(() => {
                        buildMessages(data).then(function(mensagens) {
                            General.showErrorMessages(widget, mensagens, ScreenService.i18n('Erro ao excluir.'));
                        });
                    });

                }

            }).catch((error) => {
                ScreenService.showMessage(ScreenService.i18n(error.error, widget.currentRow));
            });
        });
    };

    var buildMessages = function(data) {
        var mensagens = [];
        var msgParams = [];
        data.forEach(function(row) {
            //mensagens.push(ScreenService.i18n(row.MESSAGECODE, row));
            msgParams.push(row);
            mensagens.push(row.MESSAGECODE);
        });

        return MessageService.getMessagesByCode(mensagens).then(function(mensagensTraduzidas) {
            for (let i = 0; i < mensagensTraduzidas.length; i++) {
                mensagensTraduzidas[i] = ScreenService.i18n(mensagensTraduzidas[i], msgParams[i]);
            }
            return mensagensTraduzidas;
        });
    };

    /**
     * Função responsável por adicionar gerentes
     * à liderança do gerente selecionado.
     * 
     * @param {Args} args Objeto args.
     */
    this.addToManagerGrid = function(args) {
        let managerLeadAddGrid = args.owner.widget;
        let operToAdd = managerLeadAddGrid.getCheckedRows();

        if (operToAdd.length) {
            operToAdd.forEach((reg) => {
                delete reg.NMOPERADOR;
                delete reg.NMOPERADORLIDER;
            });

            let rowToSave = {
                cdLider: managerLeadAddGrid.getParent().getParent().currentRow.CDOPERADOR,
                toAdd: operToAdd
            };

            Tco00600_gerente_negocio_service.addToManagerGrid(rowToSave).then((response) => {

                ScreenService.closePopup();
                managerLeadAddGrid.getParent().reload();
                managerLeadAddGrid.getParent().getParent().dataSource.reload();

                if (typeof response.dataset.invalidManagers !== 'undefined' && response.dataset.invalidManagers.length) {
                    let mensagens = [];
                    let invalidManagers = response.dataset.invalidManagers;

                    invalidManagers.forEach((manager) => {
                        manager.MESSAGECODE = 'TCO00600_GERENTES_INVALIDOS';
                    });

                    buildMessages(invalidManagers).then(function(mensagens) {
                        MessageService.showMessageByCode('TCO00600_GERENTES_INVALIDOS_AVISO').then(() => {
                            General.showErrorMessages(managerLeadAddGrid.getParent(), mensagens, ScreenService.i18n('Operadores Inválidos'));
                        });
                    });

                } else {
                    ScreenService.successNotification("Adicionado com sucesso!");
                }

            }).catch((error) => {
                ScreenService.showMessage(ScreenService.i18n(error.error, managerLeadAddGrid.currentRow));
            });
        } else {
            MessageService.showMessageByCode('TCO00600_SELECIONE_REGISTROS');
        }

    };

    /**
     * Função responsável por remover gerentes
     * da liderança do gerente selecionado.
     * 
     * @param {Args} args Objeto args.
     */
    this.removeFromManagerGrid = function(args) {
        let insideGrid = args.owner.widget;
        let operToRemove = insideGrid.getCheckedRows().map(reg => {
            return {
                CDOPERADOR: reg.CDOPERADOR,
                NRGERENTENEGOCIO: reg.NRGERENTENEGOCIO
            };
        });

        let rowToRemove = {
            cdLider: insideGrid.getParent().currentRow.CDOPERADOR,
            toRemove: operToRemove
        };

        ScreenService.confirmMessage('Confirmar exclusão?').then(() => {

            Tco00600_gerente_negocio_service.removeFromManagerGrid(rowToRemove).then(() => {

                ScreenService.showMessage("Removido com sucesso!");
                insideGrid.dataSource.clearCheckedRows();
                insideGrid.reload();
                insideGrid.getParent().dataSource.reload();

            }).catch((error) => {
                ScreenService.showMessage(ScreenService.i18n(error.error, insideGrid.currentRow));
            });
        });
    };

    /**
     * Conjunto de instruções a serem executadas
     * ao alterar a row clicada no grid principal.
     * 
     * @param {Args} args Objeto args.
     */
    this.widgetMainAfterMoveRow = function(args) {
        var insideGrid = args.owner.container.getWidget('insideGrid');
        insideGrid.dataSource.clearCheckedRows();

        if (insideGrid.isActive())
            insideGrid.reload();
    };

    /**
     * Função responsável por abrir o popup
     * para a seleção de um lider para a alteração.
     * 
     * @param {Args} args Objeto args.
     */
    this.abrePossiveisLideres = function(args) {
        var mainWidget = args.owner.widget.container.getWidget('mainWidget');
        var insideGrid = args.owner.widget;
        var widgetPossiveisLideres = args.owner.widget.container.getWidget('widgetPossiveisLideres');
        var arrayOperadores = [];

        arrayOperadores.push(mainWidget.currentRow.CDOPERADOR);
        insideGrid.dataSource.data.forEach((row) => arrayOperadores.push(row.CDOPERADOR));

        var filter = [
            {
                "name": "CDOPERADOR",
                "operator": "NOT_IN",
                "value": arrayOperadores
            }
        ];

        widgetPossiveisLideres.dataSourceFilter = filter;

        widgetPossiveisLideres.reload().then((success) => {
            ScreenService.openPopup(widgetPossiveisLideres);
        });
    };

    /**
     * Função responsável por alterar o lider
     * dos colaboradores de um outro lider.
     * 
     * @param {Args} args Objeto args.
     */
    this.defineNovoLider = function(args) {

        var widgetPossiveisLideres = args.owner;
        var mainWidget = args.owner.container.getWidget('mainWidget');

        widgetPossiveisLideres.currentRow.OLD_CDOPERADORLIDER = mainWidget.currentRow.CDOPERADOR;

        ScreenService.confirmMessage(ScreenService.i18n('Esta ação irá alterar o liderança dos colaboradores. Confirma alteração?'), 'question', (success) => {
            Tco00600_gerente_negocio_service.changeManager(widgetPossiveisLideres.currentRow).then(
                (success) => {
                    ScreenService.closePopup();
                    ScreenService.closeSwipe().then(() => {
                        mainWidget.reload();
                        mainWidget.activate();
                    });
                    ScreenService.successNotification(ScreenService.i18n('Lider alterado com sucesso.'), 3000);
                },
                (err) => ScreenService.showMessage(err.error));
        });

    };
};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco00600_gerente_negocio_controller', Tco00600_gerente_negocio_controller);
});

// FILE: ./assets/js/controllers/Tco00700_tipo_de_publico_controller.js
/* global getLocalVar angular, Configuration*/
var tco00700_tipo_de_publico_controller = function(ScreenService, General, tco00700_tipo_de_publico_service, MessageService) {

    this.cancel = function(args) {

        var widget = args.owner.widget;
        if (widget.currentRow.__is_new) {
            ScreenService.closeSwipe();
        } else {
            General.backOrCancel(args);
        }
        widget.reload();
    };

    this.backDetail = function(args) {
        ScreenService.closeSwipe();
        args.owner.widget.reload();
    };

    this.editarTipoPublico = function(args) {
        var widget = args.owner.widget;
        widget.edit();
    };

    this.deletarTipoPublico = function(args) {
        var widget = args.owner.widget;
        var row = angular.copy(widget.currentRow);
        ScreenService.confirmMessage(ScreenService.i18n('Tem certeza que deseja excluir o tipo de publico?'), 'question', function() {
            tco00700_tipo_de_publico_service.apagaTipoPublico([row]).then(function(callback) {
                if (callback.dataset.tiposPublicoUtilizados.length > 0) {
                    buildMessages(callback.dataset.tiposPublicoUtilizados).then(function(tipoPublicosUtilizados) {
                        General.showErrorMessages(widget, tipoPublicosUtilizados, ScreenService.i18n("Conflitos ao excluir Tipo de Público!"));
                    });
                } else {
                    ScreenService.showMessage('Exclusão realizada com sucesso!', 'success');
                    ScreenService.closeSwipe();
                    widget.reload();
                }
            }, function(error) {
                ScreenService.showMessage(error.data.error, 'error');
            });
        });
    };

    this.adicionarTipoPublico = function(args) {
        var widget = args.owner.widget;
        widget.newRow();
        widget.applyDefaultValue();
    };

    this.formataCampo = function(args) {
        var field = args.owner.field;
        var valor = field.value();
        if (valor) {
            field.setValue(valor.padStart(5, 0));
        }
    };

    this.salvaTipoPublico = function(args) {
        var widget = args.owner.widget;
        if (widget.isValid()) {
            var row = angular.copy(widget.currentRow);
            tco00700_tipo_de_publico_service.salvaTipoPublico([row]).then(function(callback) {
                if (callback.dataset.result.success) {
                    widget.currentRow.__is_new = false;
                    widget.currentRow.NRTIPOPUBLICO = callback.dataset.result.code;
                    widget.view();
                    widget.reload();
                    ScreenService.notificationMessage(ScreenService.i18n('Tipo de público salvo com sucesso!'), 'success', 3000);
                } else if (callback.dataset.result.error) {
                    ScreenService.showMessage(callback.dataset.result.error);
                }
            }.bind(this), function(error) {
                ScreenService.showMessage(error.data.error, 'error');
            });
        }
    };

    var buildMessages = function(data) {
        var mensagens = [];
        var msgParams = [];
        data.forEach(function(row) {
            //mensagens.push(ScreenService.i18n(row.MESSAGECODE, row));
            msgParams.push(row);
            mensagens.push(row.MESSAGECODE);
        });

        return MessageService.getMessagesByCode(mensagens).then(function(mensagensTraduzidas) {
            for (let i = 0; i < mensagensTraduzidas.length; i++) {
                mensagensTraduzidas[i] = ScreenService.i18n(mensagensTraduzidas[i], msgParams[i]);
            }
            return mensagensTraduzidas;
        });
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('tco00700_tipo_de_publico_controller', tco00700_tipo_de_publico_controller);
});

// FILE: ./assets/js/controllers/Tco00800_prospect_controller.js
function Tco00800_prospect_controller(General, ScreenService, Tco00800_prospect_service, TcoUtil, templateManager, localStorageEngine, $rootScope, MessageService) {

    /**
     * Prepara registro, definindo na currentRow
     * de um novo prospect os parâmetros de newCode
     * e columnsToReturn e salva usando microservice.
     * 
     * @param {Args} args Objeto args.
     */
    this.saveProspect = function(args) {
        var widget = args.owner.widget;
        var cRow = widget.currentRow;

        var isValidEmail = cRow.DSEMAILPROS ? this.isValidEmail(cRow.DSEMAILPROS) : true;

        if (widget.isValid() && isValidEmail) {
            this.setNewProspectParams(widget);

            Tco00800_prospect_service.saveProspect([cRow]).then((response) => {
                if (response.error) {
                    MessageService.showMessageByCode(response.error);
                } else {
                    let msg = cRow.__is_new ? 'Adicionado com sucesso.' : 'Registro salvo com sucesso.';
                    ScreenService.successNotification(msg);

                    widget.view();
                    widget.reload();

                    cRow.__is_new = false;
                    cRow.CDPROSPECT = response.dataset.columnsToReturn[0].CDPROSPECT;

                    General.changeWidgetAcessibility(widget.widgets, true);
                }

            }).catch((error) => {
                if (error.error) {
                    ScreenService.showMessage(ScreenService.i18n(error.error, cRow));
                }
            });
        }

    };

    /**
     * Remove o registro de prospect e trata o acontecimento
     * de erros e registros dependentes.
     * 
     * @param {Args} args Objeto args.
     */
    this.removeProspect = function(args) {
        var widget = args.owner.widget;
        var cRow = widget.currentRow;

        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', (success) => {
            Tco00800_prospect_service.removeProspect(cRow).then((success) => {

                if (success.notifications) {

                    ScreenService.closeSwipe().then(() => {
                        widget.activate();
                        widget.reload();
                        ScreenService.showMessage('Excluído com Sucesso!');
                    });

                } else if (success.dataset && success.dataset.invalidRemove) {

                    let data = success.dataset.invalidRemove;
                    MessageService.showMessageByCode('TCO00800_PROSPECT_USED_PROPCOM_WARNING').then(() => {

                        buildMessages(data).then(function(mensagens) {
                            General.showErrorMessages(widget, mensagens, ScreenService.i18n('Erro ao excluir.'));
                        });

                    });

                }

            }).catch((error) => {
                ScreenService.showMessage(ScreenService.i18n(error.error, widget.currentRow));
            });
        });
    };

    /**
     * Define os parâmetros de newCode e columnsToReturn
     * na currentRow da widget de prospect.
     * 
     * @param {Widget} widget Widget prospect.
     */
    this.setNewProspectParams = function(widget) {
        var cRow = widget.currentRow;

        TcoUtil.setNewCodeParametersInRow(widget);
        cRow.__columns_to_return = ['CDPROSPECT'];
    };

    /**
     * Preenche o valor da inscrição com zeros
     * a esquerda.
     * 
     * @param {Args} args Objeto args.
     */
    this.setPadOnInsc = function(args) {
        var field = args.owner.field;
        field.value(TcoUtil.padLeft(field.value(), 20));
    };

    var buildMessages = function(data) {
        var mensagens = [];
        var msgParams = [];
        data.forEach(function(row) {
            //mensagens.push(ScreenService.i18n(row.MESSAGECODE, row));
            msgParams.push(row);
            mensagens.push(row.MESSAGECODE);
        });

        return MessageService.getMessagesByCode(mensagens).then(function(mensagensTraduzidas) {
            for (let i = 0; i < mensagensTraduzidas.length; i++) {
                mensagensTraduzidas[i] = ScreenService.i18n(mensagensTraduzidas[i], msgParams[i]);
            }
            return mensagensTraduzidas;
        });
    };

    /**
     * Conjunto de instruções a serem executadas
     * ao alterar o campo de "País".
     * 
     * @param {Args} args Objeto args do campo
     *  alterado.
     */
    this.onChangePais = function(args) {
        var field = args.owner.field;
        var dependentField = field.widget.getField(field.dependentFieldName);

        this.setAddressField(dependentField);
        this.onChangeAddressField(field);
    };

    /**
     * Conjunto de instruções a serem executadas
     * ao alterar o campo de "Estado".
     * 
     * @param {Args} args Objeto args do campo
     *  alterado.
     */
    this.onChangeEstado = function(args) {
        var field = args.owner.field;
        var dependentField = field.widget.getField(field.dependentFieldName);

        this.setAddressField(dependentField);
        this.onChangeAddressField(field);
    };

    /**
     * Conjunto de instruções a serem executadas
     * ao alterar o campo de "Municipio".
     * 
     * @param {Args} args Objeto args do campo
     *  alterado.
     */
    this.onChangeMunicipio = function(args) {
        var field = args.owner.field;
        var dependentField = field.widget.getField(field.dependentFieldName);

        this.setAddressField(dependentField);
        this.onChangeAddressField(field);
    };

    /**
     * Conjunto de instruções a serem executadas
     * ao alterar algum campo de endereço (Pais, 
     * Estado, Municipio, Bairro). Limpa na currentRow
     * as propriedades indicadas no JSON do 
     * field indicados por 'propertyNamesToClear';
     * Define o estado readOnly do campo dependente
     * e aplica o filtro e a própria função de forma
     * recursiva a esse field.
     * 
     * @param {Field} field Field de endereço.
     */
    this.onChangeAddressField = function(field) {
        var widget = field.widget;
        var dependentField = widget.getField(field.dependentFieldName);
        var propertyNamesToClear = field.propertyNamesToClear;
        var readOnlyState = !Boolean(field.value());

        if (field.propertyNamesToClear)
            this.clearCurrentRowValues(widget.currentRow, propertyNamesToClear);

        if (dependentField !== undefined) {
            dependentField.readOnly = readOnlyState;
            dependentField.value('');
            this.onChangeAddressField(dependentField);
        }
    };

    /**
     * Define filtro no campo de endereço de acordo com
     * os parâmetros definidos e o valores da currentRow
     * da widget onde se encontra o field.
     * 
     * @param {Field} field Field onde será aplicado
     *  filtro, deve possuir as propriedades 'filterName'
     *  e 'orderName', que indicam respectivamente, o 
     *  campo da currentRow usado para filtro e o 
     *  campo usado para ordem.
     */
    this.setAddressField = function(field) {
        let cRow = field.widget.currentRow;

        if (!Array.isArray(field.dataSourceFilter))
            field.dataSourceFilter = [];

        let orderBy = field.dataSourceFilter.find(function(f) {
            return f.operator == 'ORDER_BY';
        });
        if (orderBy) {
            orderBy.name = [{
                name: field.orderName,
                direction: 'ASC'
            }];
        } else {
            field.dataSourceFilter.push({
                operator: 'ORDER_BY',
                name: [{
                    name: field.orderName,
                    direction: 'ASC'
                }]
            });
        }

        field.filterName.forEach((fName) => {
            let clause = field.dataSourceFilter.find(function(f) {
                return f.name == fName;
            });
            if (clause) {
                clause.value = cRow[fName];
            } else {
                field.dataSourceFilter.push({
                    name: fName,
                    operator: '=',
                    value: cRow[fName]
                });
            }
        });

        field.reload();
    };

    /**
     * Limpa os valores indicados no array de nomes
     * na row indicada.
     * 
     * @param {Row} row Objeto row.
     * @param {Array[String]} propertyNamesToClear Nomes
     *  das propriedades a serem limpas.
     */
    this.clearCurrentRowValues = function(row, propertyNamesToClear) {
        propertyNamesToClear.forEach(pName => {
            row[pName] = '';
        });
    };

    /**
     * Adiciona o endereço ao referido prospect.
     * 
     * @param {Args} args Objeto args.
     */
    this.saveAddress = function(args) {
        var widget = args.owner.widget;
        var cRow = widget.currentRow;

        var isValidEmail = cRow.DSEMAILPROS ? this.isValidEmail(cRow.DSEMAILPROS) : true;

        if (widget.isValid() && isValidEmail) {
            Tco00800_prospect_service.saveAddress(cRow).then((response) => {
                if (response.error) {
                    MessageService.showMessageByCode(response.error, cRow);
                } else {
                    ScreenService.closeSwipe();

                    let msg = cRow.__is_new ? 'Adicionado com sucesso.' : 'Registro salvo com sucesso.';
                    ScreenService.successNotification(msg);
                    widget.reload();
                }
            }).catch((error) => {
                if (error.error) {
                    ScreenService.showMessage(ScreenService.i18n(error.error, cRow));
                }
            });
        }
    };

    /**
     * Remove o registro de endereço de prospect.
     * 
     * @param {Args} args Objeto args.
     */
    this.removeAddress = function(args) {
        var widget = args.owner.widget;
        var cRow = widget.currentRow;

        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', (success) => {
            Tco00800_prospect_service.removeAddress(cRow).then((success) => {
                if (success.notifications) {

                    ScreenService.closeSwipe().then(() => {
                        widget.activate();
                        widget.reload();
                        ScreenService.showMessage('Excluído com Sucesso!');
                    });

                }
            }).catch((error) => {
                ScreenService.showMessage(ScreenService.i18n(error.error, widget.currentRow));
            });
        });
    };

    /**
     * Testa validade do email.
     * 
     * @param {Email} email Email a ser testado.
     * 
     * @return {Boolean} Estado da validade do 
     *  email.
     */
    this.isValidEmail = function(email) {
        var result = false;
        var pattern = new RegExp(
            "^[_a-z0-9-]+(\\.[_a-z0-9-]+)*@[a-z0-9-]+(\\.[a-z0-9-]+)*(\\.[a-z]{2,4})$");

        if (pattern.test(email))
            result = true;
        else
            ScreenService.showMessage('Email inválido.');

        return result;
    };

    /**
     * Conjunto de instruções a serem executadas
     * ao alterar seleção do registro na widget
     * principal. 
     * Fecha o swipe aberto na widget de endereço
     * caso o usuário troque de registro na widget
     * principal.
     * 
     * @param {Args} args Objeto Args
     */
    this.afterMoveRowMainWidget = function(args) {
        this.correctCloseAddressSwipe();
        this.readOnlyStateFields(args);
    };

    /**
     * Fecha o swipe aberto na widget de endereço
     * caso ele esteja aberto.
     */
    this.correctCloseAddressSwipe = function() {
        var actWid = templateManager.container.getActiveWidget();

        if (actWid.name == 'wid_endereco' && $rootScope.swipes.length > 1) {
            ScreenService.closeSwipe();
        }
    };

    this.readOnlyStateFields = function(args) {
        var widget = templateManager.container.getWidget('gridProspect');
        var addrFields = ['NMBAIRPROS', 'NMMUNICIPIO', 'NMESTADO'];

        addrFields.forEach((fName) => {
            let field = widget.getField(fName);
            if (!field.readOnly) field.readOnly = true;
        });
    };

    /**
     * Cria novo registro definindo NRORG
     * como valor padrao na row.
     * 
     * @param {Args} args Objeto args.
     */
    this.widgetNewRow = function(args) {
        var widget = args.owner.widget;

        General.widgetNewRow(widget);
        widget.currentRow.NRORG = localStorageEngine.getLocalVar('NRORG');
    };

    /**
     * Cria novo registro na widget de 
     * endereço;
     * 
     * @param {Args} args Objeto args.
     */
    this.newRowWidgetEndereco = function(args) {
        var widget = args.owner.widget;
        this.widgetNewRow(args);

        this.onChangeAddressField(widget.getField('NMPAIS'));
    };

    /**
     * Edita registro na widget de 
     * endereço;
     * 
     * @param {Args} args Objeto args.
     */
    this.editRowWidgetEndereco = function() {
        var widEndereco = templateManager.container.getWidget('wid_endereco');
        var addrFields = ['NMBAIRPROS', 'NMMUNICIPIO', 'NMESTADO', 'NMPAIS'];

        addrFields.forEach((fName) => {
            let field = widEndereco.getField(fName);
            let dependentField = widEndereco.getField(field.dependentFieldName);

            if (dependentField && !dependentField.value()) {
                this.onChangeAddressField(field);
                this.setAddressField(dependentField);
            }
        });

        General.widgetEdit(widEndereco);
    };

    this.onChangeSegmento = function(args) {
        var segmentoField = args.owner.field;
        var cRow = segmentoField.widget.currentRow;
        var ramoField = segmentoField.widget.getField('RAMOATIVIDADE');
        if (segmentoField.value() != "") {
            ramoField.dataSourceFilter = [{
                    "operator": "ORDER_BY",
                    "name": [{
                        "name": "CDARVATIVCLI",
                        "direction": "ASC"
                    }]
                },
                {
                    "name": "IDATIVO",
                    "operator": "=",
                    "value": "S"
                },
                {
                    "name": "CDSEGMER",
                    "operator": "=",
                    "value": cRow.CDSEGMER
                }
            ];
            ramoField.reload();
            ramoField.readOnly = false;
        } else {
            ramoField.setValue("");
            ramoField.reload();
            ramoField.readOnly = true;
        }
    };

    this.defineHeaderReport = function(args) {
        var parentCurrentRow = args.owner.getParent().currentRow;

        args.e.metadata.filter = [
            {
                "label": "Razão Social",
                "value": parentCurrentRow.NRINSJURPROS + ' - ' + parentCurrentRow.NMRAZSOCPROS
            }
        ];
    };

    this.defineHeaderReportProspects = function(args) {
        args.e.metadata.filter = [];
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register("Tco00800_prospect_controller", Tco00800_prospect_controller);
});

// FILE: ./assets/js/controllers/Tco00900ParametrosComerciais.js
var Tco00900ParametrosComerciais = function(ScreenService, General, Tco00900ParametrosComerciaisService, templateManager, ZHPromise, TcoUtil, MessageService) {

    /**
     * Função responsável por fazer a requsição de find da aba
     * Geral. Após feita a requsição, é jogado o resultado na 
     * currentRow.
     * 
     * Quando essa função é chamada pelo JSON, recebe args.
     * Recebe widget quando é chamada pela função de save.
     * 
     * @param {Args} args - Args da tela Geral.
     * @param {Widget} widget - Widget da tela geral.
     */

    this.findGeral = function(args, widget) {
        if (args != null)
            widget = args.owner;
        Tco00900ParametrosComerciaisService.findGeral().then(function(response) {
            if (!Util.isEmptyOrBlank(response.dataset.tco00900_paramtco[0])) {
                widget.currentRow = response.dataset.tco00900_paramtco[0];
                this.setBackupFilial(widget);
                this.concatDescriptionWithCod(widget.currentRow, widget);
            } else {
                widget.currentRow.BKP_CDFILIAL = "first";
            }
            widget.activate();
            widget.redraw();
        }.bind(this));
    };

    this.setBackupFilial = function(widget) {
        widget.currentRow.BKP_CDFILIAL = widget.currentRow.CDFILIAL;
    };

    /**
     * Função que concatena o código e a descrição dos campos
     * necessários.
     * 
     * @param {Object} currentRow - currentRow da tela Geral.
     * @param {Widget} widget - Widget Geral.
     */

    this.concatDescriptionWithCod = function(currentRow, widget) {
        if (currentRow.CDFILIAL)
            widget.currentRow.NMFILIAL = currentRow.CDFILIAL + ' | ' + currentRow.NMFILIAL;
        if (currentRow.NRPERFILACESSO)
            widget.currentRow.NMPERFILACESSO = currentRow.NRPERFILACESSO + ' | ' + currentRow.NMPERFILACESSO;
        if (currentRow.CDTIPOCUSTO)
            widget.currentRow.NMTIPOCUSTO = currentRow.CDTIPOCUSTO + ' | ' + currentRow.NMTIPOCUSTO;
    };

    this.setDataSourceFilter = function(args) {
        var field = args.owner.field;
        field.dataSourceFilter = [
            {
                "name": "IDUTTPCUSTO",
                "value": "S",
                "operator": "="
			},
            {
                "name": "NRORG",
                "value": getLocalVar('NRORG'),
                "operator": "="
			}
        ];
        field.reload();
    };

    this.setFirstBackupFilial = function(args) {
        var widget = args.owner.field.widget;
        var bkpFilial = widget.currentRow.BKP_CDFILIAL;
        if (bkpFilial === "first")
            widget.currentRow.BKP_CDFILIAL = widget.currentRow.CDFILIAL;
    };

    /**
     * Função de save da tela Geral.
     * 
     * @param {Args} args - Args da tela Geral.
     */

    this.saveGeral = function(args) {
        var widget = args.owner.widget;
        if (widget.isValid()) {
            this.confirmMessage(widget).then(function() {
                Tco00900ParametrosComerciaisService.saveGeral(widget.currentRow).then(function() {
                    widget.view();
                    this.enableWidgets(widget.getParent());
                    this.findGeral(null, widget);
                    widget.currentRow.BKP_CDFILIAL = widget.currentRow.CDFILIAL;
                    ScreenService.successNotification("Salvo com sucesso!");
                }.bind(this), function(error) {
                    ScreenService.showMessage(error.error);
                }.bind(this));
            }.bind(this));
        }
    };

    this.confirmMessage = function(widget) {
        var promise = ZHPromise.defer();
        var cdfilial = Util.isEmptyOrBlank(widget.currentRow.CDFILIAL) ? null : widget.currentRow.CDFILIAL;

        if (cdfilial !== widget.currentRow.BKP_CDFILIAL && widget.currentRow.BKP_CDFILIAL !== "first") {
            MessageService.confirmMessageByCode('TCO00900_CONFIRMAR_ALTERACAO_FILIAL').then(function(success) {
                    promise.resolve();
                },
                function(error) {
                    promise.reject();
                }
            );
        } else {
            promise.resolve();
        }

        return promise.promise;
    };

    this.inlineEdit = function(args) {
        var widget = args.owner.field.widget;
        widget.backupRow = angular.copy(widget.currentRow);
        widget.inlineEdit();
    };

    this.newRow = function(args) {
        var widget = args.owner.widget;
        this.disableLimitador(widget);
        General.newRow(widget);
    };

    /**
     * Função que desabilita o campo Limitador de Custo
     * quando a widget Quantidade por Refeições entra no
     * modo de edição.
     * 
     * @param {Widget} widget - widgetQtdPorRefeicao.
     */

    this.disableLimitador = function(widget) {
        widget.getParent().getField('VRLIMITECUSTO').readOnly = true;
    };

    /**
     * Função que habilita o campo Limitador de Custo.
     * 
     * @param {Widget} widget - widgetQtdPorRefeicao.
     */

    this.enableLimitador = function(widget) {
        widget.getParent().getField('VRLIMITECUSTO').readOnly = false;
    };

    /**
     * Função que habilita as widgets que foram desabilitada
     * após o modo edição.
     * 
     * @param {Widget} widget - widgetTabbed.
     */

    this.enableWidgets = function(widget) {
        widget.widgets.forEach(function(widget) {
            widget.enabled = true;
        });
    };

    this.cancelGrid = function(args) {
        var widget = args.owner.widget;
        this.enableLimitador(widget);
        General.backOrCancel(args);
        widget.reload();
    };

    this.cancelInlineEdit = function(args) {
        var widget = args.owner.widget;
        var tabbedWidget = args.owner.widget.getParent();
        tabbedWidget.currentRow = tabbedWidget.backupRow;
        widget.list();
        tabbedWidget.mode = 'list';
    };

    this.back = function(args) {
        General.backOrCancel(args);
        this.enableLimitador(args.owner.widget);
        args.owner.widget.reload();
    };

    /**
     * Função responsável por bloquear a abertura do swipe na widget filha ao 
     * editar o campo limitador de custo.
     * 
     */

    this.validateMoveRow = function(args) {
        var tabbedWidget = args.owner.getParent();
        if (tabbedWidget.mode == "inline_edit") {
            args.e.preventDefault();
        }
    };

    /**
     * Função responsável por atualizar o campo VRLIMITECUSTO.
     * 
     */

    this.setLimitador = function() {
        var widget = templateManager.container.getWidget("widgetServicosComplementares");
        Tco00900ParametrosComerciaisService.findLimitador().then(function(response) {
            if (!Util.isEmptyOrBlank(response.dataset.tco00900_findLimitador[0])) {
                widget.currentRow = response.dataset.tco00900_findLimitador[0];
            }
        }.bind(this));
    };

    /**
     * Função de save da tela widgetServicosComplementares.
     * É acionada pelo botão de salvar inlineEdit.
     * Realiza validações básicas do campo Limitador de Custo.
     * 
     */

    this.saveLimitadorCustos = function() {
        var widget = templateManager.container.getWidget("widgetServicosComplementares");
        if (widget.isValid()) {
            Tco00900ParametrosComerciaisService.saveLimitadorCustos(widget.currentRow).then(function() {
                this.setLimitador();
                widget.list();
                widget.childTemplate = "widget/form.html";
                ScreenService.successNotification("Salvo com sucesso!");
            }.bind(this), function(error) {
                ScreenService.showMessage(error.error);
            });
        }
    };

    /**
     * Função de save da tela widgetQtdPorRefeicao.
     * Verifica se os campos numéricos são maiores que zero.
     * 
     * @param {Args} args - Args da widgetQtdPorRefeicao.
     */
    this.saveQtdPorRefeicao = function(args) {
        var widget = args.owner.widget;
        if (widget.isValid()) {
            if (parseFloat(TcoUtil.parseFloat(widget.currentRow.QTINICIAL)) >= 0 && parseFloat(TcoUtil.parseFloat(widget.currentRow.QTFINAL)) >= 0 && parseFloat(TcoUtil.parseFloat(widget.currentRow.QTSERVICO)) >= 0) {
                if (this.validaQuantidades(widget)) {
                    Tco00900ParametrosComerciaisService.saveQtdPorRefeicao(widget.currentRow).then(function(response) {
                        this.enableLimitador(widget);
                        widget.currentRow.__is_new = false;
                        ScreenService.successNotification("Salvo com sucesso!");
                        widget.currentRow.NRSERCOMPREFEICAO = response.dataset.tco00900_qtdPorRefeicao[0].NRSERCOMPREFEICAO;
                        widget.reload();
                        widget.view();
                    }.bind(this), function(error) {
                        ScreenService.showMessage(error.error);
                    });
                }
            } else {
                MessageService.showMessageByCode('TCO00900_VALORES_NEGATIVOS');
            }
        }
    };

    this.validaQuantidades = function(widget) {
        if (TcoUtil.parseFloat(widget.currentRow.QTINICIAL) > TcoUtil.parseFloat(widget.currentRow.QTFINAL)) {
            MessageService.showMessageByCode('TCO00900_INICIAL_MAIOR_FINAL');
            return false;
        } else {
            return true;
        }
    };

    this.remove = function(args) {
        var widget = args.owner.widget;
        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function(success) {
            Tco00900ParametrosComerciaisService.removeQtdPorRefeicao(widget.currentRow).then(function() {
                this.enableLimitador(widget);
                ScreenService.showMessage("Excluído com sucesso.", 'success');
                ScreenService.closeSwipe().then(function() {
                    widget.reload();
                });
            }.bind(this), function(error) {
                if (error.data && error.data.error.indexOf('ORA-02292') !== -1) {
                    ScreenService.showMessage('Exclusão não permitida! O registro possui registros filhos.');
                } else {
                    ScreenService.showMessage(error.data.error);
                }
            }.bind(this));
        }.bind(this));
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco00900ParametrosComerciais', Tco00900ParametrosComerciais);
});

// FILE: ./assets/js/controllers/Tco01000_cardapio_base_controller.js
/* global angular, Configuration*/
var Tco01000_cardapio_base_controller = function(ScreenService, General, Tco01000_cardapio_base_service, TcoUtil, templateManager, ZhKeyboardService, MessageService) {

    this.applyFilterByHistory = function(args) {
        var widgetMain = args.owner.getWidget('widgetMain');

        widgetMain.getField("PERIODO").defaultValue = {
            start: moment().startOf("month").format("DD/MM/YYYY"),
            end: moment().endOf("month").format("DD/MM/YYYY")
        };

        if (window.history.state && window.history.state.cardapioBaseFilter) {
            widgetMain.dataSourceFilter = window.history.state.cardapioBaseFilter;
            widgetMain.reload();
            ScreenService.closePopup();
        }

    };

    this.abreFiltro = function(keyEvent) {
        var widgetMain = templateManager.container.getWidget('widgetMain');

        if (widgetMain.filterRowBackup) {
            widgetMain.fields.forEach((field) => {
                if (widgetMain.filterRowBackup.hasOwnProperty(field.name)) {
                    field.defaultValue = widgetMain.filterRowBackup[field.name];
                }
            });
        }

        TcoUtil.startFilter(widgetMain);

        if (typeof keyEvent !== "undefined") {
            keyEvent.preventDefault();
            keyEvent.stopPropagation();
            keyEvent.stopImmediatePropagation();
        }
    };

    this.backDetail = function() {
        ScreenService.closeSwipe();
    };

    this.cancelaEdit = function(args) {
        General.widgetCancelForm(args.owner.widget);
    };

    this.fixFilterBeforeApply = function(args) {
        var widgetMain = args.owner;
        var filterRow = args.e.preference.FILTER_ROW;
        var fixedFilter = [
            {
                "name": "CDCARDAPIOBASE",
                "operator": "=",
                "value": typeof filterRow.CDCARDAPIOBASE === "undefined" || filterRow.CDCARDAPIOBASE == "" ? null : filterRow.CDCARDAPIOBASE
            },
            {
                "name": "NMSERVICOPROPOSTA",
                "operator": "=",
                "value": typeof filterRow.NMSERVICOPROPOSTA === "undefined" || filterRow.NMSERVICOPROPOSTA == "" ? null : "_" + filterRow.NMSERVICOPROPOSTA.join("_")
            },
            {
                "name": "DSSEGMER",
                "operator": "=",
                "value": typeof filterRow.DSSEGMER === "undefined" || filterRow.DSSEGMER == "" ? null : "_" + filterRow.DSSEGMER.join("_")
            },
            {
                "name": "NMPADRAOQUALIDADE",
                "operator": "=",
                "value": typeof filterRow.NMPADRAOQUALIDADE === "undefined" || filterRow.NMPADRAOQUALIDADE == "" ? null : "_" + filterRow.NMPADRAOQUALIDADE.join("_")
            },
            {
                "name": "DTINIVIGENCIA",
                "operator": "=",
                "value": filterRow.PERIODO.start,
            },
            {
                "name": "DTFIMVIGENCIA",
                "operator": "=",
                "value": filterRow.PERIODO.end,
            },
            {
                "name": "IDATIVO",
                "operator": "=",
                "value": typeof filterRow.IDATIVO === "undefined" || filterRow.IDATIVO == "" ? filterRow.IDATIVO.value = "T" : filterRow.IDATIVO
            }
        ];
        widgetMain.dataSourceFilter = fixedFilter;
        widgetMain.reload().then(function() {
            ScreenService.closePopup();
            widgetMain.activate();
            widgetMain.filterRowBackup = filterRow;
        });
        args.e.preventDefault();
    };

    this.novoCardapioBase = function(args) {
        var widgetMain = args.owner.widget;
        widgetMain.newRow();
        widgetMain.currentRow.IDATIVO = "S";
    };

    this.salvaCardapioBase = function(args) {
        var widgetMain = args.owner.widget;
        var currentRow = args.owner.widget.currentRow;
        if (widgetMain.isValid()) {
            if (isBellowSevenDays(currentRow.QTDDIASCARDBASE)) {
                currentRow.__using_unit = widgetMain.paramsNewCode.__using_unit || false;
                currentRow.__new_code_pk = widgetMain.paramsNewCode.__new_code_pk;
                currentRow.__table_pk = widgetMain.paramsNewCode.__table_pk;
                currentRow.__new_code_length = widgetMain.paramsNewCode.__new_code_length || 1;
                currentRow.__cast_number = widgetMain.paramsNewCode.__cast_number || false;
                setPeriodoCurrentRow(currentRow, 'PERIODO', 'DTINIVIGENCIA', 'DTFIMVIGENCIA');
                Tco01000_cardapio_base_service.salvaNovoCardapioBase(currentRow).then(function(success) {
                    if (typeof success.messages !== 'undefined' && success.messages.length > 0) {
                        MessageService.showMessageByCode(success.messages[0].message);
                    } else if (typeof success.dataset.cardapioBase !== "undefined" && success.dataset.cardapioBase.length > 0) {
                        MessageService.showMessageByCode(success.dataset.cardapioBase[0].MESSAGECODE, success.dataset.cardapioBase[0]);
                    } else {
                        if (typeof success.dataset.columnsToReturn !== "undefined" && success.dataset.columnsToReturn.length > 0) {
                            widgetMain.currentRow = success.dataset.columnsToReturn[0];
                        }
                        ScreenService.notificationMessage(ScreenService.i18n("Cardápio Base salvo com sucesso."), 'success', 3000);
                        widgetMain.reload();
                        widgetMain.view();
                    }
                });
            } else {
                ScreenService.showMessage("Qtd. Dias deve ser de no mínimo 7.");
            }
        }
    };

    var isBellowSevenDays = function(value) {
        if (parseInt(value) >= 7) {
            return true;
        } else {
            return false;
        }
    };

    var setPeriodoCurrentRow = function(row, nameIntervalField, nameFieldDtIni, nameFieldDtFim) {
        row[nameFieldDtIni] = row[nameIntervalField].start;
        row[nameFieldDtFim] = row[nameIntervalField].end;
    };

    this.removeCardapioBaseSelecionado = function(keyEvent) {
        var widgetMain = templateManager.container.getWidget('widgetMain');
        var currentRow = widgetMain.currentRow;

        if (typeof widgetMain.currentRow.NRCARDAPIOBASE !== "undefined") {

            ScreenService.confirmMessage(ScreenService.i18n('Remover Cardapio Base?'), 'question', function(success) {
                Tco01000_cardapio_base_service.verificaComposicaoCardapioBaseExistente(currentRow).then(function(success) {

                    if (success.dataset.compCardapioBaseExistente.length) {
                        ScreenService.confirmMessage(ScreenService.i18n('Atenção! Existem pratos lançados para o cardápio. Deseja prosseguir?'), 'alert', function(success) {
                            removeCardapio(currentRow, widgetMain);
                        });
                    } else {
                        removeCardapio(currentRow, widgetMain);
                    }

                });
            });

        }
    };
    var removeCardapio = function(currentRow, widgetMain) {
        Tco01000_cardapio_base_service.removeCardapioBase(currentRow).then(function(success) {
            if (success.dataset.cardapioBaseEmUso.length > 0) {
                buildMessages(success.dataset.cardapioBaseEmUso).then(function(cardapioBaseEmUso) {
                    General.showErrorMessages(widgetMain, cardapioBaseEmUso, ScreenService.i18n("Cardapio Base em uso."));
                });
            } else {
                ScreenService.successNotification(ScreenService.i18n("Cardapio Base excluído com sucesso."), null, "checked");
                if (ScreenService.isSwipeOpen()) {
                    ScreenService.closeSwipe().then(function() {
                        widgetMain.reload();
                    });
                } else {
                    widgetMain.reload();
                }
            }
        }, function(err) {
            ScreenService.showMessage(err.error);
        });
    };

    var buildMessages = function(data) {
        var mensagens = [];
        var msgParams = [];
        data.forEach(function(row) {
            //mensagens.push(ScreenService.i18n(row.MESSAGECODE, row));
            msgParams.push(row);
            mensagens.push(row.MESSAGECODE);
        });

        return MessageService.getMessagesByCode(mensagens).then(function(mensagensTraduzidas) {
            for (let i = 0; i < mensagensTraduzidas.length; i++) {
                mensagensTraduzidas[i] = ScreenService.i18n(mensagensTraduzidas[i], msgParams[i]);
            }
            return mensagensTraduzidas;
        });
    };

    this.buildFilterCardComplementar = function(args) {
        var widgetMain = args.owner.field.widget;
        var segmentos = [];
        var fieldCdcardcomplcardbase = args.owner.field.widget.getField('CDCARDCOMPLCARDBASE');
        widgetMain.getField('DSSEGMER').dataSource.getCheckedRows().forEach(function(segmento) {
            segmentos.push(segmento.CDSEGMER);
        });

        fieldCdcardcomplcardbase.dataSourceFilter = [
            {
                "name": "SEGMENTOS",
                "operator": "=",
                "value": segmentos.join('_')
            },
            {
                "name": "NRPADRAOQUALIDADE",
                "operator": "=",
                "value": widgetMain.currentRow.NRPADRAOQUALIDADE
            },
            {
                "name": "NRSERVICOPROPOSTA",
                "operator": "NOT_IN",
                "value": widgetMain.currentRow.NRSERVICOPROPOSTA
            },
            {
                "operator": "ORDER_BY",
                "name": [
                    {
                        "name": "CDCARDAPIOBASE",
                        "direction": "ASC"
                    }
                ]
            }
        ];

        fieldCdcardcomplcardbase.reload();
    };

    this.abreCardapio = function(args) {
        var widgetMain = args.owner.widget;

        var params = {
            elaboracaoCardapioBaseParams: widgetMain.currentRow,
            cardapioBaseFilter: widgetMain.dataSourceFilter
        };

        TcoUtil.openWindowByHistory('#/tco#tco01001_elaboracao_cardapio', params);
    };

    this.abreUtilizaSubstituiPratosCardapioBase = function(args) {
        var widgetMain = args.owner.widget.container.getWidget('widgetMain');

        var params = {
            cardapioBaseFilter: widgetMain.dataSourceFilter
        };

        TcoUtil.openWindowByHistory('#/tco#tco01007_utiliza_substitui_pratos_cardapio_base', params);
    };

    this.setDefaultPeriod = function() {};

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01000_cardapio_base_controller', Tco01000_cardapio_base_controller);
});

// FILE: ./assets/js/controllers/Tco01001_elaboracao_cardapio_controller.js
/* global angular, Configuration*/
var Tco01001_elaboracao_cardapio_controller = function(ScreenService, General, TcoUtil, Tco01001_elaboracao_cardapio_service, templateManager, MessageService) {

    const GLOBAL_COD_WEEKEND = ['TCODOM', 'TCOSAB'];
    let GLOBAL_MSG_WEEKEND = [];

    this.carregaParametros = function(args) {
        if (!window.history.state) {
            window.location.href = window.location.href.replace("#/tco#tco01001_elaboracao_cardapio", "#/tco#tco01000_cardapio_base");
            window.location.reload();
        } else {
            var widgetMain = args.owner.getWidget("widgetMain");
            var stripeWidget = args.owner.getWidget("stripeWidget");
            widgetMain.elaboracaoCardapioBaseParams = window.history.state.elaboracaoCardapioBaseParams;
            var filter = [
                {
                    "name": "NRCARDAPIOBASE",
                    "operator": "=",
                    "value": window.history.state.elaboracaoCardapioBaseParams.NRCARDAPIOBASE
                }
            ];

            TcoUtil.bindKeyboardEvents(this.addOrUpdate, this.delete);

            widgetMain.dataSourceFilter = filter;
            MessageService.getMessagesByCode(angular.copy(GLOBAL_COD_WEEKEND)).then((callback) => {
                GLOBAL_MSG_WEEKEND = callback.map(msg => ScreenService.i18n(msg));
                widgetMain.reload().then((success) => {
                    stripeWidget.currentRow = rowAjustadaParaStripe(widgetMain.elaboracaoCardapioBaseParams);
                    this.validVisibilityAction(widgetMain);
                    widgetMain.activate();
                });
            });
        }
    };

    this.validVisibilityAction = function(widgetMain) {
        var showAction = (window.history.state.elaboracaoCardapioBaseParams.IDCOMPLEMENTAR === "S");
        widgetMain.getAction("CARDAPIO_COMPLEMENTADOS_CARDAPIO_BASE").isVisible = showAction;
    };

    this.setKeyboardEvents = function(args) {
        TcoUtil.bindKeyboardEvents(this.addOrUpdate, this.delete);
    };

    var rowAjustadaParaStripe = function(row) {
        row.DSIDATIVO = row.IDATIVO == "S" ? ScreenService.i18n("Ativo") : ScreenService.i18n("Inativo");
        row.DSPERIODO = moment(row.DTINIVIGENCIA, "DD/MM/YYYY").format("DD/MM/YYYY") + " - " + moment(row.DTFIMVIGENCIA, "DD/MM/YYYY").format("DD/MM/YYYY");
        return row;
    };

    this.backCardBase = function() {
        window.history.back();
    };

    this.closePopup = function() {
        ScreenService.closePopup();
    };

    this.concatenaSemanaEDias = function(args) {
        var widgetMain = args.owner;

        MessageService.getMessageByCode('TCO01001_DIA').then(function(mensagemDia) {
            widgetMain.dataSource.data.forEach(function(row) {
                MessageService.getMessageByCode(row.LABELDIA).then(function(labelDia) {
                    row.LABELDIA = ScreenService.i18n(mensagemDia, {
                        NRSEMANA: row.NRSEMANA,
                        LABELDIA: ScreenService.i18n(labelDia)
                    });
                    row.defaultLabel = labelDia;
                });
            });
        });

    };

    this.abreConfigCardapioBase = function(args) {
        var widgetMain = args.owner.widget.container.getWidget('widgetMain');
        var stripeWidget = args.owner.widget.container.getWidget('stripeWidget');

        var params = {
            configCardapioBaseParams: widgetMain.dataSourceFilter,
            elaboracaoCardapioBaseParams: widgetMain.dataSourceFilter[0].value,
            stripeData: stripeWidget.currentRow
        };

        TcoUtil.openWindowByHistory('#/tco#tco01002_configuracao_cardapio', params);
    };

    this.definePrato = function(args) {
        if (TcoUtil.isClickOnFocusedCell()) {
            var widgetMain = args.owner;
            if (!Util.isEmptyOrBlank(widgetMain.currentRow.CDTIPOPRAT)) {
                widgetMain.currentRow.CDSERVFILI = window.history.state.elaboracaoCardapioBaseParams.CDSERVFILI;
                var params = {
                    configCardapioBaseParams: widgetMain.dataSourceFilter,
                    elaboracaoCardapioBaseParams: widgetMain.dataSourceFilter[0].value,
                    celulaElaboracaoCardapioBaseSelecionada: widgetMain.currentRow,
                    parametros: window.history.state.elaboracaoCardapioBaseParams
                };

                TcoUtil.openWindowByHistory('#/tco#tco01003_dados_do_prato', params);
            } else {
                ScreenService.showMessage('É necessário realizar a configuração do cardápio');
            }
        } else {
            TcoUtil.setCurrentNavigationElement();
        }
    };

    this.alterarVigenciaCardapioBase = function(args) {
        var popupVigencia = args.owner.widget.container.getWidget("popupVigencia");
        var widgetMain = args.owner.widget;

        popupVigencia.currentRow = widgetMain.elaboracaoCardapioBaseParams;
        popupVigencia.currentRow.PERIODO = {
            start: popupVigencia.currentRow.DTINIVIGENCIA,
            end: popupVigencia.currentRow.DTFIMVIGENCIA,
            formattedStart: popupVigencia.currentRow.DTINIVIGENCIA,
            formattedEnd: popupVigencia.currentRow.DTFIMVIGENCIA
        };

        ScreenService.openPopup(popupVigencia);
    };

    this.salvaNovaVigencia = function(args) {
        var widgetMain = args.owner.widget.container.getWidget("widgetMain");
        var stripeWidget = args.owner.widget.container.getWidget("stripeWidget");
        var currentRow = args.owner.widget.currentRow;

        setPeriodoCurrentRow(currentRow, 'PERIODO', 'DTINIVIGENCIA', 'DTFIMVIGENCIA');
        Tco01001_elaboracao_cardapio_service.alteraCardapioBase(currentRow).then(function(success) {
            if (typeof success.messages !== 'undefined' && success.messages.length > 0) {
                MessageService.showMessageByCode(success.messages[0].message);
            } else if (typeof success.dataset.cardapioBase !== "undefined" && success.dataset.cardapioBase.length > 0) {
                MessageService.showMessageByCode(success.dataset.cardapioBase[0].MESSAGECODE, success.dataset.cardapioBase[0]);
            } else {
                ScreenService.notificationMessage(ScreenService.i18n("Vigência alterada com sucesso."), 'success', 3000);
                widgetMain.reload().then((success) => {
                    ScreenService.closePopup();
                    stripeWidget.currentRow.DSPERIODO = moment(currentRow.DTINIVIGENCIA, "DD/MM/YYYY").format("DD/MM/YYYY") + " - " + moment(currentRow.DTFIMVIGENCIA, "DD/MM/YYYY").format("DD/MM/YYYY");
                });
            }
        });
    };

    var setPeriodoCurrentRow = function(row, nameIntervalField, nameFieldDtIni, nameFieldDtFim) {
        row[nameFieldDtIni] = row[nameIntervalField].start;
        row[nameFieldDtFim] = row[nameIntervalField].end;
    };

    this.alterarSituacaoCardapioBase = function(args) {
        var widgetMain = args.owner.widget.container.getWidget("widgetMain");
        var stripeWidget = args.owner.widget.container.getWidget("stripeWidget");

        widgetMain.elaboracaoCardapioBaseParams.DSIDATIVO = widgetMain.elaboracaoCardapioBaseParams.IDATIVO == "S" ? ScreenService.i18n("Ativo") : ScreenService.i18n("Inativo");
        widgetMain.elaboracaoCardapioBaseParams.descricaoSituacao = widgetMain.elaboracaoCardapioBaseParams.IDATIVO == "S" ? ScreenService.i18n("Inativar") : ScreenService.i18n("Ativar");

        var ativarCardapio = () => this.ativarCardapioBase(widgetMain, stripeWidget);

        var inativarCardapio = () => this.inativarCardapioBase(widgetMain, stripeWidget);

        ScreenService.showCustomDialog(ScreenService.i18n('Alterar situação do cardápio __CDCARDAPIOBASE__?', widgetMain.elaboracaoCardapioBaseParams), 'question', undefined, [
            {
                label: ScreenService.i18n('Ativar'),
                code: ativarCardapio
    		},
            {
                label: ScreenService.i18n('Inativar'),
                code: inativarCardapio
    		},
            {
                label: ScreenService.i18n('Cancelar'),
                code: ''
    		}
		]);

    };

    this.ativarCardapioBase = function(widgetMain, stripeWidget) {
        var row = widgetMain.elaboracaoCardapioBaseParams;
        row.PERIODO = {};
        row.PERIODO.start = row.DTINIVIGENCIA;
        row.PERIODO.end = row.DTFIMVIGENCIA;
        row.IDATIVO = "S";

        Tco01001_elaboracao_cardapio_service.alteraCardapioBase(row).then((success) => {
            if (!Util.isEmptyOrBlank(success.dataset.cardapioBase)) {
                MessageService.showMessageByCode(success.dataset.cardapioBase[0].MESSAGECODE, success.dataset.cardapioBase[0]);
            } else {
                stripeWidget.currentRow = rowAjustadaParaStripe(widgetMain.elaboracaoCardapioBaseParams);
            }
            ScreenService.successNotification(ScreenService.i18n('Situação alterada com sucesso'), 3000);
            ScreenService.closePopup();
        });
    };

    this.inativarCardapioBase = function(widgetMain, stripeWidget) {
        var row = widgetMain.elaboracaoCardapioBaseParams;
        row.PERIODO = {};
        row.PERIODO.start = row.DTINIVIGENCIA;
        row.PERIODO.end = row.DTFIMVIGENCIA;
        row.IDATIVO = "N";

        Tco01001_elaboracao_cardapio_service.alteraCardapioBase(row).then((success) => {
            if (!Util.isEmptyOrBlank(success.dataset.cardapioBase)) {
                MessageService.showMessageByCode(success.dataset.cardapioBase[0].MESSAGECODE, success.dataset.cardapioBase[0]);
            } else {
                stripeWidget.currentRow = rowAjustadaParaStripe(widgetMain.elaboracaoCardapioBaseParams);
            }
            ScreenService.successNotification(ScreenService.i18n('Situação alterada com sucesso'), 3000);
            ScreenService.closePopup();
        });
    };

    this.abrirImportacaoCardapioBase = function(args) {
        var widgetMain = args.owner.widget;
        var popupImportacaoCardapio = args.owner.widget.container.getWidget("popupImportacaoCardapio");

        popupImportacaoCardapio.getField("CDCARDAPIOBASE").dataSourceFilter = [
            {
                "name": "NRCARDAPIOBASE",
                "operator": "<>",
                "value": widgetMain.elaboracaoCardapioBaseParams.NRCARDAPIOBASE
            },
            {
                "operator": "ORDER_BY",
                "name": [
                    {
                        "name": "CDCARDAPIOBASE",
                        "direction": "ASC"
                    }
                ]
            }
        ];

        popupImportacaoCardapio.getField("CDCARDAPIOBASE").reload();
        popupImportacaoCardapio.getField("CDCARDAPIOBASE").setValue(null);
        popupImportacaoCardapio.getField("TODAS_CONFIGURACOES").setValue("T");
        General.toggleReadOnlySelect(popupImportacaoCardapio.getField("TODAS_CONFIGURACOES"), popupImportacaoCardapio.getField('NMTIPOPRAT'));
        popupImportacaoCardapio.getField("COPIAR_PRATOS").setValue("S");
        popupImportacaoCardapio.getField("COPIAR_REGRAS").setValue("S");

        popupImportacaoCardapio.currentRow.NRCARDAPIOBASEDEST = widgetMain.elaboracaoCardapioBaseParams.NRCARDAPIOBASE;

        ScreenService.openPopup(popupImportacaoCardapio);
    };

    this.importarCardapioBase = function(args) {
        var widgetMain = args.owner.widget.container.getWidget("widgetMain");
        var popupImportacaoCardapio = args.owner.widget;
        if (popupImportacaoCardapio.isValid()) {
            Tco01001_elaboracao_cardapio_service.importarCardapioBase(popupImportacaoCardapio.currentRow).then(
                (success) => {
                    if (success.error) {
                        MessageService.showMessageByCode(success.error);
                    } else {
                        MessageService.successNotificationByCode('TCO_SUCESSO');
                        widgetMain.reload().then(() => {
                            ScreenService.closePopup();
                        });
                    }
                },
                (err) => ScreenService.showMessage(err.error));
        }
    };

    this.abrirExcluirPratoCardapioBase = function(args) {
        var widgetMain = args.owner.widget;
        var popupExclusaoPrato = args.owner.widget.container.getWidget("popupExclusaoPrato");

        popupExclusaoPrato.applyDefaultValue();
        popupExclusaoPrato.getField("DSSEMANA").dataSourceFilter = widgetMain.dataSourceFilter;
        popupExclusaoPrato.getField("NMTIPOPRAT").dataSourceFilter = widgetMain.dataSourceFilter;
        popupExclusaoPrato.getField("DSSEMANA").reload().then(() => ScreenService.openPopup(popupExclusaoPrato));
        popupExclusaoPrato.getField("NMTIPOPRAT").reload();
    };

    this.montaLabelsSemanaDia = function(args) {
        var widgetMain = args.owner;

        MessageService.getMessageByCode('TCO01001_DIA').then(function(mensagemDia) {
            widgetMain.dataSource.data.forEach(function(row) {
                MessageService.getMessageByCode(row.DSSEMANA.slice(1, row.DSSEMANA.length)).then(function(labelDia) {
                    row.DSSEMANA = ScreenService.i18n(mensagemDia, {
                        NRSEMANA: row.NRSEMANA,
                        LABELDIA: labelDia
                    });
                });
            });
        });
    };

    this.excluirPratoCardapioBase = function(args) {
        var popupExclusaoPrato = args.owner.widget;
        var widgetMain = args.owner.widget.container.getWidget("widgetMain");
        let row = angular.copy(popupExclusaoPrato.currentRow);
        row.NRCARDAPIOBASE = window.history.state.elaboracaoCardapioBaseParams.NRCARDAPIOBASE;
        ScreenService.confirmMessage(ScreenService.i18n('Remover Prato(s)?'), 'question', (success) => {
            Tco01001_elaboracao_cardapio_service.excluirPratoCardapioBase(row).then(
                (success) => {
                    if (success.error) {
                        MessageService.showMessageByCode(success.error);
                    } else {
                        ScreenService.successNotification(ScreenService.i18n('Registros excluídos com sucesso'));
                        widgetMain.reload().then(() => {
                            ScreenService.closePopup(true);
                        });
                    }
                },
                (err) => ScreenService.showMessage(err.error));
        });
    };

    this.abreRegrasCardapio = function(args) {

        var widgetMain = args.owner.widget.container.getWidget('widgetMain');
        var stripeWidget = args.owner.widget.container.getWidget('stripeWidget');

        var params = {
            configCardapioBaseParams: widgetMain.dataSourceFilter,
            elaboracaoCardapioBaseParams: widgetMain.dataSourceFilter[0].value,
            stripeData: stripeWidget.currentRow
        };

        TcoUtil.openWindowByHistory('#/tco#tco01006_regras_cardapio', params);

    };

    this.abreCardapioComplementado = function(args) {

        var widgetMain = args.owner.widget.container.getWidget('widgetMain');
        var stripeWidget = args.owner.widget.container.getWidget('stripeWidget');

        var params = {
            configCardapioBaseParams: widgetMain.dataSourceFilter,
            elaboracaoCardapioBaseParams: widgetMain.dataSourceFilter[0].value,
            stripeData: stripeWidget.currentRow
        };

        TcoUtil.openWindowByHistory('#/tco#tco01008_cardapios_complementares', params);
    };

    this.abreUtilizaSubstituiPratosCardapioBase = function(args) {
        var widgetMain = args.owner.widget.container.getWidget('widgetMain');

        var params = {
            elaboracaoCardapioBaseParams: widgetMain.dataSourceFilter[0].value
        };

        TcoUtil.openWindowByHistory('#/tco#tco01007_utiliza_substitui_pratos_cardapio_base', params);
    };

    this.abreAnaliseCustos = function(args) {
        var widgetMain = args.owner.widget.container.getWidget('widgetMain');

        var params = {
            elaboracaoCardapioBaseParams: widgetMain.elaboracaoCardapioBaseParams,
            needFilter: true
        };

        TcoUtil.openWindowByHistory('#/tco#tco01009_analise_custos', params);
    };

    this.updateCustomElements = function(args) {
        var widget = args.owner.getWidget('widgetMain');
        TcoUtil.updateCustomElements(widget);
    };

    this.addOrUpdate = function(rowOrigem, rowDestino) {
        var row = {
            destino: rowDestino,
            origem: rowOrigem
        };
        Tco01001_elaboracao_cardapio_service.copiaPratos(row).then(function(callback) {
            MessageService.successNotificationByCode('TCO01001_PRATOS_COPIADOS', 3000);
            templateManager.container.getWidget('widgetMain').reload();
        }, function(erro) {
            ScreenService.showMessage(erro.data.error);
        });
    };

    this.delete = function(rowToDelete) {
        var row = {
            NRCARDAPIOBASE: rowToDelete.NRCARDAPIOBASE,
            NMTIPOPRAT: [rowToDelete.NRCONFIGCARDBASE],
            DSSEMANA: [rowToDelete.NRDIACARDBASE]
        };
        Tco01001_elaboracao_cardapio_service.excluirPratoCardapioBase(row).then(function(callback) {
                templateManager.container.getWidget('widgetMain').reload();
            },
            (err) => ScreenService.showMessage(err.error));
    };

    this.abreCopiaPratos = function(args) {
        var widgetMain = args.owner.widget;
        var popupCopiarPratos = widgetMain.container.getWidget('popupCopiarPratos');
        var row = TcoUtil.retornaRowSelecionada();
        if (row && row.NMPRATO) {
            popupCopiarPratos.currentRow = angular.copy(row);
            popupCopiarPratos.fields.forEach(function(field) {
                if (field.fieldGroup === 'DADOS_DIA') {
                    field.setValue(field.defaultValue);
                }
            });
            ScreenService.openPopup(popupCopiarPratos);
        } else {
            MessageService.showMessageByCode('TCO01001_CELULA_NAO_SELECIONADA');
        }
    };

    this.replicaPratos = function(args) {
        var widget = args.owner.widget;
        var widgetMain = widget.container.getWidget('widgetMain');
        if (widget.isValid()) {
            Tco01001_elaboracao_cardapio_service.replicaPratos(widget.currentRow).then(function(callback) {
                MessageService.successNotificationByCode('TCO01001_PRATOS_REPLICADOS', 3000);
                widgetMain.reload();
                ScreenService.closePopup(true);
            }, function(erro) {
                ScreenService.showMessage(erro.data.error);
            });
        }
    };

    this.abreAlterarDiasElaboraCardapioBase = function(args) {
        var widgetMain = args.owner.widget;
        var popupAlteraDiasElaboracaoCardapio = widgetMain.container.getWidget('popupAlteraDiasElaboracaoCardapio');
        popupAlteraDiasElaboracaoCardapio.currentRow.IDACAO = "A";
        popupAlteraDiasElaboracaoCardapio.currentRow.DIAS = "";
        ScreenService.openPopup(popupAlteraDiasElaboracaoCardapio);
    };

    this.justInteger = function() {
        General.justInteger("#DIAS");
    };

    this.alteraDiasElaboracaoCardapio = function(args) {
        var popupAlteraDiasElaboracaoCardapio = args.owner.widget;
        var widgetMain = args.owner.widget.container.getWidget("widgetMain");
        popupAlteraDiasElaboracaoCardapio.currentRow.NRCARDAPIOBASE = widgetMain.elaboracaoCardapioBaseParams.NRCARDAPIOBASE;

        if (popupAlteraDiasElaboracaoCardapio.currentRow.IDACAO === "R") {
            ScreenService.confirmMessage(ScreenService.i18n('Removendo os dias os pratos e regras serão excluídos. Remover o(s) Dia(s)?'), 'question', (success) => {
                Tco01001_elaboracao_cardapio_service.alteraDiasElaboracaoCardapio(popupAlteraDiasElaboracaoCardapio.currentRow).then(
                    (success) => {
                        if (typeof success.dataset.validacaoDiasCardapio !== "undefined" && success.dataset.validacaoDiasCardapio !== null) {
                            MessageService.showMessageByCode(success.dataset.validacaoDiasCardapio.MESSAGECODE);
                        } else {
                            ScreenService.successNotification(ScreenService.i18n('Registros excluídos com sucesso'));
                            widgetMain.reload().then(() => {
                                ScreenService.closePopup(true);
                            });
                            widgetMain.elaboracaoCardapioBaseParams.TOTALDIASCARD = success.dataset.totalDiasPosAlteracao.TOTALDIASCARD;
                        }
                    },
                    (err) => ScreenService.showMessage(err.error));
            });
        } else {
            Tco01001_elaboracao_cardapio_service.alteraDiasElaboracaoCardapio(popupAlteraDiasElaboracaoCardapio.currentRow).then(
                (success) => {
                    ScreenService.successNotification(ScreenService.i18n('Registros excluídos com sucesso'));
                    widgetMain.reload().then(() => {
                        ScreenService.closePopup(true);
                    });
                    widgetMain.elaboracaoCardapioBaseParams.TOTALDIASCARD = success.dataset.totalDiasPosAlteracao.TOTALDIASCARD;
                },
                (err) => ScreenService.showMessage(err.error));
        }
    };

    this.defineFiltroConfiguracao = function(args) {
        var currentRow = args.owner.field.widget.currentRow;
        var fieldTodasConfiguracoes = args.owner.field.widget.getField('TODAS_CONFIGURACOES');
        var fieldNmtipoprat = args.owner.field.widget.getField('NMTIPOPRAT');

        var filter = [
            {
                "name": "NRCARDAPIOBASE",
                "operator": "=",
                "value": currentRow.NRCARDAPIOBASE
            },
            {
                "operator": "ORDER_BY",
                "name": [
                    {
                        "name": "NRORDEM",
                        "direction": "ASC"
                    }
                ]
            }
        ];

        fieldNmtipoprat.value('');
        currentRow.NMTIPOPRAT = [];
        fieldNmtipoprat.validations = [];
        fieldNmtipoprat.readOnly = true;

        currentRow.TODAS_CONFIGURACOES = "T";
        fieldTodasConfiguracoes.value('T');

        fieldNmtipoprat.dataSourceFilter = filter;
        fieldNmtipoprat.reload();
    };

    this.onChangeAllConfiguracoes = function(args) {
        General.toggleReadOnlySelect(args.owner.field, args.owner.field.widget.getField('NMTIPOPRAT'));
    };

    this.abreRelatorios = function(args) {
        var widgetMain = args.owner.widget;
        var popupRelatorio = args.owner.widget.container.getWidget('popupRelatorio');

        popupRelatorio.newRow();
        popupRelatorio.currentRow.NRCARDAPIOBASE = widgetMain.elaboracaoCardapioBaseParams.NRCARDAPIOBASE;
        popupRelatorio.currentRow.DTREFERENCIA = moment().format('DD/MM/YYYY');

        ScreenService.openPopup(popupRelatorio);
    };

    this.confirmaRelatorio = function(args) {
        var popupRelatorio = args.owner.widget;

        if (popupRelatorio.isValid()) {
            if (validaPeriodoPrat(popupRelatorio) && validaPeriodoProd(popupRelatorio)) {
                popupRelatorio.currentRow.CDARVPRODINI = popupRelatorio.getField("NMPRODUTOINI").value() == "" ? "0" : popupRelatorio.currentRow.CDARVPRODINI;
                popupRelatorio.currentRow.CDARVPRODFIN = popupRelatorio.getField("NMPRODUTOFIN").value() == "" ? "zzzzzzzzzz" : popupRelatorio.currentRow.CDARVPRODFIN;
                popupRelatorio.currentRow.CDPRATOINI = popupRelatorio.getField("NMPRATOINI").value() == "" ? "0" : popupRelatorio.currentRow.CDPRATOINI;
                popupRelatorio.currentRow.CDPRATOFIN = popupRelatorio.getField("NMPRATOFIM").value() == "" ? "zzzzzzzzzz" : popupRelatorio.currentRow.CDPRATOFIM;
                popupRelatorio.reportRow = popupRelatorio.currentRow;
                TcoUtil.openReportPopup(popupRelatorio, generateReport, [popupRelatorio]);
            }
        }
    };

    var validaPeriodoProd = function(widget) {
        var valido = false;

        if (widget.getField("NMPRODUTOINI").value() != "" && widget.getField("NMPRODUTOFIN").value() == "") {
            MessageService.showMessageByCode("TCO01600_PRODUTO_FINAL_NAO_PREENCHIDO");
            valido = false;
        } else if (widget.getField("NMPRODUTOFIN").value() != "" && widget.getField("NMPRODUTOINI").value() == "") {
            MessageService.showMessageByCode("TCO01600_PRODUTO_INICIAL_NAO_PREENCHIDO");
            valido = false;
        } else {
            if (widget.currentRow.CDPRODUTOINI > widget.currentRow.CDPRODUTOFIN) {
                MessageService.showMessageByCode("TCO01600_PERIODO_PROD_INVALIDO");
                valido = false;
            } else {
                valido = true;
            }
        }

        return valido;
    };

    var validaPeriodoPrat = function(widget) {
        var valido = false;

        if (widget.getField("NMPRATOINI").value() != "" && widget.getField("NMPRATOFIM").value() == "") {
            MessageService.showMessageByCode("TCO01009_PRATO_FINAL_NAO_PREENCHIDO");
            valido = false;
        } else if (widget.getField("NMPRATOFIM").value() != "" && widget.getField("NMPRATOINI").value() == "") {
            MessageService.showMessageByCode("TCO01009_PRATO_INICIAL_NAO_PREENCHIDO");
            valido = false;
        } else {
            if (widget.currentRow.CDPRATOINI > widget.currentRow.CDPRATOFIN) {
                MessageService.showMessageByCode("TCO01009_PERIODO_PRAT_INVALIDO");
                valido = false;
            } else {
                valido = true;
            }
        }

        return valido;
    };

    var generateReport = function(widget) {
        var row = widget.currentRow;
        widget.reportRow.REPORT_NAME = row.REPORT_NAME;
        widget.reportRow.IDIOMA = getLocalVar('language');

        Tco01001_elaboracao_cardapio_service.gerarRelatorioCardapioBase(widget.reportRow).then(function(response) {
            if (response && response.messages) {
                var report = response.messages.pop();
                var reportOpened = window.open(report.message);
                if (!reportOpened) {
                    ScreenService.showMessage(ScreenService.i18n('Bloqueador de Pop-up está ativado!'));
                }
            }
        }, function(response) {
            if (response.data && response.data.dataset && response.data.dataset.ERROS) {
                TcoUtil.buildMessagesToShow(response.data.dataset.ERROS).then(function(mensagens) {
                    General.showErrorMessages(widget, mensagens, ScreenService.i18n("Erro"));
                });
            }
        });
    };

    this.isWeekend = function(row) {
        const parcialRegex = GLOBAL_MSG_WEEKEND.map(item => `(\\b${item}\\b)`).join('|');
        const regexWeekend = new RegExp(parcialRegex, 'i');
        return row.FIMSEMANA == 'S' || (row.__isYAxisProperty) && (row.label.match(regexWeekend));
    };
};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01001_elaboracao_cardapio_controller', Tco01001_elaboracao_cardapio_controller);
});

// FILE: ./assets/js/controllers/Tco01002_configuracao_cardapio_controller.js
/* global angular, Configuration*/
var Tco01002_configuracao_cardapio_controller = function(ScreenService, General, TcoUtil, Tco01002_configuracao_cardapio_service, MessageService) {

    this.carregaParametros = function(args) {
        if (!window.history.state) {
            window.location.href = window.location.href.replace("#/tco#tco01002_configuracao_cardapio", "#/tco#tco01000_cardapio_base");
            window.location.reload();
        } else {
            var widgetMain = args.owner.getWidget("widgetMain");
            var filter = window.history.state.configCardapioBaseParams;

            widgetMain.dataSourceFilter = filter;
            widgetMain.reload();
            widgetMain.dataSource.clearCheckedRows();
        }
    };

    this.backDetail = function() {
        ScreenService.closeSwipe();
    };

    this.backCardBase = function() {
        window.history.back();
    };

    this.cancelaEdit = function(args) {
        var widgetMain = args.owner.widget;
        changeReadOnly(widgetMain, true);
        General.backOrCancel(args);
    };

    this.setStripeData = function(args) {
        var stripeWidget = args.owner;
        stripeWidget.currentRow = window.history.state.stripeData || {};
    };

    this.novaConfiguracao = function(args) {
        var widgetMain = args.owner.widget;
        widgetMain.getField("IDTEMPERATURA").validations = {};
        widgetMain.newRow();
        changeReadOnly(widgetMain, false);
    };

    this.salvaConfiguracaoCardapioBase = function(args) {
        var widgetMain = args.owner.widget;
        if (widgetMain.isValid()) {

            TcoUtil.setNewCodeParametersInRow(widgetMain);
            widgetMain.currentRow.NRCARDAPIOBASE = widgetMain.dataSourceFilter[0].value;

            Tco01002_configuracao_cardapio_service.salvaConfiguracaoCardapioBase(widgetMain.currentRow).then(function(success) {
                if (typeof success.messages !== 'undefined' && success.messages.length > 0) {
                    MessageService.showMessageByCode(success.messages[0].message);
                } else if (typeof success.dataset.cardapioBase !== "undefined" && success.dataset.cardapioBase.length > 0) {
                    MessageService.showMessageByCode(success.dataset.cardapioBase[0].MESSAGECODE, success.dataset.cardapioBase[0]);
                } else {
                    if (typeof success.dataset.columnsToReturn !== "undefined" && success.dataset.columnsToReturn.length > 0) {
                        widgetMain.currentRow = success.dataset.columnsToReturn[0];
                    }
                    ScreenService.notificationMessage(ScreenService.i18n("Configuração do Cardápio salva com sucesso."), 'success', 3000);
                    widgetMain.reload();
                    ScreenService.closeSwipe();
                    changeReadOnly(widgetMain, false);
                }
            });
        }
    };

    this.preventChecked = function(args) {
        var widgetMain = args.owner;

        if (widgetMain.dataSource.getCheckedRows().length > 0) {
            widgetMain.actions.forEach((action) => {
                if (action.name == "CONFIG_ABOVE" || action.name == "CONFIG_BELLOW") {
                    action.isVisible = false;
                }
            });
        } else {
            widgetMain.actions.forEach((action) => {
                if (action.name == "CONFIG_ABOVE" || action.name == "CONFIG_BELLOW") {
                    action.isVisible = true;
                }
            });
        }
    };

    this.preventUnChecked = function(args) {
        var widgetMain = args.owner;

        if (widgetMain.dataSource.getCheckedRows().length == 2) {
            widgetMain.actions.forEach((action) => {
                if (action.name == "CONFIG_ABOVE" || action.name == "CONFIG_BELLOW") {
                    action.isVisible = true;
                }
            });
        } else {
            widgetMain.actions.forEach((action) => {
                if (action.name == "CONFIG_ABOVE" || action.name == "CONFIG_BELLOW") {
                    action.isVisible = false;
                }
            });
        }
    };

    this.preventCheckedAll = function(args) {
        args.owner.dataSource.checkedRowsCache = {};
        args.e.preventDefault();
    };

    this.changeOrdUp = function(args) {
        var widgetMain = args.owner.widget;
        var selectedRow = args.owner.widget.dataSource.getCheckedRows();
        updateOrdConfig(widgetMain, selectedRow, "up");
    };

    this.changeOrdDown = function(args) {
        var widgetMain = args.owner.widget;
        var selectedRow = args.owner.widget.dataSource.getCheckedRows();
        updateOrdConfig(widgetMain, selectedRow, "down");
    };

    var updateOrdConfig = function(widgetMain, selectedRow, direction) {

        var rowToUpdate = selectedRow[0];
        if (direction === "up") {
            rowToUpdate.ORDUP = true;
            rowToUpdate.ORDDW = false;
            rowToUpdate.changed = true;
            rowToUpdate.__is_new = false;
        } else {
            rowToUpdate.ORDUP = false;
            rowToUpdate.ORDDW = true;
            rowToUpdate.changed = true;
            rowToUpdate.__is_new = false;
        }

        Tco01002_configuracao_cardapio_service.salvaConfiguracaoCardapioBase(rowToUpdate).then(function(success) {
            ScreenService.notificationMessage(ScreenService.i18n("Configuração do Cardápio salva com sucesso."), 'success', 3000);
            widgetMain.reload();
        });

    };

    this.removeConfiguracaoCardapioBase = function(args) {
        var widgetMain = args.owner.widget;
        // var currentRow = args.owner.widget.currentRow;
        var configsSelecionadas = widgetMain.dataSource.getCheckedRows();

        ScreenService.confirmMessage(ScreenService.i18n('Remover Configuração(ões) do Cardápio Base?'), 'question', function(success) {
            Tco01002_configuracao_cardapio_service.removeConfigCardapioBase(configsSelecionadas).then(function(success) {
                if (success.dataset.configCardapioBaseEmUso.length > 0) {
                    buildMessages(success.dataset.configCardapioBaseEmUso).then(function(configCardapioBaseEmUso) {
                        General.showErrorMessages(widgetMain, configCardapioBaseEmUso, ScreenService.i18n("Configuração do Cardápio Base em uso."));
                    });
                } else {
                    ScreenService.successNotification(ScreenService.i18n("Configuração do Cardapio Base excluída com sucesso."), null, "checked");
                    if (ScreenService.isSwipeOpen()) {
                        ScreenService.closeSwipe().then(function() {
                            widgetMain.reload();
                        });
                    } else {
                        widgetMain.unSelectAll();
                        widgetMain.reload();
                    }
                }
            });
        });
    };

    var buildMessages = function(data) {
        var mensagens = [];
        var msgParams = [];
        data.forEach(function(row) {
            //mensagens.push(ScreenService.i18n(row.MESSAGECODE, row));
            msgParams.push(row);
            mensagens.push(row.MESSAGECODE);
        });

        return MessageService.getMessagesByCode(mensagens).then(function(mensagensTraduzidas) {
            for (let i = 0; i < mensagensTraduzidas.length; i++) {
                mensagensTraduzidas[i] = ScreenService.i18n(mensagensTraduzidas[i], msgParams[i]);
            }
            return mensagensTraduzidas;
        });
    };

    /**
     * Assegura que quando a configuração for do tipo alimentar
     * deve ter obrigatoriamente uma temperatura, e quando não for 
     * alimentar não deve ter temperatura.
     * 
     * @params {Object} args - Objeto args
     **/

    this.validationsTemp = function(args) {
        var classificacao = args.owner.field;
        var temperatura = args.owner.field.widget.getField("IDTEMPERATURA");
        if (classificacao.value() === "ALIMENTAR") {
            temperatura.readOnly = false;
        } else {
            temperatura.validations = {};
            temperatura.readOnly = true;
            temperatura.value("");
        }
    };

    this.editaConfiguracaoCardapioBase = function(args) {
        var widgetMain = args.owner.widget;
        General.widgetEdit(widgetMain);
        widgetMain.currentRow.ORDUP = false;
        widgetMain.currentRow.ORDDW = false;
        changeReadOnly(widgetMain, true);
    };

    function changeReadOnly(widgetMain, mode) {
        widgetMain.getField("NMTIPOPRAT").readOnly = mode;
        widgetMain.getField("IDCLASSIFICACAO").readOnly = mode;
        widgetMain.getField("IDTEMPERATURA").readOnly = mode;
    }
};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01002_configuracao_cardapio_controller', Tco01002_configuracao_cardapio_controller);
});

// FILE: ./assets/js/controllers/Tco01003_dados_do_prato_controller.js
/* global angular, Configuration*/
var Tco01003_dados_do_prato_controller = function(ScreenService, General, TcoUtil, Tco01003_dados_do_prato_service, MessageService) {

    this.carregaParametros = function(args) {
        if (!window.history.state) {
            window.location.href = window.location.href.replace("#/tco#tco01003_dados_do_prato", "#/tco#tco01000_cardapio_base");
            window.location.reload();
        } else {
            var widgetMain = args.owner.getWidget("widgetMain");

            widgetMain.celulaSelecionada = window.history.state.celulaElaboracaoCardapioBaseSelecionada;

            carregaDadosPrato(args.owner);
        }
    };

    this.backCardBase = function() {
        window.history.back();
    };

    this.defineLabelReferencia = function(args) {
        var widgetMain = args.owner;
        var mainDashboard = args.owner.container.getWidget("mainDashboard");

        mainDashboard.label = widgetMain.celulaSelecionada.LABELDIA + " - " + widgetMain.celulaSelecionada.LABELTIPOPRAT;
    };

    var carregaDadosPrato = function(container) {
        var widgetMain = container.getWidget("widgetMain");
        var widgetPratos = container.getWidget("widgetPratos");
        var widgetRegioes = container.getWidget("widgetRegioes");
        var paramFilter = ["NRCARDAPIOBASE", "NRDIACARDBASE", "NRCONFIGCARDBASE"];
        var filter = [
            {
                "operator": "ORDER_BY",
                "name": [
                    {
                        "name": "CDPRATO",
                        "direction": "ASC"
                    }
                ]
            }
        ];

        paramFilter.forEach(function(param) {
            if (widgetMain.celulaSelecionada.hasOwnProperty(param)) {
                filter.push({
                    "name": param,
                    "operator": "=",
                    "value": widgetMain.celulaSelecionada[param]
                });
            }
        });

        widgetPratos.dataSourceFilter = filter;
        widgetPratos.reload().then(function() {
            TcoUtil.maskPrato("widgetPratos");
            if (widgetPratos.dataSource.data.length) {
                var posicao = widgetPratos.dataSource.data.findIndex((row) => row.IDPRATOEXIBICAO === "S");
                widgetPratos.setCurrentRow(widgetPratos.dataSource.data[posicao < 0 ? 0 : posicao]);
                setTimeout(function() {
                    $(".zh-new-grid-" + widgetPratos.name).find(".grid-vertical-scroll-control").scrollTop(widgetPratos.currentRow.__index * 36);
                }, 200);
            } else {
                widgetRegioes.dataSource.data = [];
            }
        });
    };

    this.carregaRegioesPrato = function(args) {
        var widgetPratos = args.owner;
        var widgetRegioes = args.owner.container.getWidget("widgetRegioes");

        var filter = [
            {
                "name": "NRPRATCOMPCARDBASE",
                "operator": "=",
                "value": widgetPratos.currentRow.NRPRATCOMPCARDBASE
            },
            {
                "operator": "ORDER_BY",
                "name": [
                    {
                        "name": "DSREGIAO",
                        "direction": "ASC"
                    }
                ]
            }
        ];

        widgetRegioes.dataSourceFilter = filter;

        widgetRegioes.reload();

    };

    this.novoPrato = function(args) {
        var widgetMain = args.owner.widget.container.getWidget("widgetMain");

        var params = {
            configCardapioBaseParams: widgetMain.dataSourceFilter,
            celulaElaboracaoCardapioBaseSelecionada: widgetMain.celulaSelecionada
        };
        Object.assign(params, window.history.state);

        TcoUtil.openWindowByHistory('#/tco#tco01004_novo_prato_elaboracao_cardapio_base', params);
    };

    this.editRegiaoPrato = function(args) {
        var widgetMain = args.owner.widget.container.getWidget("widgetMain");
        var widgetRegioes = args.owner.widget.container.getWidget("widgetRegioes");

        var params = {
            configCardapioBaseParams: widgetMain.dataSourceFilter,
            celulaElaboracaoCardapioBaseSelecionada: widgetMain.celulaSelecionada,
            filterRegioes: widgetRegioes.dataSourceFilter
        };

        TcoUtil.openWindowByHistory('#/tco#tco01005_regioes_prato_elaboracao_cardapio_base', params);
    };

    this.removePrato = function(args) {
        var widgetPratos = args.owner.widget.container.getWidget("widgetPratos");
        var widgetRegioes = args.owner.widget.container.getWidget("widgetRegioes");

        ScreenService.confirmMessage(ScreenService.i18n('Remover Prato?'), 'question', function(success) {
            Tco01003_dados_do_prato_service.removePratCompCardBase(widgetPratos.currentRow).then(function(success) {
                if (typeof success.messages !== "undefined" && success.messages[0].message !== "TCO01000_ADESAOCARDBASE_INATIVADA") {
                    MessageService.showMessageByCode(success.messages[0].message);
                } else {
                    if (typeof success.messages !== "undefined") {
                        MessageService.showMessageByCode(success.messages[0].message, success.dataset.adesaoCardBaseInativada[0]);
                    }
                    ScreenService.notificationMessage(ScreenService.i18n("Prato excluído com sucesso."), 'success', 3000);
                    widgetPratos.reload().then(function() {
                        if (widgetPratos.dataSource.data.length) {
                            widgetPratos.setCurrentRow(widgetPratos.dataSource.data[0]);
                        } else {
                            widgetRegioes.dataSource.data = [];
                        }
                    });
                }
            });
        });

    };

    this.definirReferencia = function(args) {
        var widgetPratos = args.owner.widget.container.getWidget("widgetPratos");

        Tco01003_dados_do_prato_service.definirReferenciaPrato(widgetPratos.currentRow).then((success) => {
            carregaDadosPrato(args.owner.widget.container);
            ScreenService.successNotification('Operação realizada com sucesso.');
        });
    };

    this.openReview = function(args) {
        var widget = args.owner.widget;
        var widgetPratos = widget.container.getWidget('widgetPratos');
        var celulaSelecionada = window.history.state.celulaElaboracaoCardapioBaseSelecionada;
        var params = {
            "dadosPrato": {
                "CDPRATO": widgetPratos.currentRow.CDPRATO,
                "NMPRATO": widgetPratos.currentRow.NMPRATO,
                "CDFILAUXPRAT": widgetPratos.currentRow.CDFILAUXPRAT,
                "NRSERVICOPROPOSTA": celulaSelecionada.NRSERVICOPROPOSTA,
                "NRPADRAOQUALIDADE": celulaSelecionada.NRPADRAOQUALIDADE,
                "CDREGIAO": null,
                "NMREGIAO": null,
                "NRCARDAPIOTRAB": null,
                "CDSERVFILI": celulaSelecionada.CDSERVFILI ? celulaSelecionada.CDSERVFILI : null
            }
        };

        TcoUtil.openWindowByHistory("#/tco#tco01010_revisar_receita", params, "Revisar Receita");
    };
};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01003_dados_do_prato_controller', Tco01003_dados_do_prato_controller);
});

// FILE: ./assets/js/controllers/Tco01004_novo_prato_elaboracao_cardapio_base_controller.js
/* global getLocalVar, Configuration*/
var Tco01004_novo_prato_elaboracao_cardapio_base_controller = function(ScreenService, General, TcoUtil, Tco01004_novo_prato_elaboracao_cardapio_base_service, templateManager, MessageService) {

    this.carregaParametros = function(args) {
        if (!window.history.state) {
            window.location.href = window.location.href.replace("#/tco#tco01004_novo_prato_elaboracao_cardapio_base", "#/tco#tco01000_cardapio_base");
            window.location.reload();
        } else {
            var widgetMain = args.owner.getWidget("widgetMain");
            var widgetPratos = args.owner.getWidget("widgetPratos");
            var widgetReceitas = args.owner.getWidget('widgetReceitas');
            var widgetRegiaoPrato = args.owner.getWidget('widgetRegiaoPrato');
            var widgetCabecalhoFiltro = args.owner.getWidget('widgetCabecalhoFiltro');

            // $("#NMPRATO_FILTER").focus();

            widgetRegiaoPrato.getField('TODAS_REGIOES').checked = true;
            widgetRegiaoPrato.currentRow.TODAS_REGIOES = 'T';
            widgetCabecalhoFiltro.currentRow = {};

            widgetRegiaoPrato.getField('NMREGIAO_TO_SAVE').dataSourceFilter = [
                {
                    "name": "NRCOMPCARDBASE",
                    "operator": "=",
                    "value": window.history.state.celulaElaboracaoCardapioBaseSelecionada.NRCOMPCARDBASE
                },
                {
                    "operator": "ORDER_BY",
                    "name": [
                        {
                            "name": "CDREGIAO",
                            "direction": "ASC"
                        }
                    ]
                }
            ];

            General.toggleReadOnlySelect(widgetRegiaoPrato.getField('TODAS_REGIOES'), widgetRegiaoPrato.getField('NMREGIAO_TO_SAVE'));

            Tco01004_novo_prato_elaboracao_cardapio_base_service.retornaMascaraNivelPrato().then(function(success) {
                if (typeof success.dataset.mascaraNivelPrato !== "undefined" && success.dataset.mascaraNivelPrato.hasOwnProperty('CDNVPRATO') && success.dataset.mascaraNivelPrato.hasOwnProperty('CDPICTPRAT')) {

                    args.owner.CDNVPRATO = success.dataset.mascaraNivelPrato.CDNVPRATO;
                    args.owner.CDPICTPRAT = success.dataset.mascaraNivelPrato.CDPICTPRAT;

                    widgetPratos.fields.forEach(function(field) {
                        if (field.name === "CDPRATO") {
                            field.mask = {
                                "type": "fix",
                                "params": {
                                    "mask": success.dataset.mascaraNivelPrato.CDPICTPRAT.replace(/9/g, '*')
                                }
                            };
                        }
                    });

                    widgetReceitas.fields.forEach(function(field) {
                        if (field.name === "CDARVPROD") {
                            field.mask = {
                                "type": "fix",
                                "params": {
                                    "mask": getLocalVar("CDPICTPROD")
                                }
                            };
                        }
                    });

                    widgetPratos.reload().then(function() {
                        if (widgetPratos.dataSource.data.length) {
                            widgetPratos.dataSource.bkp_data = widgetPratos.dataSource.data;
                            widgetPratos.setCurrentRow(widgetPratos.dataSource.data[0]);
                            widgetPratos.activate();
                        }
                        setCurrentPrato(widgetPratos, window.history.state.celulaElaboracaoCardapioBaseSelecionada);
                    }.bind(this));

                }
            });

            widgetMain.celulaSelecionada = window.history.state.celulaElaboracaoCardapioBaseSelecionada;
            this.writeLabel(widgetMain.celulaSelecionada);
            setStripe();
        }
    };

    this.backCardBase = function() {
        window.history.back();
    };

    this.reloadBeforeOpenSelect = function(args) {
        args.owner.field.reload();
    };

    this.salvaNovoPratoCompCardapioBase = function(args) {
        var widgetPratos = args.owner.widget;
        var widgetMain = args.owner.widget.container.getWidget('widgetMain');
        var widgetRegiaoPrato = args.owner.widget.container.getWidget('widgetRegiaoPrato');

        if (widgetRegiaoPrato.isValid()) {
            var rowToSave = {};

            rowToSave.NRCARDAPIOBASE = widgetMain.celulaSelecionada.NRCARDAPIOBASE;
            rowToSave.NRDIACARDBASE = widgetMain.celulaSelecionada.NRDIACARDBASE;
            rowToSave.NRCONFIGCARDBASE = widgetMain.celulaSelecionada.NRCONFIGCARDBASE;
            rowToSave.CDPRATO = widgetPratos.currentRow.CDPRATO;
            rowToSave.NMPRATO = widgetPratos.currentRow.NMPRATO;
            rowToSave.CDFILAUXPRAT = widgetPratos.currentRow.CDFILAUXPRAT;
            rowToSave.CDREGIAO = typeof widgetRegiaoPrato.currentRow.NMREGIAO_TO_SAVE !== "undefined" && widgetRegiaoPrato.currentRow.NMREGIAO_TO_SAVE.length > 0 ? widgetRegiaoPrato.currentRow.NMREGIAO_TO_SAVE : widgetRegiaoPrato.currentRow.TODAS_REGIOES;
            rowToSave.__is_new = true;
            rowToSave.changed = true;

            Tco01004_novo_prato_elaboracao_cardapio_base_service.salvaNovoPratoComp(rowToSave).then(function(success) {
                if (typeof success.messages !== "undefined" && success.messages[0].message !== "TCO01000_ADESAOCARDBASE_INATIVADA") {
                    MessageService.showMessageByCode(success.messages[0].message);
                } else {
                    if (typeof success.messages !== "undefined") {
                        MessageService.showMessageByCode(success.messages[0].message, success.dataset.adesaoCardBaseInativada[0]);
                    }
                    MessageService.successNotificationByCode(success.notifications[0].message, 3000, success.notifications[0].variables);
                    window.history.back();

                }
            });
        }
    };

    this.carregaReceitasPratoPorRegiao = function(args) {
        var widgetPratos = args.owner.field.widget.container.getWidget('widgetPratos');
        var widgetReceitas = args.owner.field.widget.container.getWidget('widgetReceitas');
        var widgetMain = args.owner.field.widget.container.getWidget('widgetMain');
        var widgetCabecalhoFiltro = args.owner.field.widget;

        carregaReceitas(widgetPratos, widgetMain, widgetCabecalhoFiltro, widgetReceitas);
    };

    this.carregaReceitasPratoPorPrato = function(args) {
        var widgetPratos = args.owner;
        var widgetReceitas = args.owner.container.getWidget('widgetReceitas');
        var widgetMain = args.owner.container.getWidget('widgetMain');
        var widgetCabecalhoFiltro = args.owner.container.getWidget('widgetCabecalhoFiltro');

        carregaReceitas(widgetPratos, widgetMain, widgetCabecalhoFiltro, widgetReceitas);
    };

    var carregaReceitas = function(widgetPratos, widgetMain, widgetCabecalhoFiltro, widgetReceitas) {
        var row = {
            CDFILAUXPRAT: widgetPratos.currentRow.CDFILAUXPRAT,
            CDPRATO: widgetPratos.currentRow.CDPRATO,
            NRSERVICOPROPOSTA: widgetMain.celulaSelecionada.NRSERVICOPROPOSTA,
            NRPADRAOQUALIDADE: widgetMain.celulaSelecionada.NRPADRAOQUALIDADE,
            CDREGIAO: widgetCabecalhoFiltro.currentRow.CDREGIAO ? widgetCabecalhoFiltro.currentRow.CDREGIAO : null,
            NRCARDAPIOTRAB: null,
            CDSERVFILI: widgetMain.celulaSelecionada.CDSERVFILI ? widgetMain.celulaSelecionada.CDSERVFILI : null
        };

        Tco01004_novo_prato_elaboracao_cardapio_base_service.retornaReceitas(row).then(function(success) {
            widgetReceitas.dataSource.data = success.dataset.receitaPrato;
        });
    };

    this.filterPrato = function(args) {
        let field = args.owner.field;
        clearTimeOut(field);
        field.timeout = setTimeout(function() {
            let widgetPrato = templateManager.container.getWidget('widgetPratos');
            let filter = args.e.currentTarget.value;
            if (filter !== "") {
                if (filter[filter.length - 1] !== '%')
                    filter = filter + '%';
                let newData = widgetPrato.dataSource.bkp_data.filter(row => {
                    return (row.CDPRATO.toUpperCase().match(TcoUtil.buildRegExp(filter.toUpperCase())) ||
                        row.NMPRATO.toUpperCase().match(TcoUtil.buildRegExp(filter.toUpperCase())));
                });
                widgetPrato.dataSource.data = newData;

            } else {
                widgetPrato.dataSource.data = widgetPrato.dataSource.bkp_data;
                widgetPrato.setCurrentRow(widgetPrato.dataSource.data[0]);
            }
            //widgetPrato.redraw();
            templateManager.updateTemplate();
        }, 300);
    };

    let clearTimeOut = function(field) {
        if (field.timeout !== false)
            clearTimeout(field.timeout);
    };

    this.onChangeAllRegions = function(args) {
        General.toggleReadOnlySelect(args.owner.field, args.owner.field.widget.getField('NMREGIAO_TO_SAVE'));
    };

    this.writeLabel = function(label) {
        var widgetMain = templateManager.container.getWidget('widgetMain');
        widgetMain.label = label.LABELDIA + ' - ' + label.LABELTIPOPRAT;
        templateManager.updateTemplate();
    };

    function setStripe() {
        var stripeWidget = templateManager.container.getWidget('stripeWidget');
        stripeWidget.currentRow = window.history.state.parametros || {};
    }

    /**
     * Função responsável por abrir a tela de revisão da receita.
     * 
     * @params {Args} args - Objeto args.
     **/

    this.openReview = function(args) {
        var widgetPratos = args.owner.widget;
        var widgetMain = widgetPratos.container.getWidget('widgetMain');
        var widgetCabecalhoFiltro = widgetPratos.container.getWidget('widgetCabecalhoFiltro');
        var params = {
            "dadosPrato": {
                "CDPRATO": widgetPratos.currentRow.CDPRATO,
                "NMPRATO": widgetPratos.currentRow.NMPRATO,
                "CDFILAUXPRAT": widgetPratos.currentRow.CDFILAUXPRAT,
                "NRSERVICOPROPOSTA": widgetMain.celulaSelecionada.NRSERVICOPROPOSTA,
                "NRPADRAOQUALIDADE": widgetMain.celulaSelecionada.NRPADRAOQUALIDADE,
                "CDREGIAO": widgetCabecalhoFiltro.currentRow.CDREGIAO ? widgetCabecalhoFiltro.currentRow.CDREGIAO : null,
                "NMREGIAO": widgetCabecalhoFiltro.currentRow.NMREGIAO ? widgetCabecalhoFiltro.currentRow.NMREGIAO : null,
                "NRCARDAPIOTRAB": null,
                "CDSERVFILI": widgetMain.celulaSelecionada.CDSERVFILI ? widgetMain.celulaSelecionada.CDSERVFILI : null
            }
        };

        window.history.state.celulaElaboracaoCardapioBaseSelecionada.CDPRATO = widgetPratos.currentRow.CDPRATO;
        window.history.replaceState(window.history.state, "Dados do Prato");

        TcoUtil.openWindowByHistory("#/tco#tco01010_revisar_receita", params, "Revisar Receita");
    };

    function setCurrentPrato(widgetPratos, dadosPrato) {
        if (dadosPrato) {
            if (dadosPrato.CDPRATO) {
                var prato = widgetPratos.dataSource.data.filter(function(row) {
                    return row.CDPRATO === dadosPrato.CDPRATO;
                })[0];
                widgetPratos.setCurrentRow(prato);
                setTimeout(function() {
                    $(".zh-new-grid-" + widgetPratos.name).find(".grid-vertical-scroll-control").scrollTop(widgetPratos.currentRow.__index * 36);
                }, 150);
            } else {
                widgetPratos.setCurrentRow(widgetPratos.dataSource.data[0]);
            }
        }
    }

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01004_novo_prato_elaboracao_cardapio_base_controller', Tco01004_novo_prato_elaboracao_cardapio_base_controller);
});

// FILE: ./assets/js/controllers/Tco01005_regioes_prato_elaboracao_cardapio_base_controller.js
/* global angular, Configuration*/
var Tco01005_regioes_prato_elaboracao_cardapio_base_controller = function(ScreenService, General, TcoUtil, Tco01005_regioes_prato_elaboracao_cardapio_base_service) {

    this.carregaParametros = function(args) {
        var widgetMain = args.owner.getWidget("widgetMain");
        widgetMain.dataSource.clearCheckedRows();
        if (!window.history.state) {
            window.location.href = window.location.href.replace("#/tco#tco01005_regioes_prato_elaboracao_cardapio_base", "#/tco#tco01000_cardapio_base");
            window.location.reload();
        } else {
            var filterRegioes = window.history.state.filterRegioes;

            filterRegioes[1] = {
                "operator": "ORDER_BY",
                "name": [
                    {
                        "name": "CDREGIAO",
                        "direction": "ASC"
                    }
                ]
            };
            widgetMain.dataSourceFilter = filterRegioes;
            widgetMain.reload();
        }
    };

    this.backCardBase = function() {
        window.history.back();
    };

    this.closePopupRegioes = function() {
        ScreenService.closePopup();
    };

    this.novaRegiaoPrato = function(args) {
        var widgetMain = args.owner.widget;
        var popupRegioes = args.owner.widget.container.getWidget("popupRegioes");

        var regioesDefinidas = [];

        widgetMain.dataSource.data.forEach((row) => {
            regioesDefinidas.push(row.CDREGIAO);
        });

        var filter = [
            {
                "name": "NRCOMPCARDBASE",
                "operator": "=",
                "value": window.history.state.celulaElaboracaoCardapioBaseSelecionada.NRCOMPCARDBASE
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": getLocalVar('NRORG')
            },
            {
                "operator": "ORDER_BY",
                "name": [
                    {
                        "name": "CDREGIAO",
                        "direction": "ASC"
                    }
                ]
            }
        ];
        if (regioesDefinidas.length) {
            filter.push({
                "name": "CDREGIAO",
                "operator": "NOT_IN",
                "value": regioesDefinidas
            });
        }
        popupRegioes.dataSource.clearCheckedRows();
        popupRegioes.dataSourceFilter = filter;
        popupRegioes.reload().then((success) => ScreenService.openPopup(popupRegioes));

    };

    this.salvaRegiaoPrato = function(args) {
        var popupRegioes = args.owner.widget;
        var widgetMain = popupRegioes.container.getWidget("widgetMain");
        var dataset = angular.copy(popupRegioes.getCheckedRows());
        var nrprat = widgetMain.dataSourceFilter[0].value;
        if (dataset.length) {
            setNewCodeParametersInRow(dataset, popupRegioes, nrprat);

            Tco01005_regioes_prato_elaboracao_cardapio_base_service.salvaRegiaoPratoSelecionada(dataset).then(function(success) {
                ScreenService.notificationMessage(ScreenService.i18n("Região(ões) salva(s) com sucesso."), 'success', 3000);
                widgetMain.reload().then((success) => {
                    ScreenService.closePopup();
                });
            });
        }
    };

    function setNewCodeParametersInRow(dataset, widget, nrprat) {
        dataset.forEach(function(row) {
            row.__using_unit = widget.paramsNewCode.__using_unit || false;
            row.__new_code_pk = widget.paramsNewCode.__new_code_pk;
            row.__table_pk = widget.paramsNewCode.__table_pk;
            row.__new_code_length = widget.paramsNewCode.__new_code_length || 5;
            row.__new_code_column_size = widget.paramsNewCode.__new_code_column_size || 10;
            row.__cast_number = widget.paramsNewCode.__cast_number || false;

            if (!row.__columns_to_return && widget.paramsNewCode && widget.paramsNewCode.__columns_to_return)
                row.__columns_to_return = widget.paramsNewCode.__columns_to_return;

            row.__is_new = true;
            row.changed = true;
            row.NRPRATCOMPCARDBASE = nrprat;
        });
    }

    this.removeRegiaoPrato = function(args) {
        var widgetMain = args.owner.widget;
        var regioesSelecionadas = args.owner.widget.dataSource.getCheckedRows();

        ScreenService.confirmMessage(ScreenService.i18n('Remover Região(ões) do Prato?'), 'question', function(success) {
            Tco01005_regioes_prato_elaboracao_cardapio_base_service.removeRegiaoPrato(regioesSelecionadas).then(function(success) {
                ScreenService.showMessage(ScreenService.i18n("Região(ões) excluída(as) com sucesso."), null, "checked");
                widgetMain.unSelectAll();
                widgetMain.reload();
            });
        });
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01005_regioes_prato_elaboracao_cardapio_base_controller', Tco01005_regioes_prato_elaboracao_cardapio_base_controller);
});

// FILE: ./assets/js/controllers/Tco01006_regras_cardapio.js
/* global angular, Configuration*/
var Tco01006_regras_cardapio = function(ScreenService, General, TcoUtil, Tco01006_regras_cardapio_service, templateManager, metaDataFactory, MessageService) {

    var self = this;
    //carrega parametros do historico de navegação ao entrar na tela 
    this.carregaParametros = function(args) {
        if (!window.history.state) {
            window.location.href = window.location.href.replace("#/tco#tco01006_regras_cardapio", "#/tco#tco01000_cardapio_base");
            window.location.reload();
        } else {
            var filter = window.history.state.configCardapioBaseParams;

            args.owner.NRCARDAPIOBASE = filter[0].value;
            args.owner.getWidget('widgetTabbed').widgets.forEach(function(wid) {
                wid.dataSourceFilter = filter;
            });
            args.owner.getWidget('maintabAdesao').activate();
        }
    };

    this.setStripeData = function() {
        var stripeWidget = templateManager.container.getWidget('stripeWidget');
        stripeWidget.currentRow = window.history.state.stripeData || {};
    };

    //volta para tela anterior
    this.backCardBase = function() {
        window.history.back();
    };

    //recarrega a tab ao entrar nela
    this.tabOnEnter = function(args) {
        var widget = args.owner;
        widget.reload();
    };

    //volta para a widget principal da regra de adesao
    this.voltarAdesao = function(args) {
        ScreenService.closeSwipe();
        var widget = args.owner.widget;
        widget.container.getWidget('maintabAdesao').reload();
    };

    //função de retornar os fields da widget que são inline edit
    function getEditableFields(widget) {
        return widget.fields.filter(function(field) {
            return field.inlineEditable;
        });
    }
    //função de cancelar inline edit e fazer rollback
    function rollbackInlineEdit(widget) {
        let editableFields = getEditableFields(widget);

        widget.dataSource.data.forEach(function(row) {
            editableFields.forEach(function(field) {
                if (row.hasOwnProperty('changed_' + field.name)) {
                    row[field.name] = row['changed_' + field.name];
                    if (row[field.name + "_inline"]) {
                        delete row[field.name + "_inline"];
                    }
                    delete row['changed_' + field.name];
                }
            });
            delete row.changed;
        });
    }

    //cancelar inlineEdit
    this.cancelaInlineEdit = function(args) {
        var widget = args.owner.widget ? args.owner.widget : args.owner;
        if (widget.getParent().currentRow.__is_new) {
            ScreenService.closeSwipe();
        } else {
            if (widget.dataSource.backup) {
                widget.dataSource.data = angular.copy(widget.dataSource.backup);
                widget.dataSource.clearCheckedRows();
            }
            rollbackInlineEdit(widget);
            widget.container.getWidget('tabDadosGramatura').enabled = true;
            widget.activate();
            widget.list();
        }
    };

    this.cancelaInlineEditAdesao = function(args) {
        var widget = args.owner.widget ? args.owner.widget : args.owner;
        var tabDadosPrato = widget.container.getWidget('tabDadosPrato');
        if (widget.getParent().currentRow.__is_new) {
            ScreenService.closeSwipe();
        } else {
            if (tabDadosPrato.backupRow) {
                tabDadosPrato.currentRow = angular.copy(tabDadosPrato.backupRow);
            }
            if (widget.dataSource.backupData) {
                widget.dataSource.data = angular.copy(widget.dataSource.backupData);
            }
            widget.activate();
            widget.inlineEditing = false;
            widget.getAction('DELETE').isVisible = true;
            widget.getAction('COPY').isVisible = true;
            widget.list();
        }
    };

    //seta o dataSourceFilter com o NR do cardapio base
    this.setDataSourceFilter = function(args) {
        var widget = args.owner;
        var field = widget.getField('DSSEGMER');
        if (field) {
            field.dataSourceFilter = [{
                "name": "NRCARDAPIOBASE",
                "operator": "=",
                "value": widget.container.NRCARDAPIOBASE
            }];
            field.reload();
        }
    };

    //quando clicar em uma regra de adesão, cria os dados a serem mostrados e faz os filtros necessários antes de abrir a widget
    this.moveRowAdesao = function(args) {
        var maintabAdesao = args.owner;
        var tabDadosPrato = args.owner.container.getWidget('tabDadosPrato');
        var widgetTipoPrato = maintabAdesao.container.getWidget('widgetTipoPrato');
        var field = tabDadosPrato.getField('DSSEMANA');

        tabDadosPrato.currentRow = maintabAdesao.currentRow;
        tabDadosPrato.backupRow = angular.copy(tabDadosPrato.currentRow);
        widgetTipoPrato.list();

        self.carregaFieldsDinamicosAdesao(widgetTipoPrato);

        buildDataSourceToShow(tabDadosPrato.getField("DSSEGMER"), maintabAdesao.currentRow.SEGMENTOS);
        buildDataSourceToShow(tabDadosPrato.getField("DSATIVCLI"), maintabAdesao.currentRow.RAMOS);
        changeReadOnly(maintabAdesao, true);

        field.dataSourceFilter = [{
            "name": "NRCARDAPIOBASE",
            "operator": "=",
            "value": maintabAdesao.container.NRCARDAPIOBASE
        }];
        field.reload();

        widgetTipoPrato.activate();

        widgetTipoPrato.getAction('DELETE').isVisible = true;
        widgetTipoPrato.getAction('COPY').isVisible = true;
        if (!ScreenService.isSwipeOpen()) {
            ScreenService.openSwipe(tabDadosPrato);
        }
    };

    //ao clicar numa regra de gramatura, faz os filtros para inicializar as widgets
    this.moveRowGramatura = function(args) {
        var maintabGramatura = args.owner;
        var tabDadosGramatura = maintabGramatura.container.getWidget('tabDadosGramatura');
        tabDadosGramatura.dataSourceFilter = [
            {
                "name": "NRCARDAPIOBASE",
                "operator": "=",
                "value": tabDadosGramatura.container.NRCARDAPIOBASE
            },
            {
                "name": "CDGRTIPOPR",
                "operator": "=",
                "value": maintabGramatura.currentRow.CDGRTIPOPR
            }
        ];
        tabDadosGramatura.dataSource.unSelectAll();
        tabDadosGramatura.reload();
        tabDadosGramatura.activate();
        if (!ScreenService.isSwipeOpen()) {
            ScreenService.openSwipe(tabDadosGramatura.container.getWidget('tabMaeDados'));
        }
    };

    //constroi os fields dinamicamente colocando os valores e montando o datasource do grid
    this.carregaFieldsDinamicosAdesao = function(widget) {
        var widgetTipoPrato = widget.container.getWidget('widgetTipoPrato');
        var tabDadosPrato = widget.container.getWidget('tabDadosPrato');
        var row = {
            "CDGRTIPOPR": tabDadosPrato.currentRow.CDGRTIPOPR || null,
            "NRADESAOCARDBASE": tabDadosPrato.currentRow.NRADESAOCARDBASE || null,
            "NRCARDAPIOBASE": tabDadosPrato.container.NRCARDAPIOBASE
        };
        widgetTipoPrato.fields = [];
        Tco01006_regras_cardapio_service.retornaCamposDinamicosAdesao(row).then(function(callback) {
            makeFields(widgetTipoPrato, callback.porcionamento, true);

            var dataSource = [];
            callback.tipoPratoAdesao.forEach(function(tipo) {
                var row = {};
                callback.porcionamento.forEach(function(porcionamento) {
                    callback.itemadesaocbase.forEach(function(item) {
                        if (item.NRPORCIONAMENTO === porcionamento.NRPORCIONAMENTO && item.NRCONFIGCARDBASE === tipo.NRCONFIGCARDBASE) {
                            row[porcionamento.NRPORCIONAMENTO] = item.VRPEADESAO;
                        }
                    });
                    if (!row[porcionamento.NRPORCIONAMENTO]) {
                        row[porcionamento.NRPORCIONAMENTO] = 0;
                    }
                });
                row.NMTIPOPRAT = tipo.NMTIPOPRAT;
                row.NRCONFIGCARDBASE = tipo.NRCONFIGCARDBASE;
                row.__isSelected = tipo.IDMARCADO == 'S' ? true : false;
                dataSource.push(row);
            });
            widgetTipoPrato.dataSource.data = dataSource;
            widgetTipoPrato.dataSource.backupData = angular.copy(widgetTipoPrato.dataSource.data);
            widgetTipoPrato.activate();
        }, function(error) {
            ScreenService.showMessage('Erro ao carregar os dados do grid', 'error');
        });
    };

    //cria fields a partir de porcionamentos, uma field para cada porcionamento
    function makeFields(widget, porcionamento, event) {
        widget.fields = [];
        var fieldTemplate = {
            "name": "",
            "label": "",
            "template": "field/text-edit.html",
            "validations": {},
            "showOnForm": true,
            "showOnList": true,
            "descriptionOrder": false,
            "isVisible": true,
            "class": 6,
            "maxlength": 10
        };

        if (widget.name === 'tabItensGram') {
            if (event) {
                fieldTemplate.events = [
                    {
                        "name": "FieldOnChange",
                        "code": "Tco01006_regras_cardapio.checkRowAfterChange(args);"
                    }
                ];
            } else {
                fieldTemplate.name = "NMGRUPOPRATO";
                fieldTemplate.label = ScreenService.i18n("Grupo Prato");
                var aux = metaDataFactory.fieldFactory(fieldTemplate, widget);
                widget.fields.push(aux);
                fieldTemplate.events = [
                    {
                        "name": "FieldOnChange",
                        "code": "Tco01006_regras_cardapio.disableTab(args);"
                    }
                ];
            }
            fieldTemplate.name = "NMTIPOPREPAROTCO";
            fieldTemplate.label = ScreenService.i18n("Tipo de Preparo");
            var fieldNome = metaDataFactory.fieldFactory(fieldTemplate, widget);
            widget.fields.push(fieldNome);

            fieldTemplate.name = "NMPRODUTO";
            fieldTemplate.label = ScreenService.i18n("Produto");
            var fieldProduto = metaDataFactory.fieldFactory(fieldTemplate, widget);
            widget.fields.push(fieldProduto);
        } else {
            if (event) {
                fieldTemplate.events = [
                    {
                        "name": "FieldOnChange",
                        "code": "Tco01006_regras_cardapio.checkRowAfterChange(args);"
                    }
                ];
            } else {
                fieldTemplate.name = "NMTIPO";
                fieldTemplate.label = "Tipo";
                var auxField = metaDataFactory.fieldFactory(fieldTemplate, widget);
                widget.fields.push(auxField);
                fieldTemplate.events = [
                    {
                        "name": "FieldOnChange",
                        "code": "Tco01006_regras_cardapio.disableTab(args);"
                    }
                ];
            }
            fieldTemplate.name = "NMTIPOPRAT";
            fieldTemplate.label = ScreenService.i18n("Nome");
            var fieldTipoPrato = metaDataFactory.fieldFactory(fieldTemplate, widget);
            widget.fields.push(fieldTipoPrato);
        }

        porcionamento.forEach(function(porcionamento) {
            fieldTemplate.inlineEditable = "number";
            fieldTemplate.label = porcionamento.NMPORCIONAMENTO;
            fieldTemplate.name = porcionamento.NRPORCIONAMENTO;
            fieldTemplate.mask = {
                "type": "float",
                "params": {
                    "symbol": "",
                    "thousands": ".",
                    "precision": 3,
                    "allowZero": false,
                    "decimal": ",",
                    "allowNegative": false
                }
            };
            var aux = metaDataFactory.fieldFactory(fieldTemplate, widget);
            widget.fields.push(aux);
        });
        templateManager.updateTemplate();
        widget.reload();
        widget.redraw();
    }

    //cria uma currentRow nova para adicionar uma regra de adesão 
    //(widget.newRow() não é usado por motivos de manipulação do modo da widget para mostrar o grid de tipos de prato)
    this.novaRegraAdesao = function(args) {
        var widget = args.owner.widget;
        var tabDadosPrato = widget.container.getWidget('tabDadosPrato');
        var widgetTipoPrato = widget.container.getWidget('widgetTipoPrato');

        widgetTipoPrato.inlineEdit();
        tabDadosPrato.currentRow = {};

        tabDadosPrato.currentRow.CDADESAOCARDBASE = "";
        tabDadosPrato.currentRow.DSATIVCLI = "";
        tabDadosPrato.currentRow.DSSEGMER = "";
        tabDadosPrato.currentRow.DSSEMANA = "";
        tabDadosPrato.currentRow.IDATIVO = "S";
        tabDadosPrato.currentRow.NMADESAOCARDBASE = "";
        tabDadosPrato.currentRow.NMGRTIPOPR = "";
        tabDadosPrato.currentRow.__createdLocal = true;
        tabDadosPrato.currentRow.__index = 0;
        tabDadosPrato.currentRow.__is_new = true;
        tabDadosPrato.currentRow.NRCARDAPIOBASE = widget.container.NRCARDAPIOBASE;

        var field = tabDadosPrato.getField('DSSEMANA');

        field.dataSourceFilter = [{
            "name": "NRCARDAPIOBASE",
            "operator": "=",
            "value": widget.container.NRCARDAPIOBASE
        }];
        field.reload();
        var fieldDia = tabDadosPrato.getField('DSSEMANA');
        var fieldGrupo = tabDadosPrato.getField('NMGRTIPOPR');
        var dataSourceFilter = [{
            "name": "NRCARDAPIOBASE",
            "operator": "=",
            "value": widget.container.NRCARDAPIOBASE
         }];
        fieldDia.dataSourceFilter = angular.copy(dataSourceFilter);
        fieldDia.reload();
        fieldGrupo.dataSourceFilter = angular.copy(dataSourceFilter);
        fieldGrupo.reload();

        self.carregaFieldsDinamicosAdesao(widgetTipoPrato);
        widgetTipoPrato.activate();
        widgetTipoPrato.getAction('DELETE').isVisible = false;
        widgetTipoPrato.getAction('COPY').isVisible = false;

        changeReadOnly(widget, false);
        tabDadosPrato.getField('DSATIVCLI').readOnly = true;
        ScreenService.openSwipe(tabDadosPrato);
    };

    //faz a verificação de coluna preenchida a partir das linhas selecionadas e logo em seguida salva no banco
    this.salvaRegraAdesao = function(args) {
        var widgetTipoPrato = args.owner.widget;
        var tabDadosPrato = widgetTipoPrato.container.getWidget('tabDadosPrato');
        var porcionamentos = [];
        var dataSet = [];

        if (tabDadosPrato.isValid()) {

            var rowToSave = angular.copy(tabDadosPrato.currentRow);

            widgetTipoPrato.fields.forEach(function(field) {
                if (field.name !== 'NMTIPOPRAT' && field.name !== 'TIPO') {
                    porcionamentos.push({
                        'id': field.name,
                        'label': field.label
                    });
                }
            });
            var colunaPreenchida = [];
            porcionamentos.forEach(function(nrporcio) {
                var aux = false;
                var existe = false;
                widgetTipoPrato.dataSource.getCheckedRows().forEach(function(data) {
                    if (data[nrporcio.id] && !isNaN(General.valueToFloat(data[nrporcio.id]))) {
                        if (General.valueToFloat(data[nrporcio.id]) > 0) {
                            var row = {
                                'NRPORCIONAMENTO': nrporcio.id,
                                'VRPEADESAO': General.valueToFloat(data[nrporcio.id]),
                                'NRCONFIGCARDBASE': data.NRCONFIGCARDBASE,
                                '__is_new': true
                            };
                            dataSet.push(row);
                            aux = true;
                        }
                    } else {
                        if (!existe) {
                            colunaPreenchida.push(nrporcio.label);
                            existe = true;
                        }
                    }
                });
                if (!aux) {
                    colunaPreenchida.pop();
                }
            });

            if (!colunaPreenchida.length) {
                rowToSave.TIPOSPRATO = dataSet;

                rowToSave.NRCARDAPIOBASE = tabDadosPrato.container.NRCARDAPIOBASE;

                Tco01006_regras_cardapio_service.salvaRegraAdesao(rowToSave).then(function(callback) {
                    ScreenService.successNotification("Registro salvo com sucesso");
                    if (widgetTipoPrato.dataSource.data && widgetTipoPrato.dataSource.data.length) {
                        ScreenService.alertNotification("Foram salvos apenas os registros relacionados às linhas selecionadas");
                    }
                    changeReadOnly(tabDadosPrato, true);
                    tabDadosPrato.currentRow.__is_new = false;
                    tabDadosPrato.currentRow.NRADESAOCARDBASE = callback.NRADESAOCARDBASE;
                    tabDadosPrato.currentRow.CDADESAOCARDBASE = callback.CDADESAOCARDBASE;
                    tabDadosPrato.container.getWidget('maintabAdesao').reload();
                    widgetTipoPrato.list();
                    widgetTipoPrato.inlineEditing = false;
                    self.carregaFieldsDinamicosAdesao(widgetTipoPrato);
                    widgetTipoPrato.getAction('DELETE').isVisible = true;
                    widgetTipoPrato.getAction('COPY').isVisible = true;
                }, function(error) {
                    ScreenService.showMessage(error.data.error, 'error');
                });
            } else {
                var msg = colunaPreenchida.join(', ');

                MessageService.showMessageByCode('TCO01006_NECESSARIO_COLUNAS', {
                    COLUNAS: msg
                });
            }

        }
    };

    //verifica de onde virão os dados (do grid principal ou de dentro do swipe) e apaga a regra de adesão
    this.apagaRegraAdesao = function(args, isSwipe) {
        var widget = args.owner.widget;
        var dataset;
        if (isSwipe) {
            widget = widget.getParent();
            dataset = [widget.currentRow];
        } else {
            dataset = widget.getCheckedRows();
        }

        ScreenService.confirmMessage(ScreenService.i18n('Remover Regra(s) de adesão?'), 'question', function(success) {
            Tco01006_regras_cardapio_service.apagaRegraAdesao(dataset).then(function() {
                ScreenService.closeSwipe();
                ScreenService.successNotification("Registro removido com sucesso");
                widget.container.getWidget('maintabAdesao').dataSource.clearCheckedRows();
                widget.container.getWidget('maintabAdesao').reload();
            }, function(error) {
                ScreenService.showMessage(error.data.error, 'error');
            });
        });
    };

    //concatena semana e dias para mostrar na field de dia da semana
    this.concatenaSemanaEDias = function(args) {
        var widget = args.owner;
        var tabDadosPrato = widget.container.getWidget('tabDadosPrato');

        MessageService.getMessageByCode('TCO01001_DIA').then(function(mensagemDia) {
            widget.dataSource.data.forEach(function(row) {
                if (row.NRSEMANA) {
                    MessageService.getMessageByCode(row.DSSEMANA.slice(1, row.DSSEMANA.length)).then(function(labelDia) {
                        row.DSSEMANA = ScreenService.i18n(mensagemDia, {
                            NRSEMANA: row.NRSEMANA,
                            LABELDIA: ScreenService.i18n(labelDia)
                        });
                        if (row.NRDIACARDBASE === tabDadosPrato.currentRow.NRDIACARDBASE) {
                            tabDadosPrato.currentRow.DSSEMANA = angular.copy(row.DSSEMANA);
                        }
                    });
                }
            });
        });
    };

    //seleciona a checkbox referente a row após alguma linha dela ser alterada no inline edit
    this.checkRowAfterChange = function(args) {
        var widget = args.owner.field.widget;
        widget.getAction('DELETE').isVisible = false;
        widget.getAction('COPY').isVisible = false;
        widget.dataSource.addCheckedRows(args.row);
    };

    //retira seleção das rows do grid
    this.clearChecked = function(args) {
        var widget = args.owner;
        widget.dataSource.clearCheckedRows();
        widget.reload();
    };

    //quando trocar segmento, modifica o datasource e valores da field de ramo de atividade
    this.onChangeSegmento = function(args) {
        var fieldSegm = args.owner.field;
        var widget = fieldSegm.widget;
        var fieldRamo = widget.getField('DSATIVCLI');
        if (fieldSegm.value() && fieldSegm.value().length) {
            fieldRamo.readOnly = false;
            fieldRamo.dataSourceFilter = [
                {
                    "name": "NRCARDAPIOBASE",
                    "operator": "=",
                    "value": widget.container.NRCARDAPIOBASE
                },
                {
                    "name": "CDSEGMER",
                    "operator": "IN",
                    "value": widget.currentRow.CDSEGMER
                }
            ];
            fieldRamo.reload();
        } else {
            widget.currentRow.NRATIVCLISEGMER = [];
            fieldRamo.readOnly = true;
        }
        fieldRamo.clearValue();

    };

    //constroi um datasource ficticio para mostrar quando o campo for somente leitura
    function buildDataSourceToShow(field, stringData) {
        if (stringData) {
            var dados = stringData.split(', ');
            var dataSource = [];
            var indices = [];

            dados.forEach(function(data, index) {
                let obj = {};
                obj[field.valueField] = index;
                obj[field.descriptionField] = data;
                dataSource.push(obj);
                indices.push(index);
            });
            field.dataSource.data = dataSource;
            field.value(indices);
        }
    }

    //muda o readonly dos campos da widget de adesão do cardapio base
    function changeReadOnly(widget, trueOrFalse) {
        widget = widget.container.getWidget('tabDadosPrato');
        widget.getField("DSSEGMER").readOnly = trueOrFalse;
        widget.getField("NMGRTIPOPR").readOnly = trueOrFalse;
        widget.getField("DSATIVCLI").readOnly = trueOrFalse;
        widget.getField("DSSEMANA").readOnly = trueOrFalse;
    }

    //abre o swipe para adicionar uma nova regra de gramatura do cardapio base
    this.novaRegraGramatura = function(args) {
        var tabDadosGramatura = args.owner.widget;
        var maintabGramatura = tabDadosGramatura.container.getWidget('maintabGramatura');

        tabDadosGramatura.newRow();
        tabDadosGramatura.currentRow.NRCARDAPIOBASE = tabDadosGramatura.container.NRCARDAPIOBASE;
        tabDadosGramatura.currentRow.CDGRTIPOPR = maintabGramatura.currentRow.CDGRTIPOPR;

        tabDadosGramatura.container.getWidget('tabItensGram').enabled = false;
    };

    //coloca a widget em modo edição
    this.modeEdit = function(args) {
        var widget = args.owner.widget;
        General.widgetEdit(widget);
        var row = angular.copy(widget.currentRow);
        widget.bkp = {
            'NRTIPOPUBLICO': row.NRTIPOPUBLICO,
            'VRGRAMATURA': row.VRGRAMATURA.toString(),
            'SGUNIDADE': row.SGUNIDADE
        };
        widget.container.getWidget('tabItensGram').enabled = false;
    };

    //cancela o modo de edição da widget
    this.cancelEdit = function(args) {
        var widget = args.owner.widget;
        if (widget.currentRow.__is_new) {
            widget.list();
            ScreenService.closeSwipe();
        } else {
            widget.view();
        }
        General.widgetCancelForm(widget);
        widget.reload();
        widget.container.getWidget('tabItensGram').enabled = true;
    };

    //fecha swipe
    this.closeSwipe = function(args) {
        ScreenService.closeSwipe();
        var widget = args.owner.widget;
        widget.container.getWidget('tabMaeDados').editing = false;
    };

    // salva a regra de gramatura do cardapio base
    this.salvaRegraGramatura = function(args) {
        var widget = args.owner.widget;
        var verifica = true;
        if (!widget.currentRow.__is_new && widget.bkp) {
            var row = angular.copy(widget.currentRow);
            row.VRGRAMATURA = General.valueToFloat(row.VRGRAMATURA);
            row.VRGRAMATURA = row.VRGRAMATURA.toString();
            var bkp = {
                'NRTIPOPUBLICO': row.NRTIPOPUBLICO,
                'VRGRAMATURA': row.VRGRAMATURA,
                'SGUNIDADE': row.SGUNIDADE
            };
            if (JSON.stringify(bkp) === JSON.stringify(widget.bkp)) {
                verifica = false;
            } else {
                verifica = true;
            }
        } else {
            verifica = true;
        }
        widget.currentRow.VERIFICA = verifica;
        if (widget.isValid()) {
            widget.currentRow.VRGRAMATURA = General.valueToFloat(widget.currentRow.VRGRAMATURA);
            widget.currentRow.VRGRAMATURA = widget.currentRow.VRGRAMATURA.toString();
            Tco01006_regras_cardapio_service.salvaRegraGramatura(widget.currentRow).then(function(callback) {
                if (callback.error) {
                    MessageService.showMessageByCode(callback.error);
                } else {
                    ScreenService.successNotification('Registro salvo com sucesso');
                    widget.currentRow.NRGRAMATURACARDBASE = callback.NRGRAMATURACARDBASE;
                    widget.currentRow.__is_new = false;
                    widget.currentRow.changed = false;
                    widget.container.getWidget('tabItensGram').enabled = true;
                    widget.view();
                }
            }, function(error) {
                ScreenService.showMessage(error.data.error);
            });
        }
    };

    var buildMessages = function(data) {
        var mensagens = [];
        var msgParams = [];
        data.forEach(function(row) {
            //mensagens.push(ScreenService.i18n(row.MESSAGECODE, row));
            msgParams.push(row);
            mensagens.push(row.MESSAGECODE);
        });

        return MessageService.getMessagesByCode(mensagens).then(function(mensagensTraduzidas) {
            for (let i = 0; i < mensagensTraduzidas.length; i++) {
                mensagensTraduzidas[i] = ScreenService.i18n(mensagensTraduzidas[i], msgParams[i]);
            }
            return mensagensTraduzidas;
        });
    };

    //apaga uma regra de gramatura do cardapio base
    this.apagaRegraGramatura = function(args) {
        var widget = args.owner.widget;

        ScreenService.confirmMessage(ScreenService.i18n('Remover Regra de gramatura?'), 'question', function(success) {
            Tco01006_regras_cardapio_service.apagaRegraGramatura(widget.currentRow).then(function(result) {
                if (result && result.dataset && result.dataset.erros) {
                    let uso_em_propostas = result.dataset.erros.uso_em_propostas;
                    uso_em_propostas.forEach(function(p) {
                        p.MESSAGECODE = 'TCO01006_GRAMATURA_EM_USO_PROPCOM';
                    });

                    buildMessages(uso_em_propostas).then(function(mensagens) {
                        General.showErrorMessages(widget, mensagens, ScreenService.i18n("Erros"));
                    });

                } else {
                    ScreenService.closeSwipe();
                    ScreenService.successNotification("Registro removido com sucesso");
                    widget.reload();
                }
            }, function(error) {
                ScreenService.showMessage(error.data.error, 'error');
            });
        });
    };

    //abre o swipe para adicionar um agrupamento na regra de gramatura
    this.adicionarProdutoGramatura = function(args) {
        var widget = args.owner.widget;
        var popupAdicionar = widget.container.getWidget('popupAdicionar');
        var fieldProduto = popupAdicionar.getField('NMPRODUTO');
        fieldProduto.readOnly = true;
        popupAdicionar.currentRow = {
            NRGRUPOPRATO: "",
            CDGRUPOPRATO: "",
            NMGRUPOPRATO: "",
            NRTIPOPREPAROTCO: "",
            NMTIPOPREPAROTCO: "",
            CDARVPROD: "",
            CDPRODUTO: "",
            NMPRODUTO: "",
            SGUNIDADE: "",
            changed: "",
            __is_new: ""
        };
        ScreenService.openPopup(popupAdicionar);
    };

    //exibe o campo no popup referente ao tipo escolhido no campo select
    this.habilitaCamposPopup = function(args) {
        var fieldTipo = args.owner.field;
        var popup = fieldTipo.widget;
        var fieldProduto = popup.getField('NMPRODUTO');
        var fieldGrupo = popup.getField('NMSUBGRPROD');

        switch (fieldTipo.value()) {
            case 'P':
                fieldProduto.isVisible = true;
                fieldGrupo.isVisible = false;
                break;
            case 'G':
                fieldProduto.isVisible = false;
                fieldGrupo.isVisible = true;
                break;
        }
    };

    this.closePopup = function() {
        ScreenService.closePopup();
    };

    //cria mais uma linha no datasource para mostrar e adicionar dados
    this.adicionaLinhasDataSource = function(args) {
        var widget = args.owner.widget;
        var tabItensGram = widget.container.getWidget('tabItensGram');
        var newRow = {};

        newRow.__is_new = true;
        newRow.changed = true;
        tabItensGram.lastInlineEditingElement = $('#header');

        for (let column in widget.currentRow) {
            widget.currentRow[column] = widget.currentRow[column] !== "" ? widget.currentRow[column] : null;
        }

        var existe;
        if (widget.isValid()) {
            tabItensGram.dataSource.data.forEach(function(row) {

                for (let column in row) {
                    row[column] = row[column] !== "" || row[column] !== 0 ? row[column] : null;
                }

                if (row.NRGRUPOPRATO === widget.currentRow.NRGRUPOPRATO && row.NRTIPOPREPAROTCO === widget.currentRow.NRTIPOPREPAROTCO && row.CDPRODUTO === widget.currentRow.CDPRODUTO) {
                    existe = true;
                }
            });

            if (!existe) {
                for (let column in widget.currentRow) {
                    widget.currentRow[column] = widget.currentRow[column] !== 0 ? widget.currentRow[column] : "";
                }
                tabItensGram.inlineEdit();
                newRow = widget.currentRow;
                let propriedadesIgnoradas = [
                    "NRGRUPOPRATO",
                    "CDGRUPOPRATO",
                    "NMGRUPOPRATO",
                    "NRTIPOPREPAROTCO",
                    "NMTIPOPREPAROTCO",
                    "CDARVPROD",
                    "CDPRODUTO",
                    "NMPRODUTO",
                    "SGUNIDADE",
                    "changed",
                    "__is_new"
                ];

                for (let prop in newRow) {

                    let propExistente = propriedadesIgnoradas.filter((name) => name === prop);

                    newRow[prop] = propExistente.length == 0 ? 0 : newRow[prop];
                }

                tabItensGram.dataSource.data.push(newRow);
                tabItensGram.redraw();
                widget.container.getWidget('tabDadosGramatura').enabled = false;
                ScreenService.closePopup();
            } else {

                let messagesMap = [
                    "TCO01006_ITEM_JA_CADASTRADO_GRUPOPRAT",
                    "TCO01006_ITEM_JA_CADASTRADO_GRUPOPRAT_TIPOPREPARO",
                    "TCO01006_ITEM_JA_CADASTRADO_GRUPOPRAT_TIPOPREPARO_PRODUTO"
                ];
                let messageIndex = 0;

                widget.fields.forEach((field) => {
                    if (field.value() != null && field.value() != "") {
                        ++messageIndex;
                    }
                });

                MessageService.showMessageByCode(messagesMap[messageIndex - 1], widget.currentRow, 'error');
            }
        }
    };

    // carrega os fields dinamicos para regra de gramatura do cardapio base
    this.carregaFieldsDinamicosGramatura = function(widget) {
        var nrgramaturacardbase = widget.getParent().currentRow.NRGRAMATURACARDBASE;
        var row = {};
        row.NRGRAMATURACARDBASE = nrgramaturacardbase;
        row.CDGRTIPOPR = widget.getParent().currentRow.CDGRTIPOPR;
        Tco01006_regras_cardapio_service.retornaCamposDinamicosGramatura(row).then(function(callback) {
            makeFields(widget, callback.porcionamento, false);

            widget.dataSource.data = callback.campos;
            widget.dataSource.data.forEach(function(row) {
                Object.keys(row).forEach(function(key) {
                    var aux = row[key];
                    delete row[key];
                    key = key.split("'").join("");
                    if (key != "NMGRUPOPRATO" && key != "NMTIPOPREPAROTCO" && key != "NMPRODUTO" && key != "CDPRODUTO" && key != "NRTIPOPREPAROTCO" && key != "NRGRUPOPRATO") {
                        row[key] = aux ? aux : 0;
                    } else {
                        row[key] = aux;
                    }
                });

            });
            widget.dataSource.backup = angular.copy(widget.dataSource.data);
        });
    };

    this.defineFilterProduto = function(args) {
        var popupAdicionar = args.owner.field.widget.container.getWidget('popupAdicionar');
        var fieldProduto = args.owner.field.widget.getField('NMPRODUTO');

        if (popupAdicionar.currentRow.NRGRUPOPRATO != null && popupAdicionar.currentRow.NRGRUPOPRATO != "" && popupAdicionar.currentRow.NRTIPOPREPAROTCO != null && popupAdicionar.currentRow.NRTIPOPREPAROTCO != "") {
            popupAdicionar.currentRow.NMPRODUTO = "";
            popupAdicionar.currentRow.CDPRODUTO = "";
            popupAdicionar.currentRow.CDARVPROD = "";
            fieldProduto.readOnly = false;
        } else {
            popupAdicionar.currentRow.NMPRODUTO = "";
            popupAdicionar.currentRow.CDPRODUTO = "";
            popupAdicionar.currentRow.CDARVPROD = "";
            fieldProduto.readOnly = true;
        }
    };

    //apaga linhas do agrupamento de gramatura do cardapio base
    this.apagaRowAgrupamentoGramatura = function(args) {
        var widget = args.owner.widget;
        widget.inlineEdit();
        var dataSource = [];
        widget.dataSource.data.forEach(function(row) {
            if (!row.__isSelected) {
                dataSource.push(row);
            }
        });
        widget.dataSource.data = dataSource;
        widget.container.getWidget('tabDadosGramatura').enabled = false;

        if (!widget.lastInlineEditingElement) {
            widget.lastInlineEditingElement = $("#");
        }

    };

    //salva as linhas do agrupamento de gramatura do cardapio base
    this.salvaGramaturaAgrupamento = function(args) {
        var widget = args.owner;
        var porcionamentos = [];
        var dataSet = [];
        var rowToSave = angular.copy(widget.currentRow);

        widget.fields.forEach(function(field) {
            if (field.name !== 'NMTIPOPRAT') {
                porcionamentos.push({
                    'id': field.name
                });
            }
        });

        porcionamentos.forEach(function(nrporcio) {

            widget.dataSource.data.forEach(function(data) {
                if (data[nrporcio.id] && !isNaN(General.valueToFloat(data[nrporcio.id]))) {
                    if (General.valueToFloat(data[nrporcio.id]) > 0) {
                        var row = {
                            'NRPORCIONAMENTO': nrporcio.id,
                            'VRGRAMATURA': General.valueToFloat(data[nrporcio.id]),
                            'NRGRUPOPRATO': data.NRGRUPOPRATO === 0 ? null : data.NRGRUPOPRATO,
                            'NRTIPOPREPAROTCO': data.NRTIPOPREPAROTCO === 0 ? null : data.NRTIPOPREPAROTCO,
                            'CDPRODUTO': data.CDPRODUTO === 0 ? null : data.CDPRODUTO,
                            'NRGRAMATURACARDBASE': widget.getParent().currentRow.NRGRAMATURACARDBASE,
                            '__is_new': true
                        };
                        dataSet.push(row);
                    }
                }
            });

        });
        rowToSave.AGRUPAMENTO = dataSet;
        rowToSave.NRGRAMATURACARDBASE = widget.getParent().currentRow.NRGRAMATURACARDBASE;
        Tco01006_regras_cardapio_service.salvaGramaturaAgrupamento(rowToSave).then(function(callback) {
            if (dataSet.length > 0) {
                ScreenService.successNotification("Registro salvo com sucesso");
                MessageService.alertNotificationByCode("TCO01006_ITEMGRAM_APENAS_INF");
            }
            self.carregaFieldsDinamicosGramatura(widget);
            widget.list();
            widget.container.getWidget('tabDadosGramatura').enabled = true;
        }, function(error) {
            ScreenService.showMessage(error.data.error, 'error');
        });
    };

    //disabilita a tabDadosGramatura
    this.disableTab = function(args) {
        var field = args.owner.field;
        var widget = field.widget;
        widget.container.getWidget('tabDadosGramatura').enabled = false;
    };

    //abre swipe para adicionar nova regra de particularidade por segmento
    this.novaRegraParticularidade = function(args) {
        var maintabParticularidade = args.owner.widget;

        maintabParticularidade.newRow();
        General.changeWidgetAcessibility(maintabParticularidade.widgets, false);
        maintabParticularidade.currentRow.NRCARDAPIOBASE = maintabParticularidade.container.NRCARDAPIOBASE;
    };

    // salva regra de particularidade por segmento
    this.salvaRegraParticularidade = function(args) {
        var widget = args.owner.widget;
        if (widget.isValid()) {
            Tco01006_regras_cardapio_service.salvaRegraParticularidade(widget.currentRow).then(function(callback) {
                if (callback.chaves) {
                    widget.currentRow.__is_new = false;
                    widget.currentRow.NRPARTSEGCARDBASE = callback.chaves.NRPARTSEGCARDBASE;
                    widget.currentRow.CDPARTSEGCARDBASE = callback.chaves.CDPARTSEGCARDBASE;
                    ScreenService.successNotification("Registro salvo com sucesso");
                    General.changeWidgetAcessibility(widget.widgets, true);
                    widget.view();
                    widget.reload();
                } else if (callback.erro) {
                    ScreenService.successNotification("Registro sem alteração");
                    widget.view();
                    widget.reload();
                }
            }, function(error) {
                ScreenService.showMessage(error.data.error, 'error');
            });
        }
    };

    //apaga regra de particularidade por segmento
    this.apagaRegraParticularidade = function(args) {
        var widget = args.owner.widget;

        ScreenService.confirmMessage(ScreenService.i18n('Remover Regra de Particularidade do Segmento?'), 'question', function(success) {
            Tco01006_regras_cardapio_service.apagaRegraParticularidade(widget.currentRow).then(function() {
                ScreenService.closeSwipe();
                ScreenService.successNotification("Registro removido com sucesso");
                widget.reload();
            }, function(error) {
                ScreenService.showMessage(error.data.error, 'error');
            });
        });
    };

    // filtra a widget de tipo de prato na regra de particularidade
    this.setFilterTipoPratoPart = function(args) {
        var widget = args.owner;

        widget.dataSourceFilter = [
            {
                "name": "NRCARDAPIOBASE",
                "operator": "=",
                "value": widget.container.NRCARDAPIOBASE
            },
            {
                "name": "NRPARTSEGCARDBASE",
                "operator": "=",
                "value": widget.getParent().currentRow.NRPARTSEGCARDBASE
            }
        ];
        widget.reload().then(function() {
            widget.dataSource.backup = angular.copy(widget.dataSource.data);
        });
    };

    //salva tipo de prato da regra de particularidade do segmento
    this.salvaTipoPratoPart = function(args) {
        var widget = args.owner;
        widget.commitInlineEdit();
        Tco01006_regras_cardapio_service.salvaTipoPratoPart(widget.dataSource.data).then(function(callback) {
            widget.reload().then(function() {
                widget.dataSource.backup = angular.copy(widget.dataSource.data);
            });
            ScreenService.successNotification('Salvo com sucesso');
        }, function(error) {
            ScreenService.showMessage(error.data.error, 'error');
            widget.inlineEdit();
        });
    };

    //ao editar particularidade, filtra os datasources das fields
    this.editParticularidade = function(args) {
        var widget = args.owner.widget;
        General.widgetEdit(widget);
        var ramo = widget.getField('DSATIVCLI');
        ramo.dataSourceFilter = [
            {
                "name": "NRCARDAPIOBASE",
                "operator": "=",
                "value": widget.currentRow.NRCARDAPIOBASE

            },
            {
                "name": "CDSEGMER",
                "operator": "=",
                "value": widget.currentRow.CDSEGMER
            }
        ];
        ramo.reload();
    };

    //coloca a widget em modo editing para não poder clicar no primeiro grind 
    this.editing = function(args) {
        var widget = args.owner;
        widget.container.getWidget('tabMaeDados').editing = true;
    };

    /**
     * Conjunto de instruções a serem executadas
     * ao entrar na widget de descartaveis.
     */
    this.onEnterwidgetDescartaveis = function() {
        getDescartaveisData();
        setAmostraFieldsLock(false);
        setAmostraFieldsFilter();
        this.resetDescartaveisReloadFunction();
    };
    //Carrega o dataSource no select widget do
    this.LoadDataSourceWidget = function(field) {
        field.selectWidget.dataSourceFilter = field.dataSourceFilter;
    };
    /**
     * Redefine a função de reload da widget de 
     * reload da widget de Descartaveis para 
     * recarregar os dados dos campos de amostra.
     */
    this.resetDescartaveisReloadFunction = function() {
        var descartaveisGrid = templateManager.container.getWidget('descartaveisGrid');
        descartaveisGrid.reload = getDescartaveisData;
    };

    /**
     * Preenche os dados da aba de Descartáveis.
     * Busca os campos de Amostra Quente e Frio e
     * carrega o dataSource do grid inferior da tela.
     */
    var getDescartaveisData = function() {
        var configCardapioBaseParams = window.history.state.configCardapioBaseParams;

        var filterRow = {
            NRCARDAPIOBASE: configCardapioBaseParams.filter(data => data.name == 'NRCARDAPIOBASE')[0]
        };

        return Tco01006_regras_cardapio_service.getDescartaveisData(filterRow).then((success) => {
            if (success.error) {
                MessageService.showMessageByCode(success.error);
            } else if (success.dataset && success.dataset.descartData) {
                fillDescartaveisData(success.dataset.descartData);
            }
        }).catch((error) => {
            if (error.error) {
                ScreenService.showMessage(ScreenService.i18n(error.error));
            }
        });
    };

    /**
     * Preenche os dados da widget de descartaveis.
     * 
     * @param {Object} dataSet Objeto contendo
     *  os arrays 'AMOSTRA' e 'DESCART' para
     *  preencher os dados da widget.
     */
    var fillDescartaveisData = function(dataSet) {
        var widgetDescartaveis = templateManager.container.getWidget('widgetDescartaveis');
        var descartaveisGrid = templateManager.container.getWidget('descartaveisGrid');
        var tabRow = widgetDescartaveis.currentRow;

        if (dataSet.DESCART && dataSet.DESCART.length) {
            descartaveisGrid.dataSource.data = dataSet.DESCART;
        } else {
            descartaveisGrid.dataSource.data = [];
        }

        if (dataSet.AMOSTRA) {
            var amostra = dataSet.AMOSTRA;

            if (amostra.NRCONFIGCARDBASEF && amostra.NRCONFIGCARDBASEF.length) {
                tabRow.AMOSTRAFRIO = amostra.NRCONFIGCARDBASEF[0].NMTIPOPRAT + ' | ' +
                    amostra.NRCONFIGCARDBASEF[0].DSCOMPLEMENTO;
                tabRow.NRCONFIGCARDBASEF = amostra.NRCONFIGCARDBASEF[0].NRCONFIGCARDBASE;
            } else {
                tabRow.AMOSTRAFRIO = '';
                tabRow.NRCONFIGCARDBASEF = '';
            }

            if (amostra.NRCONFIGCARDBASEQ && amostra.NRCONFIGCARDBASEQ.length) {
                tabRow.AMOSTRAQUENTE = amostra.NRCONFIGCARDBASEQ[0].NMTIPOPRAT + ' | ' +
                    amostra.NRCONFIGCARDBASEQ[0].DSCOMPLEMENTO;
                tabRow.NRCONFIGCARDBASEQ = amostra.NRCONFIGCARDBASEQ[0].NRCONFIGCARDBASE;
            } else {
                tabRow.AMOSTRAQUENTE = '';
                tabRow.NRCONFIGCARDBASEQ = '';
            }

        } else {
            tabRow.AMOSTRAFRIA = '';
            tabRow.AMOSTRAQUENTE = '';
            tabRow.NRCONFIGCARDBASEF = '';
            tabRow.NRCONFIGCARDBASEQ = '';
        }
        setAmostraFieldsLock();
    };

    /**
     * Conjunto de instruções a serem executadas
     * ao (des)selecionar algum registro no grid
     * de descartaveis na aba descartaveis.
     */
    this.onChangeSelectDescart = function() {
        setAmostraFieldsLock();
    };

    /**
     * Bloqueia os campos de amostra de acordo com
     * estado do parametro. E bloqueia caso seja
     * selecionado algum registro no grid de 
     * descartaveis se não for passado state.
     * 
     * @param {Boolean} state Estado a ser
     *  definido.
     */
    var setAmostraFieldsLock = function(state) {
        var descartaveisGrid = templateManager.container.getWidget('descartaveisGrid');
        var widgetDescartaveis = templateManager.container.getWidget('widgetDescartaveis');

        widgetDescartaveis.fields.forEach(field => {
            field.readOnly = state === undefined ? descartaveisGrid.hasCheckedRows() : state;
        });
    };

    /**
     * Define filtro para campos de amostra
     * na widget tabbed de descartaveis.
     */
    var setAmostraFieldsFilter = function() {
        var widgetDescartaveis = templateManager.container.getWidget('widgetDescartaveis');
        var configCardapioBaseParams = window.history.state.configCardapioBaseParams;

        widgetDescartaveis.fields.forEach(field => {
            field.dataSourceFilter = [
                {
                    "name": "IDCLASSIFICACAO",
                    "operator": "=",
                    "value": "DESCARTAVEL"
                },
                configCardapioBaseParams[0],
                {
                    "operator": "ORDER_BY",
                    "name": [
                        {
                            "name": "NMTIPOPRAT",
                            "direction": "ASC"
                        },
                        {
                            "name": "DSCOMPLEMENTO",
                            "direction": "ASC"
                        }
                    ]
                }
            ];

            field.reload();
        });
    };

    /**
     * Conjunto de instruções a serem executadas ao alterar
     * valor nos campos de amostra.
     */
    this.onChangeAmostraFields = function() {
        var descartaveisGrid = templateManager.container.getWidget('descartaveisGrid');
        descartaveisGrid.inlineEdit();
        setDescartGridCheckboxVisibility(false);

        var widgetDescartaveis = templateManager.container.getWidget('widgetDescartaveis');
        lockSisterWidgets(widgetDescartaveis, true);
    };

    /**
     * Valida os dados selecionados para amostra
     * de descartaveis e imprime mensagem de erro
     * caso invalido.
     * 
     * @return {boolean} True para valido, false
     *  para invalido.
     */
    var validateAmostraData = function() {
        var widgetDescartaveis = templateManager.container.getWidget('widgetDescartaveis');
        var cRow = widgetDescartaveis.currentRow;
        var isValid = false;

        if (cRow.NRCONFIGCARDBASEF == cRow.NRCONFIGCARDBASEQ && cRow.NRCONFIGCARDBASEQ) {
            MessageService.showMessageByCode('TCO01006_ERROR_SAME_AMOSTRA');
        } else {
            isValid = true;
        }

        return isValid;
    };

    /**
     * Cancela o modo de edição da widget de descartaveis
     * nos campos de amostra.
     */
    this.cancelWidgetDescart = function() {
        var descartaveisGrid = templateManager.container.getWidget('descartaveisGrid');
        getDescartaveisData();
        descartaveisGrid.list();
        setDescartGridCheckboxVisibility(true);

        var widgetDescartaveis = templateManager.container.getWidget('widgetDescartaveis');
        lockSisterWidgets(widgetDescartaveis, false);
    };

    /**
     * Exibe o checkbox do grid de descartaveis caso
     * 
     * @param {Boolean} state Estado de visiblidade.
     */
    var setDescartGridCheckboxVisibility = function(state) {
        var descartaveisGrid = templateManager.container.getWidget('descartaveisGrid');
        descartaveisGrid.floatingControl.refreshAction = state;
        descartaveisGrid.floatingControl.searchAction = state;
        if (descartaveisGrid.showCheckbox != state) {
            descartaveisGrid.showCheckbox = state;
            descartaveisGrid.redraw();
        }
    };

    /**
     * Bloqueia acesso às widgets irmãs da widget
     * atual, em uma tabbed.
     * 
     * @param {Widget} widget Widget a permanecer
     *  ativa.
     * @param {Bidget} state Estado de bloqueio da
     *  widgets.
     */
    var lockSisterWidgets = function(widget, state) {
        if (widget.getParent()) {
            var widsToLock = widget.getParent().widgets.filter(wid => wid.name != widget.name);
            General.changeWidgetAcessibility(widsToLock, !state);
        }
    };

    /**
     * Salvar alterações nos campos de amostra
     * na widget de descartáveis.
     */
    this.saveChangesAmostra = function() {
        if (validateAmostraData()) {
            var widgetDescartaveis = templateManager.container.getWidget('widgetDescartaveis');
            var descartaveisGrid = templateManager.container.getWidget('descartaveisGrid');
            var configCardapioBaseParams = window.history.state.configCardapioBaseParams;
            var cRow = widgetDescartaveis.currentRow;

            cRow.NRCARDAPIOBASE = configCardapioBaseParams.filter(data => data.name == 'NRCARDAPIOBASE')[0].value;

            Tco01006_regras_cardapio_service.saveChangesAmostra(cRow).then((success) => {
                if (success.error) {
                    MessageService.showMessageByCode(success.error);
                } else {
                    ScreenService.successNotification('Registro salvo com sucesso.');
                    this.cancelWidgetDescart();
                }
            }).catch((error) => {
                if (error.error) {
                    if (error.error.indexOf('TCO') == 0) {
                        MessageService.showMessageByCode(error.error);
                    } else {
                        ScreenService.showMessage(error.error);
                    }
                }
            });
        }
    };

    /**
     * Abre widget de alteração da forma de calculo
     * de descartaveis.
     */
    this.openChangeFormaCalculo = function() {
        var changeFormaCalculoForm = templateManager.container.getWidget('changeFormaCalculoForm');
        ScreenService.openPopup(changeFormaCalculoForm);
        onEnterChangeFormaCalculo();
    };

    /**
     * Instruções a serem executadas ao abrir a widget
     * de alterar forma de calculo
     */
    var onEnterChangeFormaCalculo = function() {
        var changeFormaCalculoForm = templateManager.container.getWidget('changeFormaCalculoForm');
        changeFormaCalculoForm.currentRow = {};
    };

    /**
     * Fecha popup de forma de calculo.
     */
    this.closeChangeFormaCalculo = function() {
        var changeFormaCalculoForm = templateManager.container.getWidget('changeFormaCalculoForm');
        ScreenService.closePopup(changeFormaCalculoForm);
    };

    /**
     * Confirma operação de alterar forma
     * de calculo.
     */
    this.changeFormaCalculo = function() {
        var changeFormaCalculoForm = templateManager.container.getWidget('changeFormaCalculoForm');
        var descartaveisGrid = templateManager.container.getWidget('descartaveisGrid');
        var cRow = changeFormaCalculoForm.currentRow;
        var selectedRows = descartaveisGrid.getCheckedRows();

        var requestRow = {
            "formaCalculo": cRow,
            "selectedRows": selectedRows
        };

        Tco01006_regras_cardapio_service.changeFormaCalculo(requestRow).then((success) => {
            ScreenService.successNotification('Registro(s) salvo com sucesso.');
            this.closeChangeFormaCalculo();
            getDescartaveisData();
            setAmostraFieldsLock(false);
        }).catch((error) => {
            if (error.error) {
                ScreenService.showMessage(ScreenService.i18n(error.error, cRow));
            }
        });
    };

    /**
     * Conjunto de instruções a serem executadas
     * ao entrar na widget de limpeza.
     */
    this.onEnterLimpezaWidget = function() {
        setLimpezaGridFilter();
    };

    /**
     * Define o filtro para widget de limpeza.
     */
    var setLimpezaGridFilter = function() {
        var limpezaWidget = templateManager.container.getWidget('limpezaWidget');
        var configCardapioBaseParams = window.history.state.configCardapioBaseParams;

        limpezaWidget.dataSourceFilter = [
            {
                "name": "NRCARDAPIOBASE",
                "operator": "=",
                "value": configCardapioBaseParams.filter(data => data.name == 'NRCARDAPIOBASE')[0].value
            }
        ];

        limpezaWidget.reload();
    };

    /**
     * Rotina de salvamento dos dados da
     * widget de limpeza.
     */
    this.saveLimpeza = function() {
        var limpezaWidget = templateManager.container.getWidget('limpezaWidget');
        var changedData = limpezaWidget.dataSource.data.filter(reg => reg.changed);
        var isValidEditedReg = isValidChangedLimpeza(changedData);

        if (isValidEditedReg == 'VALID') {

            Tco01006_regras_cardapio_service.saveLimpeza(changedData).then((success) => {

                ScreenService.successNotification('Registro(s) salvo com sucesso.');
                limpezaWidget.list();

                changedData.forEach(data => {
                    Object.keys(data).forEach((prop) => {
                        if (prop.indexOf('changed_') > -1) {
                            delete data[prop];
                        }
                    });
                });

                limpezaWidget.redraw();
            }).catch((error) => {
                if (error.error && error.error.length) {
                    showErrorMessageSaveLimpeza(error.error);
                }
            });
        } else {
            showErrorMessageSaveLimpeza(isValidEditedReg);
        }
    };

    /**
     * Exibe mensagem de erro na widget de limpeza
     * de acordo com os parametros do array.
     * 
     * @param {Array} dataset Array de registros
     *  sinalizados com erros.
     */
    var showErrorMessageSaveLimpeza = function(dataset) {
        var limpezaWidget = templateManager.container.getWidget('limpezaWidget');
        var mensagens = [];
        var msgParams = [];
        dataset.forEach(function(err) {
            //mensagens.push(ScreenService.i18n(err.ERROR_CODE, err.PARAMS));
            msgParams.push(err.PARAMS);
            mensagens.push(err.ERROR_CODE);
        });
        MessageService.getMessagesByCode(mensagens).then(function(mensagensTraduzidas) {
            for (let i = 0; i < mensagensTraduzidas.length; i++) {
                mensagensTraduzidas[i] = ScreenService.i18n(mensagensTraduzidas[i], msgParams[i]);
            }
            if (mensagensTraduzidas.length > 0)
                General.showErrorMessages(limpezaWidget, mensagens, ScreenService.i18n('Operação Inválida.'));

        });
    };

    /**
     * Verifica se o dado a ser salvo é válido
     * a ser salvo na widget de limpeza.
     * 
     * @param {Array} data Dados de limpeza
     *  a serem salvos.
     */
    var isValidChangedLimpeza = function(data) {
        var invalidRegister = [];

        data.forEach((reg) => {
            if (reg.QTINICIAL && reg.QTFINAL) {
                if (reg.QTINICIAL > reg.QTFINAL) {
                    invalidRegister.push({
                        ERROR_CODE: 'TCO01006_LIMPEZACARDBASE_BEGIN_MT_END',
                        PARAMS: reg
                    });
                } else if ((reg.QTINICIAL % 1) != 0 || (reg.QTFINAL % 1) != 0) {
                    invalidRegister.push({
                        ERROR_CODE: 'TCO01006_LIMPEZACARDBASE_FLOAT_REGISTER',
                        PARAMS: reg
                    });
                }
            } else if (reg.QTINICIAL == 0 || reg.QTFINAL == 0) {
                invalidRegister.push({
                    ERROR_CODE: 'TCO01006_LIMPEZACARDBASE_ZERO_REGISTER',
                    PARAMS: reg
                });
            }
        });

        return invalidRegister.length ? invalidRegister : 'VALID';
    };

    /**
     * Abre popup para adição de novo registro de
     * quantidade de tipos de prato para limpeza.
     */
    this.openAddLimpezaCardBase = function() {
        var addLimpezaCardBase = templateManager.container.getWidget('addLimpezaCardBase');
        ScreenService.openPopup(addLimpezaCardBase);
        addLimpezaCardBase.edit();
        this.onEnterAddLimpezaCardBase();
    };

    /**
     * Cancela o formulário de adição de um novo
     * registro de limpeza de cardapio base.
     */
    this.cancelAddLimpeza = function() {
        var addLimpezaCardBase = templateManager.container.getWidget('addLimpezaCardBase');
        ScreenService.closePopup(addLimpezaCardBase);
    };

    /**
     * Conjunto de instruções a serem executadas
     * ao entrar na widget de adição de registros de
     * limpeza.
     */
    this.onEnterAddLimpezaCardBase = function() {
        General.justInteger('#QTINICIAL');
        General.justInteger('#QTFINAL');
        setAddLimpezaTipoPratFilter();
        clearAddLimpezaRow();
    };

    /**
     * Define filtro no campo de tipo de prato
     * na widget de adição de registro de limpeza
     * do cardapio.
     */
    var setAddLimpezaTipoPratFilter = function() {
        var addLimpezaCardBase = templateManager.container.getWidget('addLimpezaCardBase');
        var tipoPratField = addLimpezaCardBase.getField('NMTIPOPRAT');
        var configCardapioBaseParams = window.history.state.configCardapioBaseParams;
        var notInConfigCardBase = addLimpezaCardBase.getParent().dataSource.data.map(reg => reg.NRCONFIGCARDBASE);

        tipoPratField.dataSourceFilter = [
            configCardapioBaseParams.filter(data => data.name == 'NRCARDAPIOBASE')[0]
        ];

        if (notInConfigCardBase.length) {
            tipoPratField.dataSourceFilter.push({
                "name": "NRCONFIGCARDBASE",
                "operator": "NOT_IN",
                "value": notInConfigCardBase
            });
        }

        tipoPratField.reload();
    };

    /**
     * Limpa currentRow da widget de adição
     * de registro de limpeza de cardapio base.
     */
    var clearAddLimpezaRow = function() {
        var addLimpezaCardBase = templateManager.container.getWidget('addLimpezaCardBase');
        addLimpezaCardBase.currentRow = {};
    };

    /**
     * Confirma adição de novo registro de limpeza
     * para cardapio base;
     */
    this.confirmAddLimpeza = function() {
        var addLimpezaCardBase = templateManager.container.getWidget('addLimpezaCardBase');
        var changedData = [addLimpezaCardBase.currentRow];
        var cRow = addLimpezaCardBase.currentRow;

        if (addLimpezaCardBase.isValid()) {
            if (cRow.QTINICIAL > 0 || cRow.QTFINAL > 0) {
                if (cRow.QTINICIAL <= cRow.QTFINAL) {
                    Tco01006_regras_cardapio_service.saveLimpeza(changedData).then((success) => {

                        this.cancelAddLimpeza();
                        ScreenService.successNotification('Registro(s) salvo com sucesso.');
                        addLimpezaCardBase.getParent().reload();

                    }).catch((error) => {
                        if (error.error && error.error.length) {
                            MessageService.showMessageByCode(error.error[0].ERROR_CODE, cRow);
                        }
                    });
                } else {
                    MessageService.showMessageByCode('TCO01006_LIMPEZACARDBASE_BEGIN_MT_END', cRow);
                }
            } else {
                MessageService.showMessageByCode('TCO01006_LIMPEZACARDBASE_ZERO_REGISTER', cRow);
            }
        }
    };

    /**
     * Rotina de salvamento dos dados da
     * widget de limpeza.
     */
    this.removeLimpezaCardBase = function() {
        ScreenService.confirmMessage(ScreenService.i18n('Confirmar exclusão do registro?')).then(() => {
            var limpezaWidget = templateManager.container.getWidget('limpezaWidget');
            var selectedRows = limpezaWidget.dataSource.getCheckedRows();

            Tco01006_regras_cardapio_service.removeLimpezaCardBase(selectedRows).then((success) => {
                if (success.error) {
                    MessageService.showMessageByCode(success.error);
                } else {
                    ScreenService.showMessage('Registro(s) removido(s) com sucesso.');
                    limpezaWidget.dataSource.clearCheckedRows();
                    limpezaWidget.reload();
                }
            }).catch((error) => {
                if (error.error) {
                    ScreenService.showMessage(error.error);
                }
            });
        });
    };

    /**
     * Conjunto de instruções a serem executadas
     * ao (des)selecionar algum registro no grid
     * de limpeza.
     */
    this.onChangeSelectLimpeza = function() {
        changeLimpezaRemoveVisibility();
    };

    /**
     * Desativa exibição do botão de excluir
     * caso seja selecionado o checkbox em modo
     * 'inline_edit'.
     */
    var changeLimpezaRemoveVisibility = function() {
        var limpezaWidget = templateManager.container.getWidget('limpezaWidget');
        var removeAction = limpezaWidget.getAction('removeLimpezaCardBase');

        removeAction.isVisible = !limpezaWidget.inInlineEditMode();
    };

    this.changeToInlineEdit = function(args) {
        var widget = templateManager.container.getWidget('widgetTipoPrato');
        widget.inlineEdit();
        widget.getAction('DELETE').isVisible = false;
        widget.getAction('COPY').isVisible = false;
    };

    this.duplicaRegraAdesao = function(args) {
        var widget = args.owner.widget;
        var tabDadosPrato = widget.container.getWidget('tabDadosPrato');
        var maintabAdesao = widget.container.getWidget('maintabAdesao');
        MessageService.confirmMessageByCode('TCO01006_DUPLICAR_REGRA').then(function(success) {
            Tco01006_regras_cardapio_service.duplicaRegraAdesao(tabDadosPrato.currentRow).then(function(callback) {
                maintabAdesao.reload().then(function() {
                    ScreenService.closeSwipe().then(function() {
                        setTimeout(function() {
                            var regra = maintabAdesao.dataSource.data.filter(function(row) {
                                return row.NRADESAOCARDBASE === parseInt(callback.NRADESAOCARDBASE);
                            })[0];
                            if (regra) maintabAdesao.setCurrentRow(regra);
                        }, 150);
                    });
                });
            });
        });
    };

    this.calculaPorcentagemParticularidade = function(args) {
        var row = args.row;
        var base = row.VRPERCBASE_inline;
        var objetivo = row.VRPERCOBJ_inline;
        if (base && objetivo) {
            row.VRPERCENTUAL = (objetivo - base) / base * 100;
        }
    };

    this.cancelaInlineEditPart = function(args) {
        var widget = templateManager.container.getWidget('tabPratosPart');
        widget.reload().then(function() {
            widget.inlineEditing = false;
            widget.list();
        });
    };

    this.importarItens = function(args) {
        var tabDadosGramatura = args.owner.widget.container.getWidget('tabDadosGramatura');
        var popupImportacaoItens = args.owner.widget.container.getWidget('popupImportacaoItens');

        var filter = angular.copy(tabDadosGramatura.dataSourceFilter);
        filter.push({
            name: "NRGRAMATURACARDBASE",
            operator: "<>",
            value: tabDadosGramatura.currentRow.NRGRAMATURACARDBASE
        });

        popupImportacaoItens.dataSourceFilter = filter;
        popupImportacaoItens.currentRow = {};

        popupImportacaoItens.reload().then(() => {
            ScreenService.openPopup(popupImportacaoItens);
        });
    };

    this.confirmaImportacao = function(args) {
        var popupImportacaoItens = args.owner.widget;
        var tabItensGram = args.owner.widget.container.getWidget('tabItensGram');
        var tabDadosGramatura = args.owner.widget.container.getWidget('tabDadosGramatura');

        if (typeof popupImportacaoItens.currentRow.NRGRAMATURACARDBASE != 'undefined' && popupImportacaoItens.currentRow.NRGRAMATURACARDBASE != '') {

            ScreenService.confirmMessage(ScreenService.i18n("Os itens da Gramatura serão sobrescritos. Confirmar a importação?"), 'question', (success) => {
                let rowToImport = popupImportacaoItens.currentRow;
                rowToImport.NRGRAMATURACARDBASEDEST = tabDadosGramatura.currentRow.NRGRAMATURACARDBASE;
                Tco01006_regras_cardapio_service.importaItemGramatura(rowToImport).then((success) => {
                    if (typeof success.dataset.errormessages != 'undefined') {
                        MessageService.showMessageByCode(success.dataset.errormessages.message);
                    } else {
                        ScreenService.successNotification(ScreenService.i18n("Itens importados com sucesso."));
                        ScreenService.closePopup();
                        this.carregaFieldsDinamicosGramatura(tabItensGram);

                    }
                });
            });
        } else {
            ScreenService.showMessage(ScreenService.i18n("Selecione uma Gramatura."));
        }
    };
    this.widgetAddPorcGram = function(args) {
        ScreenService.openPopup(args.owner.container.getWidget('AddPorcGramTabbed'));
        setFilterOnPorcGramFields(args);
        args.owner.container.getWidget('AddPorcGramTabbed').currentRow = args.owner.container.getWidget('gruposWidget').currentRow;
        var translateTipoGramaturaValues = {
            "INNATURA": "0",
            "LIMPA": "1",
            "PRONTA": "2"
        };
        args.owner.container.getWidget('AddPorcGramTabbed').currentRow.IDTIPOGRAMATURA = translateTipoGramaturaValues[args.owner.container.getWidget('AddPorcGramTabbed').currentRow.IDTIPOGRAMATURA];
        args.owner.container.getWidget('AddPorcGramTabbed').getField('NMPORCIONAMENTO').reload();
        args.owner.container.getWidget('AddPorcGramTabbed').getField('VRGRAMATURA').reload().then((success) => {
            if (args.owner.container.getWidget('AddPorcGramTabbed').getField('VRGRAMATURA').dataSource.data.length != 0) {
                args.owner.container.getWidget('AddPorcGramTabbed').getField('VRGRAMATURA').validations = {
                    "required": true
                };
                args.owner.container.getWidget('AddPorcGramTabbed').getField('VRGRAMATURA').readOnly = false;
                args.owner.container.getWidget('AddPorcGramTabbed').getField('IDTIPOGRAMATURA').validations = {
                    "required": true
                };
                args.owner.container.getWidget('AddPorcGramTabbed').getField('IDTIPOGRAMATURA').readOnly = false;
            } else {
                args.owner.container.getWidget('AddPorcGramTabbed').getField('VRGRAMATURA').validations = {};
                args.owner.container.getWidget('AddPorcGramTabbed').getField('VRGRAMATURA').readOnly = true;
                args.owner.container.getWidget('AddPorcGramTabbed').getField('IDTIPOGRAMATURA').validations = {};
                args.owner.container.getWidget('AddPorcGramTabbed').getField('IDTIPOGRAMATURA').readOnly = true;

            }
        });
    };
    var setFilterOnPorcGramFields = function(args) {
        var addPorcGram = args.owner.container.getWidget('AddPorcGramTabbed');
        var detalheTab = args.owner.container.getWidget('gruposWidget');
        var stripeWidget = args.owner.container.getWidget('stripeWidget');

        addPorcGram.getField('NMPORCIONAMENTO').dataSourceFilter = [
            {
                name: "NRCARDAPIOBASE",
                operator: "=",
                value: stripeWidget.currentRow.NRCARDAPIOBASE
            },
            {
                name: "CDGRTIPOPR",
                operator: "=",
                value: detalheTab.currentRow.CDGRTIPOPR
            }
        ];

        addPorcGram.getField('VRGRAMATURA').dataSourceFilter = [
            {
                name: "NRCARDAPIOBASE",
                operator: "=",
                value: stripeWidget.currentRow.NRCARDAPIOBASE
            },
            {
                name: "CDGRTIPOPR",
                operator: "=",
                value: detalheTab.currentRow.CDGRTIPOPR
            },
            {
                name: "NRTIPOPUBLICO",
                operator: "=",
                value: detalheTab.currentRow.NRTIPOPUBLICO
            }
        ];

    };

    this.cancelAdd = function(widget) {
        var gridWidget = widget.container.getWidget("gruposWidget");
        ScreenService.closePopup(true);
        gridWidget.activate();
    };
    this.saveAndNext = function(args) {
        var widget = args.owner.widget;
        var addPorcGram = widget.container.getWidget('AddPorcGramTabbed');
        var widgetGrupos = widget.container.getWidget('gruposWidget');
        var widgetStripe = widget.container.getWidget('stripeWidget');
        var cRow = addPorcGram.currentRow;

        let executaSave = function(cRow) {
            if (addPorcGram.isValid()) {
                let row = cRow;
                Tco01006_regras_cardapio_service.saveAddPorcGram(row).then((success) => {
                    ScreenService.successNotification(ScreenService.i18n('Registro alterado com sucesso!'));

                    for (let i = 0; i < widgetGrupos.dataSource.data.length; i++) {
                        if (widgetGrupos.dataSource.data[i].CDGRTIPOPR == widgetGrupos.currentRow.CDGRTIPOPR) {
                            if (widgetGrupos.dataSource.data[i + 1] != undefined) {
                                addPorcGram.currentRow = widgetGrupos.dataSource.data[i + 1];
                                widgetGrupos.currentRow = widgetGrupos.dataSource.data[i + 1];
                                setDataSourceEValidations(args);
                                break;
                            } else {
                                ScreenService.closePopup();
                                ScreenService.showMessage(ScreenService.i18n('Todos os registros foram alterados com sucesso!'));
                            }
                        }
                    }
                    widgetGrupos.reload();
                });
            }
        };
        let setRow = function() {
            if (addPorcGram.isValid()) {
                cRow.CDGRTIPOPR = widgetGrupos.currentRow.CDGRTIPOPR;
                cRow.IDATIVO = widgetStripe.currentRow.IDATIVO;
                cRow.NRCARDAPIOBASE = widgetStripe.currentRow.NRCARDAPIOBASE;
                cRow.NRCONFGRUPTPCARDBASE = widgetGrupos.currentRow.NRCONFGRUPTPCARDBASE;
                var translateTipoGramaturaValues = {
                    "0": "INNATURA",
                    "1": "LIMPA",
                    "2": "PRONTA"
                };
                cRow.IDTIPOGRAMATURA = translateTipoGramaturaValues[cRow.IDTIPOGRAMATURA];

                executaSave(cRow);

            }
        };
        setRow();
    };

    var setDataSourceEValidations = function(args) {
        var addPorcGram = args.owner.widget.container.getWidget('AddPorcGramTabbed');
        var detalheTab = args.owner.widget.container.getWidget('gruposWidget');
        var stripeWidget = args.owner.widget.container.getWidget('stripeWidget');
        addPorcGram.getField('NMPORCIONAMENTO').dataSourceFilter = [
            {
                name: "NRCARDAPIOBASE",
                operator: "=",
                value: stripeWidget.currentRow.NRCARDAPIOBASE
            },
            {
                name: "CDGRTIPOPR",
                operator: "=",
                value: detalheTab.currentRow.CDGRTIPOPR
            }
        ];
        addPorcGram.getField('VRGRAMATURA').dataSourceFilter = [
            {
                name: "NRCARDAPIOBASE",
                operator: "=",
                value: stripeWidget.currentRow.NRCARDAPIOBASE
            },
            {
                name: "CDGRTIPOPR",
                operator: "=",
                value: detalheTab.currentRow.CDGRTIPOPR
            },
            {
                name: "NRTIPOPUBLICO",
                operator: "=",
                value: detalheTab.currentRow.NRTIPOPUBLICO
            }
        ];
        args.owner.widget.container.getWidget('AddPorcGramTabbed').getField('NMPORCIONAMENTO').reload();
        args.owner.widget.container.getWidget('AddPorcGramTabbed').getField('VRGRAMATURA').reload().then((success) => {
            if (args.owner.widget.container.getWidget('AddPorcGramTabbed').getField('VRGRAMATURA').dataSource.data.length != 0) {
                args.owner.widget.container.getWidget('AddPorcGramTabbed').getField('VRGRAMATURA').validations = {
                    "required": true
                };
                args.owner.widget.container.getWidget('AddPorcGramTabbed').getField('VRGRAMATURA').readOnly = false;
                args.owner.widget.container.getWidget('AddPorcGramTabbed').getField('IDTIPOGRAMATURA').validations = {
                    "required": true
                };
                args.owner.widget.container.getWidget('AddPorcGramTabbed').getField('IDTIPOGRAMATURA').readOnly = false;
            } else {
                args.owner.widget.container.getWidget('AddPorcGramTabbed').getField('VRGRAMATURA').validations = {};
                args.owner.widget.container.getWidget('AddPorcGramTabbed').getField('VRGRAMATURA').readOnly = true;
                args.owner.widget.container.getWidget('AddPorcGramTabbed').getField('IDTIPOGRAMATURA').validations = {};
                args.owner.widget.container.getWidget('AddPorcGramTabbed').getField('IDTIPOGRAMATURA').readOnly = true;

            }
        });
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01006_regras_cardapio', Tco01006_regras_cardapio);
});

// FILE: ./assets/js/controllers/Tco01007_utiliza_substitui_pratos_cardapio_base_controller.js
/* global angular, Configuration*/
var Tco01007_utiliza_substitui_pratos_cardapio_base_controller = function(ScreenService, General, TcoUtil, Tco01007_utiliza_substitui_pratos_cardapio_base_service, templateManager) {

    this.carregaParametros = function(args) {
        if (!window.history.state) {
            window.location.href = window.location.href.replace("#/tco#tco01007_utiliza_substitui_pratos_cardapio_base", "#/tco#tco01000_cardapio_base");
            window.location.reload();
        } else {
            var widgetMain = args.owner.getWidget("widgetMain");
            var widgetPratosSubstituicao = args.owner.getWidget("widgetPratosSubstituicao");
            widgetMain.getField('PERIODO').defaultValue = {
                start: moment().startOf('month').format('DD/MM/YYYY'),
                end: moment().endOf('month').format('DD/MM/YYYY')
            };
            TcoUtil.maskPrato("widgetMain");
            TcoUtil.maskPrato("widgetPratosSubstituicao");
            TcoUtil.startFilter(widgetMain);
        }
    };

    this.backCardBase = function() {
        var widget = templateManager.container.getWidget('widgetMain');
        widget.dataSourceFilter = [];
        widget.reload().then(() => {
            window.history.back();
        });
    };

    this.closeSwipeTiposPrato = function() {
        ScreenService.closeSwipe();
    };

    this.closePopupSubstituir = function() {
        ScreenService.closePopup();
    };

    this.preencheDatasFilter = function(args) {
        var widgetMain = args.owner;
        var filterRow = args.e.preference.FILTER_ROW;

        var novoFiltro = [
            {
                "name": "CDPRATO",
                "operator": "=",
                "value": filterRow.CDPRATO
            },
            {
                "name": "DTINI",
                "operator": "=",
                "value": filterRow.PERIODO.start
            },
            {
                "name": "DTFIM",
                "operator": "=",
                "value": filterRow.PERIODO.end
            },
            {
                "name": "CDFILAUXPRAT",
                "operator": "=",
                "value": filterRow.CDFILAUXPRAT
            }
        ];

        widgetMain.dataSourceFilter = novoFiltro;
        widgetMain.reload().then(() => {
            ScreenService.closePopup();
            widgetMain.activate();
        });

        //args.e.preventDefault();
    };

    this.preencheStripe = function(args) {
        let widgetMain = templateManager.container.getWidget("widgetMain");
        let widgetStripe = templateManager.container.getWidget("widgetStripe");
        let cdprato = TcoUtil.formatString(widgetMain.appliedFilter.FILTER_ROW.CDPRATO, getLocalVar('CDPICTPRAT'));
        widgetStripe.getField("NMPRATO").setValue(cdprato + ' | ' + widgetMain.appliedFilter.FILTER_ROW.NMPRATO);
        let de = widgetMain.appliedFilter.FILTER_ROW.PERIODO.formattedStart;
        let ate = widgetMain.appliedFilter.FILTER_ROW.PERIODO.formattedEnd;
        widgetStripe.getField("DE").setValue(de);
        widgetStripe.getField("ATE").setValue(ate);
    };

    this.abreSelecaoPratoCardapioBase = function(args) {
        var widgetPratosSubstituicao = args.owner.widget.container.getWidget("widgetPratosSubstituicao");
        ScreenService.openPopup(widgetPratosSubstituicao);
    };

    this.selecionarPratoCardapioBase = function(args) {
        var widgetMain = args.owner.container.getWidget("widgetMain");
        var widgetPratosSubstituicao = args.owner;
        var cardapiosSelecionados = widgetMain.dataSource.getCheckedRows();

        widgetMain.dataSource.data.forEach((row) => {
            cardapiosSelecionados.forEach((cardSelecionado) => {
                if (row.NRCARDAPIOBASE === cardSelecionado.NRCARDAPIOBASE) {
                    row.changed = true;
                    row.__is_new = false;
                    row.CDFILAUXPRATDEST = widgetPratosSubstituicao.currentRow.CDFILAUXPRAT;
                    row.CDPRATODEST = widgetPratosSubstituicao.currentRow.CDPRATO;
                    row.NMPRATODEST = widgetPratosSubstituicao.currentRow.NMPRATO;
                }
            });
        });

        widgetMain.edit();
        widgetMain.unSelectAll();
        ScreenService.closePopup();
    };

    this.substituirPratoCardapioBase = function(args) {
        var widgetMain = args.owner.widget;

        ScreenService.confirmMessage(ScreenService.i18n('Os cardapios terão seus pratos substituídos. Confirma Alteração?'), 'question', function(success) {
            var cardapiosAlterados = widgetMain.dataSource.data.filter((row) => row.changed == true);
            Tco01007_utiliza_substitui_pratos_cardapio_base_service.substituirPratoCardapioBase(cardapiosAlterados).then((success) => {
                widgetMain.reload().then(() => {
                    ScreenService.notificationMessage(ScreenService.i18n("Pratos alterados com sucesso."), 'success', 3000);
                    widgetMain.list();
                });
            });
        });
    };
};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01007_utiliza_substitui_pratos_cardapio_base_controller', Tco01007_utiliza_substitui_pratos_cardapio_base_controller);
});

// FILE: ./assets/js/controllers/Tco01008_cardapios_complementares_controller.js
/* global angular, Configuration*/
var Tco01008_cardapios_complementares_controller = function(ScreenService, General, TcoUtil, Tco01008_cardapios_complementares_service, templateManager, MessageService) {

    this.carregaParametros = function(args) {
        if (!window.history.state) {
            window.location.href = window.location.href.replace("#/tco#tco01006_cardapios_complementares", "#/tco#tco01000_cardapio_base");
            window.location.reload();
        } else {
            var widgetMain = args.owner.getWidget("widgetMain");

            widgetMain.dataSourceFilter = window.history.state.configCardapioBaseParams;
            widgetMain.reload();
            widgetMain.activate();
        }
    };

    this.backCardBase = function() {
        window.history.back();
    };

    this.cancelaPopup = function(args) {
        args.owner.widget.unSelectAll();
        ScreenService.closePopup();
    };

    this.setStripeData = function() {
        var stripeWidget = templateManager.container.getWidget('stripeWidget');
        stripeWidget.currentRow = window.history.state.stripeData || {};
    };

    this.closeSwipeTiposPrato = function() {
        ScreenService.closeSwipe();
    };

    this.selecionarTipoPrato = function(args) {
        var selectedRow = args.selectedRow;
        var widgetMain = args.owner.container.getWidget("widgetMain");
        var widgetTiposDePrato = args.owner;

        widgetTiposDePrato.dataSource.data.forEach((row) => {
            if (row.NRCONFIGCARDBASE == selectedRow.NRCONFIGCARDBASE && row.NRCARDAPIOBASE == selectedRow.NRCARDAPIOBASE) {
                row.changed = true;
                row.TPPRATOINTGRCUSTO = true;
                row.NRCARDCOMPLCARDBASE = widgetMain.currentRow.NRCARDCOMPLCARDBASE;
            }
        });

        widgetTiposDePrato.edit();
    };

    this.selecionarTodosTiposPrato = function(args) {
        var widgetMain = args.owner.container.getWidget("widgetMain");
        var widgetTiposDePrato = args.owner;

        widgetTiposDePrato.dataSource.data.forEach((row) => {
            row.changed = true;
            row.TPPRATOINTGRCUSTO = true;
            row.NRCARDCOMPLCARDBASE = widgetMain.currentRow.NRCARDCOMPLCARDBASE;
        });

        widgetTiposDePrato.edit();
    };

    this.deselecionarTipoPrato = function(args) {
        var selectedRow = args.selectedRow;
        var widgetMain = args.owner.container.getWidget("widgetMain");
        var widgetTiposDePrato = args.owner;

        widgetTiposDePrato.dataSource.data.forEach((row) => {
            if (row.NRCONFIGCARDBASE == selectedRow.NRCONFIGCARDBASE && row.NRCARDAPIOBASE == selectedRow.NRCARDAPIOBASE) {
                row.changed = true;
                row.TPPRATOINTGRCUSTO = false;
                row.NRCARDCOMPLCARDBASE = widgetMain.currentRow.NRCARDCOMPLCARDBASE;
            }
        });

        widgetTiposDePrato.edit();
    };

    this.deselecionarTodosTiposPrato = function(args) {
        var widgetMain = args.owner.container.getWidget("widgetMain");
        var widgetTiposDePrato = args.owner;

        widgetTiposDePrato.dataSource.data.forEach((row) => {
            row.changed = true;
            row.TPPRATOINTGRCUSTO = false;
            row.NRCARDCOMPLCARDBASE = widgetMain.currentRow.NRCARDCOMPLCARDBASE;
        });

        widgetTiposDePrato.edit();
    };

    this.cancelaEdit = function(args) {
        var widgetTiposDePrato = args.owner.widget;
        widgetTiposDePrato.list();
        widgetTiposDePrato.reload();
    };

    this.abreTiposDePrato = function(args) {
        var widgetMain = args.owner;
        var widgetTiposDePrato = args.owner.container.getWidget("widgetTiposDePrato");

        var filter = [
            {
                "name": "NRCARDAPIOBASECOMPL",
                "operator": "=",
                "value": widgetMain.currentRow.NRCARDAPIOBASECOMPL
            },
            {
                "name": "NRCARDAPIOBASE",
                "operator": "=",
                "value": widgetMain.currentRow.NRCARDAPIOBASE
            },
            {
                "operator": "ORDER_BY",
                "name": [
                    {
                        "name": "CDTIPOPRAT",
                        "direction": "ASC"
                    }
                ]
            }
        ];

        widgetTiposDePrato.dataSourceFilter = filter;
        widgetTiposDePrato.reload().then((success) => ScreenService.openSwipe(widgetTiposDePrato));
    };

    this.salvaCnfCComplCardapioBase = function(args) {
        var widgetTiposDePrato = args.owner.widget;

        var rowsAlteradas = widgetTiposDePrato.dataSource.data.filter((row) => row.changed == true);

        Tco01008_cardapios_complementares_service.salvaCnfCComplCardapioBase(rowsAlteradas).then((success) => {
            widgetTiposDePrato.reload().then((success) => {
                ScreenService.notificationMessage(ScreenService.i18n("Alterações salvas com sucesso."), 'success', 3000);
                widgetTiposDePrato.reload().then(() => {
                    widgetTiposDePrato.list();
                });
            });
        });
    };

    this.marcaRowsCnfCComplCardapioBase = function(args) {
        var widgetTiposDePrato = args.owner;

        widgetTiposDePrato.dataSource.clearCheckedRows();
        widgetTiposDePrato.dataSource.data.forEach((row) => {
            if (row.NRCNFCCOMPLCARDBASE != null) {
                row.__isSelected = true;
                widgetTiposDePrato.dataSource.checkedRows.push(row);
            }
        });
    };

    this.novoCardapioBaseComplementado = function(args) {
        var widgetCardapiosComplementados = args.owner.widget.container.getWidget('widgetCardapiosComplementados');
        var widgetMain = args.owner.widget;

        widgetCardapiosComplementados.dataSourceFilter = widgetMain.dataSourceFilter;

        widgetCardapiosComplementados.reload().then(function() {
            widgetCardapiosComplementados.dataSource.clearCheckedRows();
            ScreenService.openPopup(widgetCardapiosComplementados);
        });
    };

    this.salvaCardapiosComplementados = function(args) {
        var widgetMain = args.owner.widget.container.getWidget('widgetMain');
        var widgetCardapiosComplementados = args.owner.widget;

        var dataToSave = [];

        widgetCardapiosComplementados.getCheckedRows().forEach((row) => {
            dataToSave.push({
                NRCARDAPIOBASECOMPL: row.NRCARDAPIOBASE,
                NRCARDAPIOBASE: widgetMain.dataSourceFilter[0].value,
                __is_new: true,
                changed: true
            });
        });

        Tco01008_cardapios_complementares_service.salvaCardapiosComplementados(dataToSave).then(
            (success) => {
                ScreenService.successNotification(ScreenService.i18n('Cardapio(s) Adicionado(s)'), 3000);
                widgetMain.reload().then(() => {
                    ScreenService.closePopup();
                });
            },
            (err) => ScreenService.showMessage(err.error));
    };

    this.removeCardapiosComplementados = function(args) {
        var widgetMain = args.owner.widget;
        var rowsToDelete = args.owner.widget.getCheckedRows();

        ScreenService.confirmMessage(ScreenService.i18n('Remover Cardapio(s) Complementado(s)?'), 'question', (success) => {
            Tco01008_cardapios_complementares_service.removeCardapiosComplementados(rowsToDelete).then(
                (success) => {
                    if (typeof success.messages !== "undefined" && success.messages.length > 0) {
                        MessageService.showMessageByCode(success.messages[0].message);
                    } else {
                        widgetMain.reload().then(() => {
                            widgetMain.unSelectAll();
                            MessageService.showMessageByCode(success.notifications[0].message);
                        });
                    }
                },
                (err) => ScreenService.showMessage(err.error));
        });
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01008_cardapios_complementares_controller', Tco01008_cardapios_complementares_controller);
});

// FILE: ./assets/js/controllers/Tco01009_analise_custos_controller.js
/* global angular, Configuration, getLocalVar*/
var Tco01009_analise_custos_controller = function(ScreenService, General, TcoUtil, Tco01009_analise_custos_service, templateManager, ZhKeyboardService, Tco01000_cardapio_base_service, Tco01004_novo_prato_elaboracao_cardapio_base_service, ZHPromise, eventEngine, MessageService) {

    this.carregaParametros = function(args) {
        if (!window.history.state) {
            window.location.href = window.location.href.replace("#/tco#tco01009_analise_custos", "#/tco#tco01000_cardapio_base");
            window.location.reload();
        } else {
            var widgetMain = args.owner.getWidget("widgetMain");
            var widgetInformacoesGerais = args.owner.getWidget("widgetInformacoesGerais");
            var widgetTipoPratos = args.owner.getWidget("widgetTipoPratos");
            widgetMain.elaboracaoCardapioBaseParams = window.history.state.elaboracaoCardapioBaseParams;

            TcoUtil.bindKeyboardEvents(this.addOrUpdate, false);

            var ref = this;

            widgetMain.openFilterShortcut = eventEngine.bindEvent(ZhKeyboardService, 'mod+lPressed', function(eventData) {
                return ref.abreFiltro(eventData);
            });

            Tco01009_analise_custos_service.findParamgeral().then(function(success) {
                if (typeof success.dataset.paramgeral !== "undefined" && success.dataset.paramgeral.length > 0) {
                    args.owner.CDPICTATIVCL = success.dataset.paramgeral[0].CDPICTATIVCL;

                    if (window.history.state.needFilter) {
                        if (widgetMain.elaboracaoCardapioBaseParams.IDCOMPLEMENTAR === 'S') {
                            widgetInformacoesGerais.newRow();
                            widgetTipoPratos.dataSource.clearCheckedRows();
                            widgetTipoPratos.dataSourceFilter = [];
                            widgetTipoPratos.dataSource.data = [];
                            habilitaFieldCardapioComplementado(args);
                            widgetInformacoesGerais.getField("DSATIVCLISEGMER").readOnly = true;

                            widgetInformacoesGerais.currentRow.DTCUSTO = moment().format('DD/MM/YYYY');

                            let fieldsNumericos = ["VRPEACRESCUSTO", "QTSEGSEX", "QTSABADO", "QTDOMFERI", "QTTURNOS"];
                            fieldsNumericos.forEach((fieldName) => {
                                widgetInformacoesGerais.getField(fieldName).setValue(0);
                            });

                            this.abreFiltro();

                            widgetMain.dataSourceFilter = [];
                            widgetMain.getAction('ANALISE').isVisible = false;
                            widgetMain.getAction('RELATORIOS').isVisible = false;
                        } else {
                            widgetInformacoesGerais.newRow();
                            widgetTipoPratos.dataSource.clearCheckedRows();
                            this.aplicaFiltroDataSourceDependente(args).then((callback) => {
                                if (window.history.state.needFilter) {
                                    widgetTipoPratos.dataSource.clearCheckedRows();
                                }

                                this.abreFiltro();

                                let fieldsNumericos = ["VRPEACRESCUSTO", "QTSEGSEX", "QTSABADO", "QTDOMFERI", "QTTURNOS"];
                                fieldsNumericos.forEach((fieldName) => {
                                    widgetInformacoesGerais.getField(fieldName).setValue(0);
                                });

                                widgetInformacoesGerais.currentRow.DTCUSTO = moment().format('DD/MM/YYYY');

                                widgetMain.dataSourceFilter = [];
                                widgetMain.getAction('ANALISE').isVisible = false;
                                widgetMain.getAction('RELATORIOS').isVisible = false;
                                widgetInformacoesGerais.getField('CDCARDAPIOBASE').isVisible = false;
                                widgetInformacoesGerais.currentRow.NRCARDAPIOBASECOMPL = null;
                            });

                        }

                    } else {

                        if (widgetMain.dataSourceFilter.length) {
                            widgetMain.getAction('ANALISE').isVisible = true;
                            widgetMain.getAction('RELATORIOS').isVisible = true;
                        } else {
                            widgetMain.getAction('ANALISE').isVisible = false;
                            widgetMain.getAction('RELATORIOS').isVisible = false;
                        }
                        widgetMain.activate();
                    }

                    widgetMain.reload();
                    setStripe();
                }

            }.bind(this));
        }
    };

    this.aplicaFiltroDataSourceDependente = function(args) {
        var widgetMain = templateManager.container.getWidget("widgetMain");
        var widgetInformacoesGerais = templateManager.container.getWidget("widgetInformacoesGerais");
        var widgetTipoPratos = templateManager.container.getWidget("widgetTipoPratos");

        var nrCardComplementado = widgetInformacoesGerais.currentRow.NRCARDAPIOBASECOMPL;

        var filterTipoPratos = [
            {
                "name": "NRCARDAPIOBASE",
                "operator": "=",
                "value": typeof nrCardComplementado !== "undefined" && nrCardComplementado != null && nrCardComplementado != "" ? nrCardComplementado : widgetMain.elaboracaoCardapioBaseParams.NRCARDAPIOBASE
            },
            {
                "name": "IDCLASSIFICACAO",
                "operator": "IN",
                "value": ["ALIMENTAR", "DESCARTAVEL"]
            },
            {
                "name": "NRCARDAPIOTRAB",
                "operator": "=",
                "value": null
            },
            {
                "operator": "ORDER_BY",
                "name": [
                    {
                        "name": "NRORDEM",
                        "direction": "ASC"
                    }
                ]
            }
        ];
        var filterArvativcli = [
            {
                "name": "NRCARDAPIOBASE",
                "operator": "=",
                "value": typeof nrCardComplementado !== "undefined" && nrCardComplementado != null && nrCardComplementado != "" ? nrCardComplementado : widgetMain.elaboracaoCardapioBaseParams.NRCARDAPIOBASE
            },
            {
                "operator": "ORDER_BY",
                "name": [
                    {
                        "name": "CDARVATIVCLI",
                        "direction": "ASC"
                    }
                ]
            }
        ];

        widgetTipoPratos.dataSourceFilter = filterTipoPratos;
        widgetInformacoesGerais.getField("DSATIVCLISEGMER").dataSourceFilter = filterArvativcli;

        var promisses = [
            widgetTipoPratos.reload(),
            widgetInformacoesGerais.getField("DSATIVCLISEGMER").reload()
        ];

        widgetInformacoesGerais.getField("DSATIVCLISEGMER").readOnly = false;

        return ZHPromise.all(promisses);

    };

    this.onChangeCardComplementado = function(args) {
        var widgetInformacoesGerais = templateManager.container.getWidget("widgetInformacoesGerais");
        var currentRow = widgetInformacoesGerais.currentRow;
        if (typeof currentRow[args.owner.field.name] !== "undefined" && currentRow[args.owner.field.name] != null && currentRow[args.owner.field.name] != "") {
            this.aplicaFiltroDataSourceDependente(args);
        } else {
            currentRow.DSATIVCLISEGMER = null;
            widgetInformacoesGerais.getField("DSATIVCLISEGMER").readOnly = true;
        }
    };

    var habilitaFieldCardapioComplementado = function(args) {
        var widgetMain = args.owner.getWidget("widgetMain");
        var widgetInformacoesGerais = args.owner.getWidget("widgetInformacoesGerais");

        var filter = [
            {
                "name": "NRCARDAPIOBASE",
                "operator": "=",
                "value": widgetMain.elaboracaoCardapioBaseParams.NRCARDAPIOBASE
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": widgetMain.elaboracaoCardapioBaseParams.NRORG
            }
        ];

        widgetInformacoesGerais.getField('CDCARDAPIOBASE').dataSourceFilter = filter;
        widgetInformacoesGerais.getField('CDCARDAPIOBASE').isVisible = true;
        widgetInformacoesGerais.getField('CDCARDAPIOBASE').reload();
    };

    this.setKeyboardEvents = function(args) {
        TcoUtil.bindKeyboardEvents(this.addOrUpdate, false);
    };

    function setStripe() {
        var stripeWidget = templateManager.container.getWidget('stripeWidget');
        stripeWidget.currentRow = window.history.state.elaboracaoCardapioBaseParams || {};
    }

    this.setCostInStripe = function() {
        let stripeWidget = templateManager.container.getWidget('stripeWidget');
        let widgetMain = templateManager.container.getWidget('widgetMain');
        let widgetInformacoesGerais = templateManager.container.getWidget('widgetInformacoesGerais');
        let params = {
            decimal: ',',
            thousands: '.',
            precision: 2
        };
        if (stripeWidget.currentRow !== {} && widgetMain.dataSource.data.length) {
            stripeWidget.currentRow.CUSTOTOTAL = !Util.isEmptyOrBlank(widgetMain.dataSource.data[0].CUSTOTOTAL) ? TcoUtil.maskCurrency(widgetMain.dataSource.data[0].CUSTOTOTAL.toString(), params) : 0;
            stripeWidget.currentRow.CUSTOALIMENTAR = !Util.isEmptyOrBlank(widgetMain.dataSource.data[0].CUSTOALIMENTAR) ? TcoUtil.maskCurrency(widgetMain.dataSource.data[0].CUSTOALIMENTAR.toString(), params) : 0;
            stripeWidget.currentRow.CUSTODESCARTAVEIS = !Util.isEmptyOrBlank(widgetMain.dataSource.data[0].CUSTODESCARTAVEIS) ? TcoUtil.maskCurrency(widgetMain.dataSource.data[0].CUSTODESCARTAVEIS.toString(), params) : 0;
            stripeWidget.currentRow.CUSTOLIMPEZA = !Util.isEmptyOrBlank(widgetMain.dataSource.data[0].CUSTOLIMPEZA) ? TcoUtil.maskCurrency(widgetMain.dataSource.data[0].CUSTOLIMPEZA.toString(), params) : 0;
            stripeWidget.currentRow.DTCUSTO = typeof widgetInformacoesGerais != "undefined" && typeof widgetInformacoesGerais.currentRow.DTCUSTO != "undefined" && widgetInformacoesGerais.currentRow.DTCUSTO ? widgetInformacoesGerais.currentRow.DTCUSTO : "";
        }
    };

    this.backCardBase = function() {
        if (window.history.state && window.history.state.filter && window.history.state.filter.NRCARDAPIOTRAB) {
            var row = {
                "NRCARDAPIOTRAB": window.history.state.filter.NRCARDAPIOTRAB
            };
            Tco01009_analise_custos_service.apagaCardapioTrab(row).then(function() {
                window.history.back();
            });
        } else {
            window.history.back();
        }
    };

    this.closeSwipeTiposPrato = function() {
        ScreenService.closeSwipe();
    };

    this.closePopupGramatura = function(args) {
        var widget = args.owner.widget;
        var mainTab = widget.container.getWidget('widgetFiltro');
        ScreenService.closePopup();

        var openTab = function() {
            if (mainTab && mainTab.openTab) {
                mainTab.openTab('widgetPorcionamentoGramatura');
            } else {
                setTimeout(function() {
                    openTab();
                }, 100);
            }
        };

        openTab();
    };

    this.abreFiltro = function(args) {
        var widgetInformacoesGerais = templateManager.container.getWidget("widgetInformacoesGerais");
        var widgetFiltro = templateManager.container.getWidget("widgetFiltro");
        ScreenService.openPopup(widgetFiltro);
        widgetInformacoesGerais.currentRow.DTREFERENCIA = moment().format('DD/MM/YYYY');
    };

    this.aplicaMascara = function(args) {
        var fieldDsativcli = args.owner.getField('DSATIVCLISEGMER');

        fieldDsativcli.fields.forEach(function(field) {
            if (field.name === "CDARVATIVCLI") {
                field.mask = {
                    "type": "fix",
                    "params": {
                        "mask": args.owner.container.CDPICTATIVCL
                    }
                };
            }
        });
    };

    this.aplicaFiltroPorcGramatura = function(args) {
        var widgetFiltro = args.owner.container.getWidget("widgetFiltro");
        var widgetPorcionamentoGramatura = args.owner;
        var widgetTipoPratos = args.owner.container.getWidget("widgetTipoPratos");
        var rows = widgetTipoPratos.dataSource.getCheckedRows();

        if (rows.length) {
            if (widgetTipoPratos.changed) {
                var configs = "";
                rows.forEach((row) => {
                    if (row.IDCLASSIFICACAO === 'ALIMENTAR') configs += "_" + row.NRCONFIGCARDBASE;
                });

                widgetPorcionamentoGramatura.dataSourceFilter = [
                    {
                        "name": "NRCONFIGCARDBASE",
                        "operator": "=",
                        "value": configs
                    }
                ];

                widgetPorcionamentoGramatura.reload().then(function() {
                    widgetPorcionamentoGramatura.customDataSource = angular.copy(widgetPorcionamentoGramatura.dataSource.data);
                });
            } else {
                widgetPorcionamentoGramatura.dataSource.data = angular.copy(widgetPorcionamentoGramatura.customDataSource);
            }
            widgetTipoPratos.changed = false;
        } else {
            widgetFiltro.openTab(widgetTipoPratos.name);
            ScreenService.showMessage(ScreenService.i18n("Informe Tipos de Prato."));
            return false;
        }
    };

    this.abreWidgetGramatura = function(args) {
        var widgetMain = args.owner.container.getWidget("widgetMain");
        var widgetPorcionamentoGramatura = args.owner;
        var widgetInformacoesGerais = args.owner.container.getWidget("widgetInformacoesGerais");
        var widgetGramatura = args.owner.container.getWidget("widgetGramatura");
        var fieldGramatura = widgetGramatura.getField('VRGRAMATURA');
        var fieldTipoGramatura = widgetGramatura.getField('IDTIPOGRAMATURA');

        var nrCardComplementado = widgetInformacoesGerais.currentRow.NRCARDAPIOBASECOMPL;

        var filtro = [
            {
                "name": "NRCARDAPIOBASE",
                "operator": "=",
                "value": typeof nrCardComplementado !== "undefined" && nrCardComplementado != null && nrCardComplementado != "" ? nrCardComplementado : widgetMain.elaboracaoCardapioBaseParams.NRCARDAPIOBASE
            },
            {
                "name": "CDGRTIPOPR",
                "operator": "=",
                "value": widgetPorcionamentoGramatura.currentRow.CDGRTIPOPR
            },
            {
                "name": "NRTIPOPUBLICO",
                "operator": "=",
                "value": widgetInformacoesGerais.currentRow.NRTIPOPUBLICO
            }
        ];

        fieldGramatura.dataSourceFilter = filtro;
        fieldGramatura.reload().then(function() {
            if (!fieldGramatura.dataSource.data.length) {
                fieldGramatura.readOnly = true;
                fieldGramatura.validations = {};

                fieldTipoGramatura.readOnly = true;
                fieldTipoGramatura.validations = {};
            } else {
                fieldGramatura.readOnly = false;
                fieldGramatura.validations = {
                    "required": true
                };

                fieldTipoGramatura.readOnly = false;
                fieldTipoGramatura.validations = {
                    "required": true
                };
            }
            if (widgetPorcionamentoGramatura.currentRow.NRPORCIONAMENTO) {
                widgetGramatura.currentRow = angular.copy(widgetPorcionamentoGramatura.currentRow);
            } else {
                widgetGramatura.newRow();
                widgetGramatura.currentRow.NRCARDAPIOBASE = widgetMain.elaboracaoCardapioBaseParams.NRCARDAPIOBASE;
                widgetGramatura.currentRow.CDGRTIPOPR = widgetPorcionamentoGramatura.currentRow.CDGRTIPOPR;
                widgetGramatura.currentRow.NMGRTIPOPR = widgetPorcionamentoGramatura.currentRow.NMGRTIPOPR;
                widgetGramatura.currentRow.IDTIPOGRAMATURA = fieldGramatura.dataSource.data.length ? "0" : "";
            }
            ScreenService.openPopup(widgetGramatura);
        });

    };

    this.salvaGramCardbaseSelecionada = function(args) {
        var widgetPorcionamentoGramatura = args.owner.widget.container.getWidget('widgetPorcionamentoGramatura');
        var widgetGramatura = args.owner.widget;
        var mainTab = widgetGramatura.container.getWidget('widgetFiltro');
        if (widgetGramatura.isValid()) {
            var gramPorc = {};
            gramPorc.NRGRAMATURACARDBASE = widgetGramatura.currentRow.NRGRAMATURACARDBASE;
            gramPorc.VRGRAMATURA = widgetGramatura.currentRow.VRGRAMATURA;
            gramPorc.NMPORCIONAMENTO = widgetGramatura.currentRow.NMPORCIONAMENTO;
            gramPorc.NRPORCIONAMENTO = widgetGramatura.currentRow.NRPORCIONAMENTO;
            gramPorc.NRCARDAPIOBASE = widgetGramatura.currentRow.NRCARDAPIOBASE;
            gramPorc.CDGRTIPOPR = widgetGramatura.currentRow.CDGRTIPOPR;
            gramPorc.NMGRTIPOPR = widgetGramatura.currentRow.NMGRTIPOPR;
            gramPorc.SGUNIDADE = widgetGramatura.currentRow.SGUNIDADE;
            gramPorc.IDTIPOGRAMATURA = widgetGramatura.currentRow.IDTIPOGRAMATURA;

            var indice = widgetPorcionamentoGramatura.dataSource.data.findIndex(function(row) {
                return row.CDGRTIPOPR === gramPorc.CDGRTIPOPR;
            });

            widgetPorcionamentoGramatura.dataSource.data[indice] = gramPorc;

            widgetPorcionamentoGramatura.customDataSource = angular.copy(widgetPorcionamentoGramatura.dataSource.data);
            var setfiltro = function(args) {
                var widgetMain = args.owner.widget.container.getWidget("widgetMain");
                var widgetPorcionamentoGramatura = args.owner.widget.container.getWidget('widgetPorcionamentoGramatura');
                var widgetInformacoesGerais = args.owner.widget.container.getWidget("widgetInformacoesGerais");
                var widgetGramatura = args.owner.widget.container.getWidget("widgetGramatura");
                var fieldGramatura = widgetGramatura.getField('VRGRAMATURA');
                var fieldTipoGramatura = widgetGramatura.getField('IDTIPOGRAMATURA');

                var nrCardComplementado = widgetInformacoesGerais.currentRow.NRCARDAPIOBASECOMPL;

                var filtro = [
                    {
                        "name": "NRCARDAPIOBASE",
                        "operator": "=",
                        "value": typeof nrCardComplementado !== "undefined" && nrCardComplementado != null && nrCardComplementado != "" ? nrCardComplementado : widgetMain.elaboracaoCardapioBaseParams.NRCARDAPIOBASE
                    },
                    {
                        "name": "CDGRTIPOPR",
                        "operator": "=",
                        "value": widgetPorcionamentoGramatura.currentRow.CDGRTIPOPR
                    },
                    {
                        "name": "NRTIPOPUBLICO",
                        "operator": "=",
                        "value": widgetInformacoesGerais.currentRow.NRTIPOPUBLICO
                    }
                ];

                fieldGramatura.dataSourceFilter = filtro;
                fieldGramatura.reload().then(function() {
                    if (!fieldGramatura.dataSource.data.length) {
                        fieldGramatura.readOnly = true;
                        fieldGramatura.validations = {};

                        fieldTipoGramatura.readOnly = true;
                        fieldTipoGramatura.validations = {};
                    } else {
                        fieldGramatura.readOnly = false;
                        fieldGramatura.validations = {
                            "required": true
                        };

                        fieldTipoGramatura.readOnly = false;
                        fieldTipoGramatura.validations = {
                            "required": true
                        };
                    }
                    if (widgetPorcionamentoGramatura.currentRow.NRPORCIONAMENTO) {
                        widgetGramatura.currentRow = angular.copy(widgetPorcionamentoGramatura.currentRow);
                    } else {
                        widgetGramatura.newRow();
                        widgetGramatura.currentRow.NRCARDAPIOBASE = widgetMain.elaboracaoCardapioBaseParams.NRCARDAPIOBASE;
                        widgetGramatura.currentRow.CDGRTIPOPR = widgetPorcionamentoGramatura.currentRow.CDGRTIPOPR;
                        widgetGramatura.currentRow.NMGRTIPOPR = widgetPorcionamentoGramatura.currentRow.NMGRTIPOPR;
                        widgetGramatura.currentRow.IDTIPOGRAMATURA = fieldGramatura.dataSource.data.length ? "0" : "";
                    }
                });

            };
            for (let i = 0; i < widgetPorcionamentoGramatura.dataSource.data.length; i++) {
                if (widgetPorcionamentoGramatura.dataSource.data[i].CDGRTIPOPR == widgetPorcionamentoGramatura.currentRow.CDGRTIPOPR) {
                    if (widgetPorcionamentoGramatura.dataSource.data[i + 1] != undefined) {
                        widgetGramatura.currentRow = widgetPorcionamentoGramatura.dataSource.data[i + 1];
                        widgetPorcionamentoGramatura.currentRow = widgetPorcionamentoGramatura.dataSource.data[i + 1];
                        setfiltro(args);
                        break;
                    } else {
                        ScreenService.closePopup();
                        ScreenService.showMessage(ScreenService.i18n('Todos os registros foram alterados com sucesso!'));
                    }
                }
            }
            mainTab.reload();

            var openTab = function() {
                if (mainTab && mainTab.openTab) {

                    mainTab.openTab("widgetPorcionamentoGramatura");
                } else {
                    setTimeout(function() {
                        openTab();
                    }, 100);
                }
            };
            openTab();

        }
    };

    this.aplicaFiltroAnaliseCusto = function(args) {
        var widgetMain = args.owner.widget.container.getWidget("widgetMain");
        var widgetFiltro = args.owner.widget;
        var widgetInformacoesGerais = args.owner.widget.container.getWidget("widgetInformacoesGerais");
        var widgetTipoPratos = args.owner.widget.container.getWidget("widgetTipoPratos");
        var widgetPorcionamentoGramatura = args.owner.widget.container.getWidget("widgetPorcionamentoGramatura");

        if (validateParams(widgetFiltro, widgetInformacoesGerais, widgetTipoPratos, widgetPorcionamentoGramatura)) {
            var row = constroiRowProcedureAnaliseCusto(widgetMain);
            Tco01009_analise_custos_service.executaProcedureAnaliseCusto(row).then((success) => {
                if (success.ERRO) {
                    TcoUtil.buildMessagesToShow(success.ERRO).then(function(mensagens) {
                        General.showErrorMessages(widgetFiltro, mensagens, ScreenService.i18n("Erro"));
                    });
                } else {
                    widgetMain.dataSourceFilter = [{
                        name: "NRCARDAPIOTRAB",
                        operator: "=",
                        value: success.RESPONSE.P_NRCARDAPIOTRAB
                    }];
                    ScreenService.closePopup();
                    if (window.history.state && window.history.state.filter && window.history.state.filter.NRCARDAPIOTRAB) {
                        var row = {
                            "NRCARDAPIOTRAB": window.history.state.filter.NRCARDAPIOTRAB
                        };
                        Tco01009_analise_custos_service.apagaCardapioTrab(row);
                    }
                    window.history.state = [];
                    window.history.state.filter = [];
                    window.history.state.filter.NRCARDAPIOTRAB = success.RESPONSE.P_NRCARDAPIOTRAB;
                    widgetMain.reload();
                    widgetMain.activate();
                    widgetMain.filtered = true;
                    widgetMain.getAction('ANALISE').isVisible = true;
                    widgetMain.getAction('RELATORIOS').isVisible = true;
                }
            }, (error) => {
                ScreenService.showMessage(error.data.error);
            });
        }
    };

    var validateParams = function(widgetFiltro, widgetInformacoesGerais, widgetTipoPratos, widgetPorcionamentoGramatura) {
        var isValid = false;
        if (widgetInformacoesGerais.isValid()) {
            isValid = true;
        } else {
            widgetFiltro.openTab(widgetInformacoesGerais.name);
            return false;
        }
        var hasValue = false;
        widgetInformacoesGerais.fields.forEach(function(field) {
            if (field.name.startsWith('QT') && parseFloat(field.value()) > 0) {
                hasValue = true;
            }
        });

        if (hasValue) {
            isValid = true;
        } else {
            widgetFiltro.openTab(widgetInformacoesGerais.name);
            ScreenService.showMessage(ScreenService.i18n("Não foram informados comensais estimados."));
            return false;
        }

        if (widgetTipoPratos.dataSource.getCheckedRows().length) {
            isValid = true;
        } else {
            widgetFiltro.openTab(widgetTipoPratos.name);
            ScreenService.showMessage(ScreenService.i18n("Informe Tipos de Prato."));
            return false;
        }

        var naoPreenchido = widgetPorcionamentoGramatura.dataSource.data.findIndex(function(row) {
            return row.NRPORCIONAMENTO === '' || row.NRPORCIONAMENTO === null;
        });

        if (naoPreenchido == -1 && !widgetTipoPratos.changed) {
            isValid = true;
        } else {
            ScreenService.showMessage(ScreenService.i18n("Informe Porcionamento e Gramatura."));
            widgetFiltro.openTab(widgetPorcionamentoGramatura.name);
            return false;
        }

        return isValid;
    };

    var constroiRowProcedureAnaliseCusto = function(widget) {
        var widgetMain = widget.container.getWidget('widgetMain');
        var widgetInformacoesGerais = widget.container.getWidget('widgetInformacoesGerais');
        var widgetTipoPratos = widget.container.getWidget('widgetTipoPratos');
        var widgetPorcionamentoGramatura = widget.container.getWidget('widgetPorcionamentoGramatura');

        var row = widgetInformacoesGerais.currentRow;
        var nrconfigcardbase = [];
        var gramaturacardbase = [];

        widgetTipoPratos.dataSource.getCheckedRows().forEach((row) => nrconfigcardbase.push(row.NRCONFIGCARDBASE));
        widgetPorcionamentoGramatura.dataSource.data.forEach((args) => {
            gramaturacardbase.push(args.CDGRTIPOPR + '|' + args.NRPORCIONAMENTO + '|' + (args.NRGRAMATURACARDBASE ? args.NRGRAMATURACARDBASE : '') + '|' + (args.IDTIPOGRAMATURA ? args.IDTIPOGRAMATURA : ''));
        });

        row.NRCARDAPIOBASE = widget.elaboracaoCardapioBaseParams.IDCOMPLEMENTAR == 'S' ? row.NRCARDAPIOBASECOMPL : widgetMain.elaboracaoCardapioBaseParams.NRCARDAPIOBASE;
        row.NRCARDBASECOMPLEMENTAR = widget.elaboracaoCardapioBaseParams.IDCOMPLEMENTAR == 'S' ? widgetMain.elaboracaoCardapioBaseParams.NRCARDAPIOBASE : null;
        row.IDCOMPLEMENTAR = widget.elaboracaoCardapioBaseParams.IDCOMPLEMENTAR;
        row.NRCONFIGCARDBASE = nrconfigcardbase;
        row.PORCGRAMCARDBASE = gramaturacardbase;
        row.VRPEACRESCUSTO = row.VRPEACRESCUSTO ? row.VRPEACRESCUSTO.replace(',', '.') : 0;
        row.QTSEGSEX = row.QTSEGSEX ? row.QTSEGSEX.replace(',', '.') : 0;
        row.QTSABADO = row.QTSABADO ? row.QTSABADO.replace(',', '.') : 0;
        row.QTDOMFERI = row.QTDOMFERI ? row.QTDOMFERI.replace(',', '.') : 0;
        row.QTTURNOS = row.QTTURNOS ? row.QTTURNOS.replace(',', '.') : 0;

        return row;
    };

    this.setChanged = function(args) {
        var widget = args.owner;
        widget.changed = true;
    };

    this.verificaPreenchimentoCardapio = function(args) {
        var widget = args.owner;
        var widgetFiltro = templateManager.container.getWidget("widgetFiltro");
        var nrcardapiopreenchido = widget.dataSourceFilter.filter((condition) => condition.name === 'NRCARDAPIOBASE');

        if (typeof nrcardapiopreenchido[0] !== "undefined") {
            return true;
        } else {
            MessageService.showMessageByCode("TCO01009_CARDAPIO_COMPLEMENTADO_VAZIO").then(() => {
                widgetFiltro.openTab("widgetInformacoesGerais");
                return false;
            });
        }
    };

    this.openAnaliseCusto = function(args) {
        var widget = args.owner.widget;
        var params = {
            "filter": {
                "NRCARDAPIOTRAB": window.history.state.filter.NRCARDAPIOTRAB
            },
            "currentRow": {
                "params": widget.currentRow
            }
        };
        setFilterFalse();

        TcoUtil.openWindowByHistory("#/tco#tco01204_analise_custo", params, "Análise de Custo");
    };

    this.openDadosDoPrato = function(args) {
        if (TcoUtil.isClickOnFocusedCell()) {
            var currentRow = args.owner.currentRow;
            if (currentRow.CDPRATO) {
                var cardBaseRow = window.history.state.elaboracaoCardapioBaseParams;

                var params = {
                    "label": {
                        "LABELDIA": currentRow.LABELDIA,
                        "LABELTPPRAT": currentRow.LABELTPPRAT,
                        "DSCOMPLEMENTO": currentRow.DSCOMPLEMENTO
                    },
                    "NRCARDAPIOTRAB": window.history.state.filter.NRCARDAPIOTRAB,
                    "dadosPrato": {
                        "NRCOMPCARDTRAB": currentRow.NRCOMPCARDTRAB,
                        "NRDIACARDTRAB": currentRow.NRDIACARDTRAB,
                        "NRCONFIGCARDTRAB": currentRow.NRCONFIGCARDTRAB,
                        "CDPRATO": currentRow.CDPRATO,
                        "VRPECONSCARDTRAB": currentRow.VRPECONSCARDTRAB
                    },
                    "filtro": {
                        "NRPADRAOQUALIDADE": cardBaseRow.NRPADRAOQUALIDADE,
                        "NRSERVICOPROPOSTA": cardBaseRow.NRSERVICOPROPOSTA,
                        "CDREGIAO": currentRow.CDREGIAO,
                        "NMREGIAO": currentRow.NMREGIAO,
                        "NRCARDAPIOTRAB": window.history.state.filter.NRCARDAPIOTRAB
                    }
                };
                Object.assign(params, window.history.state);
                setFilterFalse();

                TcoUtil.openWindowByHistory("#/tco#tco01011_dados_prato", params, "Dados do Prato");
            }
        } else {
            TcoUtil.setCurrentNavigationElement();
        }
    };

    this.openRelatorio = function(args) {
        var widget = args.owner.widget;
        var reportPopup = widget.container.getWidget('reportPopup');
        reportPopup.newRow();
        aplicaFiltroFieldsDia(reportPopup);
        aplicaFiltroFieldsTipoPrato(reportPopup);
        aplicaEstadoInicialFields(reportPopup);
        //aplicaMascaraPrato(reportPopup.getField('NMPRATOINI'));
        //aplicaMascaraPrato(reportPopup.getField('NMPRATOFIM'));
        ScreenService.openPopup(reportPopup);
    };

    var aplicaEstadoInicialFields = function(reportPopup) {
        var fieldsToChange = [
            'DSSEMANA_INICIAL',
            'DSSEMANA_FINAL',
            'NMTIPOPRAT',
            'NMPRATOINI',
            'NMPRATOFIM',
            'NMPRODUTOINI',
            'NMPRODUTOFIN'
        ];

        fieldsToChange.forEach((field) => {
            reportPopup.getField(field).readOnly = true;
            reportPopup.getField(field).setValue('');
            reportPopup.currentRow[field] = null;
        });
    };

    var aplicaFiltroFieldsTipoPrato = function(reportPopup) {
        var widgetMain = reportPopup.container.getWidget('widgetMain');
        var filter = [
            {
                name: "NRCARDAPIOBASE",
                operator: "=",
                value: widgetMain.elaboracaoCardapioBaseParams.NRCARDAPIOBASE
            }
        ];

        reportPopup.getField("NMTIPOPRAT").dataSourceFilter = filter.slice();
        reportPopup.getField("NMTIPOPRAT").reload();
    };

    var aplicaFiltroFieldsDia = function(reportPopup) {
        var widgetMain = reportPopup.container.getWidget('widgetMain');
        var filter = [
            {
                name: "NRCARDAPIOBASE",
                operator: "=",
                value: widgetMain.elaboracaoCardapioBaseParams.NRCARDAPIOBASE
            }
        ];

        reportPopup.getField("DSSEMANA_INICIAL").dataSourceFilter = filter.slice();
        reportPopup.getField("DSSEMANA_FINAL").dataSourceFilter = filter.slice();
        reportPopup.getField("DSSEMANA_INICIAL").reload();
        reportPopup.getField("DSSEMANA_FINAL").reload();
    };

    var aplicaMascaraPrato = function(fieldPrato) {
        Tco01004_novo_prato_elaboracao_cardapio_base_service.retornaMascaraNivelPrato().then(function(success) {
            if (typeof success.dataset.mascaraNivelPrato !== "undefined" && success.dataset.mascaraNivelPrato.hasOwnProperty('CDNVPRATO') && success.dataset.mascaraNivelPrato.hasOwnProperty('CDPICTPRAT')) {
                fieldPrato.fields.forEach(function(field) {
                    if (field.name === "CDPRATO") {
                        field.mask = {
                            "type": "fix",
                            "params": {
                                "mask": success.dataset.mascaraNivelPrato.CDPICTPRAT.replace(/9/g, '*')
                            }
                        };
                    }
                });
            }
        });
    };

    this.closePopup = function(args) {
        var widget = args.owner.widget;
        ScreenService.closePopup(true);
        widget.getParent().activate();
    };

    this.onChangeReportType = function(args) {
        var reportPopup = args.owner.field.widget;
        var fieldTipoRelatorio = args.owner.field;
        var fieldsToChange = [
            'DSSEMANA_INICIAL',
            'DSSEMANA_FINAL',
            'NMTIPOPRAT',
            'NMPRATOINI',
            'NMPRATOFIM',
            'NMPRODUTOINI',
            'NMPRODUTOFIN'
        ];

        fieldsToChange.forEach((field) => {
            if (reportPopup.currentRow[fieldTipoRelatorio.name] === 'TCO01004' || reportPopup.currentRow[fieldTipoRelatorio.name] === 'TCO01005' || reportPopup.currentRow[fieldTipoRelatorio.name] === 'TCO01006') {
                reportPopup.getField(field).readOnly = false;
                reportPopup.getField(field).setValue('');
                reportPopup.currentRow[field] = '';
            } else {
                reportPopup.getField(field).readOnly = true;
                reportPopup.getField(field).setValue('');
                reportPopup.currentRow[field] = '';
            }
        });

        templateManager.updateTemplate();
    };

    var setFilterFalse = function() {
        window.history.state.needFilter = false;
        window.history.replaceState(window.history.state, 'Análise de Custo');
    };

    this.makeReport = function(args) {
        var widget = args.owner.widget;
        var widgetMain = args.owner.widget.container.getWidget('widgetMain');

        widget.currentRow.NRSEMANA_INICIAL = widget.getField("DSSEMANA_INICIAL").value() == "" ? null : widget.currentRow.NRSEMANA_INICIAL;
        widget.currentRow.NRDIASEMANA_INICIAL = widget.getField("DSSEMANA_INICIAL").value() == "" ? null : widget.currentRow.NRDIASEMANA_INICIAL;
        widget.currentRow.NRSEMANA_FINAL = widget.getField("DSSEMANA_FINAL").value() == "" ? null : widget.currentRow.NRSEMANA_FINAL;
        widget.currentRow.NRDIASEMANA_FINAL = widget.getField("DSSEMANA_FINAL").value() == "" ? null : widget.currentRow.NRDIASEMANA_FINAL;

        if (widget.isValid()) {
            if (validaPeriodoDia(widget) && validaPeriodoPrat(widget) && validaPeriodoProd(widget)) {
                widget.currentRow.CDARVPRODINI = widget.getField("NMPRODUTOINI").value() == "" ? "0" : widget.currentRow.CDARVPRODINI;
                widget.currentRow.CDARVPRODFIN = widget.getField("NMPRODUTOFIN").value() == "" ? "zzzzzzzzzz" : widget.currentRow.CDARVPRODFIN;
                widget.currentRow.CDPRATOINI = widget.getField("NMPRATOINI").value() == "" ? "0" : widget.currentRow.CDPRATOINI;
                widget.currentRow.CDPRATOFIM = widget.getField("NMPRATOFIM").value() == "" ? "zzzzzzzzzz" : widget.currentRow.CDPRATOFIM;
                var row = constroiRowProcedureAnaliseCusto(widgetMain);

                widget.currentRow.NRCARDAPIOBASE = row.NRCARDAPIOBASE;
                widget.currentRow.NRCARDBASECOMPLEMENTAR = row.NRCARDBASECOMPLEMENTAR;
                widget.currentRow.IDCOMPLEMENTAR = row.IDCOMPLEMENTAR;
                widget.currentRow.NRCONFIGCARDBASE = row.NRCONFIGCARDBASE;
                widget.currentRow.REGIOES = widget.currentRow.NMREGIAO;
                widget.currentRow.NRCARDAPIOBASE_PROCEDURE = row.NRCARDAPIOBASE;
                widget.currentRow.NRCONFIGCARDBASE_PROCEDURE = row.NRCONFIGCARDBASE;
                widget.currentRow.PORCGRAMCARDBASE_PROCEDURE = row.PORCGRAMCARDBASE;
                widget.currentRow.NRATIVCLISEGMER = row.NRATIVCLISEGMER;
                widget.currentRow.NRTIPOPUBLICO = row.NRTIPOPUBLICO;
                widget.currentRow.VRPEACRESCUSTO_PROCEDURE = row.VRPEACRESCUSTO;
                widget.currentRow.VRPEACRESCUSTO = row.VRPEACRESCUSTO;
                widget.currentRow.QTSEGSEX_PROCEDURE = row.QTSEGSEX;
                widget.currentRow.QTSEGSEX = row.QTSEGSEX;
                widget.currentRow.QTSABADO_PROCEDURE = row.QTSABADO;
                widget.currentRow.QTSABADO = row.QTSABADO;
                widget.currentRow.QTDOMFERI_PROCEDURE = row.QTDOMFERI;
                widget.currentRow.QTDOMFERI = row.QTDOMFERI;
                widget.currentRow.QTTURNOS_PROCEDURE = row.QTTURNOS;
                widget.currentRow.QTTURNOS = row.QTTURNOS;
                widget.currentRow.DTCUSTO = row.DTCUSTO;

                widget.reportRow = widget.currentRow;
                var reportFormats = [
                    {
                        name: "pdf",
                        value: "PDF"
    			    },
                    {
                        name: "xlsx",
                        value: "xlsx"
    			    }
    			];
                General.openReportPopup(widget, this.generateReport, [widget], reportFormats);
            }
        }
    };

    var validaPeriodoProd = function(widget) {
        var valido = false;

        if (widget.getField("NMPRODUTOINI").value() != "" && widget.getField("NMPRODUTOFIN").value() == "") {
            MessageService.showMessageByCode("TCO01600_PRODUTO_FINAL_NAO_PREENCHIDO");
            valido = false;
        } else if (widget.getField("NMPRODUTOFIN").value() != "" && widget.getField("NMPRODUTOINI").value() == "") {
            MessageService.showMessageByCode("TCO01600_PRODUTO_INICIAL_NAO_PREENCHIDO");
            valido = false;
        } else {
            if (widget.currentRow.CDPRODUTOINI > widget.currentRow.CDPRODUTOFIN) {
                MessageService.showMessageByCode("TCO01600_PERIODO_PROD_INVALIDO");
                valido = false;
            } else {
                valido = true;
            }
        }

        return valido;
    };

    var validaPeriodoPrat = function(widget) {
        var valido = false;

        if (widget.getField("NMPRATOINI").value() != "" && widget.getField("NMPRATOFIM").value() == "") {
            MessageService.showMessageByCode("TCO01009_PRATO_FINAL_NAO_PREENCHIDO");
            valido = false;
        } else if (widget.getField("NMPRATOFIM").value() != "" && widget.getField("NMPRATOINI").value() == "") {
            MessageService.showMessageByCode("TCO01009_PRATO_INICIAL_NAO_PREENCHIDO");
            valido = false;
        } else {
            if (widget.currentRow.CDPRATOINI > widget.currentRow.CDPRATOFIM) {
                MessageService.showMessageByCode("TCO01009_PERIODO_PRAT_INVALIDO");
                valido = false;
            } else {
                valido = true;
            }
        }

        return valido;
    };

    var validaPeriodoDia = function(widget) {
        var valido = false;

        if (widget.getField("DSSEMANA_INICIAL").value() != "" && widget.getField("DSSEMANA_FINAL").value() == "") {
            MessageService.showMessageByCode("TCO01009_DIA_FINAL_NAO_PREENCHIDO");
            valido = false;
        } else if (widget.getField("DSSEMANA_FINAL").value() != "" && widget.getField("DSSEMANA_INICIAL").value() == "") {
            MessageService.showMessageByCode("TCO01009_DIA_INICIAL_NAO_PREENCHIDO");
            valido = false;
        } else {
            if (widget.currentRow.NRDIASEMANA_INICIAL > widget.currentRow.NRDIASEMANA_FINAL) {
                MessageService.showMessageByCode("TCO01600_PERIODO_DIA_INVALIDO");
                valido = false;
            } else {
                valido = true;
            }
        }

        return valido;
    };

    this.generateReport = function(widget) {
        var row = widget.currentRow;
        widget.reportRow.QR2TYPE = row.QR2TYPE;
        widget.reportRow.REPORT_NAME = row.REPORT_NAME;
        widget.reportRow.NRPADRAOQUALIDADE = window.history.state.elaboracaoCardapioBaseParams.NRPADRAOQUALIDADE;
        widget.reportRow.IDIOMA = getLocalVar('language');
        widget.reportRow.P_NRLISTA = widget.hasList ? widget.reportRow.P_NRLISTA : '';

        Tco01009_analise_custos_service.criaRelatorioAnaliseCusto(widget.reportRow).then(function(response) {
            if (response && response.dataset && response.dataset.ERROS) {
                TcoUtil.buildMessagesToShow(response.dataset.ERROS).then(function(mensagens) {
                    General.showErrorMessages(widget, mensagens, ScreenService.i18n("Erro"));
                });
            } else {
                if (response && response.messages) {
                    var report = response.messages.pop();
                    var reportOpened = window.open(report.message);
                    if (!reportOpened) {
                        ScreenService.showMessage(ScreenService.i18n('Bloqueador de Pop-up está ativado!'));
                    }
                }
                if (response && response.dataset && response.dataset.RESPONSE) {
                    widget.reportRow.P_NRLISTA = response.dataset.RESPONSE.NRLISTA;
                    widget.hasList = true;
                }
            }
        });
    };

    this.updateCustomElements = function(args) {
        var widget = args.owner.getWidget('widgetMain');
        TcoUtil.updateCustomElements(widget);
    };

    this.changeNeedList = function(args) {
        var widget = args.owner.field.widget;
        widget.hasList = false;
    };

    this.addOrUpdate = function(rowOrigem, rowDestino) {
        var rowToSave = angular.copy(rowDestino);
        var valueComensal = TcoUtil.parseFloat(rowOrigem.VRPECONSCARDTRAB);

        rowToSave.CDPRATO = rowOrigem.CDPRATO;
        rowToSave.CDFILAUXPRAT = rowOrigem.CDFILAUXPRAT;
        rowToSave.VRPECONSCARDTRAB = valueComensal;
        rowToSave.NRCARDAPIOBASE = window.history.state.elaboracaoCardapioBaseParams.NRCARDAPIOBASE;

        if (!Util.isEmptyOrBlank(rowDestino.CDPRATO)) {
            ScreenService.confirmMessage(ScreenService.i18n("Sobrepor prato?"), 'question', function(success) {
                confirmAddOrUpdate(rowToSave);
            }.bind(this));
        } else {
            confirmAddOrUpdate(rowToSave);
        }
    };

    let confirmAddOrUpdate = function(rowToSave) {
        Tco01000_cardapio_base_service.addOrUpdatePratoTrab([rowToSave]).then((response) => {
            ScreenService.successNotification("Prato alterado com sucesso.");
            templateManager.container.getWidget('widgetMain').reload();
        }, function(error) {
            ScreenService.showMessage(error.error);
        });
    };

    this.setDataSourceFilterPorcionamento = function(args) {
        let field = args.owner.field;
        let widget = field.widget;
        let widgetMain = templateManager.container.getWidget("widgetMain");
        let widgetInformacoesGerais = templateManager.container.getWidget("widgetInformacoesGerais");

        let nrCardComplementado = widgetInformacoesGerais.currentRow.NRCARDAPIOBASECOMPL;

        let filter = [
            {
                "name": "NRCARDAPIOBASE",
                "operator": "=",
                "value": typeof nrCardComplementado !== "undefined" && nrCardComplementado != null && nrCardComplementado != "" ? nrCardComplementado : widgetMain.elaboracaoCardapioBaseParams.NRCARDAPIOBASE
            },
            {
                "name": "CDGRTIPOPR",
                "operator": "=",
                "value": widget.currentRow.CDGRTIPOPR
            }
        ];

        field.dataSourceFilter = filter;
        field.reload();
    };

    /**
     * Função responsável por setar o dataSourceFilter da widgetIncidencias.
     * 
     * @params {Args} args - objeto args 
     **/

    this.setDataSourceFilterIncidencia = function(args) {
        var widget = args.owner;
        var cdfilial = window.history.state.elaboracaoCardapioBaseParams.CDFILIAL;
        var cdservfili = window.history.state.elaboracaoCardapioBaseParams.CDSERVFILI;
        var nrcardtrab = window.history.state.filter.NRCARDAPIOTRAB;
        widget.activate();
        widget.dataSourceFilter = [
            {
                "name": "CDFILIAL",
                "value": cdfilial,
                "operator": "="
			},
            {
                "name": "CDSERVFILI",
                "value": cdservfili,
                "operator": "="
			},
            {
                "name": "NRCARDAPIOTRAB",
                "value": nrcardtrab,
                "operator": "="
			}
		];
        widget.reload();
    };

    this.setDataSourceFilterProduto = function(args) {
        var widget = args.owner;
        var nrcardtrab = window.history.state.filter.NRCARDAPIOTRAB;
        widget.activate();
        widget.dataSourceFilter = [
            {
                "name": "NRCARDAPIOTRAB",
                "value": nrcardtrab,
                "operator": "="
			}
		];
        widget.reload();
    };

    this.openPopupDetails = function(args) {
        let widCard = templateManager.container.getWidget('widgetMain');
        if (widCard.filtered) {
            let filterWid = templateManager.container.getWidget('widgetFiltro');
            filterWid.label = "Detalhes";
            filterWid.getAction("CONFIRMA_FILTRO").isVisible = false;
            filterWid.getAction("CANCELA_FILTRO").isVisible = false;
            filterWid.getAction("CANCEL_DETAIL").isVisible = true;
            setDataSourceFilterTipoPrato(window.history.state.filter.NRCARDAPIOTRAB, true);
            filterWid.widgets.forEach(wid => {
                if (wid.name === "widgetInformacoesGerais") {
                    wid.view();
                } else if (wid.name === "widgetTipoPratos") {
                    wid.onlyView = true;
                    wid.getField('VRCUSTO').isVisible = true;
                } else if (wid.name === "widgetPorcionamentoGramatura") {
                    wid.onlyView = true;
                }
                wid.isVisible = true;
            });

            ScreenService.openPopup(filterWid);
        } else {
            ScreenService.showMessage("Operação bloqueada. Tela não possui filtro.");
        }
    };

    this.cancelDetail = function(args) {
        let filterWid = templateManager.container.getWidget('widgetFiltro');
        filterWid.label = "Filtro";
        filterWid.getAction("CONFIRMA_FILTRO").isVisible = true;
        filterWid.getAction("CANCELA_FILTRO").isVisible = true;
        filterWid.getAction("CANCEL_DETAIL").isVisible = false;
        setDataSourceFilterTipoPrato(window.history.state.filter.NRCARDAPIOTRAB, false);
        filterWid.widgets.forEach(wid => {
            if (wid.name === "widgetInformacoesGerais") {
                wid.list();
            } else if (wid.name === "widgetTipoPratos") {
                wid.onlyView = false;
                wid.getField('VRCUSTO').isVisible = false;
            } else if (wid.name === "widgetPorcionamentoGramatura") {
                wid.onlyView = false;
            } else {
                wid.isVisible = false;
            }
        });
        ScreenService.closePopup(true);
        filterWid.getParent().activate();
    };

    this.preventDefault = function(args) {
        let widget = args.owner;
        if (widget.onlyView)
            args.e.preventDefault();
    };

    let setDataSourceFilterTipoPrato = function(nrcardtrab, setFilter) {
        let widgetTipoPrato = templateManager.container.getWidget('widgetTipoPratos');
        if (setFilter) {
            widgetTipoPrato.dataSourceFilter.forEach(filter => {
                if (filter.name === "NRCARDAPIOTRAB") {
                    filter.value = nrcardtrab;
                }
            });
        } else {
            widgetTipoPrato.dataSourceFilter.forEach(filter => {
                if (filter.name === "NRCARDAPIOTRAB") {
                    filter.value = null;
                }
            });
        }
        widgetTipoPrato.reload();
    };

    this.getWeekend = function(row) {
        if (row.NRDIASEMANA == 1 || row.NRDIASEMANA == 7) {
            return true;
        } else if (row.__isYAxisProperty) {
            return row.label.indexOf('Dom') !== -1 || row.label.indexOf('Sab') !== -1;
        } else {
            return false;
        }
    };

    this.getCelCost = function(row) {
        return (row.__isYAxisProperty && row.label === 'Custo');
    };

    this.abrePrevisaoConsumo = function(args) {
        var widgetMain = args.owner.widget.container.getWidget('widgetMain');
        if (widgetMain.filtered) {
            var widgetInformacoesGerais = args.owner.widget.container.getWidget('widgetInformacoesGerais');
            var widgetTipoPratos = args.owner.widget.container.getWidget('widgetTipoPratos');
            var widgetPorcionamentoGramatura = args.owner.widget.container.getWidget('widgetPorcionamentoGramatura');

            Object.assign(widgetMain.elaboracaoCardapioBaseParams, widgetInformacoesGerais.currentRow);

            widgetMain.elaboracaoCardapioBaseParams.NRCARDAPIOTRAB = widgetMain.dataSource.data[0].NRCARDAPIOTRAB;
            widgetMain.elaboracaoCardapioBaseParams.TIPOPRATOSSELECIONADOS = widgetTipoPratos.getCheckedRows();
            widgetMain.elaboracaoCardapioBaseParams.PORCGRAMATURA = widgetPorcionamentoGramatura.dataSource.data;

            var params = {
                elaboracaoCardapioBaseParams: widgetMain.elaboracaoCardapioBaseParams,
                filter: {
                    NRCARDAPIOTRAB: window.history.state.filter.NRCARDAPIOTRAB
                }
            };
            setFilterFalse();
            TcoUtil.openWindowByHistory('#/tco#tco01600_previsao_consumo', params);
        } else {
            MessageService.showMessageByCode("TCO01009_TELA_SEM_FILTRO");
        }
    };

    this.montaLabelsSemanaDia = function(field) {

        MessageService.getMessageByCode('TCO01001_DIA').then(function(mensagemDia) {
            field.dataSource.data.forEach(function(row) {
                if (!row.mounted) {
                    MessageService.getMessageByCode(row.DSSEMANA.slice(1, row.DSSEMANA.length)).then(function(labelDia) {
                        row.DSSEMANA = ScreenService.i18n(mensagemDia, {
                            NRSEMANA: row.NRSEMANA,
                            LABELDIA: labelDia
                        });
                    });
                }
            });
        });
    };

    this.beforeCloseContainer = function(args) {
        let widgetMain = templateManager.container.getWidget('widgetMain');
        if (widgetMain.openFilterShortcut >= 0)
            eventEngine.unbindEvent(ZhKeyboardService, 'mod+lPressed', widgetMain.openFilterShortcut);
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01009_analise_custos_controller', Tco01009_analise_custos_controller);
});

// FILE: ./assets/js/controllers/Tco01010_revisar_receita_card_base.js
var Tco01010_revisar_receita_card_base = function(ScreenService, General, Tco01004_novo_prato_elaboracao_cardapio_base_service, Tco01010_revisar_receita_card_base_service, templateManager, ZHPromise, TcoUtil, maskEngine, ZhUtil, MessageService) {
    /*global Util, getLocalVar, Configuration, angular*/

    var self = this;

    this.carregaParametrosReview = function(args) {
        if (!window.history.state) {
            window.location.href = window.location.href.replace("#/tco#tco01206_revisar_receita", "#/tco#tco01200_cardapio_da_proposta");
            window.location.reload();
        } else {
            this.defineVisibilityWidgets(window.history.state.dadosPrato);
            this.defineVisibilityActions(window.history.state.dadosPrato);
            this.defineVisibilityData(window.history.state.dadosPrato);
            this.getValuesStripe(window.history.state.dadosPrato).then(response => {
                if (!Util.isEmptyOrBlank(response.dataset.valuesStripe)) {
                    let paramStripe = response.dataset.valuesStripe[0];
                    this.setValueStripe(paramStripe);
                }
            });
            this.setDefaulValueRegion(window.history.state.dadosPrato);
        }
    };

    this.setDefaulValueRegion = function(dadosPrato) {
        let widgetReviewTabbed = templateManager.container.getWidget('widgetReviewTabbed');
        if (!Util.isEmptyOrBlank(dadosPrato.CDREGIAO)) {
            widgetReviewTabbed.getField('CDREGIAO').value(dadosPrato.CDREGIAO);
            widgetReviewTabbed.getField('NMREGIAO').value(dadosPrato.NMREGIAO);
        }
    };

    this.defineVisibilityData = function(dadosPrato) {
        var widget = templateManager.container.getWidget('tabImportacao');
        var fieldTipo = widget.getField('TIPO');
        if (dadosPrato.NRCARDAPIOTRAB) {
            let hasRecProp = fieldTipo.dataSource.data.filter(row => {
                return row.value === 'C';
            });
            if (Util.isEmptyOrBlank(hasRecProp)) {
                fieldTipo.dataSource.data.push({
                    "value": "C",
                    "name": "Proposta"
                });
            }
        } else {
            let hasRecProp = fieldTipo.dataSource.data.filter(row => {
                return row.value === 'C';
            });

            if (!Util.isEmptyOrBlank(hasRecProp)) {
                fieldTipo.dataSource.data.pop();
            }
        }
    };

    this.getValuesStripe = function(dadosPrato) {
        let params = {
            CDPRATO: dadosPrato.CDPRATO,
            NMPRATO: dadosPrato.NMPRATO,
            CDFILAUXPRAT: dadosPrato.CDFILAUXPRAT,
            NRSERVICOPROPOSTA: dadosPrato.NRSERVICOPROPOSTA,
            NRPADRAOQUALIDADE: dadosPrato.NRPADRAOQUALIDADE,
            CDREGIAO: dadosPrato.CDREGIAO,
            NRCARDAPIOTRAB: null,
            CDSERVFILI: dadosPrato.CDSERVFILI
        };
        return Tco01010_revisar_receita_card_base_service.getValuesStripe(params);
    };

    this.setValueStripe = function(paramStripe) {
        let stripeWidget = templateManager.container.getWidget("stripeWidget");
        TcoUtil.getMaskPrato().then(function(response) {
            var mask = {
                "type": "fix",
                "params": {
                    "mask": response.dataset.getMaskPrato.CDPICTPRAT.replace(/[a,9]/g, '*')
                }
            };
            paramStripe.CDPRATO = Util.applyMask(paramStripe.CDPRATO, mask, maskEngine, ZhUtil);
            stripeWidget.currentRow.CDPRATO = paramStripe.CDPRATO + ' - ' + paramStripe.NMPRATO;
        });

        stripeWidget.currentRow.CDFILIALPARAMTCO = paramStripe.CDFILIAL;
        stripeWidget.currentRow.CDFILIAL = paramStripe.CDFILIAL + ' - ' + paramStripe.NMFILIAL;
        stripeWidget.currentRow.CDSERVFILI = paramStripe.CDSERVFILI + ' - ' + paramStripe.NMSERVICO;
        if (paramStripe.CDREGIAO) {
            stripeWidget.currentRow.CDREGIAO = paramStripe.CDREGIAO + ' - ' + paramStripe.NMREGIAO;
        } else {
            stripeWidget.currentRow.CDREGIAO = '';
        }
        stripeWidget.currentRow.CDPADRAOQUALIDADE = paramStripe.CDPADRAOQUALIDADE + ' - ' + paramStripe.NMPADRAOQUALIDADE;
        stripeWidget.container.QTDRECEITA = paramStripe.QTDRECEITA;
        stripeWidget.container.CDSERVFILI = paramStripe.CDSERVFILI;

        switch (paramStripe.RECEUTIL) {
            case "F":
                stripeWidget.currentRow.RECEUTIL = "Local";
                break;
            case "P":
                stripeWidget.currentRow.RECEUTIL = "Padrão";
                break;
            case "S":
                stripeWidget.currentRow.RECEUTIL = "por Serviço";
                break;
            case "C":
                stripeWidget.currentRow.RECEUTIL = "da Proposta";
                break;
            case "Q":
                stripeWidget.currentRow.RECEUTIL = "Padrão de Qualidade";
                break;
            case "R":
                stripeWidget.currentRow.RECEUTIL = "por Região";
                break;
        }
    };

    this.getRecPad = function() {
        let widget = templateManager.container.getWidget("widRecPad");

        let filter = [{
                name: "CDPRATO",
                operator: "=",
                value: window.history.state.dadosPrato.CDPRATO
            },
            {
                name: "CDFILAUXPRAT",
                operator: "=",
                value: window.history.state.dadosPrato.CDFILAUXPRAT
            }
        ];

        widget.dataSourceFilter = filter;
        widget.reload();
    };

    this.getRecLocal = function() {
        let widget = templateManager.container.getWidget("widRecLocal");

        let filter = [{
                name: "CDPRATO",
                operator: "=",
                value: window.history.state.dadosPrato.CDPRATO
            },
            {
                name: "CDFILAUXPRAT",
                operator: "=",
                value: window.history.state.dadosPrato.CDFILAUXPRAT
            },
            {
                name: "NRORG",
                operator: "=",
                value: getLocalVar('NRORG')
            }
        ];

        widget.dataSourceFilter = filter;
        widget.reload();
    };

    this.getRecServ = function() {
        let widget = templateManager.container.getWidget("widRecServ");

        let filter = [{
                name: "CDPRATO",
                operator: "=",
                value: window.history.state.dadosPrato.CDPRATO
            },
            {
                name: "CDFILAUXPRAT",
                operator: "=",
                value: window.history.state.dadosPrato.CDFILAUXPRAT
            },
            {
                name: "NRORG",
                operator: "=",
                value: getLocalVar('NRORG')
            },
            {
                name: "CDSERVFILI",
                operator: "=",
                value: window.history.state.dadosPrato.CDSERVFILI
            }
        ];

        widget.dataSourceFilter = filter;
        widget.reload();
    };

    this.getRecRegiao = function(valueRegiao = null) {
        let widget = templateManager.container.getWidget("widRecRegiao");

        let filter = [{
                name: "CDPRATO",
                operator: "=",
                value: window.history.state.dadosPrato.CDPRATO
            },
            {
                name: "CDFILAUXPRAT",
                operator: "=",
                value: window.history.state.dadosPrato.CDFILAUXPRAT
            },
            {
                name: "CDREGIAO",
                operator: "=",
                value: window.history.state.dadosPrato.CDREGIAO
            },
            {
                name: "NRPADRAOQUALIDADE",
                operator: "=",
                value: window.history.state.dadosPrato.NRPADRAOQUALIDADE
            }
        ];

        if (valueRegiao) {
            filter[2].value = valueRegiao;
            window.history.state.dadosPrato.CDREGIAO = valueRegiao;
        } else {
            filter[2].value = '';
        }

        widget.dataSourceFilter = filter;
        widget.reload();
    };

    this.getRecPadQua = function() {
        let widget = templateManager.container.getWidget("widRecPadQua");

        let filter = [{
                name: "CDPRATO",
                operator: "=",
                value: window.history.state.dadosPrato.CDPRATO
            },
            {
                name: "CDFILAUXPRAT",
                operator: "=",
                value: window.history.state.dadosPrato.CDFILAUXPRAT
            },
            {
                name: "NRPADRAOQUALIDADE",
                operator: "=",
                value: window.history.state.dadosPrato.NRPADRAOQUALIDADE
            }
        ];

        widget.dataSourceFilter = filter;
        widget.reload();
    };

    this.getRecCardProp = function() {
        let widget = templateManager.container.getWidget("widRevRecProp");

        let filter = [{
                name: "CDPRATO",
                operator: "=",
                value: window.history.state.dadosPrato.CDPRATO
            },
            {
                name: "CDFILAUXPRAT",
                operator: "=",
                value: window.history.state.dadosPrato.CDFILAUXPRAT
            },
            {
                name: "NRCARDAPIOTRAB",
                operator: "=",
                value: window.history.state.dadosPrato.NRCARDAPIOTRAB
            }
        ];

        widget.dataSourceFilter = filter;
        widget.reload();
    };

    this.defineVisibilityWidgets = function(filterParams) {
        let widRecPadQua = templateManager.container.getWidget("widRecPadQua");
        let widRevRecProp = templateManager.container.getWidget("widRevRecProp");
        widRecPadQua.isVisible = (filterParams.NRPADRAOQUALIDADE);
        widRevRecProp.isVisible = (filterParams.NRCARDAPIOTRAB);
        if (window.history.state.veioDe && window.history.state.veioDe == "Tco01200") {
            templateManager.container.getWidget("widRecPad").isVisible = false;
            templateManager.container.getWidget("widRecLocal").isVisible = false;
            templateManager.container.getWidget("widRecServ").isVisible = false;
            templateManager.container.getWidget("widRecRegiao").isVisible = false;
            widRecPadQua.isVisible = false;
            widRevRecProp.isVisible = true;
        } else {
            templateManager.container.getWidget("widRecPad").isVisible = true;
            templateManager.container.getWidget("widRecLocal").isVisible = true;
            templateManager.container.getWidget("widRecServ").isVisible = true;
            templateManager.container.getWidget("widRecRegiao").isVisible = true;
            widRecPadQua.isVisible = (filterParams.NRPADRAOQUALIDADE);
            widRevRecProp.isVisible = (filterParams.NRCARDAPIOTRAB);
        }
    };
    this.setActiveWidgets = function() {
        let widgets = templateManager.container.getWidget("widgetReviewTabbed").widgets;
        let contador = 0;
        widgets.some(function(widget) {
            if (widget.isVisible) {
                widget.activate();
                contador++;
            }
            return contador > 0;
        });
    };

    this.defineVisibilityActions = function(filterParams) {
        let widRecServ = templateManager.container.getWidget("widRecServ");
        widRecServ.getAction("REMOVE").isVisible = (filterParams.CDSERVFILI);
        widRecServ.getAction("NEW").isVisible = (filterParams.CDSERVFILI);
        widRecServ.getAction("EDIT").isVisible = (filterParams.CDSERVFILI);
        widRecServ.getAction("IMPORT").isVisible = (filterParams.CDSERVFILI);
    };

    this.applyFilterRegion = function(args) {
        let cdRegiao = args.owner.field.widget.getField('CDREGIAO').value();
        let nmRegiao = args.owner.field.widget.getField('NMREGIAO').value();
        let widgetStripe = templateManager.container.getWidget('stripeWidget');
        if (!Util.isEmptyOrBlank(cdRegiao)) {
            this.getRecRegiao(cdRegiao);
            widgetStripe.currentRow.CDREGIAO = cdRegiao + ' - ' + nmRegiao;
        } else {
            this.getRecRegiao();
            widgetStripe.currentRow.CDREGIAO = '';
        }

        this.defineVisibilityActionsRegiao();
    };

    this.defineVisibilityStripe = function(args) {
        let widget = args.owner;
        let stripeWidget = templateManager.container.getWidget("stripeWidget");

        let fieldsStripe = {
            "widRecPad": [
                "CDPRATO",
                "RECEUTIL",
                "IDARVORE"
            ],
            "widRecLocal": [
                "CDPRATO",
                "RECEUTIL",
                "IDARVORE",
                "CDFILIAL"
            ],
            "widRecServ": [
                "CDPRATO",
                "RECEUTIL",
                "IDARVORE",
                "CDFILIAL",
                "CDSERVFILI"
            ],
            "widRecRegiao": [
                "CDPRATO",
                "RECEUTIL",
                "IDARVORE",
                "CDREGIAO",
                "CDPADRAOQUALIDADE"
            ],
            "widRecPadQua": [
                "CDPRATO",
                "RECEUTIL",
                "IDARVORE",
                "CDPADRAOQUALIDADE"
            ],
            "widRevRecProp": [
                "CDPRATO",
                "RECEUTIL",
                "IDARVORE",
                "NRCARDAPIOTRAB"
            ]
        };

        stripeWidget.fields.forEach(function(field) {
            if (fieldsStripe[widget.name].includes(field.name)) {
                field.isVisible = true;
            } else {
                field.isVisible = false;
            }
        });
    };

    /**
     * Volta para ultima tela.
     * Verifica se tem produtos que ainda não foram
     * revisados.
     **/

    this.backDadosPrato = function() {
        window.history.back();
    };

    /**
     * Fecha o swipe e volta para o grid.
     **/

    this.backGrid = function(args) {
        let widget = args.owner.widget;
        ScreenService.closeSwipe();
        widget.activate();
    };

    /**
     * Monta o data do dataSource local, antes de
     * revisar a receita.
     * 
     * @params {Object} currentRow - currentRow da widget de revisão.
     **/

    let mountData = function(widget, currentRow) {
        var cdregiao = templateManager.container.getWidget('widgetReviewTabbed').getField('CDREGIAO').value();
        currentRow.NRCARDAPIOTRAB = window.history.state.dadosPrato.NRCARDAPIOTRAB;
        currentRow.CDPRATO = window.history.state.dadosPrato.CDPRATO;
        currentRow.CDFILAUXPRAT = window.history.state.dadosPrato.CDFILAUXPRAT;
        currentRow.NRRECEPRATCARDTRAB = (currentRow.NRRECEPRATCARDTRAB) ? currentRow.NRRECEPRATCARDTRAB : null;
        currentRow.QTDBRUTA = currentRow.QTDBRUTA = TcoUtil.parseFloat(currentRow.QTDBRUTA);
        currentRow.QTPRODUTO = currentRow.QTPRODUTO = TcoUtil.parseFloat(currentRow.QTPRODUTO);
        currentRow.VRAPCOCCAO = currentRow.VRAPCOCCAO = TcoUtil.parseFloat(currentRow.VRAPCOCCAO);
        currentRow.VRAPLIMPEZA = currentRow.VRAPLIMPEZA = TcoUtil.parseFloat(currentRow.VRAPLIMPEZA);
        currentRow.VRGANHOCOCCAO = currentRow.VRGANHOCOCCAO;
        currentRow.VRPERDALIMPEZA = currentRow.VRPERDALIMPEZA;
        currentRow.CDREGIAO = !Util.isEmptyOrBlank(cdregiao) ? cdregiao : null;
        currentRow.CDPADRAOQUALIDADE = window.history.state.dadosPrato.CDPADRAOQUALIDADE;
        currentRow.CDSERVFILI = window.history.state.dadosPrato.CDSERVFILI;
        currentRow.NRPADRAOQUALIDADE = window.history.state.dadosPrato.NRPADRAOQUALIDADE;
        currentRow.NRSERVICOPROPOSTA = window.history.state.dadosPrato.NRSERVICOPROPOSTA;
        currentRow.changed = true;
        currentRow.TIPORECE = getTipoRece(widget);
        return currentRow;
    };

    let getTipoRece = function(widget) {

        let tipoRece = {
            "widRecPad": 'P',
            "widRecLocal": 'F',
            "widRecServ": 'S',
            "widRecRegiao": 'R',
            "widRecPadQua": 'Q',
            "widRevRecProp": 'C'
        };
        return tipoRece[widget.name];
    };

    /**
     * Adicona os produtos no dataSource de maneira local.
     * Valida se o produto já está adicionado, caso esteja, não deixa
     * prosseguir.
     * 
     * @params {Args} args - Object args.
     **/

    this.saveProduct = function(args) {
        let widget = args.owner.widget;
        let dataSourceToSave = widget.dataSourceToSave;
        if (widget.isValid()) {
            let dataSource = widget.dataSource.data;
            if (productValid(widget.currentRow, widget.dataSource)) {
                if (validPercentages(widget.currentRow)) {
                    let data = mountData(widget, widget.currentRow);
                    Tco01010_revisar_receita_card_base_service.saveProduct(data).then(function(response) {
                        widget.view();
                        widget.currentRow = response.dataset[dataSourceToSave][0];
                        widget.currentRow.__is_new = false;
                        ScreenService.notificationMessage(ScreenService.i18n("Adicionado com sucesso!"), 'sucess');
                    }, error => {
                        ScreenService.showMessage(error.error);
                    });
                }
            } else {
                ScreenService.showMessage("Impossível adicionar. Produto já cadastrado.");
            }
        }
    };

    let validPercentages = function(currentRow) {
        if (currentRow.VRPERDALIMPEZA > 99 || currentRow.VRPERDALIMPEZA < 0) {
            ScreenService.showMessage("Percentual de Perda Limpeza inválido.");
            return false;
        } else if (currentRow.VRAPLIMPEZA > 100 || currentRow.VRAPLIMPEZA <= 0) {
            ScreenService.showMessage("Percentual de Ap. Limpeza inválido.");
            return false;
        } else {
            return true;
        }
    };

    /**
     * Verifica se já existe o produto que está sendo adicionado.
     * 
     * @params {Object} currentRow - row que está para ser inserida.
     * @params {Object} dataSource - dataSource da tela de revisão.
     **/

    let productValid = function(currentRow, dataSource) {
        let result = dataSource.data.filter(function(data) {
            return data.CDPRODUTO === currentRow.CDPRODUTO;
        });
        return (result.length == 1);
    };

    /**
     * Faz o cálculo de cocção.
     * VRPEFATCOCCARDTRAB - Ganho Cocção (%)
     * VRAPCOCCAO - Ap. Cocção (%)
     * 
     * @params {Args} args - Object args.
     **/

    this.calcCoccao = function(args) {
        let field = args.owner.field;
        let widget = field.widget;
        if (field.name === "VRGANHOCOCCAO") {
            if (Util.isEmptyOrBlank(field.value())) field.value(0);
            widget.getField('VRAPCOCCAO').value(100 + parseFloat(field.value()));
        } else {
            if (Util.isEmptyOrBlank(field.value())) field.value(100);
            widget.getField('VRGANHOCOCCAO').value(parseFloat(field.value()) - 100);
        }
        this.calcQtBruta(args);
    };

    /**
     * Faz o cálculo de limpeza.
     * VRPEFATCORCARDTRAB - Perda Limpeza (%)
     * VRAPLIMPEZA - Ap. Limpeza (%)
     * 
     * @params {Args} args - Object args.
     **/

    this.calcLimpeza = function(args) {
        let field = args.owner.field;
        let widget = field.widget;
        if (field.name === "VRPERDALIMPEZA") {
            widget.getField('VRAPLIMPEZA').value(100 - parseFloat(field.value()));
        } else {
            widget.getField('VRPERDALIMPEZA').value(100 - parseFloat(field.value()));
        }
        this.calcQtBruta(args);
    };

    this.calcQtBruta = function(args) {
        let wid = args.owner.field.widget;
        let vrApCocc = TcoUtil.parseFloat(wid.getField('VRAPCOCCAO').value());
        let vrApLimpeza = TcoUtil.parseFloat(wid.getField('VRAPLIMPEZA').value());
        let qtdBruta = wid.getField('QTDBRUTA');
        let qtdLiq = TcoUtil.parseFloat(wid.getField('QTPRODUTO').value());
        // ap cocc
        let qtdLimpa = (qtdLiq / vrApCocc) * 100;
        // ap limpeza
        let newQtd = (qtdLimpa / vrApLimpeza) * 100;
        qtdBruta.value(TcoUtil.parseFloat(newQtd));
    };

    /**
     * Seta o valor de quantidade bruta.
     * 
     * @params {Args} args - Object args.
     **/

    this.setQtBruta = function(args) {
        let fieldQt = args.owner.field;
        let qtBruta = fieldQt.widget.getField('QTDBRUTA');
        qtBruta.value(TcoUtil.parseFloat(fieldQt.value()));
        this.calcQtBruta(args);
    };

    this.checkRangeLossCleaning = function(args) {
        let field = args.owner.field;
        if (TcoUtil.parseFloat(field.value()) > 99) {
            field.value(99);
        } else if (TcoUtil.parseFloat(field.value()) < 0) {
            field.value(0);
        } else if (Util.isEmptyOrBlank(field.value())) {
            field.value(0);
        }
    };

    this.checkRangeExploitationCleaning = function(args) {
        let field = args.owner.field;
        if (TcoUtil.parseFloat(field.value()) > 100) {
            field.value(100);
        } else if (TcoUtil.parseFloat(field.value()) <= 0) {
            field.value(100);
        } else if (Util.isEmptyOrBlank(field.value())) {
            field.value(100);
        }
    };

    /**
     * Retira a propriedade de changed dos produtos
     * revisados para que eles não caiam nas validações.
     * 
     * @params {Object} widget - Widget de revisão.
     **/

    this.showFieldRegion = function(args) {
        let widget = args.owner;
        let fieldNmRegiao = templateManager.container.getWidget("widgetReviewTabbed").getField("NMREGIAO");
        let fieldCdRegiao = templateManager.container.getWidget("widgetReviewTabbed").getField("CDREGIAO");
        if (widget.name != "widRecRegiao") {
            fieldNmRegiao.isVisible = false;
            fieldCdRegiao.isVisible = false;
        } else {
            this.getRecRegiao(window.history.state.dadosPrato.CDREGIAO);
            fieldNmRegiao.isVisible = true;
            fieldCdRegiao.isVisible = true;
        }

        this.defineVisibilityActionsRegiao();
    };

    this.newRowRegiao = function(args) {
        var widget = args.owner.widget;
        var widgetTabbed = widget.container.getWidget('widgetReviewTabbed');
        if (widgetTabbed.getField('NMREGIAO').value()) {
            General.newRow(widget);
        } else {
            ScreenService.showMessage(ScreenService.i18n("You can't add without filtering"));
        }
    };

    this.openReplRec = function(args) {
        var widget = args.owner.widget;
        var widRepRec = widget.container.getWidget('widRepRec');
        var widgetReviewTabbed = widget.container.getWidget('widgetReviewTabbed');
        if (widgetReviewTabbed.currentRow.CDREGIAO && widgetReviewTabbed.currentRow.NMREGIAO) {
            var fieldRegiaoDestino = widRepRec.getField('NMREGIAO');
            fieldRegiaoDestino.clearValue();

            fieldRegiaoDestino.dataSourceFilter = [{
                    "operator": "ORDER_BY",
                    "name": [{
                        "name": "CDREGIAO",
                        "direction": "ASC"
                    }]
                },
                {
                    "name": "CDREGIAO",
                    "operator": "<>",
                    "value": widgetReviewTabbed.currentRow.CDREGIAO
                }
            ];
            fieldRegiaoDestino.reload();
            widRepRec.currentRow.CDREGIAO_ORIGEM = widgetReviewTabbed.currentRow.CDREGIAO;
            widRepRec.currentRow.NMREGIAO_ORIGEM = widgetReviewTabbed.currentRow.NMREGIAO;
            ScreenService.openPopup(widRepRec);
        } else {
            MessageService.showMessageByCode('FILTRO_NAO_APLICADO');
        }
    };

    this.closePopup = function(args) {
        ScreenService.closePopup();
    };

    this.replicaReceitas = function(args) {
        var widget = args.owner.widget;

        if (widget.isValid()) {
            MessageService.confirmMessageByCode('CONFIRMA_REPLICA_RECEITA').then(function(success) {
                var row = {
                    CDPRATO: window.history.state.dadosPrato.CDPRATO,
                    CDFILAUXPRAT: window.history.state.dadosPrato.CDFILAUXPRAT,
                    CDREGIAO_ORIGEM: widget.currentRow.CDREGIAO_ORIGEM,
                    REGIOES_DESTINO: widget.currentRow.NMREGIAO
                };
                Tco01010_revisar_receita_card_base_service.replicaReceitas(row).then(function() {
                    MessageService.successNotificationByCode('SUCESSO_REPLICACAO');
                    ScreenService.closePopup();
                    widget.container.getWidget('widRecRegiao').reload();
                }, function(erro) {
                    ScreenService.showMessage(erro.data.error);
                });
            });
        }
    };

    this.abreImportacaoReceita = function(args) {
        var widget = args.owner.widget;
        var tabImportacao = widget.container.getWidget("tabImportacao");
        tabImportacao.widgetToActivate = widget;
        ScreenService.openSwipe(tabImportacao).then(function() {
            tabImportacao.activate();
        });
    };

    this.inicializaWidget = function(args) {
        var container = args.owner.container;
        var gridImportacao = container.getWidget('gridImportacao');
        //
        Tco01010_revisar_receita_card_base_service.findParamgeral().then(function(callback) {
            container.RECREVISADA = callback.dataset.paramgeral[0].IDRECREVICOP;
            var dadosPrato = window.history.state.dadosPrato;

            gridImportacao.currentRow.CDPRATO = dadosPrato.CDPRATO;
            gridImportacao.currentRow.NMPRATO = dadosPrato.NMPRATO;
            gridImportacao.currentRow.CDFILAUXPRAT = dadosPrato.CDFILAUXPRAT;

            container.getWidget('gridImportacao').dataSourceFilter = [{
                    "name": "CDPRATO",
                    "operator": "=",
                    "value": dadosPrato.CDPRATO
                },
                {
                    "name": "CDFILAUXPRAT",
                    "operator": "=",
                    "value": dadosPrato.CDFILAUXPRAT
                },
                {
                    "name": "TIPORECE",
                    "operator": "=",
                    "value": 'P'
                }
            ];
            container.getWidget('gridImportacao').reload();

            //Trata disponibilidade dos fields de quantidade
            if (container.QTDRECEITA == 'F') {
                container.getWidget('gridImportacao').getField('QTPRODUTO').readOnly = true;
                container.getWidget('gridImportacao').getField('QTDBRUTA').readOnly = false;
            } else {
                container.getWidget('gridImportacao').getField('QTPRODUTO').readOnly = false;
                container.getWidget('gridImportacao').getField('QTDBRUTA').readOnly = true;
            }

            setTimeout(function() {
                var widget = container.getWidget('tabImportacao');
                widget.getField('NMPRATOD').setValue(dadosPrato.NMPRATO);
                widget.getField('RECEITAD').setValue(getTipoRece(widget.widgetToActivate));
                widget.getField('CDFILIALD').setValue(getLocalVar('CDFILIAL'));
                widget.getField('NMFILIALD').setValue(getLocalVar('NMFILIAL'));
                widget.getField('CDSERVFILID').setValue(dadosPrato.CDSERVICO);
                widget.getField('NMSERVICOD').setValue('self.NMSERVFILI');

                widget.getField('CDPRATO').setValue(dadosPrato.CDPRATO);
                widget.getField('NMPRATO').setValue(dadosPrato.NMPRATO);
                widget.getField('CDFILAUXPRAT').setValue(dadosPrato.CDFILAUXPRAT);
                widget.getField('TIPO').setValue('P');
                trataField(widget.getField('NMREGIAO'), true);
                trataField(widget.getField('NMPADRAOQUALIDADE'), true);

                widget.getField('CDFILIAL').clearValue();
                trataField(widget.getField('NMFILIAL'), true);

                widget.getField('CDSERVFILI').clearValue();
                trataField(widget.getField('NMSERVICO'), true);
            }, 400);
        });
    };

    this.trataCamposPorReceita = function(widget) {
        var receita = widget.getField('TIPO').value();
        switch (receita) {
            case 'P':
                widget.getField('CDSERVFILI').clearValue();
                widget.getField('CDFILIAL').clearValue();
                trataField(widget.getField('NMFILIAL'), true);
                trataField(widget.getField('NMSERVICO'), true);
                trataField(widget.getField('NMREGIAO'), true);
                trataField(widget.getField('NMPADRAOQUALIDADE'), true);
                break;
            case 'F':
                widget.getField('CDSERVFILI').clearValue();
                widget.getField('CDFILIAL').clearValue();
                trataField(widget.getField('NMFILIAL'), false);
                widget.getField('CDFILIAL').setValue(getLocalVar('CDFILIAL'));
                widget.getField('NMFILIAL').setValue(getLocalVar('NMFILIAL'));
                trataField(widget.getField('NMSERVICO'), true);
                trataField(widget.getField('NMREGIAO'), true);
                trataField(widget.getField('NMPADRAOQUALIDADE'), true);
                break;
            case 'S':
                widget.getField('CDSERVFILI').clearValue();
                widget.getField('CDFILIAL').clearValue();
                trataField(widget.getField('NMFILIAL'), false);
                widget.getField('CDFILIAL').setValue(getLocalVar('CDFILIAL'));
                widget.getField('NMFILIAL').setValue(getLocalVar('NMFILIAL'));
                trataField(widget.getField('NMSERVICO'), false);
                trataField(widget.getField('NMREGIAO'), true);
                trataField(widget.getField('NMPADRAOQUALIDADE'), true);
                break;
            case 'Q':
                widget.getField('CDSERVFILI').clearValue();
                widget.getField('CDFILIAL').clearValue();
                trataField(widget.getField('NMFILIAL'), true);
                trataField(widget.getField('NMSERVICO'), true);
                trataField(widget.getField('NMREGIAO'), true);
                trataField(widget.getField('NMPADRAOQUALIDADE'), false);
                break;
            case 'R':
                widget.getField('CDSERVFILI').clearValue();
                widget.getField('CDFILIAL').clearValue();
                trataField(widget.getField('NMFILIAL'), true);
                trataField(widget.getField('NMSERVICO'), true);
                trataField(widget.getField('NMREGIAO'), false);
                trataField(widget.getField('NMPADRAOQUALIDADE'), false);
                break;
            case 'C':
                widget.getField('CDSERVFILI').clearValue();
                widget.getField('CDFILIAL').clearValue();
                trataField(widget.getField('NMFILIAL'), true);
                trataField(widget.getField('NMSERVICO'), true);
                trataField(widget.getField('NMREGIAO'), true);
                trataField(widget.getField('NMPADRAOQUALIDADE'), true);
                break;
        }

    };

    function trataField(field, readOnly, validations) {
        field.readOnly = readOnly;
        field.clearValue();
        if (!readOnly) {
            field.validations = {
                required: true
            };
        } else {
            field.validations = {};
        }
    }

    this.closeSwipe = function(args) {
        var widget = args.owner.widget;
        var widToAct = widget.container.getWidget("tabImportacao").widgetToActivate;
        ScreenService.closeSwipe();
        widToAct.reload();
        widToAct.activate();
    };

    this.changeTipoReceitaImportacao = function(args) {
        var widget = args.owner.field.widget;
        this.trataCamposPorReceita(widget);
        this.recarregaReceita(widget);
    };

    this.recarregaReceita = function(widget) {
        var cdPrato = widget.getField('CDPRATO').value();
        var cdFilAuxPrat = widget.getField('CDFILAUXPRAT').value();
        var receita = widget.getField('TIPO').value();
        var filial = widget.getField('CDFILIAL').value();
        if (!filial) filial = getLocalVar('CDFILIAL');
        var servico = widget.getField('CDSERVFILI').value();
        var padraoqualidade = widget.currentRow.NRPADRAOQUALIDADE;
        var regiao = widget.currentRow.CDREGIAO;
        var gridImportacao = widget.container.getWidget('gridImportacao');
        if (widget.isValid()) {
            gridImportacao.dataSourceFilter = [{
                    "name": "CDPRATO",
                    "operator": "=",
                    "value": cdPrato
                },
                {
                    "name": "CDFILAUXPRAT",
                    "operator": "=",
                    "value": cdFilAuxPrat
                },
                {
                    "name": "TIPORECE",
                    "operator": "=",
                    "value": receita
                }
            ];
            if (receita == 'S' || receita == 'F') {
                if (receita == 'S') {
                    gridImportacao.dataSourceFilter.push({
                        "name": "CDSERVFILI",
                        "operator": "=",
                        "value": servico
                    });
                }
                gridImportacao.dataSourceFilter.push({
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": filial
                });
            } else if (receita == 'R' || receita == 'Q') {
                if (receita == 'R') {
                    gridImportacao.dataSourceFilter.push({
                        "name": "CDREGIAO",
                        "operator": "=",
                        "value": regiao
                    });
                }
                gridImportacao.dataSourceFilter.push({
                    "name": "NRPADRAOQUALIDADE",
                    "operator": "=",
                    "value": padraoqualidade
                });
            } else if (receita == 'C') {
                gridImportacao.dataSourceFilter.push({
                    "name": "NRCARDAPIOTRAB",
                    "operator": "=",
                    "value": window.history.state.dadosPrato.NRCARDAPIOTRAB
                });
            }

            gridImportacao.reload();
        } else {
            gridImportacao.dataSource.data = [];
        }
    };

    this.salvaProduto = function(args) {
        var widget = args.owner.widget;
        var fieldQtdeBruta = widget.getField('QTDBRUTA');
        var value = fieldQtdeBruta.value() ? angular.copy(fieldQtdeBruta.value()) : 0;
        value = parseFloat(value.toString().split('.').join('').replace(',', '.'));
        if (value > 0) {
            if (widget.isValid()) {
                widget.currentRow.EDICAO = 'S';
                ScreenService.closeSwipe();
                widget.container.getWidget('gridImportacao').activate();
            }
        } else {
            MessageService.showMessageByCode('TCO01010_QUANTIDADE_INVALIDA');
        }
    };

    this.editing = function(args) {
        var widget = args.owner.widget;
        General.widgetEdit(widget);
    };

    this.realizaImportacao = function(args) {
        var gridImportacao = args.owner.widget;
        var stripeWidget = args.owner.widget.container.getWidget('stripeWidget');

        if (gridImportacao.getParent().isValid()) {
            var message = 'Tipo de importação';
            var type = 'alert';
            var rows = gridImportacao.dataSource.data;
            var tabImportacao = gridImportacao.container.getWidget('tabImportacao');
            var widgetReviewTabbed = gridImportacao.container.getWidget('widgetReviewTabbed');
            var dadosPrato = window.history.state.dadosPrato;
            var tipoReceita = getTipoRece(tabImportacao.widgetToActivate);

            var rowOrigem = {
                CDPRATO: tabImportacao.getField('CDPRATO').value().split('.').join(''),
                CDFILAUXPRAT: tabImportacao.getField('CDFILAUXPRAT').value(),
                TIPO: tabImportacao.getField('TIPO').value(),
                CDFILIAL: tabImportacao.getField('CDFILIAL').value(),
                CDSERVFILI: tabImportacao.getField('CDSERVFILI').value(),
                CDREGIAO: tabImportacao.currentRow.CDREGIAO ? tabImportacao.currentRow.CDREGIAO : null,
                NRPADRAOQUALIDADE: tabImportacao.currentRow.NRPADRAOQUALIDADE ? tabImportacao.currentRow.NRPADRAOQUALIDADE : null,
                NRCARDAPIOTRAB: dadosPrato.NRCARDAPIOTRAB ? dadosPrato.NRCARDAPIOTRAB : null
            };

            var rowDestino = {
                CDPRATO: dadosPrato.CDPRATO,
                CDFILAUXPRAT: dadosPrato.CDFILAUXPRAT,
                TIPO: tipoReceita,
                CDFILIAL: tipoReceita === 'S' || tipoReceita === 'F' ? stripeWidget.currentRow.CDFILIALPARAMTCO : '',
                CDSERVFILI: tipoReceita === 'S' ? gridImportacao.container.CDSERVFILI : '',
                CDREGIAO: tabImportacao.container.getWidget('widgetReviewTabbed').currentRow.CDREGIAO ? tabImportacao.container.getWidget('widgetReviewTabbed').currentRow.CDREGIAO : null,
                NRPADRAOQUALIDADE: dadosPrato.NRPADRAOQUALIDADE,
                NRCARDAPIOTRAB: dadosPrato.NRCARDAPIOTRAB ? dadosPrato.NRCARDAPIOTRAB : null
            };

            var editado = 'N';

            for (var item in rows) {
                rows[item].QUANTIDADE = General.valueToFloat(rows[item].QTPRODUTO) || 0;
                rows[item].GANHOCOC = General.valueToFloat(rows[item].VRGANHOCOCCAO) || 0;
                rows[item].PERDALIMP = General.valueToFloat(rows[item].VRPERDALIMPEZA) || 0;
                rows[item].QTREQUERIDA = General.valueToFloat(rows[item].QTDBRUTA) || 0;
                rows[item].QTBRUTA = General.valueToFloat(rows[item].QTDBRUTA) || 0;
                rows[item].SGUNIDADEPROD = rows[item].SGUNIDADE;
                rows[item].NRCARDAPIOTRAB = dadosPrato.NRCARDAPIOTRAB ? dadosPrato.NRCARDAPIOTRAB : null;

                if (rows[item].EDICAO && rows[item].EDICAO !== 'N')
                    editado = 'S';
            }

            if (rowOrigem.TIPO === rowDestino.TIPO && rowOrigem.CDFILIAL === rowDestino.CDFILIAL &&
                rowOrigem.CDSERVFILI === rowDestino.CDSERVFILI && rowOrigem.CDPRATO === rowDestino.CDPRATO && rowOrigem.CDREGIAO === rowDestino.CDREGIAO) {
                ScreenService.showMessage('Receita de origem deve ser diferente da receita de destino.', 'error', 'error');
                return;
            }

            var acrescentar = function() {
                var row = {
                    "ORIGEM": rowOrigem,
                    "DESTINO": rowDestino,
                    "TIPO_IMPORT": "A",
                    "ROWS": rows
                };
                Tco01010_revisar_receita_card_base_service.importarReceitaTCO(row).then(function() {
                    // tabImportacao.widgetToActivate.reload();
                    ScreenService.successNotification('Importação realizada com sucesso!', 3000);
                    widgetReviewTabbed.currentWidget.reload().then(() => {
                        ScreenService.closeSwipe().then(() => widgetReviewTabbed.currentWidget.activate());
                    });
                }, function(error) {
                    ScreenService.errorNotification(error.data.error);
                });
            };

            var sobrepor = function() {
                var row = {
                    "ORIGEM": rowOrigem,
                    "DESTINO": rowDestino,
                    "TIPO_IMPORT": "S",
                    "ROWS": rows
                };
                Tco01010_revisar_receita_card_base_service.importarReceitaTCO(row).then(function() {
                    // tabImportacao.widgetToActivate.reload();
                    ScreenService.successNotification('Importação realizada com sucesso!', 3000);
                    widgetReviewTabbed.currentWidget.reload().then(() => {
                        ScreenService.closeSwipe().then(() => widgetReviewTabbed.currentWidget.activate());
                    });
                }, function(error) {
                    ScreenService.errorNotification(error.data.error);
                });
            };

            ScreenService.showCustomDialog(message, type, 'exclamation', [{
                    label: ScreenService.i18n("Acrescentar"),
                    code: function(e, deferred) {
                        if (Util.isFunction(acrescentar)) {
                            acrescentar(e);
                        }
                        deferred.resolve(e);
                    }
                },
                {
                    label: ScreenService.i18n("Sobrepor"),
                    code: function(e, deferred) {
                        if (Util.isFunction(sobrepor)) {
                            sobrepor(e);
                        }
                        deferred.resolve(e);
                    }
                },
                {
                    label: ScreenService.i18n("Cancelar")
                }
            ], '');
        }
    };

    this.changeFieldUnity = function(args) {
        var widget = args.owner.field.widget;
        widget.getField('CDSERVFILI').clearValue();
        widget.getField('NMSERVICO').clearValue();
        this.recarregaReceita(widget);
    };

    this.defineVisibilityActionsRegiao = function() {
        let widRecRegiao = templateManager.container.getWidget("widRecRegiao");
        let widgetTabbed = templateManager.container.getWidget('widgetReviewTabbed');
        let regiao_informada = (widgetTabbed.getField('NMREGIAO').value() && widgetTabbed.getField('NMREGIAO').value() != "");
        widRecRegiao.getAction("REMOVE").isVisible = (regiao_informada);
        widRecRegiao.getAction("NEW").isVisible = (regiao_informada);
        widRecRegiao.getAction("EDIT").isVisible = (regiao_informada);
        widRecRegiao.getAction("IMPORT").isVisible = (regiao_informada);
        widRecRegiao.getAction("REPLICAR").isVisible = (regiao_informada);
    };
};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01010_revisar_receita_card_base', Tco01010_revisar_receita_card_base);
});

// FILE: ./assets/js/controllers/Tco01011_dados_prato_controller.js
/*global Util, Configuration*/
var Tco01011_dados_prato_controller = function(ScreenService, General, Tco01200CardapioDaPropostaService, templateManager, ZHPromise, TcoUtil, Tco01000_cardapio_base_service, Tco01011_dados_prato_service) {

    this.carregaParametrosDadosPrato = function(args) {
        if (!window.history.state) {
            window.location.href = window.location.href.replace("#/tco#tco01011_dados_prato", "#/tco#tco01000_cardapio_base");
            window.location.reload();
        } else {
            var widgetComensais = args.owner.getWidget("footerTabbed");
            var widgetProdutos = args.owner.getWidget("widgetProdutos");
            var widgetFiltro = widgetComensais.container.getWidget('pratoTabbed');
            widgetComensais.getField('VRPECONSCARDTRAB').value(window.history.state.dadosPrato.VRPECONSCARDTRAB);
            widgetFiltro.getField('CDPRATO').clearValue();
            widgetProdutos.reload();
            setStripe();
            this.writeLabel(window.history.state.label);
        }
    };

    /**
     * Volta para ultima tela.
     **/

    this.backCard = function() {
        window.history.back();
    };

    /**
     * Função resposável por montar o "description order" da tela 
     * de Dados do Prato.
     * É concatenado o dia da semana com o tipo do prato para
     * identificação da célula selecionada.
     * 
     * @params {Object} label - Objeto com dia do mês e tipo de prato.
     **/

    this.writeLabel = function(label) {
        var dashDadoPrato = templateManager.container.getWidget('dashboardDadosPrato');
        dashDadoPrato.label = label.LABELDIA + ' - ' + label.LABELTPPRAT;
        templateManager.updateTemplate();
    };

    /**
     * Função de find da tela de pratos.
     * Após buscar o dataset no backend, caso haja dados, é criado
     * uma propriedade bkp_data que será usada juntamente com o filtro.
     * No bkp_data sempre terá o dataset completo, com todos os pratos.
     * Além disso é chamada a função para setar o prato selecionado
     * na currentRow e outra para carregar o grid de produtos do prato.
     * 
     * @params {Args} args - Objeto args.
     **/

    this.getDadosPrato = function(args) {
        var widget = args.owner;
        var filter = window.history.state.filtro;
        Tco01200CardapioDaPropostaService.getDadosPrato(filter).then(function(response) {
            if (!Util.isEmptyOrBlank(response.dataset.tco_01205_pratos)) {
                widget.dataSource.data = response.dataset.tco_01205_pratos;
                widget.dataSource.bkp_data = response.dataset.tco_01205_pratos;
                widget.dataSource.reload();
                this.setCurrentPrato(widget, window.history.state.dadosPrato);
                this.carregaProdutosPrato(null, widget);
            }
        }.bind(this));
    };

    /**
     * Função responsável por setar o prato selecionado na currentRow
     * da widget de Pratos.
     * Caso não haja nenhum prato, ele seleciona por default o primeiro prato
     * do dataset.
     * 
     * @params {Object} widgetPratos - Widget de pratos.
     * @params {Object} dadosPrato - Objeto com informações da célula selecionada.
     **/

    this.setCurrentPrato = function(widgetPratos, dadosPrato) {
        if (dadosPrato) {
            if (dadosPrato.CDPRATO) {
                var prato = widgetPratos.dataSource.data.filter(function(row) {
                    return row.CDPRATO === dadosPrato.CDPRATO;
                })[0];
                widgetPratos.setCurrentRow(prato);
                setTimeout(function() {
                    $(".zh-new-grid-" + widgetPratos.name).find(".grid-vertical-scroll-control").scrollTop(widgetPratos.currentRow.__index * 36);
                }, 150);
            } else {
                widgetPratos.setCurrentRow(widgetPratos.dataSource.getFirstRow());
            }
        }
    };

    /**
     * Função de filtro da tela de pratos.
     * Chamada no evento onChange, ela tenta dar um match em uma regex
     * previamente criada para trazer o resultado da pesquisa, respeitando
     * o padrão de pesquisa da GR.
     * A cada pesquisa o dataSource.data é sobreescrito, caso seja pesquisado
     * um valor vazio, a função garante que seja retornado o dataset completo,
     * buscado na propriedade bkp_data.
     * 
     * @params {Args} args - Objeto args.
     **/

    this.filterPrato = function(args) {
        let field = args.owner.field;
        clearTimeOut(field);
        field.timeout = setTimeout(function() {
            let widgetPrato = templateManager.container.getWidget('widgetPratos');
            let filter = args.e.currentTarget.value;
            if (filter !== "") {
                if (filter[filter.length - 1] !== '%')
                    filter = filter + '%';
                let newData = widgetPrato.dataSource.bkp_data.filter(row => {
                    return (row.CDPRATO.toUpperCase().match(TcoUtil.buildRegExp(filter.toUpperCase())) ||
                        row.NMPRATO.toUpperCase().match(TcoUtil.buildRegExp(filter.toUpperCase())));
                });
                widgetPrato.dataSource.data = newData;

            } else {
                widgetPrato.dataSource.data = widgetPrato.dataSource.bkp_data;
                widgetPrato.setCurrentRow(widgetPrato.dataSource.data[0]);
            }
            templateManager.updateTemplate();
        }, 300);
    };

    let clearTimeOut = function(field) {
        if (field.timeout !== false)
            clearTimeout(field.timeout);
    };

    /**
     * Função responsável por criar o filtro da tela de produtos
     * de acordo com o prato selecionado.
     * 
     * @params {Args} args - Objeto args.
     * @params {Object} widgetPrato - Parâmetro passado quando essa função é 
     * chamada diretamente do js.
     **/

    this.carregaProdutosPrato = function(args, widgetPrato = null) {
        if (args)
            widgetPrato = args.owner;
        var widgetProdutos = templateManager.container.getWidget("widgetProdutos");
        var filter = [
            {
                "name": "CDPRATO",
                "operator": "=",
                "value": widgetPrato.currentRow.CDPRATO
            }
        ];
        widgetProdutos.dataSourceFilter = filter;
        widgetProdutos.reload();
        var row = {
            CDFILAUXPRAT: widgetPrato.currentRow.CDFILAUXPRAT,
            CDPRATO: widgetPrato.currentRow.CDPRATO,
            NRSERVICOPROPOSTA: window.history.state.filtro.NRSERVICOPROPOSTA,
            NRPADRAOQUALIDADE: window.history.state.filtro.NRPADRAOQUALIDADE,
            CDREGIAO: window.history.state.filtro.CDREGIAO ? window.history.state.filtro.CDREGIAO : null,
            NRCARDAPIOTRAB: window.history.state.filter.NRCARDAPIOTRAB,
            CDSERVFILI: window.history.state.elaboracaoCardapioBaseParams.CDSERVFILI ? window.history.state.elaboracaoCardapioBaseParams.CDSERVFILI : null
        };

        Tco01011_dados_prato_service.retornaReceitas(row).then(success => {
            widgetProdutos.dataSource.data = success.dataset.receitaPrato;
            this.calcTotalyCost(widgetProdutos);
        });
    };

    this.calcTotalyCost = function(widProdutos) {
        let totalyCost = 0;
        let fieldTotalyCost = templateManager.container.getWidget('footerTabbed').getField('CUSTO');
        let params = {
            decimal: ',',
            thousands: '.',
            precision: 3
        };
        widProdutos.dataSource.data.forEach(row => {
            totalyCost += row.CUSTOTOTAL;
        });
        fieldTotalyCost.value(TcoUtil.maskCurrency(totalyCost, params));
    };

    /**
     * Função responsável por adicionar ou updatar o prato ao dia e tipo
     * de prato selecionado.
     * São montados os parâmetros para o upsert e em caso de sucesso, a
     * tela de dados do prato é "fechada", voltando então para o crosstab.
     * 
     **/

    this.addOrUpdate = function() {
        var widgetPratos = templateManager.container.getWidget("widgetPratos");
        var widgetProdutos = templateManager.container.getWidget("widgetProdutos");
        var valueComensal = templateManager.container.getWidget("footerTabbed").getField('VRPECONSCARDTRAB').value() || 0;
        valueComensal = TcoUtil.parseFloat(valueComensal);
        if (widgetPratos.currentRow.CDPRATO && valueComensal > 0 && !Util.isEmptyOrBlank(widgetPratos.currentRow.TIPO_RECEITA)) {

            var params = {
                "NRCOMPCARDTRAB": window.history.state.dadosPrato.NRCOMPCARDTRAB,
                "NRCARDAPIOTRAB": window.history.state.NRCARDAPIOTRAB,
                "NRDIACARDTRAB": window.history.state.dadosPrato.NRDIACARDTRAB,
                "NRCONFIGCARDTRAB": window.history.state.dadosPrato.NRCONFIGCARDTRAB,
                "CDPRATO": widgetPratos.currentRow.CDPRATO,
                "CDFILAUXPRAT": widgetPratos.currentRow.CDFILAUXPRAT,
                "VRPECONSCARDTRAB": valueComensal,
                "NRCARDAPIOBASE": window.history.state.elaboracaoCardapioBaseParams.NRCARDAPIOBASE,
                "CDREGIAO": window.history.state.filtro.CDREGIAO,
                "PRODUTOS": widgetProdutos.dataSource.data
            };
            Tco01000_cardapio_base_service.addOrUpdatePratoTrab([params]).then(function(response) {
                window.history.back();
                ScreenService.successNotification("Prato alterado com sucesso.");
            }.bind(this), function(error) {
                ScreenService.showMessage(error.error);
            });

        } else {
            ScreenService.showMessage("Selecione um prato com receita e informe um valor maior que zero para o Total de Comensais.");
        }
    };

    /**
     * Função responsável por abrir a tela de revisão da receita.
     * 
     * @params {Args} args - Objeto args.
     **/
    this.openReview = function(args) {
        var widgetPratos = templateManager.container.getWidget('widgetPratos');
        var params = {
            "dadosPrato": {
                "CDPRATO": widgetPratos.currentRow.CDPRATO,
                "NMPRATO": widgetPratos.currentRow.NMPRATO,
                "CDFILAUXPRAT": widgetPratos.currentRow.CDFILAUXPRAT,
                "NRSERVICOPROPOSTA": window.history.state.filtro.NRSERVICOPROPOSTA,
                "NRPADRAOQUALIDADE": window.history.state.filtro.NRPADRAOQUALIDADE,
                "CDREGIAO": window.history.state.filtro.CDREGIAO,
                "NMREGIAO": window.history.state.filtro.CDREGIAO,
                "NRCARDAPIOTRAB": window.history.state.filtro.NRCARDAPIOTRAB,
                "CDSERVFILI": window.history.state.elaboracaoCardapioBaseParams.CDSERVFILI
            }
        };
        window.history.state.dadosPrato.CDPRATO = widgetPratos.currentRow.CDPRATO;
        window.history.replaceState(window.history.state, "Dados do Prato");

        TcoUtil.openWindowByHistory("#/tco#tco01010_revisar_receita", params, "Revisar Receita");
    };

    /**
     *  Função responsável por colocar valores na widget Stripe
     **/
    function setStripe() {
        var stripeWidget = templateManager.container.getWidget('stripeWidget');
        stripeWidget.currentRow = window.history.state.elaboracaoCardapioBaseParams || {};
    }

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01011_dados_prato_controller', Tco01011_dados_prato_controller);
});

// FILE: ./assets/js/controllers/Tco01100_document_controller.js
var Tco01100_document_controller = function(ScreenService, General, localStorageEngine, templateManager, TcoUtil, Tco01100_document_service, MessageService) {
    /**
     * @var {Array[String]} allowedFileNamePattern Extensões
     * de arquivos que são aceitas no sistema.
     */
    var allowedFileNamePattern = [
        "doc", "docx", "rtf",
        "xls", "pdf", "txt",
        "ppt", "png", "jpg",
        "jpeg", "bmp", "xlsx",
        "pptx"
    ];

    /**
     * Abre widget para upload de arquivo.
     */
    this.openUploadFileWidget = function() {
        var uploadFileWidget = getWidget('uploadFileWidget');
        ScreenService.openPopup(uploadFileWidget);
    };

    /**
     * Fecha widget para upload de arquivo.
     */
    this.closeUploadFilePopup = function() {
        var uploadFileWidget = getWidget('uploadFileWidget');
        ScreenService.closePopup(uploadFileWidget);
    };

    /**
     * Conjunto de instruções a serem executadas
     * ao selecionar um arquivo na tela de novo
     * documento.
     */
    this.afterSelectFile = function() {
        validateFileToUpload();
    };

    /**
     * Realiza o upload do arquivo.
     */
    this.uploadFile = function() {
        var uploadFileWidget = getWidget('uploadFileWidget');
        var widgetMain = getWidget('widgetMain');

        if (uploadFileWidget.isValid()) {
            var uploadedFile = {
                NRPROPOSTACOMERCIAL: widgetMain.currentRow.NRPROPOSTACOMERCIAL,
                FILE: uploadFileWidget.currentRow._FILE
            };

            if (!uploadedFile.FILE.type) {
                uploadedFile.FILE.type = 'application/octet-stream';
            }

            Tco01100_document_service.uploadFile(uploadedFile).then((response) => {
                if (response && response.error) {
                    MessageService.showMessageByCode(response.error, uploadFileWidget.currentRow);
                } else {
                    ScreenService.closePopup();
                    ScreenService.successNotification('Adicionado com sucesso.');
                    uploadFileWidget.getParent().reload();
                }
            }).catch((error) => {
                ScreenService.showMessage(error.error, uploadFileWidget.currentRow);
            });
        }
    };

    /**
     * Remove o arquivo da lista de arquivos
     * do documento.
     */
    this.removeFile = function() {
        var documentWidget = getWidget('documentWidget');
        var filesToRemove = documentWidget.dataSource.getCheckedRows().map(row => {
            return {
                NRDOCUMENTOPROPCOM: row.NRDOCUMENTOPROPCOM
            };
        });
        var rowToRemove = {
            FILES: filesToRemove,
            NRPROPOSTACOMERCIAL: documentWidget.getParent().currentRow.NRPROPOSTACOMERCIAL
        };

        Tco01100_document_service.removeFile(rowToRemove).then((response) => {
            if (response && response.error) {
                MessageService.showMessageByCode(response.error, documentWidget.currentRow);
            } else {
                documentWidget.dataSource.clearCheckedRows();
                documentWidget.reload();
            }
        }).catch((error) => {
            ScreenService.showMessage(ScreenService.i18n(error.error, documentWidget.currentRow));
        });
    };

    /**
     * Conjunto de instruções a serem executadas ao
     * (des)marcar um registro na widget de documento.
     */
    this.onChangeSelectDocument = function() {
        this.setDownloadButtonVisibility();
    };

    /**
     * Define a visibilidade do botão de download
     * para visivel somente quando um registro for
     * selecionado.
     */
    this.setDownloadButtonVisibility = function() {
        var documentWidget = getWidget('documentWidget');
        var downloadButton = documentWidget.getAction('downloadFile');
        var selectedRows = documentWidget.dataSource.getCheckedRows();

        downloadButton.isVisible = selectedRows.length == 1;
    };

    /**
     * Realiza o download o arquivo selecionado.
     */
    this.downloadFile = function() {
        var documentWidget = getWidget('documentWidget');
        var selectedRows = documentWidget.dataSource.getCheckedRows();
        var fileToDownload = selectedRows[0];

        if (selectedRows.length == 1) {

            Tco01100_document_service.downloadFile(fileToDownload).then((response) => {
                if (response && response.error) {
                    MessageService.showMessageByCode(response.error, documentWidget.currentRow);
                } else {
                    documentWidget.dataSource.clearCheckedRows();
                }
            }).catch((error) => {
                MessageService.showMessageByCode(error.error, documentWidget.currentRow);
            });

        } else {
            MessageService.showMessageByCode('TCO01100_JUST_ONE_FILE_ALLOWED');
        }
    };

    /**
     * Valida arquivo selecionado na tela de
     * novo documento e exibe mensagem ao 
     * usuário caso seja inválido.
     * 
     * @return {Boolean} 'true' caso o arquivo
     *  seja válido, 'false' caso seja invalido.
     */
    var validateFileToUpload = () => {
        var uploadFileWidget = getWidget('uploadFileWidget');
        var uploadedFile = uploadFileWidget.currentRow._FILE[0];

        var fileNameToCheck = uploadedFile.name.split('.');

        if (fileNameToCheck.length != 2 || !allowedFileNamePattern.includes(fileNameToCheck[1].trim())) {

            MessageService.showMessageByCode('TCO01100_INVALID_FILE_TYPE').then(() => {
                uploadFileWidget.currentRow._FILE = [];
                uploadFileWidget.getField('_FILE').element[0].parentElement.querySelector('.zh-file-name').textContent = ScreenService.i18n('Nenhum arquivo selecionado');
                return false;
            });

        } else return true;
    };

    /**
     * Retorna widget cujo nome é passado por parâmetro.
     * 
     * @param {String} widgetName Nome da widget a ser
     *  retornada.
     */
    var getWidget = (widgetName) => {
        return templateManager.container.getWidget(widgetName);
    };
};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01100_document_controller', Tco01100_document_controller);
});

// FILE: ./assets/js/controllers/Tco01100_proposta_comercial.js
/* global angular, Configuration*/
var Tco01100_proposta_comercial = function(ScreenService, General, localStorageEngine, templateManager, Tco01100_proposta_comercial_service, TcoUtil, $rootScope, TcoEmailService, ZhKeyboardService, ZHPromise, eventEngine, MessageService) {
    /**
     * @var {Array} defaultValues
     *  Valores padrões para propriedades usadas 
     * em toda a tela.
     */
    var defaultValues = {
        NMGERENTENEGOCIO: localStorageEngine.getLocalVar('CDOPERADOR') +
            ' - ' + localStorageEngine.getLocalVar('NMOPERADOR'),
        NMRESPONSAVEL: localStorageEngine.getLocalVar('CDOPERADOR') +
            ' - ' + localStorageEngine.getLocalVar('NMOPERADOR'),
        DTLANCAMENTO: moment().format('DD/MM/YYYY HH:mm:ss'),
        SITUACAO: "Pendente"
    };

    /**
     * Conjunto de instruções a serem executadas
     * ao iniciar o container.
     */
    this.afterInitContainer = function() {
        let widgetMain = this.getWidget('widgetMain');
        var ref = this;

        widgetMain.openFilterShortcut = eventEngine.bindEvent(ZhKeyboardService, 'mod+lPressed', function(eventData) {
            return ref.openFilterMainWidget(eventData);
        });

        widgetMain.getField("VIGENCIA").defaultValue = {
            start: moment().startOf("month").format("DD/MM/YYYY"),
            end: moment().endOf("month").format("DD/MM/YYYY")
        };
        prepareWidgetIfComeFromDetail();
    };

    /**
     * Abre filtro na widget main.
     */
    this.openFilterMainWidget = (keyEvent) => {
        var widgetMain = this.getWidget('widgetMain');

        if (widgetMain.filterRowBackup) {
            widgetMain.fields.forEach((field) => {
                if (widgetMain.filterRowBackup.hasOwnProperty(field.name)) {
                    field.defaultValue = widgetMain.filterRowBackup[field.name];
                }
            });
        }

        TcoUtil.startFilter(widgetMain);
    };

    /**
     * Abre swipe apontando para o registro correto
     * caso a chamada da tela seja feita a partir
     * da tela de detalhes.
     */
    var prepareWidgetIfComeFromDetail = () => {
        var widgetMain = this.getWidget('widgetMain');

        if (window.history.state && window.history.state.backFromDetail) {
            window.history.replaceState({}, 'Proposta Comercial', '#/tco#tco01100_proposta_comercial');

            widgetMain.reload().then(() => {
                restoreServicoPropostaSwipe();
            });
        }
    };

    /**
     * Abre a tab da widget de versao da proposta
     * e posiciona no serviço correto, quando 
     * retorna do detalhe do serviço.
     */
    var restoreServicoPropostaSwipe = () => {
        var widgetMain = this.getWidget('widgetMain');
        var versaoWidget = this.getWidget('versaoWidget');

        widgetMain.afterMoveRow();

        var waitForParent = () => {

            if (versaoWidget.parent && versaoWidget.parent.openTab) {

                versaoWidget.parent.openTab('versaoWidget');
                versaoWidget.editing = false;
                versaoWidget.afterMoveRow();
                setActionsVisibilityVersaoWidget();

            } else {
                setTimeout(function() {
                    waitForParent();
                }, 100);
            }
        };

        waitForParent();
    };

    /**
     * Conjunto de instruções a serem executadas
     * ao alterar o valor do campo de retenção.
     * 'clearClienteProspectValues' deve ser executada
     * antes de 'switchClienteProspectFields'.
     * 
     * @param {Args} args Objeto args.
     */
    this.onChangeRetencao = function(args) {
        var widget = this.getWidget('widgetMain');
        var field = widget.getField('IDRETENCAO');
        if (!widget.currentRow.oldIdretencao || (widget.currentRow.oldIdretencao !== field.value())) {
            widget.currentRow.oldIdretencao = field.value();

            var fieldShowParam = field.value() == 'S' ? 'C' : 'P';
            this.clearClienteProspectValues(widget, fieldShowParam);

            var fieldToClear = ["SEGMENTO", "RAMOATIVIDADE", "TIPOCONTRATO", "CDPAIS", "NMPAIS", "SGESTADO", "NMMUNICIPIO", "CDMUNICIPIO", "NMREGIAO", "DSENDERECO"];

            for (var j = 0; j < fieldToClear.length; ++j) {
                if (fieldToClear[j] !== "SEGMENTO" && fieldToClear[j] !== "TIPOCONTRATO" && fieldToClear[j] !== "NMPAIS" && fieldToClear[j] !== "DSENDERECO") {
                    widget.currentRow[fieldToClear[j]] = "";
                    widget.getField(fieldToClear[j]).readOnly = true;
                    widget.getField(fieldToClear[j]).validations = {};
                } else {
                    widget.currentRow[fieldToClear[j]] = "";
                }
            }

            this.switchClienteProspectFields(widget, fieldShowParam);

            widget.getField('CLIENTEPROSPECT').showOnForm = false;
            TcoUtil.changeRequiredField(widget.getField('CLIENTEPROSPECT'), false);
        }
    };

    /**
     * Limpa os campos de Cliente e Prospect na widget
     * principal.
     * 
     * @param {Widget} widget Widget principal contendo
     *  os fields de Cliente e Prospect.
     * @param {String} fieldToShow Campo visivel.
     */
    this.clearClienteProspectValues = function(widget, visibleField) {
        var clearCondition = widget.getField('PROSPECT').isVisible;
        var namesToClear = [
            'CLIENTE',
            'CDCLIENTE',
            'PROSPECT',
            'CDPROSPECT'
        ];

        if (visibleField) {
            let condition = visibleField == 'C';

            if (clearCondition == condition) {
                this.clearRowValues(widget.currentRow, namesToClear);
            }
        } else {
            this.clearRowValues(widget.currentRow, namesToClear);
        }
    };

    /**
     * Alterna os campos de Cliente e Prospect na widget
     * principal.
     * 
     * @param {Widget} widget Widget principal contendo
     *  os fields de Cliente e Prospect.
     * @param {String} fieldToShow Campo a ser exibido 
     *  independente de alternância. (OPCIONAL)
     */
    this.switchClienteProspectFields = function(widget, fieldToShow) {
        var showCondition = widget.getField('PROSPECT').isVisible;

        if (fieldToShow) {
            let condition = fieldToShow == 'C';

            if (showCondition != condition)
                showCondition = condition;
        }

        widget.getField('CLIENTE').isVisible = showCondition;
        TcoUtil.changeRequiredField(widget.getField('CLIENTE'), showCondition);

        widget.getField('PROSPECT').isVisible = !showCondition;
        TcoUtil.changeRequiredField(widget.getField('PROSPECT'), !showCondition);
    };

    /**
     * Define como string vazia as propriedades, que estejam
     * com o nome em 'namesToClear', no objeto passado por 
     * parâmetro.
     * 
     * @param {Object} row Objeto contendo as propriedades
     *  a serem limpas.
     * @param {Array[String]} namesToClear Nomes da propriedades
     *  a serem limpas.
     */
    this.clearRowValues = function(row, namesToClear) {
        namesToClear.forEach((name) => {
            row[name] = '';
        });
    };

    /**
     * Cria novo registro na widget de proposta comercial.
     * 
     * @param {Args} args Objeto args.
     */
    this.addPropostaComercial = function(args) {
        var widget = args.owner.widget;
        var widgetMain = args.owner.widget.container.getWidget('widgetMain');

        widget.getField('CLIENTE').isVisible = false;
        TcoUtil.changeRequiredField(widget.getField('CLIENTE'), false);
        widget.getField('PROSPECT').isVisible = false;
        TcoUtil.changeRequiredField(widget.getField('PROSPECT'), false);

        widget.getField('CLIENTEPROSPECT').showOnForm = true;
        TcoUtil.changeRequiredField(widget.getField('CLIENTEPROSPECT'), true);

        widget.getField('RAMOATIVIDADE').readOnly = true;

        this.newRow(widget);
        this.applyDefaultValuesOnWidget(widget);

        // Busca datas default

        Tco01100_proposta_comercial_service.findParamTCO().then((success) => {
            widgetMain.QTDIASVIGENPROPCOM = success.dataset.paramtco[0].QTDIASVIGENPROPCOM;
            widget.getField("VIGENCIA").setValue({
                start: moment().format("DD/MM/YYYY"),
                end: typeof widgetMain.QTDIASVIGENPROPCOM !== 'undefined' && widgetMain.QTDIASVIGENPROPCOM !== null ? moment().add(widgetMain.QTDIASVIGENPROPCOM, 'days').format("DD/MM/YYYY") : moment().endOf("month").format("DD/MM/YYYY"),
                formattedStart: moment().format("DD/MM/YYYY"),
                formattedEnd: typeof widgetMain.QTDIASVIGENPROPCOM !== 'undefined' && widgetMain.QTDIASVIGENPROPCOM !== null ? moment().add(widgetMain.QTDIASVIGENPROPCOM, 'days').format("DD/MM/YYYY") : moment().endOf("month").format("DD/MM/YYYY")
            });
            widget.getField("VIGENCIA").readOnly = widgetMain.QTDIASVIGENPROPCOM != null && widgetMain.QTDIASVIGENPROPCOM != "";

            widget.currentRow.VIGENCIA = widget.getField("VIGENCIA").value();
        });
        widget.getField('IDRETENCAO').setValue("N");
        this.onChangeRetencao(args);
    };

    /**
     * Cria um novo registro na widget.
     * 
     * @param {Args} args Objeto args.
     */
    this.newRow = function(widget) {
        widget.newRow();
        General.changeWidgetAcessibility(widget.widgets, false);
    };

    /**
     * Define os valores padrões de acordo com
     * 'defaultValues' na widget passada por parâmetro.
     * 
     * @param {Widget} widget Widget contendo os campos
     */
    this.applyDefaultValuesOnWidget = function(widget) {
        Object.keys(defaultValues).forEach(key => {
            widget.currentRow[key] = defaultValues[key];
        });
    };

    /**
     * Rotina que define o endereço de um cliente ou prospect

 selecionado 
     * 
     * @param {Args} args Objeto args.
     */

    this.setEnderecoClienteProspect = function(args) {
        var widgetMain = args.owner.field.widget;
        const fieldMunicipio = widgetMain.getField('NMMUNICIPIO');
        var filterRegiao = {
            name: "CDREGIAO",
            operator: "=",
            value: ''
        };
        const filterMunicipio = [
            {
                name: "CDPAIS",
                operator: "=",
                value: ''
            },
            {
                name: "SGESTADO",
                operator: "=",
                value: ''
            },
            {
                name: "CDMUNICIPIO",
                operator: "=",
                value: ''
            }
        ];
        if (args.owner.field.name === 'CLIENTE') {

            let filter = [
                {
                    "name": "CDCLIENTE",
                    "operator": "=",
                    "value": widgetMain.currentRow.CDCLIENTE
                }
            ];

            Tco01100_proposta_comercial_service.findDadosFormCliente(filter).then((success) => {
                if (success.dataset.dadosformcliente.length > 0) {
                    for (var prop in success.dataset.dadosformcliente[0]) {
                        if (widgetMain.currentRow.hasOwnProperty(prop) && prop !== "__is_new") {
                            widgetMain.currentRow[prop] = success.dataset.dadosformcliente[0][prop];
                        }
                    }

                    const rowResult = success.dataset.dadosformcliente[0];
                    widgetMain.currentRow.CDMUNICIPIO = rowResult.CDMUNICIPIO;
                    var fieldsEndereco = ["NMPAIS", "SGESTADO", "NMMUNICIPIO"];

                    for (var i = 0; i < fieldsEndereco.length; ++i) {
                        widgetMain.getField(fieldsEndereco[i]).readOnly = false;
                        widgetMain.getField(fieldsEndereco[i]).validations = {
                            required: true
                        };
                    }

                    if (typeof widgetMain.currentRow.SEGMENTO !== "undefined" && widgetMain.currentRow.SEGMENTO !== null && widgetMain.currentRow.SEGMENTO !== "") {
                        widgetMain.getField("RAMOATIVIDADE").readOnly = false;
                        widgetMain.getField("RAMOATIVIDADE").validations = {
                            required: true
                        };

                        widgetMain.getField("RAMOATIVIDADE").dataSourceFilter = [
                            {
                                "name": "CDSEGMER",
                                "operator": "=",
                                "value": widgetMain.currentRow.CDSEGMER
                            },
                            {
                                name: "IDATIVO",
                                operator: "=",
                                value: "S"
                            }
                        ];

                        widgetMain.getField("RAMOATIVIDADE").reload();

                    } else {
                        widgetMain.getField("RAMOATIVIDADE").readOnly = true;
                        widgetMain.getField("RAMOATIVIDADE").validations = {};
                    }

                    const {
                        CDMUNICIPIO,
                        CDPAIS,
                        SGESTADO
                    } = rowResult;
                    if (CDPAIS && SGESTADO && CDMUNICIPIO) {
                        filterMunicipio.forEach(filter => filter.value = rowResult[filter.name]);
                        let promise = fieldMunicipio.dataSource.filter(filterMunicipio);
                        promise.then((municipio) => {
                            if (municipio[0].CDREGIAO) {
                                filterRegiao.value = municipio[0].CDREGIAO;
                                Tco01100_proposta_comercial_service.getRegiao([filterRegiao]).then((success) => {
                                    var dataset = success.dataset.regiao;

                                    if (dataset && dataset.length) {
                                        widgetMain.currentRow.NMREGIAO = success.dataset.regiao[0].NMREGIAO;
                                    } else {
                                        widgetMain.currentRow.CDREGIAO = '';
                                        widgetMain.currentRow.NMREGIAO = '';
                                    }

                                }).catch((error) => {
                                    ScreenService.showMessage(ScreenService.i18n(error.error, widgetMain.currentRow));
                                });
                            } else {
                                widgetMain.currentRow.CDREGIAO = '';
                                widgetMain.currentRow.NMREGIAO = '';
                            }
                        });
                    } else {
                        widgetMain.currentRow.CDREGIAO = '';
                        widgetMain.currentRow.NMREGIAO = '';
                    }
                } else {
                    var fieldToClear = ["SEGMENTO", "RAMOATIVIDADE", "CDPAIS", "NMPAIS", "SGESTADO", "NMMUNICIPIO", "CDMUNICIPIO", "NMREGIAO", "DSENDERECO"];

                    for (var j = 0; j < fieldToClear.length; ++j) {
                        if (fieldToClear[j] !== "SEGMENTO" && fieldToClear[j] !== "NMPAIS" && fieldToClear[j] !== "DSENDERECO") {
                            widgetMain.currentRow[fieldToClear[j]] = "";
                            widgetMain.getField(fieldToClear[j]).readOnly = true;
                            widgetMain.getField(fieldToClear[j]).validations = {};
                        } else {
                            widgetMain.currentRow[fieldToClear[j]] = "";
                        }
                    }
                }
            });
        } else {

            let filter = [
                {
                    "name": "CDPROSPECT",
                    "operator": "=",
                    "value": widgetMain.currentRow.CDPROSPECT
                }
            ];

            Tco01100_proposta_comercial_service.findDadosFormProspect(filter).then((success) => {
                if (success.dataset.dadosformprospect.length > 0) {
                    for (var prop in success.dataset.dadosformprospect[0]) {
                        if (widgetMain.currentRow.hasOwnProperty(prop) && prop !== "__is_new") {
                            widgetMain.currentRow[prop] = success.dataset.dadosformprospect[0][prop];
                        }
                    }

                    let rowResult = success.dataset.dadosformprospect[0];
                    widgetMain.currentRow.CDMUNICIPIO = rowResult.CDMUNICIPIO;

                    var fieldsEndereco = ["NMPAIS", "SGESTADO", "NMMUNICIPIO"];

                    for (var i = 0; i < fieldsEndereco.length; ++i) {
                        widgetMain.getField(fieldsEndereco[i]).readOnly = false;
                        widgetMain.getField(fieldsEndereco[i]).validations = {
                            required: true
                        };
                    }

                    if (typeof widgetMain.currentRow.SEGMENTO !== "undefined" && widgetMain.currentRow.SEGMENTO !== null && widgetMain.currentRow.SEGMENTO !== "") {
                        widgetMain.getField("RAMOATIVIDADE").readOnly = false;
                        widgetMain.getField("RAMOATIVIDADE").validations = {
                            required: true
                        };

                        widgetMain.getField("RAMOATIVIDADE").dataSourceFilter = [
                            {
                                "name": "CDSEGMER",
                                "operator": "=",
                                "value": widgetMain.currentRow.CDSEGMER
                            },
                            {
                                name: "IDATIVO",
                                operator: "=",
                                value: "S"
                            }
                        ];

                        widgetMain.getField("RAMOATIVIDADE").reload();
                    } else {
                        widgetMain.getField("RAMOATIVIDADE").readOnly = true;
                        widgetMain.getField("RAMOATIVIDADE").validations = {};
                    }

                    const {
                        CDMUNICIPIO,
                        CDPAIS,
                        SGESTADO
                    } = rowResult;

                    if (CDPAIS && SGESTADO && CDMUNICIPIO) {
                        filterMunicipio.forEach(filter => filter.value = rowResult[filter.name]);
                        let promise = fieldMunicipio.dataSource.filter(filterMunicipio);
                        promise.then((municipio) => {
                            if (municipio[0].CDREGIAO) {
                                filterRegiao.value = municipio[0].CDREGIAO;
                                Tco01100_proposta_comercial_service.getRegiao([filterRegiao]).then((success) => {
                                    var dataset = success.dataset.regiao;

                                    if (dataset && dataset.length) {
                                        widgetMain.currentRow.NMREGIAO = success.dataset.regiao[0].NMREGIAO;
                                    } else {
                                        widgetMain.currentRow.CDREGIAO = '';
                                        widgetMain.currentRow.NMREGIAO = '';
                                    }
                                }).catch((error) => {
                                    ScreenService.showMessage(ScreenService.i18n(error.error, widgetMain.currentRow));
                                });
                            } else {
                                widgetMain.currentRow.CDREGIAO = '';
                                widgetMain.currentRow.NMREGIAO = '';
                            }
                        });
                    } else {
                        widgetMain.currentRow.CDREGIAO = '';
                        widgetMain.currentRow.NMREGIAO = '';
                    }
                } else {
                    var fieldToClear = ["SEGMENTO", "RAMOATIVIDADE", "CDPAIS", "NMPAIS", "SGESTADO", "NMMUNICIPIO", "CDMUNICIPIO", "NMREGIAO", "DSENDERECO"];

                    for (var j = 0; j < fieldToClear.length; ++j) {
                        if (fieldToClear[j] !== "SEGMENTO" && fieldToClear[j] !== "NMPAIS" && fieldToClear[j] !== "DSENDERECO") {
                            widgetMain.currentRow[fieldToClear[j]] = "";
                            widgetMain.getField(fieldToClear[j]).readOnly = true;
                            widgetMain.getField(fieldToClear[j]).validations = {};
                        } else {
                            widgetMain.currentRow[fieldToClear[j]] = "";
                        }
                    }
                }
            });
        }

    };

    /**
     * Conjunto de instruções a serem executadas
     * ao entrar pela primeira vez na widget 
     * principal.
     * 
     * @param {Args} args Objeto args.
     */
    this.onEnterOnceMainWidget = function(args) {
        this.setAtivClieMask();

        TcoUtil.startFilter(args.owner);

    };

    this.onChangeGerenteNegocio = function(args) {
        General.toggleReadOnlySelect(args.owner.field, args.owner.field.widget.getField('NMGERENTENEGOCIO'));
        args.owner.field.widget.currentRow.NRGERENTENEGOCIO = args.owner.field.widget.currentRow.TODAS_RESPONSAVEIS;
        templateManager.updateTemplate();
    };

    this.onChangeCliente = function(args) {
        var fieldCliente = args.owner.field.widget.getField('CLIENTE');
        var fieldTodas_Clientes = args.owner.field;
        if (fieldTodas_Clientes.checked === true) {
            fieldCliente.readOnly = false;
            templateManager.updateTemplate();
        } else {
            fieldCliente.setValue("");
            fieldCliente.readOnly = true;
            templateManager.updateTemplate();
        }
    };

    this.onChangeProspect = function(args) {
        var fieldProspect = args.owner.field.widget.getField('PROSPECT');
        var fieldTodas_Prospects = args.owner.field;
        if (fieldTodas_Prospects.checked === true) {
            fieldProspect.readOnly = false;
            templateManager.updateTemplate();
        } else {
            fieldProspect.setValue("");
            fieldProspect.readOnly = true;
            templateManager.updateTemplate();
        }
    };

    /**
     * Cancela formulário de edição de registro.
     */
    this.cancelForm = function() {
        var widgetMain = this.getWidget('widgetMain');

        General.widgetCancelForm(widgetMain);
        lockAddressFields();
    };

    /**
     * Bloqueia a inserção nos campos de endereço
     * na widget principal;
     */
    var lockAddressFields = () => {
        var widgetMain = this.getWidget('widgetMain');

        widgetMain.getField('SGESTADO').readOnly = true;
        widgetMain.getField('NMMUNICIPIO').readOnly = true;
    };

    /**
     * Define mascara de Atividade de Cliente
     * no field de ramo de atividade.
     */
    this.setAtivClieMask = function() {
        var widgetMain = this.getWidget('widgetMain');
        var ramoAtividadeField = widgetMain.getField('RAMOATIVIDADE');
        var ativClieMask = '';

        Tco01100_proposta_comercial_service.findParamgeral().then((success) => {
            if (success && success.error) {
                MessageService.showMessageByCode(success.error, widgetMain.currentRow);
            } else {
                if (typeof success.dataset.paramgeral !== "undefined" && success.dataset.paramgeral.length > 0) {
                    ativClieMask = success.dataset.paramgeral[0].CDPICTATIVCL;
                }

                ramoAtividadeField.fields.filter(field => field.name == 'CDARVATIVCLI')[0].mask = {
                    "type": "fix",
                    "params": {
                        "mask": ativClieMask
                    }
                };
            }

        }).catch((error) => {
            ScreenService.showMessage(ScreenService.i18n(error.error, widgetMain.currentRow));
        });
    };

    /**
     * Retorna widget cujo nome é passado por parâmetro.
     * 
     * @param {String} widgetName Nome da widget a ser
     *  retornada.
     */
    this.getWidget = function(widgetName) {
        return templateManager.container.getWidget(widgetName);
    };

    /**
     * Salva inserção ou alteração de proposta comercial
     * e trata possíveis erros.
     */
    this.saveProposta = function() {
        var widgetMain = this.getWidget('widgetMain');
        var cRow = widgetMain.currentRow;
        validaPeriodoProposta(widgetMain).then((callback) => {
            if (callback) {
                if (widgetMain.isValid()) {
                    Tco01100_proposta_comercial_service.saveProposta(cRow).then((success) => {
                        if (success && success.error) {
                            MessageService.showMessageByCode(success.error, cRow);
                        } else {
                            widgetMain.reload();

                            let colToRtrn = success.dataset.columnsToReturn;
                            let msg = cRow.__is_new ? 'Adicionado com sucesso.' : 'Registro salvo com sucesso.';

                            ScreenService.successNotification(msg);
                            widgetMain.view();

                            lockAddressFields();

                            cRow.__is_new = false;
                            cRow.DTULTATU = moment().format('DD/MM/YYYY');
                            cRow.CLIENTEPROSPECT = cRow.CLIENTE || cRow.PROSPECT;
                            cRow.NRPROPOSTACOMERCIAL = colToRtrn.NRPROPOSTACOMERCIAL;
                            cRow.CDPROPOSTACOMERCIAL = colToRtrn.CDPROPOSTACOMERCIAL;

                            General.changeWidgetAcessibility(widgetMain.widgets, true);
                        }
                    }).catch((error) => {
                        if (error.error) {
                            ScreenService.showMessage(ScreenService.i18n(error.error, cRow));
                        }
                    });
                }
            } else {
                MessageService.showMessageByCode('TCO01100_PERIODO_VG_INVALIDO');
            }
        });
    };

    /**
     * Valida período da proposta comercial de acordo com os dias definidos 
     * nos parâmetros comerciais.
     */
    var validaPeriodoProposta = function(widgetMain) {
        var row = widgetMain.currentRow;
        var dtInicial = moment(row.VIGENCIA.start, 'DD/MM/YYYY');
        var dtFinal = moment(row.VIGENCIA.end, 'DD/MM/YYYY');

        return Tco01100_proposta_comercial_service.findParamTCO().then((success) => {
            if (success.dataset.paramtco[0].QTDIASVIGENPROPCOM != null && success.dataset.paramtco[0].QTDIASVIGENPROPCOM != "" && success.dataset.paramtco[0].QTDIASVIGENPROPCOM != "0") {
                var periodoVigenciaPropcom = parseInt(success.dataset.paramtco[0].QTDIASVIGENPROPCOM);
                if (dtFinal.diff(dtInicial, 'days') > periodoVigenciaPropcom) {
                    widgetMain.QTDIASVIGENPROPCOM = widgetMain.QTDIASVIGENPROPCOM != success.dataset.paramtco[0].QTDIASVIGENPROPCOM ? success.dataset.paramtco[0].QTDIASVIGENPROPCOM : widgetMain.QTDIASVIGENPROPCOM;
                    return false;
                } else {
                    return true;
                }
            } else {
                return true;
            }
        });
    };

    /**
     * Define filtro no campo de endereço de acordo com
     * os parâmetros definidos e o valores da currentRow
     * da widget onde se encontra o field.
     * 
     * @param {Field} field Field onde será aplicado
     *  filtro, deve possuir as propriedades 'filterName'
     *  e 'orderName', que indicam respectivamente, o 
     *  campo da currentRow usado para filtro e o 
     *  campo usado para ordem.
     */
    this.setAddressFieldFilter = function(field) {
        this.applyFieldOrderName(field);
        this.applyFieldFilterName(field);
    };

    /**
     * Aplica a propriedade 'orderName' ao filtro do
     * field passado por parametro.
     * 
     * @param {Field} field Field a ser aplicado.
     */
    this.applyFieldOrderName = function(field) {
        let cRow = field.widget.currentRow;

        if (field.orderName) {
            if (Array.isArray(field.dataSourceFilter)) {
                let filterClause = field.dataSourceFilter.find(function(f) {
                    return (f.operator == 'ORDER_BY');
                });
                if (filterClause) {
                    filterClause.name = [{
                        name: field.orderName,
                        direction: 'ASC'
                    }];
                } else {
                    field.dataSourceFilter.push({
                        operator: 'ORDER_BY',
                        name: [{
                            name: field.orderName,
                            direction: 'ASC'
                        }]
                    });
                }
            } else {
                field.dataSourceFilter = [{
                    operator: 'ORDER_BY',
                    name: [{
                        name: field.orderName,
                        direction: 'ASC'
                    }]
                }];
            }
        }
    };

    /**
     * Aplica a propriedade 'filterName' ao filtro do
     * field passado por parametro.
     * 
     * @param {Field} field Field a ser aplicado.
     */
    this.applyFieldFilterName = function(field) {
        let cRow = field.widget.currentRow;

        if (field.filterName) {
            field.filterName.forEach((fName) => {
                if (Array.isArray(field.dataSourceFilter)) {
                    let filterClause = field.dataSourceFilter.find(function(f) {
                        return (f.operator == '=') && (f.name == fName);
                    });
                    if (filterClause) {
                        filterClause.value = cRow[fName];
                    } else {
                        field.dataSourceFilter.push({
                            name: fName,
                            operator: '=',
                            value: cRow[fName]
                        });
                    }
                }

            });
        }
    };

    /**
     * Conjunto de instruções a serem executadas
     * ao alterar algum campo de endereço (Pais, 
     * Estado, Municipio, Bairro). Limpa na currentRow
     * as propriedades indicadas no JSON do 
     * field indicados por 'propertyNamesToClear';
     * Define o estado readOnly do campo dependente
     * e aplica o filtro e a própria função de forma
     * recursiva a esse field.
     * 
     * @param {Args|Field} args Function argument.
     */
    this.onChangeAddressField = function(args) {
        var field = args instanceof Field ? args : args.owner.field;
        var widget = field.widget;
        var readOnlyState = !(field.value());
        var dependentField = widget.getField(field.dependentFieldName);
        var propertyNamesToClear = field.propertyNamesToClear;

        if (field.propertyNamesToClear)
            this.clearRowValues(widget.currentRow, propertyNamesToClear);

        if (dependentField !== undefined) {
            this.setAddressFieldFilter(dependentField);

            TcoUtil.changeRequiredField(dependentField, !readOnlyState);
            dependentField.readOnly = readOnlyState;
            dependentField.value('');
            dependentField.reload();

            this.onChangeAddressField(dependentField);
        }
    };

    /**
     * Conjunto de instruções a serem executadas ao 
     * alterar o municipio.
     * 
     * @param {Args} args Function arguments.
     */
    this.onChangeMunicipio = function(args) {
        var field = args.owner.field;
        var widget = field.widget;
        var cRow = widget.currentRow;

        var filter = [{
            name: "CDREGIAO",
            operator: "=",
            value: cRow.CDREGIAO
        }];

        Tco01100_proposta_comercial_service.getRegiao(filter).then((success) => {
            if (success && success.error) {
                MessageService.showMessageByCode(success.error, cRow);
            }
            var dataset = success.dataset.regiao;

            if (dataset && dataset.length) {
                cRow.NMREGIAO = success.dataset.regiao[0].NMREGIAO;
            } else {
                cRow.CDREGIAO = '';
                cRow.NMREGIAO = '';
            }
        }).catch((error) => {
            ScreenService.showMessage(ScreenService.i18n(error.error, cRow));
        });
    };

    /**
     * Filtra campo de ramo de atividade
     * de acordo com segmento.
     */
    this.onChangeSegmento = function(args) {
        var segmentoField = args.owner.field;
        var cRow = segmentoField.widget.currentRow;
        var ramoField = segmentoField.widget.getField('RAMOATIVIDADE');
        if (segmentoField.value() != "") {
            ramoField.dataSourceFilter = [{
                    "operator": "ORDER_BY",
                    "name": [{
                        "name": "CDARVATIVCLI",
                        "direction": "ASC"
                    }]
                },
                {
                    "name": "IDATIVO",
                    "operator": "=",
                    "value": "S"
                },
                {
                    "name": "CDSEGMER",
                    "operator": "=",
                    "value": cRow.CDSEGMER
                }
            ];

            ramoField.reload();
            ramoField.readOnly = false;
            ramoField.validations = {
                required: true
            };
        } else {
            ramoField.setValue("");
            ramoField.reload();
            ramoField.readOnly = true;
            ramoField.validations = {};
        }

    };

    /**
     * Aplica filtro na widget de acordo com o filtro passado
     * por parâmetro. Modelo do filtro deve ser o seguinte:
     * '{columnName : value}'
     * 
     * @param {Widget} widget Widget a ser aplicado filtro.
     * @param {Object} filter Objeto contendo chaves e 
     *  valores a serem aplicados como filtro na widget.
     */
    this.setWidgetFilter = function(widget, filter) {
        widget.dataSourceFilter = [];

        Object.keys(filter).forEach(key => {
            widget.dataSourceFilter.push({
                name: key,
                operator: "=",
                value: filter[key]
            });
        });

        widget.dataSourceFilter.push({
            operator: "ORDER_BY",
            name: [{
                name: widget.orderName,
                direction: 'ASC'
            }]
        });
    };

    /**
     * Conjunto de instruções a serem executadas
     * ao selecionar registro no grid.
     * 
     * @param {Args} args Objeto args.
     */
    this.afterMoveRowMainWidget = function(args) {
        this.changeClienteProspectFieldsByRetencao(args);
        this.setVigenciaValuesOnCurrentRow(args);
    };

    /**
     * Define os valores de vigencia no campo de vigencia
     * ao selecionar um registro.
     * 
     * @param {Args} args Objeto args. 
     */
    this.setVigenciaValuesOnCurrentRow = function(args) {
        var widget = args.owner;
        var cRow = widget.currentRow;

        cRow.VIGENCIA = {
            start: cRow.DTINIVIGENCIA,
            end: cRow.DTFIMVIGENCIA
        };
    };

    /**
     * Altera a visibilidade dos fields de prospect
     * ou cliente de acordo com o estado da retenção
     * ao selecionar registro, e define a descrição do(s) campo(s)
     * referentes ao cliente/prospect.
     * 
     * @param {Args} args Objeto args.
     */
    this.changeClienteProspectFieldsByRetencao = function(args) {
        var widget = args.owner;
        var cRow = widget.currentRow;
        var fieldDescription = args.row.CLIENTEPROSPECT;

        if (!args.row.__is_new) {
            if (args.row.IDRETENCAO == 'S') {
                cRow.CLIENTE = fieldDescription;
                this.switchClienteProspectFields(widget, 'C');
            } else {
                cRow.PROSPECT = fieldDescription;
                this.switchClienteProspectFields(widget, 'P');
            }
        }

    };

    /**
     * Adiciona nova função à proposta selecionada.
     * 
     * @param {Args} args Objeto args.
     */
    this.newVersion = function(args) {
        var widget = args.owner.widget;
        var rowToSave = {
            NRPROPOSTACOMERCIAL: widget.getParent().currentRow.NRPROPOSTACOMERCIAL
        };

        Tco01100_proposta_comercial_service.newVersion(rowToSave).then((response) => {
            if (response && response.error) {
                MessageService.showMessageByCode(response.error, widget.getParent().currentRow);
            } else {
                ScreenService.successNotification('Adicionado com sucesso.');
                widget.reload();
            }
        }).catch((error) => {
            if (error.error) {
                ScreenService.showMessage(ScreenService.i18n(error.error, widget.getParent().currentRow));
            }
        });
    };

    /**
     * Conjunto de instruções a serem executadas ao selecionar
     * registro no grid de versões.
     * 
     * @param {Args} args Objeto args.
     */
    this.afterMoveRowVersaoWidget = function(args) {
        args.owner.showCheckbox = false;
        this.openServicoVersaoGrid();
        this.filterServicoVersaoGrid();
        setActionsVisibilityVersaoWidget();
    };

    /**
     * Abre grid de serviços da proposta comercial.
     */
    this.openServicoVersaoGrid = function() {
        var servicoVersaoTabbed = this.getWidget('servicoVersaoTabbed');
        var servicoVersaoGrid = this.getWidget('servicoVersaoGrid');
        ScreenService.openSwipe(servicoVersaoTabbed);
        servicoVersaoTabbed.getParent().editing = true;
        servicoVersaoGrid.dataSource.clearCheckedRows();
    };

    /**
     * Conjunto de instruções a serem executadas ao fechar
     * o swipe na widget de Serviço da Proposta.
     */
    this.onCloseSwipeServicoVersaoGrid = function() {
        var servicoVersaoTabbed = this.getWidget('servicoVersaoTabbed');

        if (servicoVersaoTabbed) {
            var versaoWidget = servicoVersaoTabbed.getParent();

            versaoWidget.editing = false;
            versaoWidget.showCheckbox = true;
            versaoWidget.redraw();
            versaoWidget.reload();
        }
    };

    /**
     * 
     * 
     */
    this.openReport = function(args) {
        var widget = args.owner.widget;
        var rowRelatorio;
        var relatorioPorByWidget = {
            versaoWidget: "TCO01100",
            servicoVersaoGrid: "TCO01101",
            widgetDetalhes: "TCO01102",
            widgetHistorico: "TCO01103"
        };

        if (widget.name === "versaoWidget") {
            rowRelatorio = widget.container.getWidget("widgetMain").currentRow;
            rowRelatorio.REPORT_NAME = relatorioPorByWidget[widget.name];

            chamaraRelatorio(rowRelatorio, widget);

        } else if (widget.name === "servicoVersaoGrid") {
            rowRelatorio = widget.container.getWidget("versaoWidget").currentRow;
            rowRelatorio.REPORT_NAME = relatorioPorByWidget[widget.name];

            chamaraRelatorio(rowRelatorio, widget);

        } else if (widget.name === "widgetDetalhes") {
            rowRelatorio = widget.currentRow;
            rowRelatorio.REPORT_NAME = relatorioPorByWidget[widget.name];

            chamaraRelatorio(rowRelatorio, widget);

        } else if (widget.name === "widgetHistorico") {
            rowRelatorio = widget.container.getWidget('widgetDetalhes').currentRow;
            if (typeof widget.currentRow.NRHISTNOTIFPROPCOM !== "undefined" && widget.currentRow.NRHISTNOTIFPROPCOM !== null) {
                rowRelatorio.REPORT_NAME = relatorioPorByWidget[widget.name];
                rowRelatorio.NRDETALHESERVPROPCOM = widget.currentRow.NRDETALHESERVPROPCOM;
                rowRelatorio.NRHISTNOTIFPROPCOM = widget.currentRow.NRHISTNOTIFPROPCOM;

                chamaraRelatorio(rowRelatorio, widget);

            } else {
                ScreenService.showMessage("Nenhum registro selecionado!");
            }
        }
    };

    var chamaraRelatorio = function(row, widget) {

        Tco01100_proposta_comercial_service.chamadaRelatorio(row).then(function(response) {
            if (response && response.messages) {
                var report = response.messages.pop();
                var reportOpened = window.open(report.message);
                if (!reportOpened) {
                    ScreenService.showMessage(ScreenService.i18n('Bloqueador de Pop-up está ativado!'));
                }
            }
            if (response && response.dataset && response.dataset.RESPONSE) {
                widget.reportRow.P_NRLISTA = response.dataset.RESPONSE.NRLISTA;
                widget.hasList = true;
            }
        }, function(response) {
            if (response.data && response.data.dataset && response.data.dataset.ERROS) {
                TcoUtil.buildMessagesToShow(response.data.dataset.ERROS).then(function(mensagens) {
                    General.showErrorMessages(widget, mensagens, ScreenService.i18n("Erro"));
                });
            }
        });

    };

    /**
     * Aplica o filtro na widget principal.
     * 
     * @param {Args} args Objeto args.
     */
    this.applyCustomFilterMainWidget = function(args) {
        args.e.preventDefault();
        if (filterMainWidget(args.e.preference.FILTER_ROW)) {
            afterApplyFilterMainWidget();
        }
    };

    /**
     * Define filtro customizado na widget principal 
     * de acordo com os parâmetros escolhidos 
     * no popup filtro.
     * 
     * @param {Args} args Objeto args.
     */
    var filterMainWidget = (filter) => {
        var filterRow = Object.assign({}, filter);
        var widgetMain = this.getWidget('widgetMain');

        filterRow.DTINICIAL = filterRow.VIGENCIA.start;

        filterRow.DTFINAL = filterRow.VIGENCIA.end;

        var filterNames = {
            "DTINICIAL": "DTINICIAL",
            "DTFINAL": "DTFINAL",
            "CLIENTE": "CLIENTES",
            "NRGERENTENEGOCIO": "GERENTES",
            "PROSPECT": "PROSPECTS",
            "SITUACAO": "SITUACAO"
        };

        filter = [];

        filterRow.NRGERENTENEGOCIO = filterRow.TODAS_RESPONSAVEIS === 'T' ? filterRow.TODAS_RESPONSAVEIS : filterRow.NRGERENTENEGOCIO;
        // verificando o que há e adicionando valores em filterRow.CLIENTE e filterRow.PROSPECT

        if (filterRow.TODAS_CLIENTES === 'N') {
            filterRow.CLIENTE = "";
        }

        if (filterRow.TODAS_PROSPECTS === 'N') {
            filterRow.PROSPECT = "";
        }

        if (filterRow.TODAS_CLIENTES === 'N' && filterRow.TODAS_PROSPECTS === 'N') {
            MessageService.showMessageByCode("TCO01100_FILTRO_INVALIDO");
            return false;
        }

        Object.keys(filterNames).forEach(fName => {
            let value = Util.isEmptyOrBlank(filterRow[fName]) ? 'T' : filterRow[fName];
            if (fName === "CLIENTE") {
                if (filterRow.TODAS_CLIENTES === 'N') {
                    value = "";
                }
            }
            if (fName === "PROSPECT") {
                if (filterRow.TODAS_PROSPECTS === 'N') {
                    value = "";
                }
            }

            filter.push({
                name: filterNames[fName],
                operator: "=",
                value: value
            });
        });

        filter = filter.concat([{
            name: "CDPROPOSTACOMERCIAL",
            operator: "=",
            value: filterRow.CDPROPOSTACOMERCIAL || ''
        }]);

        widgetMain.dataSourceFilter = filter;
        widgetMain.reload().then(() => {
            widgetMain.activate();
            widgetMain.filterRowBackup = filterRow;
        });
        return true;
    };

    /**
     * Executa as ações após a aplicação do filtro.
     */
    var afterApplyFilterMainWidget = () => {
        ScreenService.closePopup(true);
    };

    /**
     * Implementa outData no campo de vigencia.
     * 
     * @param {Args} args Objeto args.
     */
    this.outDataVigencia = function(args) {
        var field = args.owner.field;
        var widget = field.widget;

        try {
            widget.currentRow.DTINICIAL = field.value().start || '';
            widget.currentRow.DTFINAL = field.value().end || '';
            widget.currentRow.VIGENCIA.formattedStart = field.value().start || '';
            widget.currentRow.VIGENCIA.formattedEnd = field.value().end || '';
        } catch (e) {
            console.warn(ScreenService.i18n('TCO_INTERVAL_OUTDATA_ERROR'));
        }
    };

    /**
     * Abre formulário para adição de novo serviço da 
     * proposta.
     */
    this.addServicoProposta = function() {
        var widgetServicoProposta = this.getWidget('servicoVersaoGrid');

        if (isNotAprovedPropCom()) {

            var fieldsToJustInteger = [
                "QTSEGSEX",
                "QTSABADO",
                "QTDOMFERI",
                "QTTOTDIAS",
                "QTTURNOS",
                "QTTOTREFEICOES"
            ];

            var promises = [
                widgetServicoProposta.getField("SERVICO").reload(),
                widgetServicoProposta.getField("PADRAOQUALIDADE").reload(),
                widgetServicoProposta.getField("TIPOPUBLICO").reload()
            ];

            ZHPromise.all(promises).then(() => {
                this.applyJustInteger(fieldsToJustInteger);

                this.newRow(widgetServicoProposta);
            });
        }
    };

    /**
     * Aplica máscara de digitação que permite somente
     * inteiros no campo passado por parâmetro.
     * 
     * @param {Array{String}} fieldsToJustInteger Campos
     *  para aplicar mascara de digitação.
     */
    this.applyJustInteger = function(fieldsToJustInteger) {
        fieldsToJustInteger.forEach((fName) => {
            General.justInteger('#' + fName);
        });
    };

    /**
     * Conjunto de instruções a serem executadas ao 
     * alterar valor de algum campo de alguma refeição
     * estimada.
     */
    this.onChangeRefeicoesEstimadas = function() {
        var servicoVersaoGrid = this.getWidget('servicoVersaoGrid');
        var cRow = servicoVersaoGrid.currentRow;

        calculateTotalDiasTrabalhados(cRow);
        calculateTotalRefeicoesEstimadas(cRow);
    };

    /**
     * Calcula os totais de dias trabalhados
     * e define o valor na curretRow da widget de 
     * serviço da proposta.
     * 
     * @param {Object} cRow Current row da widget
     *  de proposta.
     */
    var calculateTotalDiasTrabalhados = (cRow) => {
        cRow.QTTOTDIAS = 0;
        if (cRow.QTSEGSEX) cRow.QTTOTDIAS += 21;
        if (cRow.QTSABADO) cRow.QTTOTDIAS += 5;
        if (cRow.QTDOMFERI) cRow.QTTOTDIAS += 4;
    };

    /**
     * Calcula os totais de refeições estimadas
     * e define o valor na curretRow da widget de 
     * serviço da proposta.
     * 
     * @param {Object} cRow Current row da widget
     *  de proposta.
     */
    var calculateTotalRefeicoesEstimadas = (cRow) => {
        cRow.QTTOTREFEICOES = 0;
        if (cRow.QTSEGSEX) cRow.QTTOTREFEICOES += cRow.QTSEGSEX * 21;
        if (cRow.QTSABADO) cRow.QTTOTREFEICOES += cRow.QTSABADO * 5;
        if (cRow.QTDOMFERI) cRow.QTTOTREFEICOES += cRow.QTDOMFERI * 4;
    };

    /**
     * Salva inserção de serviço da proposta comercial.
     */
    this.insertServicoPropCom = function() {
        var servicoVersaoGrid = this.getWidget('servicoVersaoGrid');
        var widgetMain = this.getWidget('widgetMain');
        var cRow = servicoVersaoGrid.currentRow;

        Object.keys(widgetMain.currentRow).forEach(prop => {
            cRow[prop] = widgetMain.currentRow[prop];
        });

        var versaoRow = servicoVersaoGrid.getParent().getParent().currentRow;

        if (servicoVersaoGrid.isValid() && isNotAprovedPropCom()) {
            delete cRow.$error;

            cRow.NRVERSAOPROPCOM = versaoRow.NRVERSAOPROPCOM;
            cRow.CDVERSAOPROPCOM = versaoRow.CDVERSAOPROPCOM;

            Tco01100_proposta_comercial_service.insertServicoPropCom(cRow).then((success) => {
                let colToRtrn = success.dataset.columnsToReturn;

                cRow.__is_new = false;
                cRow.DTULTATU = moment().format('DD/MM/YYYY HH:mm:ss');
                cRow.IDSERVICOPROPCOM = 'PENDENTE';
                cRow.NRSERVICOPROPCOM = colToRtrn.NRSERVICOPROPCOM;
                cRow.NRDETALHESERVPROPCOM = colToRtrn.NRDETALHESERVPROPCOM || null;

                if (colToRtrn && colToRtrn.ERROR_CODE) {
                    MessageService.confirmMessageByCode(colToRtrn.ERROR_CODE).then(() => {
                        cRow.MOTIVO = 'SEMCARDBASE';

                        Tco01100_proposta_comercial_service.notificarPlan(cRow).then((success) => {
                            if (success && success.error) {
                                MessageService.showMessageByCode(success.error);
                            } else {
                                cRow.IDSERVICOPROPCOM = 'AGUARDANDOPLANEJAMENTO';
                                ScreenService.alertNotification('Equipe de planejamento notificada');
                                cardBaseErrorHandler();
                            }
                        }).catch((error) => {
                            if (error.error) {
                                if (error.error.indexOf('TCOPROC_') >= 0) {
                                    MessageService.showMessageByCode(error.error);
                                } else {
                                    if (error.error.indexOf('TCO01100_') >= 0 || error.error.indexOf('TCO_') >= 0)
                                        MessageService.showMessageByCode(error.error);
                                    else
                                        ScreenService.showMessage(ScreenService.i18n(error.error, cRow));
                                }
                            }
                        });
                    }).catch(() => {
                        cardBaseErrorHandler();
                    });

                    var cardBaseErrorHandler = () => {
                        this.openDetailServPage(cRow);
                        MessageService.alertNotificationByCode('TCO01100_SUCCESS_WITH_NO_CARDBASE');
                    };
                } else {
                    cRow.NRCARDAPIOBASE = colToRtrn.NRCARDAPIOBASE[0].NRCARDAPIOBASE || null;
                    this.openDetailServPage(cRow);
                    ScreenService.successNotification('Adicionado com sucesso.');
                }
            }).catch((error) => {
                if (error.error) {
                    if (error.error.indexOf('TCO01100_') >= 0 || error.error.indexOf('TCO_') >= 0)
                        MessageService.showMessageByCode(error.error);
                    else
                        ScreenService.showMessage(ScreenService.i18n(error.error, cRow));
                }
            });
        }
    };

    /**
     * Define filtro na widget de Serviço da Proposta.
     */
    this.filterServicoVersaoGrid = function() {
        var versaoWidget = this.getWidget('versaoWidget');
        var servicoVersaoGrid = this.getWidget('servicoVersaoGrid');

        servicoVersaoGrid.dataSourceFilter = [{
                name: "NRVERSAOPROPCOM",
                operator: "=",
                value: versaoWidget.currentRow.NRVERSAOPROPCOM
        },
            {
                operator: "ORDER_BY",
                name: [{
                    name: "NRSERVICOPROPCOM",
                    direction: 'ASC'
                }]
        }];

        versaoWidget.reload().then(() => {
            let rowCorreta = versaoWidget.dataSource.data.filter((row) => row.NRVERSAOPROPCOM == versaoWidget.currentRow.NRVERSAOPROPCOM && row.NRPROPOSTACOMERCIAL == versaoWidget.currentRow.NRPROPOSTACOMERCIAL);
            versaoWidget.currentRow = rowCorreta[0];
            setActionsVisibilityVersaoWidget();
        });

        servicoVersaoGrid.reload();
    };

    /**
     * Remove serviços da versão que estiverem 
     * selecionados.
     */
    this.removeServicoProposta = function() {
        var servicoVersaoGrid = this.getWidget('servicoVersaoGrid');
        var widgetMain = this.getWidget('widgetMain');
        var widgetVersao = this.getWidget('versaoWidget');

        let currentRow = angular.copy(widgetMain.currentRow);
        currentRow.CDVERSAOPROPCOM = widgetVersao.currentRow.CDVERSAOPROPCOM;
        var selectedRows = servicoVersaoGrid.getCheckedRows();

        if (selectedRows.length) {
            ScreenService.confirmMessage(ScreenService.i18n('Delete Register(s)?'), 'question', (success) => {
                selectedRows = selectedRows.map(row => {
                    return {
                        NRSERVICOPROPCOM: row.NRSERVICOPROPCOM,
                        NRVERSAOPROPCOM: servicoVersaoGrid.getParent().getParent().currentRow.NRVERSAOPROPCOM

                    };
                });

                Tco01100_proposta_comercial_service.removeServicoProposta(selectedRows).then((success) => {
                    if (success && success.error) {
                        MessageService.showMessageByCode(success.error);
                    } else {
                        ScreenService.showMessage('Removido com Sucesso.');
                        this.filterServicoVersaoGrid();
                        servicoVersaoGrid.dataSource.clearCheckedRows();
                        templateManager.updateTemplate();
                    }

                }).catch((error) => {
                    if (error.data && error.data.error && error.data.error.indexOf('TCOPROC_') > -1) {
                        MessageService.showMessageByCode(error.data.error);
                    } else if (error.error) {
                        ScreenService.showMessage(ScreenService.i18n(error.error, currentRow));
                    }

                });
            });
        }
    };

    /**
     * Remove versões da proposta que estiverem 
     * selecionados.
     */
    this.removeVersion = function() {
        var versaoWidget = this.getWidget('versaoWidget');
        var selectedRows = versaoWidget.getCheckedRows();

        if (selectedRows.length) {
            ScreenService.confirmMessage(ScreenService.i18n('Delete Register(s)?'), 'question', (success) => {
                selectedRows = selectedRows.map(row => {
                    return {
                        NRVERSAOPROPCOM: row.NRVERSAOPROPCOM
                    };
                });

                Tco01100_proposta_comercial_service.removeVersion(selectedRows).then((success) => {
                    if (success && success.error) {
                        MessageService.showMessageByCode(success.error, versaoWidget.currentRow);
                    } else {
                        afterRemoveRegister(versaoWidget, success);
                        versaoWidget.reload();
                        versaoWidget.dataSource.clearCheckedRows();
                    }
                }).catch((error) => {
                    if (error.error) {
                        ScreenService.showMessage(ScreenService.i18n(error.error, versaoWidget.currentRow));
                    }
                });
            });
        }
    };

    /**
     * Remove versões da proposta que estiverem 
     * selecionados.
     */
    this.removeProposta = function() {
        var widgetMain = this.getWidget('widgetMain');
        var rowToRemove = {
            NRPROPOSTACOMERCIAL: widgetMain.currentRow.NRPROPOSTACOMERCIAL
        };

        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', (success) => {
            Tco01100_proposta_comercial_service.removeProposta(rowToRemove).then((success) => {
                if (success && success.error) {
                    MessageService.showMessageByCode(success.error, widgetMain.currentRow);
                } else {
                    afterRemoveRegister(widgetMain, success);

                    if (!success.dataset) {
                        ScreenService.closeSwipe();
                        widgetMain.reload();
                    }
                }
            }).catch((error) => {
                if (error.error) {
                    ScreenService.showMessage(ScreenService.i18n(error.error, widgetMain.currentRow));
                }
            });
        });
    };

    /**
     * Conjunto de instruções a serem executadas
     * ao selecionar uma row na widget de serviço
     * da proposta.
     * 
     * @param {Args} args Objeto args do 
     *  evento.
     */
    this.onSelectRowServicoProposta = function(args) {
        var event = args.e;
        openDetailServPropCom(event);
    };

    /**
     * Função responsável por realizar rotina
     * de abertura de detalhe do serviço
     * 
     * @param {Event} event AfterMoveRow Event.
     */
    var openDetailServPropCom = (event) => {
        var widget = this.getWidget('servicoVersaoGrid');
        var versaoWidget = this.getWidget('versaoWidget');
        var widgetMain = this.getWidget('widgetMain');
        var cRow = widget.currentRow;
        cRow.NRPROPOSTACOMERCIAL = widgetMain.currentRow.NRPROPOSTACOMERCIAL;

        if (!cRow.__is_new) {
            event.stopPropagation();
            cRow.CDVERSAOPROPCOM = versaoWidget.currentRow.CDVERSAOPROPCOM;
            Tco01100_proposta_comercial_service.openServPropComDetail(cRow).then((response) => {
                let rData = response.dataset;

                if (rData.backParams && !rData.backParams.errorCode) {
                    cRow.NRCARDAPIOBASE = rData.backParams.cardBase;

                    var detalheServ = rData.backParams.detalheServ;
                    Object.keys(detalheServ).forEach(key => {
                        cRow[key] = detalheServ[key];
                    });
                } else {
                    MessageService.confirmMessageByCode(rData.backParams.errorCode).then(() => {
                        this.notifyPlanej(cRow);
                    });
                }

                this.openDetailServPage(cRow);
            });
        }
    };

    /**
     * Envia notificação ao planejamento caso o 
     * serviço não possua cardápio base relacionado.
     * 
     * @param {Object} row
     */
    this.notifyPlanej = function(row) {
        row.MOTIVO = 'SEMCARDBASE';
        Tco01100_proposta_comercial_service.notificarPlan(row).then((success) => {
            if (success && success.error) {
                MessageService.showMessageByCode(success.error);
            } else {
                ScreenService.alertNotification('Equipe de planejamento notificada');
            }
        }).catch((error) => {
            if (error.error) {
                if (error.error.indexOf('TCOPROC_') >= 0) {
                    MessageService.showMessageByCode(error.error);
                } else
                    ScreenService.showMessage(ScreenService.i18n(error.error));
            }
        });
    };

    /**
     * Abre a tela de detalhes de serviço.
     * 
     * @param {Object} selectedRow Registro a ser
     *  exibido no detalhe.
     */
    this.openDetailServPage = function(selectedRow) {
        $rootScope.__bkpWatchers = $rootScope.$$watchers;
        $rootScope.$$watchers = [];

        TcoUtil.openWindowByHistory('#/tco#tco01101_detalhes_servico', {
            'startedByPropCom': true,
            'row': selectedRow
        }, 'Detalhes do Serviço');
    };

    /**
     * Executa verificação e impressão do 
     * relatório de erros caso haja ou imprime
     * mensagem de sucesso na tela.
     * 
     * @param {Widget} widget Widget de qual
     *  foram removidos os dados.
     * @param {Object} successRemove Response de sucesso
     *  da requisição que excluiu os dados.
     */
    var afterRemoveRegister = (widget, successRemove) => {
        if (successRemove.dataset) {
            if (successRemove.dataset.invalidRows) {

                ScreenService.showMessage('Erro ao excluir registro(s).').then(() => {
                    let invalidRows = successRemove.dataset.invalidRows;

                    buildMessages(invalidRows).then(function(mensagem) {
                        General.showErrorMessages(widget, mensagem, ScreenService.i18n('Exclusão Inválida.'));
                    });
                });

            } else if (successRemove.dataset.invalidPropCom[0]) {

                MessageService.showMessageByCode('TCO01100_PROPCOM_INVALID_STATE', {
                    CDPROPOSTACOMERCIAL: this.getWidget('widgetMain').currentRow.CDPROPOSTACOMERCIAL
                });

            }
        } else {
            ScreenService.showMessage('Removido com Sucesso.');
        }
    };

    var buildMessages = function(data) {
        var mensagens = [];
        var msgParams = [];
        data.forEach(function(row) {
            //mensagens.push(ScreenService.i18n(row.MESSAGECODE, row));
            msgParams.push(row);
            mensagens.push(row.MESSAGECODE);
        });

        return MessageService.getMessagesByCode(mensagens).then(function(mensagensTraduzidas) {
            for (let i = 0; i < mensagensTraduzidas.length; i++) {
                mensagensTraduzidas[i] = ScreenService.i18n(mensagensTraduzidas[i], msgParams[i]);
            }
            return mensagensTraduzidas;
        });
    };

    /**
     * Abre widget para troca de responsavel
     * pela proposta comercial.
     */
    this.openChangeResponsavelPropCom = function() {
        var popupRespPropCom = this.getWidget('changeResponsavelPropComPopup');
        ScreenService.openPopup(popupRespPropCom);
        popupRespPropCom.edit();
        popupRespPropCom.getField('NMGERENTENEGOCIO').reload();
    };

    /**
     * Altera o responsável pela proposta comercial.
     */
    this.changeResponsavelPropCom = function() {
        var responsavelPropComPopup = this.getWidget('changeResponsavelPropComPopup');
        var rowToSend = {
            NRGERENTENEGOCIO: responsavelPropComPopup.currentRow.NRGERENTENEGOCIO,
            NRPROPOSTACOMERCIAL: this.getWidget('widgetMain').currentRow.NRPROPOSTACOMERCIAL
        };

        if (responsavelPropComPopup.isValid()) {
            Tco01100_proposta_comercial_service.changeResponsavelPropCom(rowToSend).then((success) => {
                if (success && success.error) {
                    MessageService.showMessageByCode(success.error, responsavelPropComPopup.currentRow);
                } else {
                    var widgetMain = this.getWidget('widgetMain');
                    var cRow = responsavelPropComPopup.currentRow;

                    ScreenService.successNotification('Registro alterado com sucesso.');
                    this.cancelChangeResponsavelPropCom();

                    widgetMain.currentRow.NMRESPONSAVEL = cRow.CDOPERADOR + ' - ' + cRow.NMOPERADOR;
                }
            }).catch((error) => {
                if (error.error) {
                    ScreenService.showMessage(ScreenService.i18n(error.error, responsavelPropComPopup.currentRow));
                }
            });
        }
    };

    /**
     * Cancela alteração de responsável pela proposta
     * comercial.
     */
    this.cancelChangeResponsavelPropCom = function() {
        var responsavelPropComPopup = this.getWidget('changeResponsavelPropComPopup');
        var gerenteField = responsavelPropComPopup.getField('NMGERENTENEGOCIO');
        ScreenService.closePopup();
        responsavelPropComPopup.currentRow = {};
    };

    /**
     * Conjunto de instruções a serem executadas ao
     * sair da widget de simulações.
     */
    this.backToPropostas = function(args) {
        let mainWidget = args.owner.widget.getParent();
        General.widgetGoBackToList(args.owner.widget);
        mainWidget.reload();
    };

    /**
     * Conjunto de instruções a serem executadas ao
     * entrar na widget de serviços da versão.
     */
    this.onEnterServicoTabbed = function() {
        setActionsVisibilityVersaoWidget();
    };

    /**
     * Altera visibilidade das ações de aprovação
     * e conclusão, e cancelamentos, de acordo com
     * o estado da versão.
     * 
     * @param {Widget} versaoWidget 
     */
    var setActionsVisibilityVersaoWidget = () => {
        var versaoWidget = this.getWidget('versaoWidget');
        var servicoVersaoGrid = this.getWidget('servicoVersaoGrid');

        let actionBySit = {
            aproveVersion: 'CONCLUIDA',
            concludeVersion: 'CALCULADA',
            cancelAproveVersion: 'APROVADA',
            cancelConcludeVersion: 'CONCLUIDA'
        };

        servicoVersaoGrid.actions.forEach(act => {
            if (actionBySit[act.name]) {
                act.isVisible = versaoWidget.currentRow.SITVERSAO == actionBySit[act.name];
            }
        });

        templateManager.updateTemplate();
        if (servicoVersaoGrid.redraw) servicoVersaoGrid.redraw();
    };

    /**
     * Conclui versão selecionada
     * na tela de versão. 
     */
    this.concludeVersion = function() {
        var versaoWidget = this.getWidget('versaoWidget');
        var servicoVersaoGrid = this.getWidget('servicoVersaoGrid');

        Tco01100_proposta_comercial_service.concludeVersion(versaoWidget.currentRow).then((respResult) => {
            if (respResult && respResult.error) {
                MessageService.showMessageByCode(respResult.error, versaoWidget.currentRow);
            } else {
                if (respResult && respResult.dataset && respResult.dataset.invalidOperation) {
                    var rData = respResult.dataset.invalidOperation;
                    if (rData.invalidServ) {
                        rData.invalidServ.forEach(invServ => {
                            invServ.MESSAGECODE = 'TCO01100_SERV_INVALID_STATE';
                        });
                    }

                    if (rData.invalidVer) {
                        let a = angular.copy(versaoWidget.currentRow);
                        a.MESSAGECODE = 'TCO01100_VER_INVALID_STATE';
                        rData.invalidServ.push(a);
                    }
                    buildMessages(rData.invalidServ).then(function(mensagem) {
                        ScreenService.showMessage('Erro ao realizar operação.').then(() => {
                            General.showErrorMessages(servicoVersaoGrid, mensagem, ScreenService.i18n('Operação Inválida.'));
                        });
                    });
                } else {
                    servicoVersaoGrid.getParent().currentRow.IDVERSAOPROPCOM = 'CONCLUIDA';
                    this.filterServicoVersaoGrid();
                    versaoWidget.currentRow.IDVERSAOPROPCOM = 'CONCLUIDA';
                    versaoWidget.currentRow.SITVERSAO = 'CONCLUIDA';

                    ScreenService.successNotification('Operação realizada com sucesso.');
                    setActionsVisibilityVersaoWidget();

                    templateManager.updateTemplate();
                }
            }

        }).catch((error) => {
            if (error.error) {
                ScreenService.showMessage(ScreenService.i18n(error.error, versaoWidget.currentRow));
            }
        });
    };

    /**
     * Cancela conclusão versão selecionada
     * na tela de versão.
     */
    this.cancelConcludeVersion = function() {
        var versaoWidget = this.getWidget('versaoWidget');
        var servicoVersaoGrid = this.getWidget('servicoVersaoGrid');

        Tco01100_proposta_comercial_service.cancelConcludeVersion(versaoWidget.currentRow).then((respResult) => {
            if (respResult && respResult.error) {
                MessageService.showMessageByCode(respResult.error, versaoWidget.currentRow);
            } else {
                if (respResult && respResult.dataset && respResult.dataset.invalidOperation) {
                    var rData = respResult.dataset.invalidOperation;

                    if (rData.invalidServ) {
                        rData.invalidServ.forEach(invServ => {
                            invServ.MESSAGECODE = 'TCO01100_SERV_INVALID_STATE';
                        });
                    }

                    if (rData.invalidVer) {
                        let a = angular.copy(versaoWidget.currentRow);
                        a.MESSAGECODE = 'TCO01100_VER_INVALID_STATE';
                        rData.invalidServ.push(a);
                    }
                    buildMessages(rData.invalidServ).then(function(mensagem) {
                        ScreenService.showMessage('Erro ao realizar operação').then(() => {
                            General.showErrorMessages(servicoVersaoGrid, mensagem, ScreenService.i18n('Operação Inválida.'));
                        });
                    });
                } else {
                    ScreenService.successNotification('Operação realizada com sucesso.');

                    servicoVersaoGrid.getParent().currentRow.IDVERSAOPROPCOM = 'CALCULADA';
                    this.filterServicoVersaoGrid();
                    versaoWidget.currentRow.IDVERSAOPROPCOM = 'CALCULADA';
                    versaoWidget.currentRow.SITVERSAO = 'CALCULADA';

                    setActionsVisibilityVersaoWidget();
                    ScreenService.closePopup();
                }
            }

        }).catch((error) => {
            if (error.error) {
                ScreenService.showMessage(ScreenService.i18n(error.error, versaoWidget.currentRow));
            }
        });
    };

    /**
     * Abre widget de seleção de filiais por serviço
     * para aprovação de versão.
     */
    this.openAproveWidget = function() {
        var aproveVersaoWidget = this.getWidget('aproveVersaoWidget');
        var versaoWidget = this.getWidget('versaoWidget');

        if (typeof aproveVersaoWidget.changedData !== 'undefined' &&
            aproveVersaoWidget.changedData !== null &&
            aproveVersaoWidget.changedData.length > 0) {
            aproveVersaoWidget.changedData = [];
        }

        Tco01100_proposta_comercial_service.openAproveWidget(versaoWidget.currentRow).then((success) => {
            if (success && success.error) {
                MessageService.showMessageByCode(success.error, versaoWidget.getParent().currentRow);
            } else {
                var rData = success.dataset;
                var propComRow = versaoWidget.getParent().currentRow;

                if (rData && rData.validOpen && rData.validOpen[0]) {

                    aproveVersaoWidget.isVisible = true;
                    aproveVersaoWidget.edit();
                    aproveVersaoWidget.getAction('aproveVersion').isVisible = false;
                    ScreenService.openPopup(aproveVersaoWidget);
                    setFilterOnAproveWidget();

                } else {
                    MessageService.showMessageByCode('TCO01100_PROPCOM_ALREADY_APROVED', propComRow);
                }
            }

        }).catch((error) => {
            if (error.error) {
                MessageService.showMessageByCode(error.error, versaoWidget.getParent().currentRow);
            }
        });
    };

    /**
     * Define filtro na widget de seleção de filial
     * para aprovação da versão.
     */
    var setFilterOnAproveWidget = () => {
        var versaoWidget = this.getWidget('versaoWidget');
        var aproveVersaoWidget = this.getWidget('aproveVersaoWidget');

        aproveVersaoWidget.dataSourceFilter = [{
            "name": "NRVERSAOPROPCOM",
            "operator": "=",
            "value": versaoWidget.currentRow.NRVERSAOPROPCOM
        }];

        aproveVersaoWidget.reload();
    };

    /**
     * Aprova versão selecionada
     * na tela de versão.
     */
    this.aproveVersion = function() {
        var widgetMain = this.getWidget('widgetMain');
        var versaoWidget = this.getWidget('versaoWidget');
        var servicoVersaoGrid = this.getWidget('servicoVersaoGrid');
        var aproveVersaoWidget = this.getWidget('aproveVersaoWidget');

        var objToSend = {
            MOTIVO: "APROVADO",
            NRVERSAOPROPCOM: versaoWidget.currentRow.NRVERSAOPROPCOM,
            versaoData: versaoWidget.currentRow,
            aproveData: aproveVersaoWidget.dataSource.data.map(reg => {
                return {
                    NRSERVICOPROPCOM: reg.NRSERVICOPROPCOM,
                    CDFILIAL: reg.CDFILIAL
                };
            })
        };

        Tco01100_proposta_comercial_service.aproveVersion(objToSend).then((respResult) => {
            if (respResult && respResult.error) {
                MessageService.showMessageByCode(respResult.error);
            } else {

                var rData = respResult.dataset;

                if (rData && rData.invalidOperation) {

                    if (rData.invalidOperation.invalidVersaoToAprove) {
                        MessageService.showMessageByCode('TCO01100_PROPCOM_ALREADY_APROVED', widgetMain.currentRow);
                    } else if (rData.invalidOperation.invalidServToAprove) {
                        invalidServToAproveErrorHandler(rData.invalidOperation.invalidServToAprove);
                    }

                } else {

                    versaoWidget.currentRow.IDVERSAOPROPCOM = 'APROVADA';
                    versaoWidget.currentRow.SITVERSAO = 'APROVADA';

                    widgetMain.currentRow.SITUACAO = 'APROVADA';
                    widgetMain.currentRow.NRVERSAOPROPCOM = versaoWidget.currentRow.NRVERSAOPROPCOM;

                    servicoVersaoGrid.reload().then(() => {
                        ScreenService.showMessage('Operação realizada com sucesso.');
                        ScreenService.closePopup();
                        servicoVersaoGrid.reload();
                        setActionsVisibilityVersaoWidget();
                        aproveVersaoWidget.changedData = [];
                    });
                }
            }
        }).catch((error) => {

            if (error.error) {
                ScreenService.showMessage(error.error);
            }

        });
    };

    /**
     * Trata erro de tentativa de aprovação
     * de filial usada em serviço de outra proposta.
     * 
     * @param {Array} invalidServ Serviços invalidos
     */
    var invalidServToAproveErrorHandler = (invalidServ) => {
        var selectFilialWidget = this.getWidget('selectFilialWidget');

        var mensagem = [];
        var msgParams = [];
        invalidServ.forEach((row) => {
            var completeRow = selectFilialWidget.dataSource.data.filter(reg => reg.CDFILIAL == row.CDFILIAL)[0];

            //mensagem.push(ScreenService.i18n('TCO01100_FILIAL_ALREADY_USED', completeRow));
            mensagem.push('TCO01100_FILIAL_ALREADY_USED');
            msgParams.push(completeRow);
        });

        MessageService.getMessagesByCode(mensagem).then(function(mensagensTraduzidas) {
            for (let i = 0; i < mensagensTraduzidas.length; i++) {
                mensagensTraduzidas[i] = ScreenService.i18n(mensagensTraduzidas[i], msgParams[i]);
            }
            General.showErrorMessages(selectFilialWidget, mensagensTraduzidas, ScreenService.i18n('Erro ao aprovar'));
        });
    };

    /**
     * Cancela aprovação versão selecionada
     * na tela de versão.
     */
    this.cancelAproveVersion = function() {
        var widgetMain = this.getWidget('widgetMain');
        var versaoWidget = this.getWidget('versaoWidget');
        var servicoVersaoGrid = this.getWidget('servicoVersaoGrid');

        Tco01100_proposta_comercial_service.cancelAproveVersion(versaoWidget.currentRow).then((respResult) => {
            if (respResult && respResult.error) {
                MessageService.showMessageByCode(respResult.error);
            } else {
                var rData = respResult.dataset;

                if (rData && rData.invalidOperation) {
                    if (rData.invalidOperation.invalidSit) {
                        MessageService.showMessageByCode('TCO01100_VER_INVALID_STATE', versaoWidget.currentRow);
                    }
                } else {
                    versaoWidget.currentRow.IDVERSAOPROPCOM = 'CONCLUIDA';
                    versaoWidget.currentRow.SITVERSAO = 'CONCLUIDA';

                    widgetMain.currentRow.SITVERSAO = 'CONCLUIDA';

                    widgetMain.currentRow.NRVERSAOPROPCOM = null;

                    ScreenService.showMessage('Operação realizada com sucesso.');
                    setActionsVisibilityVersaoWidget();
                    servicoVersaoGrid.reload();
                }
            }
        }).catch((error) => {
            if (error.error) {
                ScreenService.showMessage(error.error);
            }
        });
    };

    /**
     * Seleciona filial para registros de serviço
     * na widget de aprovação de versão.
     */
    this.selectFilialOnAproveWidget = function() {
        var selectFilialWidget = this.getWidget('selectFilialWidget');
        ScreenService.openPopup(selectFilialWidget);
    };

    /**
     * Conjunto de instruções a serem executadas
     * após selecionar uma filial para serviço.
     * @param {Object} args 
     */
    this.afterSelectFilial = function(args) {
        args.e.preventDefault();
        args.e.stopPropagation();
        setFilialOnServicos(args);
    };

    /**
     * Define filial selecionada no registro
     * de serviço escolhido.
     * 
     * @param {Row} row Filial selecionada.
     */
    var setFilialOnServicos = (args) => {
        var aproveVersaoWidget = this.getWidget('aproveVersaoWidget');
        var dataBackup = angular.copy(aproveVersaoWidget.dataSource.data);
        dataBackup.forEach(row => {
            if (row.__isSelected) {
                row.CDFILIAL = args.row.CDFILIAL;
                row.NMFILIAL = args.row.NMFILIAL;
                row.__isSelected = false;
            }
        });

        aproveVersaoWidget.changedData = dataBackup;
        aproveVersaoWidget.dataSource.data = dataBackup;

        ScreenService.closePopup();
        aproveVersaoWidget.dataSource.clearCheckedRows();
        aproveVersaoWidget.getAction('aproveVersion').isVisible = true;
    };

    /**
     * Rotina que aplica modificações no dataSource.data da widget de "Aprovar 
     * Versão" caso ela exista
     * 
     * @param {Object} args
     */
    this.setChangedData = (args) => {
        var aproveVersaoWidget = args.owner;

        if (typeof aproveVersaoWidget.changedData !== "undefined" && aproveVersaoWidget.changedData !== null && aproveVersaoWidget.changedData.length > 0) {
            aproveVersaoWidget.dataSource.data = aproveVersaoWidget.changedData;
        }
    };

    /**
     * Duplica versão selecionada.
     */
    this.duplicateVersion = function() {
        var versaoWidget = this.getWidget('versaoWidget');
        var widgetMain = this.getWidget('widgetMain');
        var rowsToSend = versaoWidget.dataSource.getCheckedRows();

        var cRow = Object.assign({}, versaoWidget.currentRow);

        Object.keys(widgetMain.currentRow).forEach(prop => {
            cRow[prop] = widgetMain.currentRow[prop];
        });

        Tco01100_proposta_comercial_service.duplicateVersion(rowsToSend).then((respResult) => {
            if (respResult && respResult.error) {
                MessageService.showMessageByCode(respResult.error, cRow);
            } else {
                versaoWidget.reload();
                versaoWidget.dataSource.clearCheckedRows();
                ScreenService.showMessage('Operação realizada com sucesso.');
            }

        }).catch((error) => {
            if (error.error) {
                ScreenService.showMessage(ScreenService.i18n(error.error, cRow));
            }
        });
    };

    /**
     * Verifica se proposta comercial atual
     * está aprovada.
     * 
     * @return {bool} Estado de aprovação da proposta
     */
    var isNotAprovedPropCom = () => {
        var widgetMain = this.getWidget('widgetMain');
        if (widgetMain.currentRow.SITUACAO == 'APROVADA') {
            MessageService.showMessageByCode('TCO01100_PROPCOM_INVALID_STATE', widgetMain.currentRow);
            return false;
        }

        return true;
    };

    /**
     * Rotina que abre o popup de importação de serviços.
     */
    this.openImportServico = function(args) {
        var widgetImportacaoServicos = args.owner.widget.container.getWidget("widgetImportacaoServicos");
        widgetImportacaoServicos.getField("CDPROPOSTACOMERCIAL").dataSourceFilter = [
            {
                name: "CDOPERADOR",
                operator: "=",
                value: getLocalVar("CDOPERADOR")
            },
            {
                "operator": "ORDER_BY",
                "name": [
                    {
                        "name": "CDPROPOSTACOMERCIAL",
                        "direction": "DESC"
                    }
                ]
            }
        ];

        widgetImportacaoServicos.getField("CDPROPOSTACOMERCIAL").reload().then(() => {
            widgetImportacaoServicos.getField("CDVERSAOPROPCOM").template = widgetImportacaoServicos.getField("CDVERSAOPROPCOM").templateOriginal;
            widgetImportacaoServicos.getField("CDVERSAOPROPCOM").readOnly = true;
            widgetImportacaoServicos.newRow();
            ScreenService.openPopup(widgetImportacaoServicos);
        });
    };

    /**
     * Rotina que aplica e busca dados do field dependente
     */
    this.onChangeProposta = function(args) {
        var currentRow = args.owner.field.widget.currentRow;
        var fieldSimulacoes = args.owner.field.widget.getField("CDVERSAOPROPCOM");
        var fieldServicos = args.owner.field.widget.getField("NMSERVICOPROPOSTA");
        var fieldTodosServicos = args.owner.field.widget.getField("TODOS_SERVICOPROPOSTA");
        var widgetVersaoProposta = args.owner.field.widget.container.getWidget("versaoWidget");

        if (typeof currentRow.NRPROPOSTACOMERCIAL !== "undefined" && currentRow.NRPROPOSTACOMERCIAL !== "" && currentRow.NRPROPOSTACOMERCIAL !== null) {

            fieldSimulacoes.dataSourceFilter = [
                {
                    name: "NRPROPOSTACOMERCIAL",
                    operator: "=",
                    value: currentRow.NRPROPOSTACOMERCIAL
                },
                {
                    name: "NRVERSAOPROPCOM",
                    operator: "<>",
                    value: widgetVersaoProposta.currentRow.NRVERSAOPROPCOM
                },
                {
                    operator: "ORDER_BY",
                    name: [
                        {
                            name: "CDVERSAOPROPCOM",
                            direction: "ASC"
                        }
                    ]
                }
            ];

            fieldSimulacoes.reload().then((success) => {
                fieldSimulacoes.readOnly = false;
                fieldSimulacoes.value("");
                fieldSimulacoes.template = fieldSimulacoes.templateAutoComplete;
                currentRow.CDVERSAOPROPCOM = null;
                currentRow.NRVERSAOPROPCOM = null;

                fieldServicos.readOnly = true;
                fieldServicos.value("");
                fieldServicos.validations = {};
                currentRow.NMSERVICOPROPOSTA = null;

                fieldTodosServicos.readOnly = false;
                fieldTodosServicos.value("T");
                currentRow.TODOS_SERVICOPROPOSTA = "T";
                templateManager.updateTemplate();
            });

        } else {
            fieldSimulacoes.readOnly = true;
            fieldSimulacoes.value("");
            currentRow.CDVERSAOPROPCOM = null;
            currentRow.NRVERSAOPROPCOM = null;

            fieldServicos.readOnly = true;
            fieldServicos.value("");
            fieldServicos.validations = {};
            currentRow.NMSERVICOPROPOSTA = null;

            fieldTodosServicos.readOnly = true;
            fieldTodosServicos.value("T");
            currentRow.TODOS_SERVICOPROPOSTA = "T";
            templateManager.updateTemplate();
        }

    };

    /**
     * Rotina que aplica e busca dados do field dependente
     */
    this.onChangeSimulacao = function(args) {
        var currentRow = args.owner.field.widget.currentRow;
        var fieldServicos = args.owner.field.widget.getField("NMSERVICOPROPOSTA");
        var fieldTodosServicos = args.owner.field.widget.getField("TODOS_SERVICOPROPOSTA");

        if (typeof currentRow.NRVERSAOPROPCOM !== "undefined" && currentRow.NRVERSAOPROPCOM !== "" && currentRow.NRVERSAOPROPCOM !== null) {

            fieldServicos.dataSourceFilter = [
                {
                    name: "NRVERSAOPROPCOM",
                    operator: "=",
                    value: currentRow.NRVERSAOPROPCOM
                },
                {
                    name: "IDCOMPLEMENTAR",
                    operator: "=",
                    value: "N"
                },
                {
                    operator: "ORDER_BY",
                    name: [
                        {
                            name: "NMSERVICOPROPOSTA",
                            direction: "ASC"
                        }
                    ]
                }
            ];

            fieldServicos.reload().then((success) => {
                fieldServicos.readOnly = true;
                fieldServicos.value("");
                fieldServicos.validations = {};
                currentRow.NMSERVICOPROPOSTA = null;

                fieldTodosServicos.readOnly = false;
                fieldTodosServicos.value("T");
                currentRow.TODOS_SERVICOPROPOSTA = "T";
            });

        } else {
            fieldServicos.readOnly = true;
            fieldServicos.value("");
            fieldServicos.validations = {};
            currentRow.NMSERVICOPROPOSTA = null;

            fieldTodosServicos.readOnly = true;
            fieldTodosServicos.value("T");
            currentRow.TODOS_SERVICOPROPOSTA = "T";
        }

        templateManager.updateTemplate();
    };

    /**
     * Rotina de controle do checkbox Todos Serviços.
     */
    this.onChangeTodosServicos = function(args) {
        General.toggleReadOnlySelect(args.owner.field, args.owner.field.widget.getField('NMSERVICOPROPOSTA'));
        templateManager.updateTemplate();

    };

    /**
     * Rotina para fechamento do popup.
     */
    this.cancelImportServ = function(args) {
        ScreenService.closePopup();
    };

    /**
     * Rotina para importação dos serviços.
     * 
     * @param {Object} args
     */
    this.confirmImportServ = function(args) {
        var widgetImportacaoServicos = args.owner.widget;
        var widgetServicoProposta = args.owner.widget.container.getWidget("servicoVersaoGrid");
        var widgetVersaoProposta = args.owner.widget.container.getWidget("versaoWidget");

        var rowImportacao = angular.copy(widgetImportacaoServicos.currentRow);
        rowImportacao.NRVERSAOPROPCOMDEST = widgetVersaoProposta.currentRow.NRVERSAOPROPCOM;

        if (widgetImportacaoServicos.isValid()) {
            Tco01100_proposta_comercial_service.importServicos(rowImportacao).then((response) => {
                if (typeof response.dataset.ERROS === "undefined") {
                    widgetServicoProposta.reload().then(() => {
                        ScreenService.closePopup(true);
                        widgetServicoProposta.activate();
                    });
                } else {
                    if (response.dataset.ERROS.length > 0) {
                        TcoUtil.buildMessagesToShow(response.dataset.ERROS).then(function(mensagens) {
                            General.showErrorMessages(widgetImportacaoServicos, mensagens, ScreenService.i18n("Erro", response.dataset.ERROS[0].PARAMS));
                        });
                    }
                }
            }, (response) => {
                if (response.data && response.data.dataset && response.data.dataset.ERROS) {
                    TcoUtil.buildMessagesToShow(response.data.dataset.ERROS).then(function(mensagens) {
                        General.showErrorMessages(widgetImportacaoServicos, mensagens, ScreenService.i18n("Erro"));
                    });
                }
            });
        }
    };

    this.beforeCloseContainer = function(args) {
        let widgetMain = this.getWidget('widgetMain');
        if (widgetMain.openFilterShortcut >= 0)
            eventEngine.unbindEvent(ZhKeyboardService, 'mod+lPressed', widgetMain.openFilterShortcut);
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01100_proposta_comercial', Tco01100_proposta_comercial);
});

// FILE: ./assets/js/controllers/Tco01101_detalhes_controller.js
var Tco01101_detalhes_controller = function(ScreenService, General, localStorageEngine, templateManager, TcoUtil, Tco01101_detalhes_service, $rootScope, TcoEmailService, Tco01100_proposta_comercial_service, Tco01200CardapioDaPropostaService, ZHPromise, MessageService) {
    var _backupRow = {};
    var _servPropSituations = {
        "PENDENTE": "Pendente",
        "CALCULADO": "Calculado",
        "AGUARDANDOPLANEJAMENTO": "Aguardando Planejamento",
        "FINALIZADOPLANEJAMENTO": "Finalizado Planejamento",
        "AGUARDANDOGN": "Aguardando Gerente de Negócio",
        "RECUSADO": "Recusado",
        "EFETIVADO": "Efetivado",
        "EMANDAMENTO": "Em Andamento",
        "NAOSOLICITADO": "Não Solicitado",
        "EMANDAMENTO_RETORNOGN": "Aguardando Planejamento"
    };

    var _calcErrorNotifyAction = new Action();
    _calcErrorNotifyAction.name = "calcErrorNotifyAction";
    _calcErrorNotifyAction.label = ScreenService.i18n("Notificar Planejamento");
    _calcErrorNotifyAction.showAsAction = "never";
    _calcErrorNotifyAction.events = [{
        "name": "ActionEvent",
        "code": "Tco01101_detalhes_controller.calcErrorNotifyAction(args);"
    }];

    /**
     * Conjunto de instruções a serem executadas
     * após a inicialização do container.
     */
    this.afterInitContainer = function() {
        startContainer();
        Tco01101_detalhes_service.getUltimaSolicitacao(getWidget('widgetDetalhes').currentRow).then((response) => {
            let historico = response.dataset;
            if (historico) historico = historico.ultima[0];
            if ((historico === 'DUVIDA') || (historico === 'FINALIZADO') || (historico === 'RECUSADO')) {
                let widgetTabbed = getWidget('widgetDetalhesTabbed');
                widgetTabbed.openTab('widgetObservacoes');
            }
        });
    };

    /**
     * Retorna à tela de proposta comercial
     * indicando a origem na tela de detalhes.
     */
    this.backToPropCom = function() {
        TcoUtil.openWindowByHistory('#/tco#tco01100_proposta_comercial', {
            'backFromDetail': true
        }, 'Proposta Comercial');
    };

    /**
     * Inicializa container caso seja válido,
     * e redireciona para tela de proposta comercial
     * caso nao seja.
     */
    var startContainer = () => {
        $rootScope.$$watchers = $rootScope.__bkpWatchers;

        if (!window.history.state || !window.history.state.startedByPropCom) {
            backReplacingToPropCom();
        } else {
            initializeWidget();
            window.history.replaceState({}, 'Detalhes do Serviço', '#/tco#tco01101_detalhes_servico');
        }
    };

    /**
     * Retorna a tela de Proposta Comercial bloqueando
     * o acesso pelo historico a tela de detalhes.
     */
    var backReplacingToPropCom = () => {
        window.location.href = window.location.href.replace('#/tco#tco01101_detalhes_servico', '#/tco#tco01100_proposta_comercial');
        window.location.reload();
    };

    /**
     * Inicializa a widget principal de detalhes.
     */
    var initializeWidget = () => {
        var widgetStripe = getWidget('widgetStripe');
        var widgetDetalhes = getWidget('widgetDetalhes');
        var widgetDetalhesTabbed = getWidget('widgetDetalhesTabbed');

        var stateRow = window.history.state.row;

        widgetDetalhes.currentRow = stateRow;
        widgetDetalhes.view();
        setComplFieldVisibility(!!widgetDetalhes.currentRow.NMSERVCOMPLEMENTADO);

        var params = {
            decimal: ',',
            thousands: '.',
            precision: 2
        };

        widgetStripe.currentRow = stateRow;
        widgetStripe.currentRow.CUSTALISERVPROPCOM = widgetStripe.currentRow.CUSTALISERVPROPCOM ? TcoUtil.maskCurrency(widgetStripe.currentRow.CUSTALISERVPROPCOM.toString(), params) : "0,00";
        widgetStripe.currentRow.CUSTDESSERVPROPCOM = widgetStripe.currentRow.CUSTDESSERVPROPCOM ? TcoUtil.maskCurrency(widgetStripe.currentRow.CUSTDESSERVPROPCOM.toString(), params) : "0,00";
        widgetStripe.currentRow.CUSTLIMERVPROPCOM = widgetStripe.currentRow.CUSTLIMERVPROPCOM ? TcoUtil.maskCurrency(widgetStripe.currentRow.CUSTLIMERVPROPCOM.toString(), params) : "0,00";
        widgetStripe.currentRow.CUSTTOTSERVPROPCOM = widgetStripe.currentRow.CUSTTOTSERVPROPCOM ? TcoUtil.maskCurrency(widgetStripe.currentRow.CUSTTOTSERVPROPCOM.toString(), params) : "0,00";
        widgetStripe.currentRow.IDSITUACAO = _servPropSituations[stateRow.IDSERVICOPROPCOM];

        widgetDetalhesTabbed.currentRow = widgetDetalhes.currentRow;

        showActionsByParams();
    };

    /**
     * Define visibilidade do campo de complemetar
     * a.
     * @param {bool} state 
     */
    var setComplFieldVisibility = (state) => {
        var widgetDetalhes = getWidget('widgetDetalhes');
        widgetDetalhes.getField('NMSERVCOMPLEMENTADO').isVisible = state;
    };

    /**
     * Retorna widget cujo nome é passado por parâmetro.
     * 
     * @param {String} widgetName Nome da widget a ser
     *  retornada.
     */
    var getWidget = (widgetName) => {
        return templateManager.container.getWidget(widgetName);
    };

    /**
     * Coloca widget de detalhes do serviço em
     * modo de edição e cria uma row com backup 
     * dos valores.
     */
    this.editServicoProposta = function() {
        var widgetDetalhes = getWidget('widgetDetalhes');
        _backupRow = angular.copy(widgetDetalhes.currentRow);
        widgetDetalhes.edit();
    };

    /**
     * Coloca widget de detalhes do serviço em
     * modo de visualização e restaura a row 
     * com backup dos valores.
     */
    this.cancelEditServicoProposta = function() {
        var widgetDetalhes = getWidget('widgetDetalhes');
        widgetDetalhes.currentRow = _backupRow;
        widgetDetalhes.view();
    };

    /**
     * Salva alterações no registro de serviço
     * da proposta.
     */
    this.saveEditServicoProposta = function() {
        var widgetDetalhes = getWidget('widgetDetalhes');
        var widgetStripe = getWidget('widgetStripe');
        var cRow = widgetDetalhes.currentRow;
        let executaSave = function() {
            Tco01101_detalhes_service.saveEditServicoProposta(cRow).then((success) => {
                if (success && success.error) {
                    MessageService.showMessageByCode(success.error, cRow);
                } else {
                    ScreenService.successNotification('Registro salvo com sucesso.');
                    widgetDetalhes.view();
                    widgetDetalhes.currentRow.NRCARDAPIOTRAB = null;

                    widgetDetalhes.currentRow.IDSERVICOPROPCOM = 'PENDENTE';
                    widgetStripe.currentRow.IDSITUACAO = _servPropSituations.PENDENTE;

                    showActionsByParams();
                }

            }).catch((error) => {
                if (error.error) {
                    ScreenService.showMessage(ScreenService.i18n(error.error, cRow));
                }
            });
        };

        if (widgetDetalhes.currentRow.NRCARDAPIOTRAB) {
            MessageService.confirmMessageByCode('TCO01101_CONFIRM_ALT').then(() => {
                executaSave();
            });
        } else {
            executaSave();
        }
    };

    /**
     * Conjunto de instruções a serem executadas ao 
     * alterar valor de algum campo de alguma refeição
     * estimada.
     */
    this.onChangeRefeicoesEstimadas = function() {
        var widgetDetalhes = getWidget('widgetDetalhes');
        var cRow = widgetDetalhes.currentRow;

        calculateTotalDiasTrabalhados(cRow);
        calculateTotalRefeicoesEstimadas(cRow);
        templateManager.updateTemplate();
    };

    /**
     * Calcula os totais de dias trabalhados
     * e define o valor na curretRow da widget de 
     * serviço da proposta.
     * 
     * @param {Object} cRow Current row da widget
     *  de proposta.
     */
    var calculateTotalDiasTrabalhados = (cRow) => {
        cRow.QTTOTDIAS = 0;
        if (cRow.QTSEGSEX) cRow.QTTOTDIAS += 21;
        if (cRow.QTSABADO) cRow.QTTOTDIAS += 5;
        if (cRow.QTDOMFERI) cRow.QTTOTDIAS += 4;
    };

    /**
     * Calcula os totais de refeições estimadas
     * e define o valor na curretRow da widget de 
     * serviço da proposta.
     * 
     * @param {Object} cRow Current row da widget
     *  de proposta.
     */
    var calculateTotalRefeicoesEstimadas = (cRow) => {
        cRow.QTTOTREFEICOES = 0;
        if (cRow.QTSEGSEX) cRow.QTTOTREFEICOES += cRow.QTSEGSEX * 21;
        if (cRow.QTSABADO) cRow.QTTOTREFEICOES += cRow.QTSABADO * 5;
        if (cRow.QTDOMFERI) cRow.QTTOTREFEICOES += cRow.QTDOMFERI * 4;
    };

    /**
     * Conjunto de instruções a serem executadas
     * ao entrar na widget de detalhes.
     */
    this.onEnterWidgetDetalhes = function() {
        General.justInteger('#NRPROPOSTACOMERCIAL');
        setActionButtonsVisibility();
    };

    /**
     * Envia notificação ao planejamento caso o 
     * serviço não possua cardápio base relacionado.
     */
    var notifyPlanej = function(motivo) {
        var widgetDetalhes = getWidget('widgetDetalhes');
        var cRow = widgetDetalhes.currentRow;

        cRow.MOTIVO = motivo;
        Tco01101_detalhes_service.notificarPlan(cRow).then((success) => {
            if (success && success.error) {
                MessageService.showMessageByCode(success.error);
            } else {
                ScreenService.alertNotification('Equipe de planejamento notificada');
            }
        }).catch((error) => {
            if (error.error) {
                if (error.error.indexOf('TCO') >= 0) {
                    MessageService.showMessageByCode(error.error);
                } else
                    ScreenService.showMessage(ScreenService.i18n(error.error));
            }
        });
    };

    /**
     * Altera visibilidade dos botões de verificar cardapio
     * para exibir somente se o serviço não possuir cardapio
     * base e de calcular custo somente se serviço possuir
     * cardápio.
     */
    var setActionButtonsVisibility = function() {
        var widgetDetalhes = getWidget('widgetDetalhes');
        var widgetTabs = widgetDetalhes.getParent().widgets;
        var cRow = widgetDetalhes.currentRow;
        var checkCardBaseAction = widgetDetalhes.getAction('checkCardBase');

        checkCardBaseAction.isVisible = !cRow.NRCARDAPIOBASE;

        widgetTabs.forEach(wid => {
            var calcAct = wid.getAction('calcularCalcServ');

            if (calcAct) {
                calcAct.isVisible = Boolean(cRow.NRCARDAPIOBASE);
            }
        });
    };

    /**
     * Verifica existencia do cardápio base e preenche
     * seu valor caso exista, do contrário dá opção de 
     * notificar planejamento.
     */
    this.checkCardBaseServPropCom = function() {
        var widgetDetalhes = getWidget('widgetDetalhes');
        var cRow = widgetDetalhes.currentRow;

        Tco01100_proposta_comercial_service.openServPropComDetail(cRow).then((response) => {
            let rData = response.dataset;

            if (rData.backParams && !rData.backParams.errorCode) {
                cRow.NRCARDAPIOBASE = rData.backParams.cardBase;
                ScreenService.successNotification('Cardápio base relacionado');
                setActionButtonsVisibility();
            } else {
                MessageService.confirmMessageByCode(rData.backParams.errorCode).then(() => {
                    notifyPlanej('SEMCARDBASE');
                });
            }
        });
    };

    /**
     * Conjunto de instruções a serem executadas
     * após carregar o dataSource da widget de tipo
     * de prato.
     */
    this.afterLoadDataSourceTipoPrato = function() {
        this.setSelectsTipoPrato();
    };

    /**
     * Função responsável marcar as rows com o 
     * UTLSERVPROPCOM igual a 'S' ou 'B'.
     */
    this.setSelectsTipoPrato = function() {
        var widgetTiposPrato = getWidget('widgetTiposPrato');

        widgetTiposPrato.dataSource.clearCheckedRows();
        widgetTiposPrato.dataSource.data.forEach(function(reg) {
            reg.__isSelected = (reg.UTLSERVPROPCOM === 'S' || reg.UTLSERVPROPCOM === 'B');
        });
        widgetTiposPrato.dataSource.updateCheckedRows();
    };

    /**
     * Conjunto de instruções a serem executadas
     * após (des)selecionar registro no grid de
     * tipo de prato.
     */
    this.onChangeSelectTipoPrat = function() {
        var widgetTiposPrato = getWidget('widgetTiposPrato');
        widgetTiposPrato.inlineEdit();
        lockTipoPratSisterWidgets(true);
        widgetTiposPrato.getAction('calcularCalcServ').isVisible = false;
    };

    /**
     * Bloqueia acesso as widgets irmãs da
     * widget tipo de prato.
     * 
     * @param {Boolean} state Estado de bloqueio.
     */
    var lockTipoPratSisterWidgets = function(state) {
        var widgetTiposPrato = getWidget('widgetTiposPrato');

        var sisterWdgts = widgetTiposPrato.getParent().widgets.filter(wid => wid != widgetTiposPrato);
        General.changeWidgetAcessibility(sisterWdgts, !state);
    };

    /**
     * Conjunto de instruções a serem executadas
     * ao (des)selecionar um registro no grid de
     * tipo de prato.
     * 
     * @param {Object} args 
     */
    this.afterChangeSelectOneRowTipoPrat = function(args) {
        if (args.selectedRow.UTLSERVPROPCOM != 'B') {
            args.selectedRow.changed = !args.selectedRow.changed;
            this.onChangeSelectTipoPrat();
        } else {
            args.selectedRow.__isSelected = true;
        }
    };

    /**
     * Conjunto de instruções a serem executadas
     * ao (des)selecionar todos os registros no grid de
     * tipo de prato.
     */
    this.afterChangeSelectAllRowTipoPrat = function() {
        var widgetTiposPrato = getWidget('widgetTiposPrato');
        widgetTiposPrato.dataSource.data.forEach((reg) => {
            if (reg.UTLSERVPROPCOM != 'B') {
                reg.changed = reg.__isSelected != (reg.UTLSERVPROPCOM == "S");
            } else {
                reg.__isSelected = true;
                reg.changed = false;
            }
        });
        this.onChangeSelectTipoPrat();
    };

    /**
     * Cancela modo de edição grid de tipo de prato.
     */
    this.cancelEditTipoPrat = function() {
        var widgetTiposPrato = getWidget('widgetTiposPrato');
        widgetTiposPrato.list();
        widgetTiposPrato.reload();
        lockTipoPratSisterWidgets(false);
        widgetTiposPrato.getAction('calcularCalcServ').isVisible = true;
    };

    /**
     * Salva alteração nos registros de tipo de prato
     * do serviço da proposta.
     */
    this.saveEditTipoPrat = function() {
        var widgetDetalhes = getWidget('widgetDetalhes');
        var widgetTabbed = getWidget('widgetDetalhesTabbed');
        var widgetStripe = getWidget('widgetStripe');
        var widgetTiposPrato = getWidget('widgetTiposPrato');
        var data = widgetTiposPrato.dataSource.data.filter(reg => reg.changed);
        let executaSave = function() {
            if (data && data.length) {
                Tco01101_detalhes_service.saveEditTipoPrat(data).then((success) => {
                    if (success && success.error) {
                        MessageService.showMessageByCode(success.error, widgetDetalhes.currentRow);
                    } else {
                        widgetTiposPrato.list();
                        widgetTiposPrato.reload();

                        widgetDetalhes.currentRow.NRCARDAPIOTRAB = null;
                        widgetDetalhes.currentRow.IDSERVICOPROPCOM = 'PENDENTE';
                        widgetStripe.currentRow.IDSITUACAO = _servPropSituations.PENDENTE;

                        showActionsByParams();
                        ScreenService.successNotification('Registro alterado com sucesso!');

                        lockTipoPratSisterWidgets(false);
                        widgetTiposPrato.getAction('calcularCalcServ').isVisible = true;

                        widgetTabbed.openTab('widgetPorcGram');
                    }

                }).catch((error) => {
                    if (error.error && !error.error.includes('TCOAUTH_')) {
                        ScreenService.showMessage(ScreenService.i18n(error.error, widgetDetalhes.currentRow));
                    }
                });

            } else {

                widgetTiposPrato.list();
                widgetTiposPrato.getAction('calcularCalcServ').isVisible = true;
                lockTipoPratSisterWidgets(false);

            }
        };
        if (data && data.length) {
            if (widgetDetalhes.currentRow.NRCARDAPIOTRAB) {
                MessageService.confirmMessageByCode('TCO01101_CONFIRM_ALT').then(() => {
                    executaSave();
                });
            } else {
                executaSave();
            }
        }
    };

    /**
     * Conjunto de instruções a serem executadas ao
     * selecionar registro no grid de porcionamento
     * por gramatura.
     * 
     * @param {Object} args
     */
    this.afterMoveRowPorcGram = function(args) {
        openAddPorcGram(Object.assign({}, args.row));
    };

    /**
     * Abre widget para adição de porcionamento e 
     * gramatura ao registro selecionado.
     * 
     * @param {Object} row
     */
    var openAddPorcGram = (row) => {
        var addPorcGram = getWidget('addPorcGram');

        var translateTipoGramaturaValues = {
            INNATURA: "0",
            LIMPA: "1",
            PRONTA: "2"
        };

        addPorcGram.currentRow = row;
        addPorcGram.currentRow.IDTIPOGRAMATURA = translateTipoGramaturaValues[addPorcGram.currentRow.IDTIPOGRAMATURA];
        addPorcGram.currentRow.NRDETALHESERVPROPCOM = getWidget('widgetDetalhesTabbed').currentRow.NRDETALHESERVPROPCOM;
        setFilterOnPorcGramFields();
        let promises = [
            addPorcGram.getField('NMPORCIONAMENTO').reload(),
            addPorcGram.getField('VRGRAMATURA').reload()
        ];

        ZHPromise.all(promises).then(() => {
            lockGramFieldIfEmpty();
            ScreenService.openPopup(addPorcGram);
        });
    };

    /**
     * Define filtro nos campos da widget de 
     * adição de porcionamento e gramatura.
     */
    var setFilterOnPorcGramFields = function() {
        var addPorcGram = getWidget('addPorcGram');
        var detalheTab = getWidget('widgetDetalhesTabbed');

        addPorcGram.getField('NMPORCIONAMENTO').dataSourceFilter = [
            {
                name: "NRCARDAPIOBASE",
                operator: "=",
                value: detalheTab.currentRow.NRCARDAPIOBASE
            },
            {
                name: "CDGRTIPOPR",
                operator: "=",
                value: addPorcGram.currentRow.CDGRTIPOPR
            }
        ];

        addPorcGram.getField('VRGRAMATURA').dataSourceFilter = [
            {
                name: "NRCARDAPIOBASE",
                operator: "=",
                value: detalheTab.currentRow.NRCARDAPIOBASE
            },
            {
                name: "CDGRTIPOPR",
                operator: "=",
                value: addPorcGram.currentRow.CDGRTIPOPR
            },
            {
                name: "NRTIPOPUBLICO",
                operator: "=",
                value: detalheTab.currentRow.NRTIPOPUBLICO
            }
        ];

    };

    /**
     * Bloqueia ou desbloqueia campo de gramatura
     * na widget de adicionar porcionamento e 
     * gramatura caso o campo não apresente opções.
     */
    var lockGramFieldIfEmpty = function() {
        var addPorcGram = getWidget('addPorcGram');
        var gramField = addPorcGram.getField('VRGRAMATURA');
        var aplicacaoField = addPorcGram.getField('IDTIPOGRAMATURA');
        var gramData = gramField.dataSource.data;

        if (!gramData || !gramData.length) {
            gramField.readOnly = true;

            if (gramField.validations.required) {
                delete gramField.validations.required;
            }

            aplicacaoField.readOnly = true;

            if (aplicacaoField.validations.required) {
                delete aplicacaoField.validations.required;
            }
        } else {
            gramField.readOnly = false;
            gramField.validations.required = true;

            aplicacaoField.readOnly = false;
            aplicacaoField.validations.required = true;
        }
    };

    /**
     * Cancela adição de dados de porcionamento
     * e gramatura.
     */
    this.cancelAddPorcGram = function() {
        var addPorcGram = getWidget('addPorcGram');
        ScreenService.closePopup();
        addPorcGram.currentRow = {};
    };

    /**
     * Salva alterações de porcionamento e gramatura
     * para o grupo de tipo de prato relacionado.
     */
    this.saveAddPorcGram = function() {
        var addPorcGram = getWidget('addPorcGram');
        var widgetDetalhes = getWidget('widgetDetalhes');
        var widgetStripe = getWidget('widgetStripe');
        var widgetPorcGram = getWidget('widgetPorcGram');
        var cRow = addPorcGram.currentRow;

        var translateTipoGramaturaValues = {
            "0": "INNATURA",
            "1": "LIMPA",
            "2": "PRONTA"
        };

        cRow.IDTIPOGRAMATURA = translateTipoGramaturaValues[cRow.IDTIPOGRAMATURA];

        let controller = this;

        let executaSave = function() {
            if (addPorcGram.isValid()) {
                cRow.NRVERSAOPROPCOM = widgetDetalhes.currentRow.NRVERSAOPROPCOM;

                Tco01101_detalhes_service.saveAddPorcGram(cRow).then((success) => {
                    ScreenService.successNotification('Registro alterado com sucesso!');
                    for (let i = 0; i < widgetPorcGram.dataSource.data.length; i++) {
                        if (widgetPorcGram.dataSource.data[i].CDGRTIPOPR == widgetPorcGram.currentRow.CDGRTIPOPR) {
                            if (widgetPorcGram.dataSource.data[i + 1] != undefined) {
                                var addPorcGram = getWidget('addPorcGram');
                                var translateTipoGramaturaValues = {
                                    INNATURA: "0",
                                    LIMPA: "1",
                                    PRONTA: "2"
                                };
                                addPorcGram.currentRow = widgetPorcGram.dataSource.data[i + 1];
                                widgetPorcGram.currentRow = widgetPorcGram.dataSource.data[i + 1];

                                addPorcGram.currentRow.IDTIPOGRAMATURA = translateTipoGramaturaValues[addPorcGram.currentRow.IDTIPOGRAMATURA];
                                addPorcGram.currentRow.NRDETALHESERVPROPCOM = getWidget('widgetDetalhesTabbed').currentRow.NRDETALHESERVPROPCOM;
                                setFilterOnPorcGramFields();
                                let promises = [
                                    addPorcGram.getField('NMPORCIONAMENTO').reload(),
                                    addPorcGram.getField('VRGRAMATURA').reload()
                                ];
                                ZHPromise.all(promises).then(() => {
                                    lockGramFieldIfEmpty();
                                });
                                break;
                            } else {
                                ScreenService.closePopup();
                                ScreenService.showMessage(ScreenService.i18n('Todos os registros foram alterados com sucesso!'));
                                widgetDetalhes.currentRow.NRCARDAPIOTRAB = null;
                                widgetDetalhes.currentRow.IDSERVICOPROPCOM = 'PENDENTE';
                                widgetStripe.currentRow.IDSITUACAO = _servPropSituations.PENDENTE;
                                showActionsByParams();
                            }
                        }
                    }
                    widgetPorcGram.reload();

                });
            }
        };

        if (addPorcGram.isValid()) {
            if (widgetDetalhes.currentRow.NRCARDAPIOTRAB) {
                MessageService.confirmMessageByCode('TCO01101_CONFIRM_ALT').then(() => {
                    executaSave();
                });
            } else {
                executaSave();
            }
        }
    };

    /**
     * Executa a rotina de calculo do custo do
     * serviço.
     */
    this.calcularCustServ = function() {
        var widgetDetalhes = getWidget('widgetDetalhes');
        var widgetDetalhesTabbed = getWidget('widgetDetalhesTabbed');
        var widgetStripe = getWidget('widgetStripe');
        var cRow = widgetDetalhes.currentRow;

        let executaCalc = function() {
            Tco01101_detalhes_service.calcularCustServ(cRow).then((success) => {
                if (success && success.error) {
                    MessageService.showMessageByCode(success.error, cRow);
                } else {
                    var rData = success.dataset;
                    //Caso esteja faltando informações do GN
                    if (rData.faltaInfo && rData.faltaInfo[0]) {
                        let widgetAInformar = '';
                        if (rData.faltaInfo[0] === 'TCO01101_SELECIONE_TIPOS_PRATO') {
                            widgetAInformar = 'widgetTiposPrato';
                        } else if (rData.faltaInfo[0] === 'TCO01101_INFORME_PORC_GRAM') {
                            widgetAInformar = 'widgetPorcGram';
                        }
                        widgetDetalhesTabbed.openTab(widgetAInformar);
                        MessageService.showMessageByCode(rData.faltaInfo[0]);
                    } else if (rData.errorCode) {

                        TcoUtil.buildMessagesToShow(rData.errorCode).then(function(mensagens) {
                            ScreenService.showMessage('Erro ao realizar cálculo').then(() => {
                                widgetDetalhes.currentRow.NRCARDAPIOTRAB = null;
                                General.showErrorMessages(widgetDetalhes, mensagens, ScreenService.i18n("Cálculo não realizado"));

                                addNotifyActionToMessageWidget();
                            });
                        });

                    } else if (rData.procedureResult) {
                        widgetDetalhes.currentRow.NRCARDAPIOTRAB = rData.procedureResult.P_NRCARDAPIOTRAB;

                        var detalheServ = rData.procedureResult.detalheServ;
                        Object.keys(detalheServ).forEach(key => {
                            widgetDetalhes.currentRow[key] = detalheServ[key];
                        });

                        widgetDetalhes.currentRow.IDSERVICOPROPCOM = 'CALCULADO';
                        widgetStripe.currentRow.IDSITUACAO = _servPropSituations.CALCULADO;

                        widgetDetalhesTabbed.widgets.forEach(wid => wid.reload());

                        ScreenService.successNotification('Cálculo realizado com sucesso.');
                    }

                    showActionsByParams();
                }
            }).catch((error) => {
                if (error.error) {
                    if (error.error.indexOf('TCO') === 0) {
                        MessageService.showMessageByCode(error.error, cRow);
                    } else {
                        ScreenService.showMessage(ScreenService.i18n(error.error, cRow));
                    }
                }
            });
        };

        if (widgetDetalhes.currentRow.NRCARDAPIOTRAB) {
            MessageService.confirmMessageByCode('TCO01101_CONFIRM_CALC').then(() => {
                executaCalc();
            });
        } else {
            executaCalc();
        }
    };

    /**
     * Esconde campos de custo caso o serviço
     * não possua cardápio de trabalho, ou seja,
     * o calculo não foi efetuado.
     */
    var setStripeCustFieldsVisibility = function() {
        var widgetDetalhes = getWidget('widgetDetalhes');
        var widgetStripe = getWidget('widgetStripe');

        widgetStripe.fields.forEach(field => {
            if (field.hideIfNoCardTrab) {
                field.isVisible = Boolean(widgetDetalhes.currentRow.NRCARDAPIOTRAB);
            }
        });

        var params = {
            decimal: ',',
            thousands: '.',
            precision: 2
        };

        widgetStripe.currentRow.CUSTALISERVPROPCOM = widgetStripe.currentRow.CUSTALISERVPROPCOM ? TcoUtil.maskCurrency(widgetStripe.currentRow.CUSTALISERVPROPCOM.toString(), params) : "0,00";
        widgetStripe.currentRow.CUSTDESSERVPROPCOM = widgetStripe.currentRow.CUSTDESSERVPROPCOM ? TcoUtil.maskCurrency(widgetStripe.currentRow.CUSTDESSERVPROPCOM.toString(), params) : "0,00";
        widgetStripe.currentRow.CUSTLIMERVPROPCOM = widgetStripe.currentRow.CUSTLIMERVPROPCOM ? TcoUtil.maskCurrency(widgetStripe.currentRow.CUSTLIMERVPROPCOM.toString(), params) : "0,00";
        widgetStripe.currentRow.CUSTTOTSERVPROPCOM = widgetStripe.currentRow.CUSTTOTSERVPROPCOM ? TcoUtil.maskCurrency(widgetStripe.currentRow.CUSTTOTSERVPROPCOM.toString(), params) : "0,00";

        templateManager.updateTemplate();
    };

    /**
     * Conjunto de instruções a serem executadas
     * ao entrar na widget de tipo de prato.
     */
    this.onEnterTabTipoPrat = function() {
        setTipoPratCustFieldVisibility();
    };

    /**
     * Exibe campo de custos da widget de tipo
     * de prato somente para serviços que tenham
     * sido calculados.
     */
    var setTipoPratCustFieldVisibility = function() {
        var widgetTiposPrato = getWidget('widgetTiposPrato');
        var widgetDetalhes = getWidget('widgetDetalhes');

        widgetTiposPrato.getField('CUSTOTIPOPRAT').isVisible = Boolean(widgetDetalhes.currentRow.NRCARDAPIOTRAB);

        if (widgetTiposPrato.redraw) {
            widgetTiposPrato.redraw();
        }

        templateManager.updateTemplate();
    };

    /**
     * Notifica planejamento dos erros encontrados
     * ao tentar calcular cardápio.
     * 
     * @param {object} args
     */
    this.calcErrorNotifyAction = function(args) {
        var messagesWidget = args.owner.widget;
        var widgetDetalhes = getWidget('widgetDetalhes');
        var widgetStripe = getWidget('widgetStripe');

        widgetDetalhes.currentRow.MOTIVO = 'ERROCALCULO';
        Tco01101_detalhes_service.notificarPlan(widgetDetalhes.currentRow).then((success) => {
            if (success && success.error) {
                MessageService.showMessageByCode(success.error);
            } else {
                ScreenService.alertNotification('Equipe de planejamento notificada');
                widgetDetalhes.currentRow.IDSERVICOPROPCOM = _servPropSituations.AGUARDANDOPLANEJAMENTO;
                widgetStripe.currentRow.IDSITUACAO = _servPropSituations.AGUARDANDOPLANEJAMENTO;
            }

        }).catch((error) => {
            if (error.error) {
                if (error.error.indexOf('TCOPROC_') >= 0) {
                    MessageService.showMessageByCode(error.error);
                } else
                    ScreenService.showMessage(ScreenService.i18n(error.error));
            }
        });
    };

    /**
     * Adiciona ação de notificar planejamento a 
     * widget de mensagens de erro ao calcular.
     */
    var addNotifyActionToMessageWidget = function() {
        var activeWidget = templateManager.container.getActiveWidget();

        if (activeWidget.name == 'messagesWidget') {
            _calcErrorNotifyAction.widget = activeWidget;
            activeWidget.actions.push(_calcErrorNotifyAction);
            templateManager.updateTemplate();
        }
    };

    /**
     * Rotina para salvar alterações em incidencias.
     */
    this.onSaveInlineEditIncidencias = function() {
        var widgetDetalhes = getWidget('widgetDetalhes');
        var widgetIncidencias = getWidget('widgetIncidencias');
        var changedData = widgetIncidencias.dataSource.data.filter(reg => reg.changed);

        changedData.forEach(data => {
            data.NRDETALHESERVPROPCOM = widgetDetalhes.currentRow.NRDETALHESERVPROPCOM;
            data.NRSERVICOPROPCOM = widgetDetalhes.currentRow.NRSERVICOPROPCOM;
            data.NRVERSAOPROPCOM = widgetDetalhes.currentRow.NRVERSAOPROPCOM;
        });

        saveIncidencias(changedData);
    };

    /**
     * Salva incidencias alteradas.
     * 
     * @param {array[object]} changedData
     */
    var saveIncidencias = function(changedData) {
        var widgetDetalhes = getWidget('widgetDetalhes');
        var widgetIncidencias = getWidget('widgetIncidencias');

        Tco01101_detalhes_service.saveIncidencias(changedData).then((success) => {
            if (success && success.error) {
                MessageService.showMessageByCode(success.error, widgetDetalhes.currentRow);
            } else {
                widgetIncidencias.list();
                widgetIncidencias.reload();
            }

        }).catch((error) => {
            if (error.error) {
                ScreenService.showMessage(ScreenService.i18n(error.error, widgetDetalhes.currentRow));
            }
        });

        lockIncidenciaSisterWidgets(false);
    };

    /**
     * Conjunto de instruções a serem executadas
     * ao selecionar uma row na widget de produtos.
     */
    this.afterMoveRowProdutos = function(args) {
        var widgetProdutos = getWidget("widgetProdutos");
        widgetProdutos.selectedRow = args.row;
    };

    /**
     * Inicia Alteração de marca do produto 
     * selecionado.
     */
    this.changeMarcaProduto = function() {
        var widgetProdutos = getWidget("widgetProdutos");

        if (!Util.isEmptyObject(widgetProdutos.selectedRow)) {
            openChangeMarcaWidget();
        } else {
            ScreenService.showMessage("Nenhum produto selecionado!");
        }
    };

    /**
     * Abre widget para alteração de marca do 
     * produto selecionado.
     */
    var openChangeMarcaWidget = function() {
        var widgetProdutos = getWidget("widgetProdutos");
        var widgetDetalhes = getWidget("widgetDetalhes");
        var changeMarcaPopup = getWidget("changeMarcaPopup");

        changeMarcaPopup.dataSourceFilter = [
            {
                name: "CDPRODUTO",
                operator: "=",
                value: widgetProdutos.selectedRow.CDPRODUTO
            },
            {
                name: "NRDETALHESERVPROPCOM",
                operator: "=",
                value: widgetDetalhes.currentRow.NRDETALHESERVPROPCOM
            }
        ];

        changeMarcaPopup.reload();
        ScreenService.openPopup(changeMarcaPopup);
    };

    /**
     * Cancela edição de marca do produto.
     */
    this.cancelChangeMarca = function() {
        var changeMarcaPopup = getWidget('changeMarcaPopup');
        ScreenService.closePopup();
    };

    /**
     * Verifica se entrada é valida e corrige
     * valor em caso de inserção de zero ou nulo.
     */
    this.onEditQtSolicitada = function(args) {
        var widget = args.owner.field.widget;
        var row = args.row;

        if (row.QTSOLICITADA_inline === "" || isNaN(row.QTSOLICITADA_inline)) {

            row.QTSOLICITADA = null;
            row.QTSOLICITADA_inline = null;
            row.changed_QTSOLICITADA = null;
            row.changed = true;
            widget.inlineEdit();

        }

        templateManager.updateTemplate();
    };

    /**
     * Altera marca do produto selecionado.
     */
    this.onSelectMarca = function() {
        var widgetProdutos = getWidget("widgetProdutos");
        var widgetDetalhes = getWidget("widgetDetalhes");
        var changeMarcaPopup = getWidget("changeMarcaPopup");
        var rowToSend = changeMarcaPopup.currentRow;

        rowToSend.NRDETALHESERVPROPCOM = widgetDetalhes.currentRow.NRDETALHESERVPROPCOM;
        rowToSend.CDPRODUTO = widgetProdutos.selectedRow.CDPRODUTO;
        rowToSend.NRPRODUTOPROPCOM = widgetProdutos.selectedRow.NRPRODUTOPROPCOM;
        rowToSend.CDMARCAANT = widgetProdutos.selectedRow.CDMARCA;
        rowToSend.CDMARCASOLICITADA = rowToSend.CDMARCA;
        rowToSend.NRVERSAOPROPCOM = widgetDetalhes.currentRow.NRVERSAOPROPCOM;
        rowToSend.NRSERVICOPROPCOM = widgetDetalhes.currentRow.NRSERVICOPROPCOM;

        Tco01101_detalhes_service.selectMarca(rowToSend).then((success) => {
            if (success && success.error) {
                MessageService.showMessageByCode(success.error, widgetDetalhes.currentRow);
            } else {
                this.cancelChangeMarca();
                ScreenService.successNotification('Registro alterado com sucesso!');
                widgetProdutos.reload();
                widgetProdutos.selectedRow = {};
            }

        }).catch((error) => {
            if (error.error) {
                ScreenService.showMessage(ScreenService.i18n(error.error, widgetDetalhes.currentRow));
            }
        });
    };

    /**
     * Conjunto de instruções a serem executadas 
     * ao entra na widget de produtos.
     */
    this.onEnterProdutos = function() {
        var widgetProdutos = getWidget("widgetProdutos");
        widgetProdutos.selectedRow = {};
    };

    /**
     * Limpa marca selecionada no produto.
     */
    this.clearMarcaProduto = function() {
        var widgetDetalhes = getWidget("widgetDetalhes");
        var widgetProdutos = getWidget("widgetProdutos");

        widgetProdutos.selectedRow.NRVERSAOPROPCOM = widgetDetalhes.currentRow.NRVERSAOPROPCOM;
        widgetProdutos.selectedRow.NRSERVICOPROPCOM = widgetDetalhes.currentRow.NRSERVICOPROPCOM;

        if (!Util.isEmptyObject(widgetProdutos.selectedRow)) {

            Tco01101_detalhes_service.clearMarcaProduto(widgetProdutos.selectedRow).then((success) => {
                if (success && success.error) {
                    MessageService.showMessageByCode(success.error, widgetDetalhes.currentRow);
                } else {
                    widgetProdutos.reload();
                    widgetProdutos.selectedRow = {};
                }

            }).catch((error) => {
                if (error.error) {
                    ScreenService.showMessage(ScreenService.i18n(error.error, widgetDetalhes.currentRow));
                }
            });

        } else {
            ScreenService.showMessage("Nenhum produto selecionado!");
        }
    };

    /**
     * Abre popup de customização.
     */
    this.openCustomize = function() {
        var widCustomize = getWidget("widCustomize");
        ScreenService.openPopup(widCustomize);
    };

    /**
     * Cancela customização do serviço.
     */
    this.cancelCustomize = function() {
        var widCustomize = getWidget("widCustomize");
        ScreenService.closePopup();
        widCustomize.currentRow = {};
    };

    /**
     * Confirma customização e salva registro.
     */
    this.confirmCustomize = function() {
        var widgetDetalhes = getWidget('widgetDetalhes');
        var widCustomize = getWidget('widCustomize');
        var widgetStripe = getWidget('widgetStripe');

        var cRow = widgetDetalhes.currentRow;
        cRow.DSOBSERVACAO = widCustomize.currentRow.DSOBSERVACAO;

        Tco01101_detalhes_service.confirmCustomize(cRow).then(success => {
            if (success && success.error) {
                MessageService.showMessageByCode(success.error, cRow);
            } else {
                this.cancelCustomize();
                ScreenService.successNotification('Operação realizada com sucesso.');

                widgetDetalhes.currentRow.IDSERVICOPROPCOM = 'AGUARDANDOPLANEJAMENTO';
                widgetStripe.currentRow.IDSITUACAO = _servPropSituations.AGUARDANDOPLANEJAMENTO;

                showActionsByParams();
            }

        }).catch((error) => {
            if (error.error) {
                if (error.error.indexOf('TCO') >= 0) {
                    MessageService.showMessageByCode(error.error);
                } else
                    ScreenService.showMessage(ScreenService.i18n(error.error, cRow));
            }
        });
    };

    /**
     * Exibe as actions corretas de acordo com
     * os parametros do serviço.
     */
    var showActionsByParams = () => {
        setStripeCustFieldsVisibility();
        setRespostaVisibility();
        setTipoPratCustFieldVisibility();
        setOpenServComplVisibility();
    };

    /**
     * Define visibilidade do botão e responder
     * para serviços que possuam status aguardando planejamento.
     */
    var setOpenServComplVisibility = function() {
        var widgetDetalhes = getWidget('widgetDetalhes');
        var widgetDetalhesTabbed = getWidget('widgetDetalhesTabbed');
        var cRow = widgetDetalhes.currentRow;

        widgetDetalhes.getAction('openServCompl').isVisible = Boolean(cRow.NRCARDAPIOTRAB) && !cRow.NRSERVICOPROPCOMCOMPL;
        widgetDetalhes.redraw();

        widgetDetalhesTabbed.widgets.forEach(wid => {
            wid.getAction('openServCompl').isVisible = Boolean(cRow.NRCARDAPIOTRAB) && !cRow.NRSERVICOPROPCOMCOMPL;
        });

        templateManager.updateTemplate();
    };

    /**
     * Define visibilidade do botão e responder
     * para serviços que possuam status aguardando planejamento.
     */
    var setRespostaVisibility = function() {
        var widgetDetalhes = getWidget('widgetDetalhes');
        var widgetDetalhesTabbed = getWidget('widgetDetalhesTabbed');
        var cRow = widgetDetalhes.currentRow;

        widgetDetalhesTabbed.widgets.forEach(wid => {
            wid.getAction('responderPlanej').isVisible = cRow.IDSERVICOPROPCOM == 'AGUARDANDOGN';
        });

        templateManager.updateTemplate();
    };

    /**
     * Abre widget de responder ao planejamento.
     */
    this.openWidgetResponder = function() {
        var widResponder = getWidget('widResponder');
        widResponder.currentRow = {};
        ScreenService.openPopup(widResponder);
    };

    /**
     * Cancela resposta ao planejamento.
     */
    this.cancelResposta = function() {
        var widResponder = getWidget("widResponder");
        ScreenService.closePopup();
        widResponder.currentRow = {};
    };

    /**
     * Envia resposta ao operador do planejamento.
     */
    this.enviaResposta = function() {
        var widgetDetalhes = getWidget('widgetDetalhes');
        var widResponder = getWidget('widResponder');

        var cRow = widgetDetalhes.currentRow;
        cRow.DSOBSERVACAO = widResponder.currentRow.DSOBSERVACAO;

        Tco01101_detalhes_service.enviaResposta(cRow).then(success => {

            if (success && success.error) {
                MessageService.showMessageByCode(success.error, cRow);
            } else {
                this.cancelCustomize();
                ScreenService.successNotification('Resposta enviada com sucesso.');
            }

        }).catch((error) => {
            if (error.error) {
                if (error.error.indexOf('TCOPROC_') >= 0) {
                    MessageService.showMessageByCode(error.error);
                } else
                    ScreenService.showMessage(ScreenService.i18n(error.error));
            }
        });
    };

    /**
     * Conjunto de instruções a serem executadas ao
     * entrar na tela de histórico.
     */
    this.onEnterHistorico = function() {
        var widgetHistorico = getWidget('widgetHistorico');
        widgetHistorico.selectedRow = {};
    };

    /**
     * Abre widget de escopo para exibição de 
     * informações sobre a solicitação.
     */
    this.openScopeWidget = function() {
        var widScope = getWidget('widScope');
        var widgetHistorico = getWidget('widgetHistorico');

        if (!Util.isEmptyObject(widgetHistorico.selectedRow)) {
            setTimeout(function() {
                document.getElementById("empty").innerHTML = widgetHistorico.selectedRow.TXHISTORICO;
                var elements = document.querySelector("#empty").getElementsByTagName("script");
                while (elements[0]) {
                    elements[0].parentNode.removeChild(elements[0]);
                }
            }, 500);

            ScreenService.openPopup(widScope);
        } else {
            ScreenService.showMessage("Nenhum registro selecionado!");
        }
    };

    /**
     * Conjunto de instruções a serem executadas ao
     * entrar na tela de histórico.
     */
    this.afterMoveRowHist = function(args) {
        var widgetHistorico = getWidget('widgetHistorico');
        widgetHistorico.selectedRow = args.row;
    };

    /**
     * Conjunto de instruções a serem executadas ao
     * entrar na tela de observações.
     */
    this.onEnterObservacoes = function() {
        var widgetObservacoes = getWidget('widgetObservacoes');
        setValueOnObservacao(widgetObservacoes);
    };

    /**
     * Define valor no campo de observação
     * da widget de Observações.
     * @param {Widget} widgetObservacoes 
     */
    var setValueOnObservacao = function(widgetObservacoes) {
        var widgetDetalhes = getWidget('widgetDetalhes');
        var filter = [
            {
                name: "NRSERVICOPROPOSTA",
                operator: "=",
                value: widgetDetalhes.currentRow.NRSERVICOPROPOSTA
            },
            {
                name: "NRSERVICOPROPCOM",
                operator: "=",
                value: widgetDetalhes.currentRow.NRSERVICOPROPCOM
            }
        ];

        Tco01200CardapioDaPropostaService.findObservacoesServComercial(filter).then(function(response) {
            if (response && response.error) {
                MessageService.showMessageByCode(response.error, widgetDetalhes.currentRow);
            } else {
                if (!Util.isEmptyOrBlank(response.dataset.observacoesServComercial))
                    widgetObservacoes.currentRow = response.dataset.observacoesServComercial[0];
            }
        }).catch((error) => {
            if (error.error) {
                ScreenService.showMessage(ScreenService.i18n(error.error, widgetDetalhes.currentRow));
            }
        });
    };

    /**
     * Abre widget de serviços complementares.
     */
    this.openServCompl = function() {
        var widgetDetalhes = getWidget('widgetDetalhes');

        Tco01101_detalhes_service.openServCompl(widgetDetalhes.currentRow).then(function(response) {
            if (response && response.error) {
                MessageService.showMessageByCode(response.error, widgetDetalhes.currentRow);
            } else {
                var servComplWidget = getWidget('servComplWidget');
                servComplWidget.activeWidget = widgetDetalhes.container.getActiveWidget().name;

                servComplWidget.dataSource.clearCheckedRows();
                servComplWidget.dataSource.data = response.dataset.servComplData;
                ScreenService.openPopup(servComplWidget);
            }

        }).catch((error) => {
            if (error.error) {
                if (error.error.indexOf('TCOPROC_') >= 0) {
                    MessageService.showMessageByCode(error.error);
                } else
                    ScreenService.showMessage(ScreenService.i18n(error.error));
            }
        });
    };

    /**
     * Selecionar serviços complementares
     * para integrar custo do serviço.
     */
    this.selectServCompl = function() {
        var widgetDetalhes = getWidget('widgetDetalhes');
        var servComplWidget = getWidget('servComplWidget');
        var activeWidget = getWidget(servComplWidget.activeWidget);

        var objToSend = {
            servData: {
                NRSERVICOPROPCOM: widgetDetalhes.currentRow.NRSERVICOPROPCOM,
                NRVERSAOPROPCOM: widgetDetalhes.currentRow.NRVERSAOPROPCOM,
                NRCARDAPIOBASE: widgetDetalhes.currentRow.NRCARDAPIOBASE,
                NRCARDAPIOTRAB: widgetDetalhes.currentRow.NRCARDAPIOTRAB,
                NRTIPOPUBLICO: widgetDetalhes.currentRow.NRTIPOPUBLICO,
                NRPADRAOQUALIDADE: widgetDetalhes.currentRow.NRPADRAOQUALIDADE,
                QTSEGSEX: widgetDetalhes.currentRow.QTSEGSEX
            },
            servCompl: servComplWidget.dataSource.getCheckedRows()
        };

        Tco01101_detalhes_service.selectServCompl(objToSend).then((response) => {
            if (response && response.error) {
                MessageService.showMessageByCode(response.error, widgetDetalhes.currentRow);
            } else {
                var rData = response.dataset;

                if (rData.selectServCompl) {
                    if (rData.selectServCompl == 'VALID') {

                        servComplWidget.dataSource.clearCheckedRows();
                        ScreenService.closePopup(true);

                        if (activeWidget)
                            activeWidget.activate();

                        ScreenService.successNotification('Operação realizada com Sucesso.');

                    } else if (Array.isArray(rData.selectServCompl[0])) {

                        var mensagem = [];
                        var msgParams = [];
                        rData.selectServCompl[0].forEach((row) => {
                            mensagem.push('TCO01100_SERV_COMPL_EXIST');
                            msgParams.push(row);
                        });

                        MessageService.getMessagesByCode(mensagem).then(function(mensagensTraduzidas) {
                            for (let i = 0; i < mensagensTraduzidas.length; i++) {
                                mensagensTraduzidas[i] = ScreenService.i18n(mensagensTraduzidas[i], msgParams[i]);
                            }
                            General.showErrorMessages(servComplWidget, mensagensTraduzidas, 'Operação Inválida.');
                        });

                    } else {
                        MessageService.showMessageByCode(rData.selectServCompl);
                    }
                }
            }

        }).catch((error) => {
            if (error.error) {
                ScreenService.showMessage(ScreenService.i18n(error.error, widgetDetalhes.currentRow));
            }
        });
    };

    /**
     * Abre widget de notificação de planejamento
     * para serviços complementares.
     */
    this.openNotifyServCompl = function() {
        var widNotifyPlanej = getWidget('widNotifyPlanej');
        widNotifyPlanej.currentRow = {};
        ScreenService.openPopup(widNotifyPlanej);
    };

    /**
     * Envia notificação ao planejamento
     */
    this.confirmNotifyPlanejServCompl = function() {
        var widgetDetalhes = getWidget('widgetDetalhes');
        var widNotifyPlanej = getWidget('widNotifyPlanej');
        var cRow = widgetDetalhes.currentRow;

        cRow.MOTIVO = 'SERVICOCOMPLEMENTAR';
        cRow.DSOBSERVACAO = widNotifyPlanej.currentRow.DSOBSERVACAO;

        Tco01101_detalhes_service.notificarPlan(cRow).then((success) => {
            if (success && success.error) {
                MessageService.showMessageByCode(success.error);
            } else {
                ScreenService.alertNotification('Equipe de planejamento notificada');
                ScreenService.closePopup();
            }

        }).catch((error) => {
            if (error.error) {
                if (error.error.indexOf('TCOPROC_') >= 0) {
                    MessageService.showMessageByCode(error.error);
                } else
                    ScreenService.showMessage(ScreenService.i18n(error.error));
            }
        });
    };

    /**
     * Bloqueia acesso as widgets irmãs da
     * widget tipo de prato.
     * 
     * @param {Boolean} state Estado de bloqueio.
     */
    var lockIncidenciaSisterWidgets = function(state) {
        var widgetIncidencias = getWidget('widgetIncidencias');

        var sisterWdgts = widgetIncidencias.getParent().widgets.filter(wid => wid != widgetIncidencias);
        General.changeWidgetAcessibility(sisterWdgts, !state);
    };

    this.onChangeIncidencias = function() {
        var widgetIncidencias = getWidget('widgetIncidencias');
        lockIncidenciaSisterWidgets(true);
    };
    this.onCancelInlineEditIncidencias = function() {
        lockIncidenciaSisterWidgets(false);
        var widgetIncidencias = getWidget('widgetIncidencias');
        widgetIncidencias.activate();
        widgetIncidencias.inlineEditing = false;
        widgetIncidencias.list();
        widgetIncidencias.reload();
    };
};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01101_detalhes_controller', Tco01101_detalhes_controller);
});

// FILE: ./assets/js/controllers/Tco01200_cardapio_da_proposta.js
let Tco01200_cardapio_da_proposta = function(ScreenService, General, Tco01200CardapioDaPropostaService, templateManager, ZHPromise, TcoUtil, Tco01200_ActionsValidator, MessageService) {

    this.setDefaultValueFilter = function() {
        let widget = templateManager.container.getWidget("widgetCardProp");
        let start = moment().startOf('month').format('DD/MM/YYYY');
        let end = moment().endOf('month').format('DD/MM/YYYY');
        widget.getField("DTLANCAMENTO").defaultValue = {
            start: start,
            end: end
        };
    };

    /**
     * Função responsável por abrir as telas de detalhes. 
     * É acionado pela action DETAILSERVICE.
     * 
     * Antes de habilitar as telas é verificado se o 
     * usuário selecionou algum serviço do grid principal.
     * 
     * @params {Args} args - objeto args
     **/

    this.openDetails = function(args) {
        let widget = args.owner.widget;
        let rowCardProp = widget.currentRow;
        let action = args.owner.name;
        if (!Util.isEmptyOrBlank(rowCardProp.NRSERVICOPROPCOM)) {
            if (Tco01200_ActionsValidator.verifyPermission(action, rowCardProp)) {
                let params = {
                    "filter": {
                        "params": widget._filterInfo
                    },
                    "currentRow": {
                        "params": widget.currentRow
                    },
                    "filterInfo": {
                        "FILTER_ROW": widget.appliedFilter.FILTER_ROW
                    }
                };

                TcoUtil.openWindowByHistory("#/tco#tco01201_detalhes", params, "Detalhes do Serviço");
            }
        } else {
            ScreenService.showMessage("Selecione um Serviço.");
        }
    };

    /**
     * Função que valida se o usuário selecionou algum
     * serviço do grid principal.
     * 
     * @params {rowCardProp} rowCardProp - objeto currentRow da widgetCardProp 
     **/

    this.hasCardTrab = function(rowCardProp) {
        return (!Util.isEmptyOrBlank(rowCardProp.NRCARDAPIOTRAB));
    };

    /**
     * Função responsável por abrir as telas de detalhes. 
     * É acionado pela action DETAILSERVICE.
     * 
     * Antes de chamar o openSwipe é verificado se o 
     * usuário selecionou algum serviço do grid principal.
     * 
     * @params {Args} args - objeto args
     **/

    this.openCardapio = function(args) {
        let widget = args.owner.widget;
        let rowCardProp = widget.currentRow;
        let action = args.owner.name;
        if (this.hasCardTrab(rowCardProp)) {
            if (Tco01200_ActionsValidator.verifyPermission(action, rowCardProp)) {

                let params = {
                    "stripe": {
                        "CDPROPOSTACOMERCIAL": rowCardProp.CDPROPOSTACOMERCIAL,
                        "NMSERVICOPROPOSTA": rowCardProp.NMSERVICOPROPOSTA,
                        "NRPADRAOQUALIDADE": rowCardProp.NRPADRAOQUALIDADE,
                        "NMPADRAOQUALIDADE": Util.isEmptyOrBlank(rowCardProp.NRPADRAOQUALIDADE) ? "" : rowCardProp.NRPADRAOQUALIDADE + ' - ' + rowCardProp.NMPADRAOQUALIDADE,
                        "IDSERVICOPROPCOM": rowCardProp.IDSERVICOPROPCOM,
                        "DSSERVICOPROPCOM": rowCardProp.DSSERVICOPROPCOM,
                        "DTLANCAMENTO": rowCardProp.DTLANCAMENTO,
                        "NMTIPOPUBLICO": rowCardProp.NMTIPOPUBLICO,
                        "DTCUSTO": moment(rowCardProp.DTCUSTO, 'DD/MM/YYYY').format('L')
                    },
                    "filter": {
                        "NRCARDAPIOTRAB": rowCardProp.NRCARDAPIOTRAB
                    },
                    "filterInfo": {
                        "FILTER_ROW": widget.appliedFilter.FILTER_ROW
                    },
                    "currentRow": {
                        "params": widget.currentRow
                    }
                };

                TcoUtil.openWindowByHistory("#/tco#tco01202_elaboracao_cardapio", params, "Elaboração de Cardápio");
            }
        } else {
            MessageService.showMessageByCode("TCO01200_SEM_CARD_TRAB");
        }
    };

    /**
     * Função responsável por abrir o popup da responsável. 
     * É acionado pela action RESPONSIBLE.
     * 
     * Antes de chamar o openSwipe é verificado se o 
     * usuário selecionou algum serviço do grid principal.
     * 
     * @params {Args} args - objeto args
     **/

    this.openPopupResponsible = function(args) {
        let widCardProp = templateManager.container.getWidget('widgetCardProp');
        let action = args.owner.name;
        if (Tco01200_ActionsValidator.verifyPermission(action, widCardProp.currentRow)) {
            let widResponsible = templateManager.container.getWidget('responsible');
            ScreenService.openPopup(widResponsible).then(function() {
                let widgetResponsible = templateManager.container.getWidget("responsible");
                this.onEnterResponsible(widgetResponsible);
            }.bind(this));
        }
    };

    /**
     * Função responsável por criar a propriedade __currentManager 
     * no operador responsável do serviço da proposta, caso exista.
     * Essa propriedade é verificada no styles do container para aplicar
     * um css de identificação no operador atual.
     * 
     * @params {Object} widgetResponsible - widgetResponsible
     **/

    this.onEnterResponsible = function(widgetResponsible) {
        let rowCardProp = templateManager.container.getWidget("widgetCardProp").currentRow;
        let dataResponsible = widgetResponsible.dataSource.data;
        dataResponsible.forEach(function(row, count) {
            if (row.CDOPERADOR === rowCardProp.CDRESPONSAVELPLAN) {
                widgetResponsible.dataSource.data[count].__currentManager = true;
            } else {
                widgetResponsible.dataSource.data[count].__currentManager = undefined;
            }
        });
        widgetResponsible.currentManager = rowCardProp.CDRESPONSAVELPLAN;
    };

    /**
     * Função responsável por atualizar o grid dos serviços da proposta
     * e manter o escopo anterior.
     * 
     * Caso houvesse algum serviço selecionado, ele será mantido.
     * 
     * Função chamada no onEnter. Util quando alterações também referentes
     * a essa tela são feitas na tela de elaboração do cardápio.
     * 
     * @params {Args} args - objeto args
     **/

    this.refreshInScope = function(args) {
        let widget = args.owner;
        let nrpropostacomercial = widget.currentRow.NRSERVICOPROPCOM ? angular.copy(widget.currentRow.NRSERVICOPROPCOM) : "";
        if (nrpropostacomercial) {
            widget.reload().then(f => {
                let currentRow = widget.dataSource.data.filter(row => {
                    return row.NRSERVICOPROPCOM === nrpropostacomercial;
                })[0];
                widget.setCurrentRow(currentRow);
            });
        }
    };

    this.abrirPopupImportacao = function(args) {
        let widgetMain = templateManager.container.getWidget('widgetCardProp');
        let action = 'IMPORT';
        if (Util.isEmptyOrBlank(widgetMain.currentRow.NRSERVICOPROPCOM)) {
            MessageService.showMessageByCode('TCO01200_SERV_N_SEL');
        } else {
            if (Util.isEmptyOrBlank(widgetMain.currentRow.NRCARDAPIOBASE)) {
                MessageService.showMessageByCode("TCO01200_SEM_CARD_BASE");
            } else {
                if (Tco01200_ActionsValidator.verifyPermission(action, widgetMain.currentRow)) {
                    let widgetImport = templateManager.container.getWidget('popupCalc');
                    widgetImport.currentRow.DTREFERENCIA = moment().format('DD/MM/YYYY');
                    widgetImport.getField('NMREGIAO').value(widgetMain.currentRow.CDREGIAO);
                    widgetImport.currentRow.VRPEACRESCIMO = null;
                    ScreenService.openPopup(widgetImport);
                }
            }
        }
    };

    this.confirmImport = function(args) {
        let widgetImport = templateManager.container.getWidget('popupCalc');
        if (widgetImport.isValid()) {
            MessageService.confirmMessageByCode('TCO01200_CONFIRM_IMPORT').then(this.importCardTrab);
        }
    };

    this.importCardTrab = function() {
        let widgetMain = templateManager.container.getWidget('widgetCardProp');
        let widgetImport = templateManager.container.getWidget('popupCalc');
        let action = 'IMPORT';
        let currentRow = angular.copy(widgetMain.currentRow);
        currentRow.CDREGIAO = widgetImport.currentRow.CDREGIAO;
        currentRow.DTCUSTO = widgetImport.currentRow.DTREFERENCIA;
        currentRow.VRPEACRESCUSTO = TcoUtil.parseFloat(widgetImport.currentRow.VRPEACRESCIMO);

        if (Tco01200_ActionsValidator.verifyPermission(action, currentRow)) {
            Tco01200CardapioDaPropostaService.importCardTrab(currentRow).then(function(response) {
                if (response.dataset.ERROR) {
                    TcoUtil.buildMessagesToShow(response.dataset.ERROR).then(function(mensagens) {
                        widgetMain.currentRow.CDFILIALLISTA = response.dataset.LISTA.CDFILIAL;
                        widgetMain.currentRow.NRLISTA = response.dataset.LISTA.NRLISTA;
                        General.showErrorMessages(widgetMain, mensagens, ScreenService.i18n("Erro"));
                    });
                } else {
                    widgetMain.reload().then(() => {
                        var servico = widgetMain.dataSource.data.filter(function(row) {
                            return row.NRSERVICOPROPCOM === currentRow.NRSERVICOPROPCOM;
                        })[0];

                        if (!Util.isEmptyOrBlank(servico)) {
                            widgetMain.setCurrentRow(servico);
                        }
                    });
                    ScreenService.successNotification('Cardápio importado com sucesso!');
                    ScreenService.closePopup();
                }
            });
        }
    };
    this.setMetaDataReport = function(args) {
        const filterRow = args.e.preference.FILTER_ROW;
        let row = {
            DTINI: filterRow.DTLANCAMENTO.formattedStart,
            DTFIN: filterRow.DTLANCAMENTO.formattedEnd
        };
        TcoUtil.newFilterReport(args.owner, row);
    };
};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01200_cardapio_da_proposta', Tco01200_cardapio_da_proposta);
});

// FILE: ./assets/js/controllers/Tco01201_detalhe_servico_proposta.js
var Tco01201_detalhe_servico_proposta = function(ScreenService, General, Tco01200CardapioDaPropostaService, templateManager, ZHPromise, TcoUtil, Tco01100_document_service, Tco01200_ActionsValidator, maskEngine, MessageService) {

    this.carregaParametros = function(args) {
        if (!window.history.state) {
            window.location.href = window.location.href.replace("#/tco#tco01201_detalhes", "#/tco#tco01200_cardapio_da_proposta");
            window.location.reload();
        } else {
            var widgetDetalhes = args.owner.getWidget("widgetDetalhes");
            widgetDetalhes.activate();
            widgetDetalhes.currentRow = window.history.state.currentRow.params;
            widgetDetalhes.redraw();
            let t = this;
            //Busca páginas a destacar de acordo com a solicitação feita ao planejamento
            Tco01200CardapioDaPropostaService.getInfoADestacar(widgetDetalhes.currentRow).then(function(response) {
                setTimeout(function() {
                    t.destacaTabs(response.dataset.paginasADestacar);
                }, 100);

            });
        }
    };

    /**
     * Volta para ultima tela.
     **/

    this.backCardProp = function() {
        window.history.back();
    };

    /**
     * Fecha o swipe e volta para o grid.
     **/

    this.backGrid = function(args) {
        let widget = args.owner.widget;
        ScreenService.closeSwipe();
        widget.activate();
    };

    /**
     * Função responsável por montar o dataSourceFilter
     * das widget de detalhe do serviço da proposta comercial.
     * 
     * @params {Args} args - objeto args 
     **/

    this.setDataSourceFilterWidgetsDetails = function(args) {
        var widget = args.owner;
        var rowDetails = widget.container.getWidget('widgetDetalhes').currentRow;
        var servicoProposta = !Util.isEmptyOrBlank(rowDetails.NRSERVICOPROPOSTA) ? rowDetails.NRSERVICOPROPOSTA : '';
        var servicoPropostaComercial = !Util.isEmptyOrBlank(rowDetails.NRSERVICOPROPCOM) ? rowDetails.NRSERVICOPROPCOM : '';
        widget.activate();
        widget.dataSourceFilter = [
            {
                "name": "NRSERVICOPROPOSTA",
                "value": servicoProposta,
                "operator": "="
			},
            {
                "name": "NRSERVICOPROPCOM",
                "value": servicoPropostaComercial,
                "operator": "="
			}
		];
        widget.reload();
    };

    this.getListaError = function() {
        var widget = templateManager.container.getWidget('widgetErrosCalc');
        var rowDetails = templateManager.container.getWidget('widgetDetalhes').currentRow;
        var nrlista = !Util.isEmptyOrBlank(rowDetails.NRLISTA) ? rowDetails.NRLISTA : null;
        var cdfiliallista = !Util.isEmptyOrBlank(rowDetails.CDFILIALLISTA) ? rowDetails.CDFILIALLISTA : null;
        widget.activate();
        let filter = [
            {
                "NRLISTA": nrlista,
                "CDFILIALLISTA": cdfiliallista
            }
        ];
        Tco01200CardapioDaPropostaService.getListaError(filter).then(response => {
            let newData = [];
            TcoUtil.buildMessagesToShow(response.dataset.LISTAERROR).then(mensagens => {
                mensagens.forEach((msg, index) => {
                    newData.push({
                        "DSERRO": msg
                    });
                });
                widget.dataSource.data = newData;
            });
        });

    };

    /**
     * Função responsável marcar as rows com o UTLSERVPROPCOM igual a 'S'.
     * 
     * @params {Args} args - objeto args 
     **/

    this.setSelectsTipoPrato = function(args) {
        var widget = args.owner;
        widget.dataSource.clearCheckedRows();
        widget.dataSource.data.forEach(function(data, count) {
            widget.dataSource.data[count].__isSelected = (data.UTLSERVPROPCOM === 'S');
        });
        widget.dataSource.updateCheckedRows();
    };

    /**
     * Função responsável por não deixar o usuário mudar o state de
     * nenhuma row pelo checkedRow.
     * 
     * @params {Args} args - objeto args 
     **/

    this.preventDefault = function(args) {
        args.owner.dataSource.checkedRowsCache = {};
        args.e.preventDefault();
    };

    /**
     * Função responsável por setar a currentRow da widgetObservacoes.
     * 
     * @params {Args} args - objeto args 
     **/

    this.setObservacoesOnCurrentRow = function(args) {
        var widget = args.owner;
        var rowDetails = templateManager.container.getWidget('widgetDetalhes').currentRow;
        var servicoProposta = !Util.isEmptyOrBlank(rowDetails.NRSERVICOPROPOSTA) ? rowDetails.NRSERVICOPROPOSTA : '';
        var servicoPropostaComercial = !Util.isEmptyOrBlank(rowDetails.NRSERVICOPROPCOM) ? rowDetails.NRSERVICOPROPCOM : '';
        var filter = [
            {
                name: "NRSERVICOPROPOSTA",
                operator: "=",
                value: servicoProposta
            },
            {
                name: "NRSERVICOPROPCOM",
                operator: "=",
                value: servicoPropostaComercial
            }
        ];
        Tco01200CardapioDaPropostaService.findObservacoesServComercial(filter).then(function(response) {
            if (!Util.isEmptyOrBlank(response.dataset.observacoesServComercial))
                widget.currentRow = response.dataset.observacoesServComercial[0];
            widget.activate();
        });
    };

    /**
     * Função responsável por setar o dataSourceFilter da widgetIncidencias.
     * 
     * @params {Args} args - objeto args 
     **/

    this.setDataSourceFilterProdIncidencia = function(args) {
        var widget = args.owner;
        var rowDetails = widget.container.getWidget('widgetDetalhes').currentRow;
        var detServPropPla = !Util.isEmptyOrBlank(rowDetails.NRDETSERVPROPCOMPLAN) ? rowDetails.NRDETSERVPROPCOMPLAN : null;
        var detServPropCom = !Util.isEmptyOrBlank(rowDetails.NRDETALHESERVPROPCOM) ? rowDetails.NRDETALHESERVPROPCOM : null;
        widget.activate();
        widget.dataSourceFilter = [
            {
                "name": "NRDETALHESERVPROPCOM",
                "value": detServPropCom,
                "operator": "="
			},
            {
                "name": "NRDETSERVPROPCOMPLAN",
                "value": detServPropPla,
                "operator": "="
			}
		];
        widget.reload();
    };

    /**
     * Função que valida se o usuário selecionou algum
     * histórico de solicitações do grid da widgetHistorico.
     * 
     * @params {Args} args - objeto args 
     **/

    this.validActionEscopo = function(args) {
        var widget = args.owner.widget;
        if (!Util.isEmptyOrBlank(widget.currentRow.DTINCLUSAO)) {
            this.scope(widget.currentRow);
        } else {
            ScreenService.showMessage("Selecione alguma solicitação para continuar.");
        }
    };

    this.scope = function(currentRow) {
        var widScope = templateManager.container.getWidget("widScope");
        widScope.strHtml = currentRow.TXHISTORICO;
        ScreenService.openPopup(widScope);
    };

    /**
     * Função responsável por colocar o html retornado do banco no innerHTML
     * do template vazio criado apenás para receber tags html.
     * 
     * O while procura qualquer script que possa ser injetado no html e remove.
     * 
     * @params {Args} args - objeto args 
     **/

    this.onEnterScope = function(args) {
        var widget = args.owner;
        setTimeout(function() {
            if (widget.strHtml) {
                document.getElementById("empty").innerHTML = widget.strHtml;
                var elements = document.querySelector("#empty").getElementsByTagName("script");
                while (elements[0]) {
                    elements[0].parentNode.removeChild(elements[0]);
                }
                widget.strHtml = null;
            }
        }, 500);
    };

    /**
     * Realiza o download o arquivo selecionado.
     * 
     * @params {Args} args - objeto args
     */
    this.download = function(args) {
        let action = args.owner.name;
        let widget = args.owner.widget;

        if (widget.currentRow.NMDOCUMENTOPROPCOM) {
            if (Tco01200_ActionsValidator.verifyPermission(action, widget.currentRow)) {
                widget.currentRow.NRPROPOSTACOMERCIAL = window.history.state.currentRow.params.NRPROPOSTACOMERCIAL;
                var fileToDownload = widget.currentRow;
                Tco01100_document_service.downloadFile(fileToDownload).then((success) => {
                    if (success && success.error) {
                        MessageService.showMessageByCode(success.error, widget.currentRow);
                    }
                }).catch((error) => {
                    ScreenService.showMessage(ScreenService.i18n(error.error, widget.currentRow));
                });
            }
        } else {
            ScreenService.showMessage("Selecione um documento para fazer o dowload.");
        }
    };

    /**
     * Função resposável por abrir o popup de seleção de marca para o produto.
     * Nela é verificado se o usuário selecionou algum produto e se a ação é
     * permitida para a situação filtrada.
     * 
     * @params {Args} args - objeto args
     */

    this.openPopupSelectBrand = function(args) {
        let action = args.owner.name;
        let widget = args.owner.widget;
        let currentRow = window.history.state.currentRow.params;
        if (widget.currentRow.CDARVPROD) {
            if (Tco01200_ActionsValidator.verifyPermission(action, currentRow)) {
                let popupSelectBrand = templateManager.container.getWidget('widPopupSelectBrand');
                popupSelectBrand.newRow();
                ScreenService.openPopup(popupSelectBrand).then(() => {
                    this.setDataSourceFilterSelectBrand();
                });
            }
        } else {
            ScreenService.showMessage("Selecione um Produto.");
        }
    };

    /**
     * Função resposável por setar o dataSource da widget de seleção de produtos.
     * 
     */

    this.setDataSourceFilterSelectBrand = function() {
        let widget = templateManager.container.getWidget('widPopupSelectBrand');
        let widgetProdutos = templateManager.container.getWidget('widgetProdutos');
        let rowDetails = widget.container.getWidget('widgetDetalhes').currentRow;
        let nrDetalheServPropCom = (!Util.isEmptyOrBlank(rowDetails.NRDETSERVPROPCOMPLAN)) ? rowDetails.NRDETSERVPROPCOMPLAN : rowDetails.NRDETALHESERVPROPCOM;
        let filter = [
            {
                "name": "CDPRODUTO",
                "operator": "=",
                "value": widgetProdutos.currentRow.CDPRODUTO
            },
            {
                "name": "NRDETALHESERVPROPCOM",
                "operator": "=",
                "value": nrDetalheServPropCom
            }
        ];

        widget.dataSourceFilter = filter;
        widget.reload();
    };

    /**
     * Função resposável por abrir a widget de percentual de acrésimo, para que 
     * o usuário o informe o acréscimo da marca selecionada.
     * 
     *  @params {Args} args - objeto args
     */

    this.selectedBrand = function(args) {
        let widBrands = args.owner.widget;
        if (widBrands.currentRow.CDMARCA) {
            let widPercent = templateManager.container.getWidget('widPopupPercent');
            widPercent.newRow();
            ScreenService.openPopup(widPercent);
        } else {
            ScreenService.showMessage("Selecione uma marca.");
        }
    };

    /**
     * Função resposável por montar a currentRow da marca selecionada.
     * 
     * @params {Object} rowPercent - objeto currentRow.
     * @params {Object} rowProducts - objeto currentRow.
     * @params {Object} rowBrands - objeto currentRow.
     */

    let buildCurrentRow = function(rowPercent, rowProducts, rowBrands) {
        let rowToSave;
        rowToSave = rowBrands;
        rowToSave.NRPRODUTOPROPCOM = rowProducts.NRPRODUTOPROPCOM;
        rowToSave.NRPRODUTOCARDTRAB = rowProducts.NRPRODUTOCARDTRAB;
        rowToSave.CDMARCAESCOLHIDA = rowProducts.CDMARCAESCOLHIDA;
        rowToSave.CDMARCASOLICITADA = rowProducts.CDMARCASOLICITADA;
        rowToSave.VRPEACRESCIMO = TcoUtil.parseFloat(rowPercent.VRPEACRESCIMO);
        rowToSave.NRDETALHESERVPROPCOM = window.history.state.currentRow.params.NRDETALHESERVPROPCOM;
        rowToSave.NRDETSERVPROPCOMPLAN = window.history.state.currentRow.params.NRDETSERVPROPCOMPLAN;
        rowToSave.NRCARDAPIOTRAB = window.history.state.currentRow.params.NRCARDAPIOTRAB;
        return rowToSave;
    };

    /**
     * Função resposável por confirmar a seleção de marca para o produto 
     * selecionado.
     * 
     * @params {Args} args - objeto args
     */

    this.confirmSelectedBrand = function(args) {
        let widgetProdutos = templateManager.container.getWidget('widgetProdutos');
        let rowPercent = args.owner.widget.currentRow;
        let rowProducts = widgetProdutos.currentRow;
        let rowBrands = templateManager.container.getWidget('widPopupSelectBrand').currentRow;
        let rowToSave;
        rowToSave = buildCurrentRow(rowPercent, rowProducts, rowBrands);
        Tco01200CardapioDaPropostaService.confirmSelectedBrand(rowToSave).then(response => {
            ScreenService.successNotification("Marca Selecionada salva com sucesso!");
            ScreenService.closePopup(true);
            widgetProdutos.activate();
            widgetProdutos.reload();
        }, error => {
            ScreenService.showMessage("Ocorreu um erro inesperado.");
        });
    };

    /**
     * Função resposável por destacar as páginas de acordo com o motivo da solicitação
     *
     */

    this.destacaTabs = function(paginasADestacar) {
        let lista = document.querySelectorAll('.zh-tab-cell a');
        if (lista.length == 0) {
            setTimeout(function() {
                this.destacaTabs(paginasADestacar);
            }, 100);
        } else {
            for (let i = 0; i < lista.length; i++) {
                if (paginasADestacar.indexOf(lista[i].text) >= 0)
                    lista[i].text = lista[i].text + '*';
            }
        }
    };

    /**
     * Função responsável pela chamada do pop de substituição de produto.
     * 
     */

    this.openPopupSubstituiProduto = function(args) {
        var widgetProdutos = args.owner.widget;
        var popupSubstituicaoProduto = args.owner.widget.container.getWidget('popupSubstituicaoProduto');

        if (typeof widgetProdutos.currentRow.CDPRODUTO != "undefined" && widgetProdutos.currentRow.CDPRODUTO != null && widgetProdutos.currentRow.CDPRODUTO != null) {
            var filterProdutos = [
                {
                    "name": "NRLISTAPRECOTCO",
                    "operator": "=",
                    "value": window.history.state.currentRow.params.NRLISTAPRECOTCO
                },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": window.history.state.currentRow.params.NRORG
                },
                {
                    "operator": "ORDER_BY",
                    "name": [
                        {
                            "name": "CDARVPROD",
                            "direction": "ASC"
                        }
                    ]
                }
            ];

            var filterPrato = [
                {
                    "name": "NRCARDAPIOTRAB",
                    "operator": "=",
                    "value": window.history.state.currentRow.params.NRCARDAPIOTRAB
                },
                {
                    "name": "CDPRODUTO",
                    "operator": "=",
                    "value": widgetProdutos.currentRow.CDPRODUTO
                }
            ];

            popupSubstituicaoProduto.newRow();

            popupSubstituicaoProduto.currentRow.QTSUBSPRODUTO = "100";
            popupSubstituicaoProduto.currentRow.PRODATUAL = widgetProdutos.currentRow.CDPRODUTO;
            popupSubstituicaoProduto.currentRow.NMPRODUTOSELECIONADO = widgetProdutos.currentRow.NMPRODUTO;
            popupSubstituicaoProduto.currentRow.NRLISTAPRECOTCO = window.history.state.currentRow.params.NRLISTAPRECOTCO;
            popupSubstituicaoProduto.getField('NMPRODESTO').dataSourceFilter = filterProdutos;
            popupSubstituicaoProduto.getField('NMPRATO').dataSourceFilter = filterPrato;

            var promises = [
                popupSubstituicaoProduto.getField('NMPRODESTO').reload(),
                popupSubstituicaoProduto.getField('NMPRATO').reload()
            ];

            ZHPromise.all(promises).then((success) => ScreenService.openPopup(popupSubstituicaoProduto));

        } else {
            MessageService.showMessageByCode('TCO01200_ERRO_VALIDACAO_SUBS_PROD').then(() => ScreenService.closePopup(true));
        }

    };

    /**
     * Função responsável por controlar o field de Prato e Todos
     * 
     */

    this.alteraFieldPrato = function(args) {
        General.toggleReadOnlySelect(args.owner.field, args.owner.field.widget.getField('NMPRATO'));
    };

    /**
     * Função responsável pela substituição do produto selecionado.
     * 
     */

    this.confirmaSubstituicao = function(args) {
        var popupSubstituicaoProduto = args.owner.widget;
        var widgetProdutos = args.owner.widget.container.getWidget('widgetProdutos');

        var params = window.history.state.currentRow.params;

        if (popupSubstituicaoProduto.isValid()) {
            var row = buildRowProd(params, popupSubstituicaoProduto.currentRow);

            Tco01200CardapioDaPropostaService.substituiProduto(row).then((success) => {
                if (success.dataset.NAO_ATUALIZADOS.length > 0) {
                    aplicaMascaraPratoProduto(success.dataset.NAO_ATUALIZADOS);
                    TcoUtil.buildMessagesToShow(success.dataset.NAO_ATUALIZADOS).then(function(mensagens) {
                        General.showErrorMessages(popupSubstituicaoProduto, mensagens, ScreenService.i18n("Produtos não Atualizados"));
                    });
                } else {
                    widgetProdutos.reload().then(() => {
                        ScreenService.successNotification('Produto Substituido');
                        ScreenService.closePopup(true);
                        widgetProdutos.setCurrentRow(widgetProdutos.dataSource.data[0]);
                        widgetProdutos.activate();
                    });
                }
            });
        }

    };

    var buildRowProd = function(params, currentRow) {
        var row = currentRow;

        row.NRCARDAPIOTRAB = params.NRCARDAPIOTRAB;

        return row;
    };

    var aplicaMascaraPratoProduto = function(rows) {
        rows.forEach((row) => {
            row.PARAMS.CDPRATO = maskEngine.fixMaskParser(row.PARAMS.CDPRATO, {
                mask: getLocalVar("CDPICTPRAT")
            });
            row.PARAMS.CDPRODUTO = maskEngine.fixMaskParser(row.PARAMS.CDPRODUTO, {
                mask: getLocalVar("CDPICTPROD")
            });
        });
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01201_detalhe_servico_proposta', Tco01201_detalhe_servico_proposta);
});

// FILE: ./assets/js/controllers/Tco01202_elaboracao_card_proposta.js
var Tco01202_elaboracao_card_proposta = function(ScreenService, General, Tco01200CardapioDaPropostaService, templateManager, ZHPromise, TcoUtil, Tco01200_ActionsValidator, Tco01004_novo_prato_elaboracao_cardapio_base_service, Tco01009_analise_custos_service, MessageService) {

    this.carregaParametros = function(args) {
        if (!window.history.state || !window.history.state.currentRow.params.NRCARDAPIOTRAB) {
            window.location.href = window.location.href.replace("#/tco#tco01202_elaboracao_cardapio", "#/tco#tco01200_cardapio_da_proposta");
            window.location.reload();
        } else {
            var widgetMain = args.owner.getWidget("cardapio");
            var filter = [{
                "name": "NRCARDAPIOTRAB",
                "operator": "=",
                "value": window.history.state.filter.NRCARDAPIOTRAB
            }];
            TcoUtil.bindKeyboardEvents(this.addPratoCardTrab, this.removePratoCardTrab);
            widgetMain.parentRow = window.history.state.currentRow.params;
            this.buildStripe(window.history.state.stripe);
            widgetMain.dataSourceFilter = filter;
            widgetMain.reload();
        }
    };

    /**
     * Ativa a widget.
     **/

    this.activate = function(args) {
        args.owner.activate();
    };

    /**
     * Volta para ultima tela.
     **/

    this.backCardProp = function() {
        window.history.back();
    };

    /**
     * Monta o stripe da tela de elaboração de cardápio.
     * 
     * @params {Object} data - Dados do stripe.
     **/

    this.buildStripe = function(data) {
        var widgetStripe = templateManager.container.getWidget("widgetStripe");
        data.IDSERVICOPROPCOM = this.dicionary(data.IDSERVICOPROPCOM);
        widgetStripe.currentRow = data;
    };

    /**
     * Função responsável por abrir as telas de detalhes. 
     * É acionado pela action DETAILSERVICE.
     * 
     * Antes de habilitar as telas é verificado se o 
     * usuário selecionou algum serviço do grid principal.
     * 
     * @params {Args} args - objeto args
     **/

    this.openDetails = function(args) {
        let action = args.owner.name;
        let currentRow = window.history.state.currentRow.params;
        let appliedFilter = window.history.state.filterInfo;
        if (Tco01200_ActionsValidator.verifyPermission(action, currentRow)) {
            var params = {
                "filter": {
                    "params": window.history.state.filterInfo
                },
                "currentRow": {
                    "params": window.history.state.currentRow.params
                }
            };

            TcoUtil.openWindowByHistory("#/tco#tco01201_detalhes", params, "Detalhes do Serviço");
        }
    };

    /**
     * Função responsável por abrir as telas de configuracao. 
     * É acionado pela action CONFIG.
     * 
     * @params {Args} args - objeto args
     **/

    this.openConfig = function(args) {
        let action = args.owner.name;
        let appliedFilter = window.history.state.filterInfo;
        let currentRow = window.history.state.currentRow.params;
        if (Tco01200_ActionsValidator.verifyPermission(action, currentRow)) {
            let params = {
                "filter": {
                    "NRCARDAPIOTRAB": window.history.state.filter.NRCARDAPIOTRAB
                },
                "filterInfo": {
                    "FILTER_ROW": window.history.state.filterInfo.FILTER_ROW
                },
                "currentRowCardapioBase": {
                    "FILTER_ROW": currentRow
                }
            };

            TcoUtil.openWindowByHistory("#/tco#tco01203_configuracao_elab_card", params, "Configuração");
        }
    };

    /**
     * Função responsável por abrir as telas de Análise de Custo. 
     * É acionado pela action ANALISE.
     * 
     * @params {Args} args - objeto args
     **/

    this.openAnaliseCusto = function(args) {
        let action = args.owner.name;
        let appliedFilter = window.history.state.filterInfo;
        let currentRow = window.history.state.currentRow.params;
        if (Tco01200_ActionsValidator.verifyPermission(action, currentRow)) {
            let widget = args.owner.widget;
            let rowCardProp = widget.currentRow;
            let params = {
                "filter": {
                    "NRCARDAPIOTRAB": window.history.state.filter.NRCARDAPIOTRAB
                },
                "currentRow": {
                    "params": currentRow
                }
            };

            TcoUtil.openWindowByHistory("#/tco#tco01204_analise_custo", params, "Análise de Custo");
        }
    };

    /**
     * Função responsável por abrir as o popup da action Efetivar.
     * 
     **/

    this.openPopupEffect = function(args) {
        let action = args.owner.name;
        let currentRow = window.history.state.currentRow.params;
        let appliedFilter = window.history.state.filterInfo;
        if (Tco01200_ActionsValidator.verifyPermission(action, currentRow)) {
            var widEffect = templateManager.container.getWidget('effect');
            ScreenService.openPopup(widEffect);
        }
    };

    /**
     * Função responsável por abrir as telas de Dados do Prato. 
     * É acionado pelo WidgetAfterMoveRow.
     * 
     * @params {Args} args - objeto args
     **/

    this.openDadosDoPrato = function(args) {
        if (TcoUtil.isClickOnFocusedCell()) {
            let appliedFilter = window.history.state.filterInfo;
            appliedFilter.FILTER_ROW.IDSERVICOPROPCOM = window.history.state.currentRow.params.IDSERVICOPROPCOM;
            var currentRow = args.owner.currentRow;
            if (isValidCell(currentRow)) {
                let rowCardProp = args.owner.parentRow;
                let currentRow = args.owner.currentRow;
                let params = {
                    "label": {
                        "LABELDIA": currentRow.LABELDIA,
                        "LABELTPPRAT": currentRow.LABELTPPRAT
                    },
                    "filterReceita": {
                        "NRPADRAOQUALIDADE": rowCardProp.NRPADRAOQUALIDADE,
                        "NMPADRAOQUALIDADE": rowCardProp.NMPADRAOQUALIDADE,
                        "CDSERVICOPROPOSTA": rowCardProp.CDSERVICOPROPOSTA,
                        "NMSERVICOPROPOSTA": rowCardProp.NMSERVICOPROPOSTA,
                        "NRSERVICOPROPOSTA": rowCardProp.NRSERVICOPROPOSTA,
                        "CDREGIAO": currentRow.CDREGIAO,
                        "NMREGIAO": currentRow.NMREGIAO,
                        "NRCARDAPIOTRAB": window.history.state.filter.NRCARDAPIOTRAB
                    },
                    "filter": {
                        "NRPADRAOQUALIDADE": rowCardProp.NRPADRAOQUALIDADE,
                        "NMSERVICOPROPOSTA": rowCardProp.NMSERVICOPROPOSTA,
                        "NRSERVICOPROPOSTA": rowCardProp.NRSERVICOPROPOSTA,
                        "CDREGIAO": currentRow.CDREGIAO,
                        "NRCARDAPIOTRAB": window.history.state.filter.NRCARDAPIOTRAB,
                        "CDSERVFILI": window.history.state.currentRow.params.CDSERVFILI
                    },
                    "NRCARDAPIOTRAB": window.history.state.filter.NRCARDAPIOTRAB,
                    "dadosPrato": {
                        "NRCOMPCARDTRAB": currentRow.NRCOMPCARDTRAB,
                        "NRDIACARDTRAB": currentRow.NRDIACARDTRAB,
                        "NRCONFIGCARDTRAB": currentRow.NRCONFIGCARDTRAB,
                        "CDPRATO": currentRow.CDPRATO,
                        "NMPRATO": currentRow.NMPRATO,
                        "VRPECONSCARDTRAB": currentRow.VRPECONSCARDTRAB
                    },
                    "filterInfo": {
                        "FILTER_ROW": window.history.state.filterInfo.FILTER_ROW
                    }

                };

                TcoUtil.openWindowByHistory("#/tco#tco01205_dados_prato", params, "Dados do Prato");
            }
        } else {
            TcoUtil.setCurrentNavigationElement();
        }
    };

    let isValidCell = function(currentRow) {
        let valid = true;
        if (Util.isEmptyOrBlank(currentRow.NRCONFIGCARDTRAB)) {
            valid = false;
        } else if (currentRow.NRDIASEMANA === 0) {
            valid = false;
        }

        return valid;
    };

    /**
     * Função responsável por abrir o popup da responsável. 
     * É acionado pela action RESPONSIBLE.
     * 
     * @params {Args} args - objeto args
     **/

    this.openPopupResponsible = function(args) {
        let action = args.owner.name;
        let currentRow = window.history.state.currentRow.params;
        let appliedFilter = window.history.state.filterInfo;
        if (Tco01200_ActionsValidator.verifyPermission(action, currentRow)) {
            let widResponsible = templateManager.container.getWidget('responsible');
            ScreenService.openPopup(widResponsible).then(function() {
                this.onEnterResponsible(widResponsible);
            }.bind(this));
        }
    };

    /**
     * Função responsável por criar a propriedade __currentManager 
     * no operador responsável do serviço da proposta, caso exista.
     * Essa propriedade é verificada no styles do container para aplicar
     * um css de identificação no operador atual.
     * 
     * @params {Object} widgetResponsible - widgetResponsible
     **/

    this.onEnterResponsible = function(widgetResponsible) {
        var rowCardProp = window.history.state.currentRow.params;
        var dataResponsible = widgetResponsible.dataSource.data;
        dataResponsible.forEach(function(row, count) {
            if (row.CDOPERADOR === rowCardProp.CDRESPONSAVELPLAN) {
                widgetResponsible.dataSource.data[count].__currentManager = true;
            } else {
                widgetResponsible.dataSource.data[count].__currentManager = undefined;
            }
        });
        widgetResponsible.currentManager = rowCardProp.CDRESPONSAVELPLAN;
    };

    /**
     * Dicionário do campo IDSERVICOPROPCOM.
     * 
     * @params {String} code - value do IDSERVICOPROPCOM.
     **/

    this.dicionary = function(code) {
        var dicionary = {
            "NAOSOLICITADO": "Não Solicitado",
            "AGUARDANDOPLANEJAMENTO": "Aguardando Planejamento",
            "EMANDAMENTO": "Em Andamento",
            "RECUSADO": "Recusado",
            "AGUARDANDOGN": "Aguardando GN",
            "EMANDAMENTO_RETORNOGN": "Em andamento (Retorno GN)",
            "FINALIZADOPLANEJAMENTO": "Finalizado Planejamento",
            "CONCLUIDO": "Concluido",
            "APROVADO": "Aprovado",
            "EFETIVADO": "Efetivado"
        };
        return dicionary[code];
    };

    this.updateCustomElements = function(args) {
        let widget = args.owner.getWidget('cardapio');
        TcoUtil.updateCustomElements(widget);
    };

    /**
     * Função responsável por adicionar ou updatar o prato ao dia e tipo
     * de prato selecionado.
     * São montados os parâmetros para o upsert e em caso de sucesso, a
     * tela de dados do prato é "fechada", voltando então para o crosstab.
     * 
     **/

    this.addPratoCardTrab = function(rowOrigem, rowDestino) {
        let currentRow = angular.copy(window.history.state.currentRow.params);
        if (Tco01200_ActionsValidator.blockActionsCard(currentRow)) {
            var rowToSave = angular.copy(rowDestino);
            var valueComensal = TcoUtil.parseFloat(rowOrigem.VRPECONSCARDTRAB);

            rowToSave.CDPRATO = rowOrigem.CDPRATO;
            rowToSave.CDFILAUXPRAT = rowOrigem.CDFILAUXPRAT;
            rowToSave.VRPECONSCARDTRAB = valueComensal;
            rowToSave.NRCARDAPIOTRAB = window.history.state.filter.NRCARDAPIOTRAB;
            rowToSave.NRSERVICOPROPOSTA = window.history.state.currentRow.params.NRSERVICOPROPOSTA;
            rowToSave.NRPADRAOQUALIDADE = window.history.state.currentRow.params.NRPADRAOQUALIDADE;
            rowToSave.CDREGIAO = window.history.state.currentRow.params.CDREGIAO;
            rowToSave.CDSERVFILI = window.history.state.currentRow.params.CDSERVFILI;

            if (!Util.isEmptyOrBlank(rowDestino.CDPRATO)) {
                ScreenService.confirmMessage(ScreenService.i18n("Sobrepor prato?"), 'question', function(success) {
                    confirmAddOrUpdate(rowToSave);
                }.bind(this));
            } else {
                confirmAddOrUpdate(rowToSave);
            }
        }
    };

    let confirmAddOrUpdate = function(rowToSave) {
        Tco01200CardapioDaPropostaService.addOrUpdatePrato(rowToSave).then((response) => {
            ScreenService.successNotification("Prato alterado com sucesso.");
            templateManager.container.getWidget('cardapio').reload();
        }, function(error) {
            ScreenService.showMessage(error.error);
        });
    };

    /**
     * Função responsável por deletar o prato da célula selecionada.
     * Em caso de sucesso, a tela de dados do prato é "fechada",
     * voltando então para o crosstab.
     * 
     **/

    this.removePratoCardTrab = function(rowToDelete) {
        let currentRow = angular.copy(window.history.state.currentRow.params);
        if (Tco01200_ActionsValidator.blockActionsCard(currentRow)) {
            let widget = templateManager.container.getWidget("cardapio");
            let params = {
                "NRCOMPCARDTRAB": rowToDelete.NRCOMPCARDTRAB
            };
            Tco01200CardapioDaPropostaService.removePrato(params).then(function(response) {
                ScreenService.showMessage(response.notifications[0].message, response.notifications[0].type);
                widget.reload();
            }.bind(this), function(error) {
                ScreenService.showMessage(error.error);
            });
        }
    };

    /**
     * Função resposável por realizar a importação do cardapio de trabalho
     * No .then os erros são tratados.
     * 
     * @params {Args} args - objeto args
     */

    this.abrirPopupImportacao = function(args) {
        let servico = window.history.state.currentRow.params;
        let widgetImport = templateManager.container.getWidget('popupCalc');
        let action = 'IMPORT';
        if (Util.isEmptyOrBlank(servico.NRSERVICOPROPCOM)) {
            MessageService.showMessageByCode('TCO01200_SERV_N_SEL');
        } else {
            if (Util.isEmptyOrBlank(servico.NRCARDAPIOBASE)) {
                MessageService.showMessageByCode("TCO01200_SEM_CARD_BASE");
            } else {
                if (Tco01200_ActionsValidator.verifyPermission(action, servico)) {
                    let widgetImport = templateManager.container.getWidget('popupCalc');
                    widgetImport.currentRow.DTREFERENCIA = moment().format('DD/MM/YYYY');
                    widgetImport.getField('NMREGIAO').value(servico.CDREGIAO);
                    widgetImport.currentRow.VRPEACRESCIMO = null;
                    ScreenService.openPopup(widgetImport);
                }
            }
        }
    };

    this.confirmImport = function(args) {
        let widgetImport = templateManager.container.getWidget('popupCalc');
        if (widgetImport.isValid()) {
            MessageService.confirmMessageByCode('TCO01200_CONFIRM_IMPORT').then(this.importCardTrab);
        }
    };

    this.importCardTrab = function() {
        let widgetMain = templateManager.container.getWidget('cardapio');
        let widgetImport = templateManager.container.getWidget('popupCalc');
        let action = 'IMPORT';
        let currentRow = angular.copy(window.history.state.currentRow.params);
        currentRow.CDREGIAO = widgetImport.currentRow.CDREGIAO;
        currentRow.DTCUSTO = widgetImport.currentRow.DTREFERENCIA;
        currentRow.VRPEACRESCUSTO = widgetImport.currentRow.VRPEACRESCIMO;

        if (Tco01200_ActionsValidator.verifyPermission(action, currentRow)) {
            Tco01200CardapioDaPropostaService.importCardTrab(currentRow).then((response) => {
                if (response.dataset.ERROR) {
                    TcoUtil.buildMessagesToShow(response.dataset.ERROR).then(function(mensagens) {
                        window.history.state.currentRow.params.CDFILIALLISTA = response.dataset.LISTA.CDFILIAL;
                        window.history.state.currentRow.params.NRLISTA = response.dataset.LISTA.NRLISTA;
                        General.showErrorMessages(widgetMain, mensagens, ScreenService.i18n("Erro"));
                    });
                } else {
                    window.history.state.currentRow.params.NRCARDAPIOTRAB = response.dataset.SUCCESS.P_NRCARDAPIOTRAB;
                    window.history.state.currentRow.params.CDFILIALLISTA = null;
                    window.history.state.currentRow.params.NRLISTA = null;
                    window.history.state.currentRow.params.NRLISTA = null;
                    window.history.state.stripe.DTCUSTO = response.dataset.SUCCESS.DTCUSTO;
                    let filter = [{
                        name: "NRCARDAPIOTRAB",
                        operator: "=",
                        value: response.dataset.SUCCESS.P_NRCARDAPIOTRAB
                    }];
                    widgetMain.dataSourceFilter = filter;
                    widgetMain.reload().then(() => {
                        var servico = widgetMain.dataSource.data.filter(function(row) {
                            return row.NRSERVICOPROPCOM === currentRow.NRSERVICOPROPCOM;
                        })[0];

                        if (!Util.isEmptyOrBlank(servico)) {
                            widgetMain.setCurrentRow(servico);
                        }
                    });
                    ScreenService.successNotification('Cardápio importado com sucesso!');
                    ScreenService.closePopup();
                }
            });
        }
    };

    /**
     * Função resposável por identificar os finais de semana do cardápio para 
     * aplicar um css de background.
     * 
     * @params {Args} args - objeto args
     */

    this.getWeekend = function(row) {
        if (row.NRDIASEMANA == 1 || row.NRDIASEMANA == 7) {
            return true;
        } else if (row.__isYAxisProperty) {
            return row.label.indexOf('Dom') !== -1 || row.label.indexOf('Sab') !== -1;
        } else {
            return false;
        }
    };

    /**
     * Função resposável por identificar a célula de custo do cardápio para 
     * aplicar um css de bold.
     * 
     * @params {Object} row - celula do cardápio.
     */

    this.getCelCost = function(row) {
        return (row.__isYAxisProperty && row.label === 'Custo');
    };

    /**
     * Função responsável por atualizar as informações de custo do cardápio
     * no stripe.
     */

    this.setCostInStripe = function() {
        let stripeWidget = templateManager.container.getWidget('widgetStripe');
        let widgetCardapio = templateManager.container.getWidget('cardapio');
        let params = {
            decimal: ',',
            thousands: '.',
            precision: 2
        };
        if (stripeWidget.currentRow !== {}) {
            stripeWidget.currentRow.CUSTOTOTAL = !Util.isEmptyOrBlank(widgetCardapio.dataSource.data) ? TcoUtil.maskCurrency(widgetCardapio.dataSource.data[0].CUSTOTOTAL.toString(), params) : 0;
            stripeWidget.currentRow.CUSTOALIMENTAR = !Util.isEmptyOrBlank(widgetCardapio.dataSource.data) ? TcoUtil.maskCurrency(widgetCardapio.dataSource.data[0].CUSTOALIMENTAR.toString(), params) : 0;
            stripeWidget.currentRow.CUSTODESCARTAVEIS = !Util.isEmptyOrBlank(widgetCardapio.dataSource.data) ? TcoUtil.maskCurrency(widgetCardapio.dataSource.data[0].CUSTODESCARTAVEIS.toString(), params) : 0;
            stripeWidget.currentRow.CUSTOLIMPEZA = !Util.isEmptyOrBlank(widgetCardapio.dataSource.data) ? TcoUtil.maskCurrency(widgetCardapio.dataSource.data[0].CUSTOLIMPEZA.toString(), params) : 0;
        }
    };

    this.abrePrevisaoConsumo = function(args) {

        var params = {
            elaboracaoCardapioBaseParams: window.history.state.currentRow.params,
        };

        params.elaboracaoCardapioBaseParams.PORCGRAMATURA = [];
        params.elaboracaoCardapioBaseParams.TIPOPRATOSSELECIONADOS = [];
        params.elaboracaoCardapioBaseParams.origemCardProposta = true;

        TcoUtil.openWindowByHistory('#/tco#tco01600_previsao_consumo', params);
    };

    this.abreCopiaPratos = function(args) {
        var widgetMain = args.owner.widget;
        var popupCopiarPratos = widgetMain.container.getWidget('popupCopiarPratos');
        var row = TcoUtil.retornaRowSelecionada();
        var actionName = args.owner.name;
        var currentRowParams = window.history.state.currentRow.params;

        if (Tco01200_ActionsValidator.verifyPermission(actionName, currentRowParams)) {
            if (row && row.NMPRATO) {
                popupCopiarPratos.currentRow = angular.copy(row);
                popupCopiarPratos.fields.forEach(function(field) {
                    if (field.fieldGroup === 'DADOS_DIA') {
                        field.setValue(field.defaultValue);
                    }
                });
                ScreenService.openPopup(popupCopiarPratos);
            } else {
                MessageService.showMessageByCode('TCO01001_CELULA_NAO_SELECIONADA');
            }
        }
    };

    this.replicaPratos = function(args) {
        var widget = args.owner.widget;
        var widgetCardapio = widget.container.getWidget('cardapio');
        var currentRowParams = window.history.state.currentRow.params;
        if (Tco01200_ActionsValidator.blockActionsCard(currentRowParams)) {
            if (widget.isValid()) {
                Tco01200CardapioDaPropostaService.replicaPratos(widget.currentRow).then(function(callback) {
                    MessageService.successNotificationByCode('TCO01001_PRATOS_REPLICADOS', 3000);
                    widgetCardapio.reload();
                    ScreenService.closePopup(true);
                }, function(erro) {
                    ScreenService.showMessage(erro.data.error);
                });
            }
        }
    };

    this.openRelatorio = function(args) {
        var widget = args.owner.widget;
        var reportPopup = widget.container.getWidget('reportPopup');
        reportPopup.newRow();
        aplicaFiltroFieldsDia(reportPopup);
        aplicaFiltroFieldsTipoPrato(reportPopup);
        aplicaMascaraPrato(reportPopup.getField('NMPRATOINI'));
        aplicaMascaraPrato(reportPopup.getField('NMPRATOFIM'));
        ScreenService.openPopup(reportPopup);
    };

    var aplicaFiltroFieldsTipoPrato = function(reportPopup) {
        var widgetMain = reportPopup.container.getWidget('widgetMain');
        var filter = [
            {
                name: "NRCARDAPIOBASE",
                operator: "=",
                value: window.history.state.currentRow.params.NRCARDAPIOBASE
            }
        ];

        reportPopup.getField("NMTIPOPRAT").dataSourceFilter = filter.slice();
        reportPopup.getField("NMTIPOPRAT").reload();
    };

    var aplicaFiltroFieldsDia = function(reportPopup) {
        var widgetMain = reportPopup.container.getWidget('widgetMain');
        var filter = [
            {
                name: "NRCARDAPIOBASE",
                operator: "=",
                value: window.history.state.currentRow.params.NRCARDAPIOBASE
            }
        ];

        reportPopup.getField("DSSEMANA_INICIAL").dataSourceFilter = filter.slice();
        reportPopup.getField("DSSEMANA_FINAL").dataSourceFilter = filter.slice();
        reportPopup.getField("DSSEMANA_INICIAL").reload();
        reportPopup.getField("DSSEMANA_FINAL").reload();
    };

    var aplicaMascaraPrato = function(fieldPrato) {
        Tco01004_novo_prato_elaboracao_cardapio_base_service.retornaMascaraNivelPrato().then(function(success) {
            if (typeof success.dataset.mascaraNivelPrato !== "undefined" && success.dataset.mascaraNivelPrato.hasOwnProperty('CDNVPRATO') && success.dataset.mascaraNivelPrato.hasOwnProperty('CDPICTPRAT')) {
                fieldPrato.fields.forEach(function(field) {
                    if (field.name === "CDPRATO") {
                        field.mask = {
                            "type": "fix",
                            "params": {
                                "mask": success.dataset.mascaraNivelPrato.CDPICTPRAT.replace(/9/g, '*')
                            }
                        };
                    }
                });
            }
        });
    };

    this.montaLabelsSemanaDia = function(field) {
        MessageService.getMessageByCode('TCO01001_DIA').then(function(mensagemDia) {
            field.dataSource.data.forEach(function(row) {
                MessageService.getMessageByCode(row.DSSEMANA.slice(1, row.DSSEMANA.length)).then(function(labelDia) {
                    row.DSSEMANA = ScreenService.i18n(mensagemDia, {
                        NRSEMANA: row.NRSEMANA,
                        LABELDIA: labelDia
                    });
                    row.mounted = true;
                });
            });
        });
    };

    this.changeNeedList = function(args) {
        var widget = args.owner.field.widget;
        widget.hasList = false;
    };

    this.closePopup = function(args) {
        var widget = args.owner.widget;
        ScreenService.closePopup(true);
        widget.getParent().activate();
    };

    var constroiRowProcedureAnaliseCusto = function(widget) {
        var widgetMain = widget.container.getWidget('widgetMain');
        var widgetInformacoesGerais = widget.container.getWidget('widgetInformacoesGerais');
        var widgetTipoPratos = widget.container.getWidget('widgetTipoPratos');
        var widgetPorcionamentoGramatura = widget.container.getWidget('widgetPorcionamentoGramatura');

        var row = widgetInformacoesGerais.currentRow;
        var nrconfigcardbase = [];
        var gramaturacardbase = [];

        widgetTipoPratos.dataSource.getCheckedRows().forEach((row) => nrconfigcardbase.push(row.NRCONFIGCARDBASE));
        widgetPorcionamentoGramatura.dataSource.data.forEach((args) => {
            gramaturacardbase.push(args.CDGRTIPOPR + '|' + args.NRPORCIONAMENTO + '|' + (args.NRGRAMATURACARDBASE ? args.NRGRAMATURACARDBASE : ''));
        });

        row.NRCARDAPIOBASE = widgetMain.elaboracaoCardapioBaseParams.NRCARDAPIOBASE;
        row.NRCONFIGCARDBASE = nrconfigcardbase;
        row.PORCGRAMCARDBASE = gramaturacardbase;
        row.VRPEACRESCUSTO = row.VRPEACRESCUSTO ? row.VRPEACRESCUSTO.replace(',', '.') : 0;
        row.QTSEGSEX = row.QTSEGSEX ? row.QTSEGSEX.replace(',', '.') : 0;
        row.QTSABADO = row.QTSABADO ? row.QTSABADO.replace(',', '.') : 0;
        row.QTDOMFERI = row.QTDOMFERI ? row.QTDOMFERI.replace(',', '.') : 0;
        row.QTTURNOS = row.QTTURNOS ? row.QTTURNOS.replace(',', '.') : 0;

        return row;
    };

    this.makeReport = function(args) {
        var widget = args.owner.widget;
        var widgetMain = args.owner.widget.container.getWidget('widgetMain');

        if (widget.isValid()) {
            if (validaPeriodoDia(widget) && validaPeriodoPrat(widget) && validaPeriodoProd(widget)) {
                widget.currentRow.CDARVPRODINI = widget.getField("NMPRODUTOINI").value() == "" ? "0" : widget.currentRow.CDARVPRODINI;
                widget.currentRow.CDARVPRODFIN = widget.getField("NMPRODUTOFIN").value() == "" ? "zzzzzzzzzz" : widget.currentRow.CDARVPRODFIN;
                widget.currentRow.CDPRATOINI = widget.getField("NMPRATOINI").value() == "" ? "0" : widget.currentRow.CDPRATOINI;
                widget.currentRow.CDPRATOFIM = widget.getField("NMPRATOFIM").value() == "" ? "zzzzzzzzzz" : widget.currentRow.CDPRATOFIM;
                var row = constroiRowProcedureAnaliseCusto(widget);

                widget.currentRow.NRCARDAPIOBASE = widgetMain.elaboracaoCardapioBaseParams.NRCARDAPIOBASE;
                widget.currentRow.NRCARDAPIOBASE_PROCEDURE = row.NRCARDAPIOBASE;
                widget.currentRow.NRCONFIGCARDBASE_PROCEDURE = row.NRCONFIGCARDBASE;
                widget.currentRow.PORCGRAMCARDBASE_PROCEDURE = row.PORCGRAMCARDBASE;
                widget.currentRow.VRPEACRESCUSTO_PROCEDURE = row.VRPEACRESCUSTO;
                widget.currentRow.QTSEGSEX_PROCEDURE = row.QTSEGSEX;
                widget.currentRow.QTSABADO_PROCEDURE = row.QTSABADO;
                widget.currentRow.QTDOMFERI_PROCEDURE = row.QTDOMFERI;
                widget.currentRow.QTTURNOS_PROCEDURE = row.QTTURNOS;

                widget.reportRow = widget.currentRow;
                General.openReportPopup(widget, this.generateReport, [widget]);
            }
        }
    };

    var validaPeriodoProd = function(widget) {
        var valido = false;

        if (widget.getField("NMPRODUTOINI").value() != "" && widget.getField("NMPRODUTOFIN").value() == "") {
            MessageService.showMessageByCode("TCO01600_PRODUTO_FINAL_NAO_PREENCHIDO");
            valido = false;
        } else if (widget.getField("NMPRODUTOFIN").value() != "" && widget.getField("NMPRODUTOINI").value() == "") {
            MessageService.showMessageByCode("TCO01600_PRODUTO_INICIAL_NAO_PREENCHIDO");
            valido = false;
        } else {
            if (widget.currentRow.CDPRODUTOINI > widget.currentRow.CDPRODUTOFIN) {
                MessageService.showMessageByCode("TCO01600_PERIODO_PROD_INVALIDO");
                valido = false;
            } else {
                valido = true;
            }
        }

        return valido;
    };

    var validaPeriodoPrat = function(widget) {
        var valido = false;

        if (widget.getField("NMPRATOINI").value() != "" && widget.getField("NMPRATOFIM").value() == "") {
            MessageService.showMessageByCode("TCO01009_PRATO_FINAL_NAO_PREENCHIDO");
            valido = false;
        } else if (widget.getField("NMPRATOFIM").value() != "" && widget.getField("NMPRATOINI").value() == "") {
            MessageService.showMessageByCode("TCO01009_PRATO_INICIAL_NAO_PREENCHIDO");
            valido = false;
        } else {
            if (widget.currentRow.CDPRATOINI > widget.currentRow.CDPRATOFIM) {
                MessageService.showMessageByCode("TCO01009_PERIODO_PRAT_INVALIDO");
                valido = false;
            } else {
                valido = true;
            }
        }

        return valido;
    };

    var validaPeriodoDia = function(widget) {
        var valido = false;

        if (widget.getField("DSSEMANA_INICIAL").value() != "" && widget.getField("DSSEMANA_FINAL").value() == "") {
            MessageService.showMessageByCode("TCO01009_DIA_FINAL_NAO_PREENCHIDO");
            valido = false;
        } else if (widget.getField("DSSEMANA_FINAL").value() != "" && widget.getField("DSSEMANA_INICIAL").value() == "") {
            MessageService.showMessageByCode("TCO01009_DIA_INICIAL_NAO_PREENCHIDO");
            valido = false;
        } else {
            if (widget.currentRow.NRDIASEMANA_INICIAL > widget.currentRow.NRDIASEMANA_FINAL) {
                MessageService.showMessageByCode("TCO01600_PERIODO_DIA_INVALIDO");
                valido = false;
            } else {
                valido = true;
            }
        }

        return valido;
    };

    this.generateReport = function(widget) {
        var row = widget.currentRow;
        widget.reportRow.QR2TYPE = row.QR2TYPE;
        widget.reportRow.REPORT_NAME = row.REPORT_NAME;
        widget.reportRow.NRPADRAOQUALIDADE = window.history.state.elaboracaoCardapioBaseParams.NRPADRAOQUALIDADE;
        widget.reportRow.IDIOMA = getLocalVar('language');
        widget.reportRow.P_NRLISTA = widget.hasList ? widget.reportRow.P_NRLISTA : '';

        Tco01009_analise_custos_service.criaRelatorioAnaliseCusto(widget.reportRow).then(function(response) {
            if (response && response.messages) {
                var report = response.messages.pop();
                var reportOpened = window.open(report.message);
                if (!reportOpened) {
                    ScreenService.showMessage(ScreenService.i18n('Bloqueador de Pop-up está ativado!'));
                }
            }
            if (response && response.dataset && response.dataset.RESPONSE) {
                widget.reportRow.P_NRLISTA = response.dataset.RESPONSE.NRLISTA;
                widget.hasList = true;
            }
        }, function(response) {
            if (response.data && response.data.dataset && response.data.dataset.ERROS) {
                TcoUtil.buildMessagesToShow(response.data.dataset.ERROS).then(function(mensagens) {
                    General.showErrorMessages(widget, mensagens, ScreenService.i18n("Erro"));
                });
            }
        });
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01202_elaboracao_card_proposta', Tco01202_elaboracao_card_proposta);
});

// FILE: ./assets/js/controllers/Tco01203_configuracao_elab_card.js
var Tco01203_configuracao_elab_card = function(ScreenService, General, Tco01200CardapioDaPropostaService, templateManager, ZHPromise, TcoUtil, Tco01200_ActionsValidator) {

    this.carregaParametrosConfig = function(args) {
        if (!window.history.state) {
            window.location.href = window.location.href.replace("#/tco#tco01203_configuracao_elab_card", "#/tco#tco01200_cardapio_da_proposta");
            window.location.reload();
        } else {
            let widgetMain = args.owner.getWidget("configuracao");
            let filter = [
                {
                    "name": "NRCARDAPIOTRAB",
                    "operator": "=",
                    "value": window.history.state.filter.NRCARDAPIOTRAB
                }
            ];
            widgetMain.dataSourceFilter = filter;
            widgetMain.reload();
        }
    };

    /**
     * Volta para ultima tela.
     **/

    this.backCard = function() {
        window.history.back();
    };

    /**
     * Fecha o swipe a ativa a widget de configuração.
     **/

    this.backGrid = function() {
        let widget = templateManager.container.getWidget('configuracao');
        ScreenService.closeSwipe();
        widget.activate();
    };

    this.newConfig = function(args) {
        let row = window.history.state.currentRowCardapioBase.FILTER_ROW;
        if (Tco01200_ActionsValidator.blockActionsCard(row)) {
            args.owner.widget.newRow();
        }
    };

    /**
     * Função de salvar a configuração do prato.
     * 
     * @params {Object} args - Objeto args
     **/

    this.saveConfigTipoPrato = function(args) {
        let appliedFilter = window.history.state.currentRowCardapioBase;
        if (Tco01200_ActionsValidator.blockActionsCard(appliedFilter.FILTER_ROW)) {
            let widget = args.owner.widget;
            if (widget.isValid()) {
                widget.currentRow.NRCARDAPIOTRAB = window.history.state.filter.NRCARDAPIOTRAB;
                Tco01200CardapioDaPropostaService.saveConfigTipoPrato(widget.currentRow).then(function(response) {
                    widget.currentRow.NRCONFIGCARDTRAB = response.dataset.tco01200_config_elab_card[0].NRCONFIGCARDTRAB;
                    ScreenService.successNotification("Salvo com sucesso!");
                    widget.view();
                    widget.reload();
                }.bind(this), function(error) {
                    ScreenService.showMessage(error.data.error);
                }.bind(this));
            }
        }
    };

    /**
     * Função de deletar a configuração do prato.
     * 
     * @params {Object} args - Objeto args
     **/

    this.deleteConfigTipoPrato = function(args) {
        let appliedFilter = window.history.state.currentRowCardapioBase;
        if (Tco01200_ActionsValidator.blockActionsCard(appliedFilter.FILTER_ROW)) {
            let widget = args.owner.widget;
            ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function(success) {
                Tco01200CardapioDaPropostaService.deleteConfigTipoPrato(widget.currentRow).then(function() {
                    ScreenService.showMessage("Excluído com sucesso.", 'success');
                    ScreenService.closeSwipe().then(function() {
                        widget.reload();
                    });
                }.bind(this), function(error) {
                    if (error.data && error.data.error.indexOf('ORA-02292') !== -1) {
                        ScreenService.showMessage('Exclusão não permitida! O registro possui registros filhos.');
                    } else {
                        ScreenService.showMessage(error.data.error);
                    }
                }.bind(this));
            }.bind(this), function(error) {
                ScreenService.showMessage(error.data.error);
            }.bind(this));
        }
    };

    /**
     * Assegura que quando a configuração for do tipo alimentar
     * deve ter obrigatoriamente uma temperatura, e quando não for 
     * alimentar não deve ter temperatura.
     * 
     * @params {Object} args - Objeto args
     **/

    this.validationsTemp = function(args) {
        let classificacao = args.owner.field;
        let temperatura = args.owner.field.widget.getField("IDTEMPERATURA");
        if (classificacao.value() === "ALIMENTAR") {
            temperatura.readOnly = false;
        } else {
            temperatura.validations = {};
            temperatura.readOnly = true;
            temperatura.value("");
        }
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01203_configuracao_elab_card', Tco01203_configuracao_elab_card);
});

// FILE: ./assets/js/controllers/Tco01204_analise_custo.js
var Tco01204_analise_custo = function(ScreenService, General, Tco01200CardapioDaPropostaService, templateManager, ZHPromise, TcoUtil, MessageService) {

    this.carregaParametrosAnaliseCusto = function(args) {
        if (!window.history.state) {
            window.location.href = window.location.href.replace("#/tco#tco01204_analise_custo", "#/tco#tco01200_cardapio_da_proposta");
            window.location.reload();
        }
    };

    /**
     * Volta para ultima tela.
     **/

    this.backCard = function() {
        window.history.back();
    };

    /**
     * Função de find da tela análise de custo.
     * 
     * @params {Object} args - Objeto args.
     **/

    this.getAnaliseCusto = function(args) {
        var widget = args ? args.owner : templateManager.container.getWidget("analiseCusto");
        var filter = [
            {
                "name": "NRCARDAPIOTRAB",
                "operator": "=",
                "value": window.history.state.filter.NRCARDAPIOTRAB
            }
        ];
        widget.dataSourceFilter = filter;
        Tco01200CardapioDaPropostaService.getAnaliseCusto(widget.dataSourceFilter).then(function(response) {
            widget.dataSource.data = response.dataset.tco01200_analise_custo;
            if (widget.dataSource.data.length > 0)
                widget.setCurrentRow(widget.dataSource.data[0]);
            TcoUtil.concatenaSemanaEDias(args, widget);
        });
    };

    this.abrePratoReceita = function(args) {
        let analisecusto = templateManager.container.getWidget("analiseCusto");
        if (analisecusto.currentRow) {
            analisecusto.pratoreceitaaberto = true;
            this.getPratos(args);
            let pratoreceita = templateManager.container.getWidget("pratoreceita");
            ScreenService.openSwipe(pratoreceita);
        } else {
            MessageService.showMessageByCode('TCO01204_SELECIONE_DIA');
        }
    };

    this.closePratoReceita = function(args) {
        let analisecusto = templateManager.container.getWidget("analiseCusto");
        analisecusto.pratoreceitaaberto = false;
        ScreenService.closeSwipe();
    };

    this.getPratos = function(args) {
        let analisecusto = templateManager.container.getWidget("analiseCusto");
        if (analisecusto.currentRow && analisecusto.pratoreceitaaberto) {
            let prato = templateManager.container.getWidget("gridprato");
            let receita = templateManager.container.getWidget("gridreceita");
            let row = {
                NRCARDAPIOTRAB: analisecusto.currentRow.NRCARDAPIOTRAB,
                NRDIACARDTRAB: analisecusto.currentRow.NRDIACARDTRAB
            };
            Tco01200CardapioDaPropostaService.getPratos(row).then(function(response) {
                if (response.dataset && response.dataset.pratos) {
                    prato.dataSource.data = response.dataset.pratos;
                    if (prato.dataSource.data.length > 0) {
                        prato.setCurrentRow(prato.dataSource.data[0]);
                    } else {
                        receita.dataSource.data = [];
                    }
                }
            });
        }
    };

    this.getReceita = function(args) {
        let prato = templateManager.container.getWidget("gridprato");
        if (prato.currentRow) {
            let analisecusto = templateManager.container.getWidget("analiseCusto");
            let receita = templateManager.container.getWidget("gridreceita");
            let row = {
                QTPRATO: prato.currentRow.QTCOMENSAIS,
                NRCARDAPIOTRAB: analisecusto.currentRow.NRCARDAPIOTRAB,
                CDFILAUXPRAT: prato.currentRow.CDFILAUXPRAT,
                CDPRATO: prato.currentRow.CDPRATO
            };
            Tco01200CardapioDaPropostaService.getReceita(row).then(function(response) {
                if (response.dataset && response.dataset.receita) {
                    receita.dataSource.data = response.dataset.receita;
                }
            });
        }
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01204_analise_custo', Tco01204_analise_custo);
});

// FILE: ./assets/js/controllers/Tco01205_dados_prato.js
var Tco01205_dados_prato = function(ScreenService, General, Tco01200CardapioDaPropostaService, templateManager, ZHPromise, TcoUtil, Tco01200_ActionsValidator) {

    this.carregaParametrosDadosPrato = function(args) {
        if (!window.history.state) {
            window.location.href = window.location.href.replace("#/tco#tco01205_dados_prato", "#/tco#tco01200_cardapio_da_proposta");
            window.location.reload();
        } else {
            let widgetComensais = args.owner.getWidget("footerTabbed");
            let widgetProdutos = args.owner.getWidget("widgetProdutos");
            let pratoTabbed = args.owner.getWidget("pratoTabbed");
            widgetComensais.getField('VRPECONSCARDTRAB').value(window.history.state.dadosPrato.VRPECONSCARDTRAB);
            pratoTabbed.getField('PRATO').value(window.history.state.dadosPrato.NMPRATO);
            widgetProdutos.reload();
            this.writeLabel(window.history.state.label);
        }
    };

    /**
     * Volta para ultima tela.
     **/

    this.backCard = function() {
        window.history.back();
    };

    /**
     * Função resposável por montar o "description order" da tela 
     * de Dados do Prato.
     * É concatenado o dia da semana com o tipo do prato para
     * identificação da célula selecionada.
     * 
     * @params {Object} label - Objeto com dia do mês e tipo de prato.
     **/

    this.writeLabel = function(label) {
        var dashDadoPrato = templateManager.container.getWidget('dashboardDadosPrato');
        dashDadoPrato.label = label.LABELDIA + ' - ' + label.LABELTPPRAT;
        templateManager.updateTemplate();
    };

    /**
     * Função de find da tela de pratos.
     * Após buscar o dataset no backend, caso haja dados, é criado
     * uma propriedade bkp_data que será usada juntamente com o filtro.
     * No bkp_data sempre terá o dataset completo, com todos os pratos.
     * Além disso é chamada a função para setar o prato selecionado
     * na currentRow e outra para carregar o grid de produtos do prato.
     * 
     * @params {Args} args - Objeto args.
     **/

    this.getDadosPrato = function(args) {
        var widget = args.owner;
        var filter = window.history.state.filter;
        var fieldPrato = templateManager.container.getWidget("pratoTabbed").getField("CDPRATO");
        Tco01200CardapioDaPropostaService.getDadosPrato(filter).then(function(response) {
            if (!Util.isEmptyOrBlank(response.dataset.tco_01205_pratos)) {
                widget.dataSource.data = response.dataset.tco_01205_pratos;
                widget.dataSource.bkp_data = response.dataset.tco_01205_pratos;
                widget.dataSource.reload();
                this.setCurrentPrato(widget, window.history.state.dadosPrato);
                this.carregaProdutosPrato(null, widget);
            }
        }.bind(this));
    };

    /**
     * Função responsável por setar o prato selecionado na currentRow
     * da widget de Pratos.
     * Caso não haja nenhum prato, ele seleciona por default o primeiro prato
     * do dataset.
     * 
     * @params {Object} widgetPratos - Widget de pratos.
     * @params {Object} dadosPrato - Objeto com informações da célula selecionada.
     **/

    this.setCurrentPrato = function(widgetPratos, dadosPrato) {
        if (dadosPrato.CDPRATO) {
            var prato = widgetPratos.dataSource.data.filter(function(row) {
                return row.CDPRATO === dadosPrato.CDPRATO;
            })[0];
            widgetPratos.setCurrentRow(prato);
            setTimeout(function() {
                $(".zh-new-grid-" + widgetPratos.name).find(".grid-vertical-scroll-control").scrollTop(widgetPratos.currentRow.__index * 36);
            }, 150);
        } else {
            widgetPratos.setCurrentRow(widgetPratos.dataSource.getFirstRow());
        }
    };

    /**
     * Função de filtro da tela de pratos.
     * Chamada no evento onChange, ela tenta dar um match em uma regex
     * previamente criada para trazer o resultado da pesquisa, respeitando
     * o padrão de pesquisa da GR.
     * A cada pesquisa o dataSource.data é sobreescrito, caso seja pesquisado
     * um valor vazio, a função garante que seja retornado o dataset completo,
     * buscado na propriedade bkp_data.
     * 
     * @params {Args} args - Objeto args.
     **/

    this.filterPrato = function(args) {
        let field = args.owner.field;
        clearTimeOut(field);
        field.timeout = setTimeout(function() {
            let widgetPrato = templateManager.container.getWidget('widgetPratos');
            let filter = args.e.currentTarget.value;
            if (filter !== "") {
                if (filter[filter.length - 1] !== '%')
                    filter = filter + '%';
                let newData = widgetPrato.dataSource.bkp_data.filter(row => {
                    return (row.CDPRATO.toUpperCase().match(TcoUtil.buildRegExp(filter.toUpperCase())) ||
                        row.NMPRATO.toUpperCase().match(TcoUtil.buildRegExp(filter.toUpperCase())));
                });
                widgetPrato.dataSource.data = newData;

            } else {
                widgetPrato.dataSource.data = widgetPrato.dataSource.bkp_data;
                widgetPrato.setCurrentRow(widgetPrato.dataSource.data[0]);
            }
            templateManager.updateTemplate();
        }, 400);
    };

    /**
     * Limpa o timeout da pesquisa de prato sempre que o usuário digitar, para
     * que a pesquisa seja realizada apenas ao fim da digitação.
     * 
     * @params {Object} field - field da pesquisa de Pratos
     **/

    let clearTimeOut = function(field) {
        if (field.timeout !== false)
            clearTimeout(field.timeout);
    };

    /**
     * Função responsável por criar o filtro da tela de produtos
     * de acordo com o prato selecionado.
     * 
     * @params {Args} args - Objeto args.
     * @params {Object} widgetPrato - Parâmetro passado quando essa função é 
     * chamada diretamente do js.
     **/

    this.carregaProdutosPrato = function(args, widgetPrato = null) {
        if (args)
            widgetPrato = args.owner;
        var widgetProdutos = templateManager.container.getWidget("widgetProdutos");
        var row = {
            CDFILAUXPRAT: widgetPrato.currentRow.CDFILAUXPRAT,
            CDPRATO: widgetPrato.currentRow.CDPRATO,
            NRSERVICOPROPOSTA: window.history.state.filter.NRSERVICOPROPOSTA,
            NRPADRAOQUALIDADE: window.history.state.filter.NRPADRAOQUALIDADE,
            CDREGIAO: window.history.state.filter.CDREGIAO ? window.history.state.filter.CDREGIAO : null,
            NRCARDAPIOTRAB: window.history.state.filter.NRCARDAPIOTRAB,
            CDSERVFILI: window.history.state.filter.CDSERVFILI ? window.history.state.filter.CDSERVFILI : null
        };

        Tco01200CardapioDaPropostaService.retornaReceitas(row).then(success => {
            widgetProdutos.dataSource.data = success.dataset.receitaPrato;
            widgetProdutos.reload().then(() => {
                this.calcTotalyCost(widgetProdutos);
            });
        });
    };

    /**
     * Função responsável por calcular o custo total da receita e setar seu 
     * valor, já com a mascara aplicada, no field de custo.
     * 
     * @params {Object} widProdutos - widget de Produtos
     **/

    this.calcTotalyCost = function(widProdutos) {
        let totalyCost = 0;
        let fieldTotalyCost = templateManager.container.getWidget('footerTabbed').getField('CUSTO');
        let params = {
            decimal: ',',
            thousands: '.',
            precision: 3
        };
        widProdutos.dataSource.data.forEach(row => {
            totalyCost += row.CUSTOTOTAL;
        });
        fieldTotalyCost.value(TcoUtil.maskCurrency(totalyCost, params));
    };

    /**
     * Função responsável por adicionar ou updatar o prato ao dia e tipo
     * de prato selecionado.
     * São montados os parâmetros para o upsert e em caso de sucesso, a
     * tela de dados do prato é "fechada", voltando então para o crosstab.
     * 
     **/

    this.addOrUpdate = function() {
        let appliedFilter = window.history.state.filterInfo;
        if (Tco01200_ActionsValidator.blockActionsCard(appliedFilter.FILTER_ROW)) {
            var widgetPratos = templateManager.container.getWidget("widgetPratos");
            var widgetProdutos = templateManager.container.getWidget("widgetProdutos");
            var valueComensal = templateManager.container.getWidget("footerTabbed").getField('VRPECONSCARDTRAB').value() || 0;
            valueComensal = TcoUtil.parseFloat(valueComensal);
            if (widgetPratos.currentRow.CDPRATO && valueComensal > 0 && !Util.isEmptyOrBlank(widgetPratos.currentRow.TIPO_RECEITA)) {

                var params = {
                    "NRCOMPCARDTRAB": window.history.state.dadosPrato.NRCOMPCARDTRAB,
                    "NRCARDAPIOTRAB": window.history.state.NRCARDAPIOTRAB,
                    "NRSERVICOPROPOSTA": window.history.state.filter.NRSERVICOPROPOSTA,
                    "NRPADRAOQUALIDADE": window.history.state.filter.NRPADRAOQUALIDADE,
                    "CDREGIAO": window.history.state.filter.CDREGIAO,
                    "NRDIACARDTRAB": window.history.state.dadosPrato.NRDIACARDTRAB,
                    "NRCONFIGCARDTRAB": window.history.state.dadosPrato.NRCONFIGCARDTRAB,
                    "CDSERVFILI": window.history.state.filter.CDSERVFILI,
                    "CDPRATO": widgetPratos.currentRow.CDPRATO,
                    "CDFILAUXPRAT": widgetPratos.currentRow.CDFILAUXPRAT,
                    "VRPECONSCARDTRAB": valueComensal,
                    "PRODUTOS": widgetProdutos.dataSource.data
                };
                Tco01200CardapioDaPropostaService.addOrUpdatePrato(params).then(function(response) {
                    ScreenService.successNotification("Prato adicionado com sucesso.");
                    this.backCard();
                }.bind(this), function(error) {
                    ScreenService.showMessage(error.error);
                });

            } else {
                ScreenService.showMessage("Selecione um prato com receita e informe um valor maior que zero para o Total de Comensais.");
            }
        }
    };

    /**
     * Função responsável por deletar o prato da célula selecionada.
     * Em caso de sucesso, a tela de dados do prato é "fechada",
     * voltando então para o crosstab.
     * 
     **/

    this.removePrato = function() {
        let appliedFilter = window.history.state.filterInfo;
        if (Tco01200_ActionsValidator.blockActionsCard(appliedFilter.FILTER_ROW)) {
            var widgetPratos = templateManager.container.getWidget("widgetPratos");
            var fieldComensal = templateManager.container.getWidget("footerTabbed").getField('VRPECONSCARDTRAB');
            var params = {
                "NRCOMPCARDTRAB": window.history.state.dadosPrato.NRCOMPCARDTRAB
            };
            Tco01200CardapioDaPropostaService.removePrato(params).then(function(response) {
                ScreenService.showMessage(response.notifications[0].message, response.notifications[0].type);
                this.backCard();
            }.bind(this), function(error) {
                ScreenService.showMessage(error.error);
            });
        }
    };

    /**
     * Função responsável por abrir a tela de revisão da receita.
     * 
     * @params {Args} args - Objeto args.
     **/

    this.openReview = function(args) {
        let appliedFilter = window.history.state.filterInfo;
        if (Tco01200_ActionsValidator.blockActionsCard(appliedFilter.FILTER_ROW)) {
            var widgetPratos = templateManager.container.getWidget("widgetPratos");
            var widProdutos = templateManager.container.getWidget("widgetProdutos");
            var params = {
                "dataSource": widProdutos.dataSource.data,
                "NRCARDAPIOTRAB": window.history.state.NRCARDAPIOTRAB,
                "veioDe": "Tco01200",
                "dadosPrato": {
                    "CDPRATO": widgetPratos.currentRow.CDPRATO,
                    "NMPRATO": widgetPratos.currentRow.NMPRATO,
                    "CDFILAUXPRAT": widgetPratos.currentRow.CDFILAUXPRAT,
                    "NRSERVICOPROPOSTA": window.history.state.filter.NRSERVICOPROPOSTA,
                    "NRPADRAOQUALIDADE": window.history.state.filter.NRPADRAOQUALIDADE,
                    "CDREGIAO": window.history.state.filter.CDREGIAO,
                    "NMREGIAO": window.history.state.filter.NMREGIAO,
                    "NRCARDAPIOTRAB": window.history.state.filter.NRCARDAPIOTRAB,
                    "CDSERVFILI": window.history.state.filter.CDSERVFILI,
                    "NMSERVICO": window.history.state.filter.NMSERVICOPROPOSTA
                }
            };
            TcoUtil.openWindowByHistory("#/tco#tco01010_revisar_receita", params, "Revisar Receita");
        }
    };

    this.clearFilter = function(args) {
        // to do....
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01205_dados_prato', Tco01205_dados_prato);
});

// FILE: ./assets/js/controllers/Tco01206_revisao_receita.js
var Tco01206_revisao_receita = function(ScreenService, General, Tco01200CardapioDaPropostaService, templateManager, ZHPromise, TcoUtil, MessageService) {

    this.carregaParametrosReview = function(args) {
        if (!window.history.state) {
            window.location.href = window.location.href.replace("#/tco#tco01206_revisar_receita", "#/tco#tco01200_cardapio_da_proposta");
            window.location.reload();
        } else {
            var widgetReview = templateManager.container.getWidget("widRevisao");
            var filter = [
                {
                    "name": "CDPRATO",
                    "operator": "=",
                    "value": window.history.state.dadosPrato.CDPRATO
                },
                {
                    "name": "NRCARDAPIOTRAB",
                    "operator": "=",
                    "value": window.history.state.NRCARDAPIOTRAB
                }
            ];
            widgetReview.dataSourceFilter = filter;
            widgetReview.reload();
        }
    };

    /**
     * Volta para ultima tela.
     * Verifica se tem produtos que ainda não foram
     * revisados.
     **/

    this.backDadosPrato = function() {
        var widget = templateManager.container.getWidget("widRevisao");
        var data = widget.dataSource.data.filter(function(data) {
            return data.changed;
        });
        if (!Util.isEmptyOrBlank(data)) {
            MessageService.confirmMessageByCode('TCO01200_CONFIRM_REVIEW').then(function(success) {
                window.history.back();
            });
        } else {
            window.history.back();
        }
    };

    /**
     * Fecha o swipe e volta para o grid.
     **/

    this.backGrid = function() {
        var widget = templateManager.container.getWidget('widRevisao');
        ScreenService.closeSwipe();
        widget.activate();
    };

    /**
     * Monta o data do dataSource local, antes de
     * revisar a receita.
     * 
     * @params {Object} currentRow - currentRow da widget de revisão.
     **/

    var mountData = function(currentRow) {
        var data = {
            "NRRECEPRATCARDTRAB": (currentRow.NRRECEPRATCARDTRAB) ? currentRow.NRRECEPRATCARDTRAB : "",
            "NRCARDAPIOTRAB": window.history.state.NRCARDAPIOTRAB,
            "CDARVPROD": currentRow.CDARVPROD,
            "CDPRODUTO": currentRow.CDPRODUTO,
            "CDPRATO": window.history.state.dadosPrato.CDPRATO,
            "CDFILAUXPRAT": window.history.state.dadosPrato.CDFILAUXPRAT,
            "IDPRODBASECARDTRAB": currentRow.IDPRODBASECARDTRAB,
            "IDQTDEFIXCARDTRAB": currentRow.IDQTDEFIXCARDTRAB,
            "NMPRODUTO": currentRow.NMPRODUTO,
            "QTBRUTACARDTRABUNR": currentRow.QTBRUTACARDTRABUNR = TcoUtil.parseFloat(currentRow.QTBRUTACARDTRABUNR),
            "QTPRODRECCARDTRAB": currentRow.QTPRODRECCARDTRAB = TcoUtil.parseFloat(currentRow.QTPRODRECCARDTRAB),
            "SGUNIDADE": currentRow.SGUNIDADE,
            "VRAPCOCCAO": currentRow.VRAPCOCCAO = TcoUtil.parseFloat(currentRow.VRAPCOCCAO),
            "VRAPLIMPEZA": currentRow.VRAPLIMPEZA = TcoUtil.parseFloat(currentRow.VRAPLIMPEZA),
            "VRPEFATCOCCARDTRAB": currentRow.VRPEFATCOCCARDTRAB,
            "VRPEFATCORCARDTRAB": currentRow.VRPEFATCORCARDTRAB,
            "changed": true,
            "__is_new": currentRow.__is_new
        };
        return data;
    };

    /**
     * Adicona os produtos no dataSource de maneira local.
     * Valida se o produto já está adicionado, caso esteja, não deixa
     * prosseguir.
     * 
     * @params {Args} args - Object args.
     **/

    this.saveProductLocal = function(args) {
        var widget = args.owner.widget;
        if (widget.isValid()) {
            var dataSource = widget.dataSource.data;
            if (widget.currentRow.__is_new) {
                if (productValid(widget.currentRow, widget.dataSource)) {
                    var data = mountData(widget.currentRow);
                    dataSource[dataSource.length - 1] = data;
                    widget.view();
                    ScreenService.notificationMessage(ScreenService.i18n("Adicionado com sucesso!"), 'sucess');
                } else {
                    ScreenService.showMessage("Impossível adicionar. Produto já cadastrado.");
                }
            } else {
                widget.dataSource.data.forEach(function(data, count) {
                    if (widget.currentRow.CDARVPROD === data.CDARVPROD) {
                        widget.dataSource.data[count] = mountData(widget.currentRow);
                        widget.view();
                        ScreenService.notificationMessage(ScreenService.i18n("Adicionado com sucesso!"), 'sucess');
                    }
                });
            }
        }
    };

    /**
     * Verifica se já existe o produto que está sendo adicionado.
     * 
     * @params {Object} currentRow - row que está para ser inserida.
     * @params {Object} dataSource - dataSource da tela de revisão.
     **/

    var productValid = function(currentRow, dataSource) {
        var result = dataSource.data.filter(function(data) {
            return data.CDARVPROD === currentRow.CDARVPROD;
        });
        return (result.length == 1);
    };

    /**
     * Faz o cálculo de cocção.
     * VRPEFATCOCCARDTRAB - Ganho Cocção (%)
     * VRAPCOCCAO - Ap. Cocção (%)
     * 
     * @params {Args} args - Object args.
     **/

    this.calcCoccao = function(args) {
        var field = args.owner.field;
        var widget = field.widget;
        if (field.name === "VRPEFATCOCCARDTRAB") {
            widget.getField('VRAPCOCCAO').value(100 + parseFloat(field.value()));
        } else {
            widget.getField('VRPEFATCOCCARDTRAB').value(parseFloat(field.value()) - 100);
        }
    };

    /**
     * Faz o cálculo de limpeza.
     * VRPEFATCORCARDTRAB - Perda Limpeza (%)
     * VRAPLIMPEZA - Ap. Limpeza (%)
     * 
     * @params {Args} args - Object args.
     **/

    this.calcLimpeza = function(args) {
        var field = args.owner.field;
        var widget = field.widget;
        if (field.name === "VRPEFATCORCARDTRAB") {
            widget.getField('VRAPLIMPEZA').value(100 - parseFloat(field.value()));
        } else {
            widget.getField('VRPEFATCORCARDTRAB').value(parseFloat(100 - field.value()));
        }
    };

    /**
     * Seta o valor de quantidade bruta.
     * 
     * @params {Args} args - Object args.
     **/

    this.setQtBruta = function(args) {
        var fieldQt = args.owner.field;
        var qtBruta = fieldQt.widget.getField('QTBRUTACARDTRABUNR');
        qtBruta.value(fieldQt.value());
    };

    /**
     * Verifica quais produtos foram alterados 
     * e os manda para a revisão da receita.
     * 
     * @params {Args} args - Object args.
     **/

    this.confirmReview = function(args) {
        var widget = args.owner.widget;
        var data = widget.dataSource.data.filter(function(data) {
            return data.changed;
        });
        if (!Util.isEmptyOrBlank(data)) {
            Tco01200CardapioDaPropostaService.confirmReview(data).then(function(response) {
                ScreenService.showMessage("Revisado com Sucesso!", "sucess");
                removePropretyChanged(widget);
            }, function(error) {
                if (error.error === "invalidRows") {
                    mountMessageError(error.dataset);
                } else {
                    ScreenService.showMessage(error.error);
                }
            });
        } else {
            ScreenService.showMessage("Nenhum produto foi revisado.");
        }
    };

    /**
     * Retira a propriedade de changed dos produtos
     * revisados para que eles não caiam nas validações.
     * 
     * @params {Object} widget - Widget de revisão.
     **/

    var removePropretyChanged = function(widget) {
        widget.dataSource.data.forEach(data => {
            if (data.changed) data.changed = undefined;
        });
    };

    var mountMessageError = function(error) {
        // to do...
    };

    /**
     * Função de reload da tela. Foi sobreescrita para verificar
     * se existem produtos adicionados no dataset que ainda não foram
     * revisados.
     * 
     * @params {Args} args - Object args.
     **/

    this.reload = function(args) {
        var widget = templateManager.container.getWidget("widRevisao");
        var data = widget.dataSource.data.filter(function(data) {
            return data.changed;
        });
        if (!Util.isEmptyOrBlank(data)) {
            MessageService.confirmMessageByCode('TCO01200_CONFIRM_REVIEW').then(function(success) {
                widget.reload();
            });
        } else {
            widget.reload();
        }
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01206_revisao_receita', Tco01206_revisao_receita);
});

// FILE: ./assets/js/controllers/Tco01207_controller_geral.js
var Tco01207_controller_geral = function(ScreenService, General, Tco01200CardapioDaPropostaService, templateManager, ZHPromise, TcoUtil, TcoEmailService, Tco01200_ActionsValidator, MessageService) {

    /**
     * Função responsável por definir o defaultValue dos campos
     * da widget effect.
     * 
     * @params {Args} args - objeto args
     **/

    this.setDefaultValueEffect = function(args) {
        let rowCardProp = Util.isEmptyOrBlank(templateManager.container.getWidget('widgetCardProp')) ?
            window.history.state.currentRow.params : templateManager.container.getWidget('widgetCardProp').currentRow;
        let widgetEffect = args.owner;
        let value = moment().add(1, "month").format("MM/YYYY");
        let cdfilial = rowCardProp.CDFILIAL;
        let filial = (Util.isEmptyOrBlank(cdfilial)) ? "" : cdfilial + ' | ' + rowCardProp.NMFILIAL;
        widgetEffect.currentRow.DTINICARD = value;
        widgetEffect.currentRow.NMFILIAL = filial;
        widgetEffect.currentRow.CDFILIAL = cdfilial;
        widgetEffect.currentRow.NUMMESES = 01;
        widgetEffect.currentRow.NRSERVICOPROPCOM = rowCardProp.NRSERVICOPROPCOM;
    };

    /**
     * Função responsável por definir o dataSourceFilter do campo de
     * serviço da widget effect.
     * 
     * @params {Args} args - objeto args
     **/

    var setDataSourceFilterServEffect = function(field) {
        let filter = [
            {
                "operator": "ORDER_BY",
                "name": [
                    {
                        "name": "CDSERVFILI",
                        "direction": "ASC"
                    }
                ]
            },
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": field.widget.currentRow.CDFILIAL
            }
        ];
        field.dataSourceFilter = filter;
        field.reload();
    };

    /**
     * Função responsável por atualizar o operador responsável do serviço
     * da proposta comercial. Como esse é um controller geral, essa função
     * será chamada por mais de uma tela (widgetCardProp || cardapio).
     * Na definição dos parâmetros isso é analisado, para que os parâmetros
     * possam ser passador corretamente.
     * 
     * @params {Args} args - objeto args
     **/

    this.confirmResponsible = function(args) {
        let widgetResponsible = args.owner.widget;
        let widCardProp = templateManager.container.getWidget("widgetCardProp");
        let rowCardProp = widCardProp ? widCardProp.currentRow : window.history.state.currentRow.params;
        let widToActive = widCardProp || templateManager.container.getWidget("cardapio");
        let params = {
            "CDOPERADOR": widgetResponsible.currentRow.CDOPERADOR,
            "NRDETALHESERVPROPCOM": rowCardProp.NRDETALHESERVPROPCOM,
            "NRDETSERVPROPCOMPLAN": rowCardProp.NRDETSERVPROPCOMPLAN,
            "NRCARDAPIOTRAB": rowCardProp.NRCARDAPIOTRAB,
            "NRSERVICOPROPCOM": rowCardProp.NRSERVICOPROPCOM
        };
        Tco01200CardapioDaPropostaService.confirmResponsible(params).then(function(response) {
            ScreenService.successNotification("Responsável atualizado com Sucesso!");
            this.afterConfirmResponsible(widgetResponsible, widCardProp, widToActive, response);
            if (widCardProp) {
                this.refreshInScope(widCardProp);
            }
        }.bind(this), function(error) {
            ScreenService.showMessage(error.error);
        });
    };

    /**
     * Função chamada após atualizar o responsável com sucesso.
     * O CDRESPONSAVELPLAN é atualizado para manter a consistência dos dados
     * mesmo sem atualizar a tela, evitando assim uma nova requisição.
     * 
     * @params {Object} widgetResponsible - widget widgetResponsible.
     * @params {Object} widCardProp - widget widCardProp.
     * @params {Object} widToActive - widget que será ativada. 
     * @params {Object} response - response da requisição do confirmResponsible.
     **/

    this.afterConfirmResponsible = function(widgetResponsible, widCardProp, widToActive, response) {
        widgetResponsible.reload();
        ScreenService.closePopup(true);
        widToActive.activate();
        if (widToActive.name === "cardapio") {
            window.history.state.currentRow.params.CDRESPONSAVELPLAN = response.dataset.confirmResponsible[0];
        } else {
            widCardProp.currentRow.CDRESPONSAVELPLAN = response.dataset.confirmResponsible[0];
        }
    };

    /**
     * Função responsável por abrir o popup de notificações e configurá-lo de
     * acordo com a ação de Finalizar.
     * 
     * @params {Args} args - objeto args
     **/

    this.openPopupFinish = function(args) {
        let action = args.owner.name;
        let widget = args.owner.widget;
        let appliedFilter = getFilterInfo(widget);
        let currentRow = setCurrentRowByWid(args);

        if (Tco01200_ActionsValidator.verifyPermission(action, currentRow)) {
            let popupFinish = args.owner.widget.container.getWidget('widPopup');
            popupFinish.label = ScreenService.i18n("Finalizar");
            popupFinish.getAction('CONFIRM').events[0].code = "Tco01207_controller_geral.finish(args);";
            popupFinish.currentRow = {};
            ScreenService.openPopup(popupFinish);
        }
    };

    let setCurrentRowByWid = function(args) {
        let wid = args.owner.widget;
        let currentRow;
        if (wid.name === "widgetCardProp") {
            currentRow = args.owner.widget.currentRow;
        } else {
            currentRow = window.history.state.currentRow.params;
        }
        return currentRow;
    };

    /**
     * Função que chama a rota responsável por acionar as procedures
     * VALIDA_CARDTRAB e SOL_ALT_SERVPROPCOM.
     * 
     * Na função de erro os tipos possíveis de mensagens retornadas são tratados.
     * 
     * @params {Args} args - objeto args
     **/

    this.finish = function(args) {
        let popupFinish = args.owner.widget;
        let widget = popupFinish.container.getWidget('cardapio') ||
            popupFinish.container.getWidget('widgetCardProp');
        if (widget.isValid()) {
            let row = widget.parentRow || widget.currentRow;
            let filter = this.buildFilter(row);
            filter.P_DSOBSERVACAO = popupFinish.currentRow.DSOBSERVACAO;
            filter.P_IDMOTIVO = "FINALIZADO";
            Tco01200CardapioDaPropostaService.finish(filter).then(response => {
                this.afterSuccess(widget, filter);
            }, error => {
                this.afterError(widget, error);
            });
        }
    };

    /**
     * Função responsável por abrir o popup de notificações e configurá-lo de
     * acordo com a ação de Recusar.
     * 
     * @params {Args} args - objeto args
     **/

    this.openPopupRecuse = function(args) {
        let action = args.owner.name;
        let widget = args.owner.widget;
        let appliedFilter = getFilterInfo(widget);
        let currentRow = setCurrentRowByWid(args);

        if (Tco01200_ActionsValidator.verifyPermission(action, currentRow)) {
            let popupRecuse = args.owner.widget.container.getWidget('widPopup');
            popupRecuse.label = ScreenService.i18n("Recusar");
            popupRecuse.getAction('CONFIRM').events[0].code = "Tco01207_controller_geral.recuse(args);";
            popupRecuse.currentRow = {};
            ScreenService.openPopup(popupRecuse);
        }

    };

    let getFilterInfo = function(widget) {
        if (widget.name === "widgetCardProp") {
            return widget.appliedFilter;
        } else {
            return window.history.state.filterInfo;
        }
    };

    /**
     * Função que chama a rota responsável por acionar a procedure 
     * de solicitar informações.
     * 
     * Na função de erro os tipos possíveis de mensagens retornadas são tratados.
     * 
     * @params {Args} args - objeto args
     **/

    this.recuse = function(args) {
        let popupRecuse = args.owner.widget;
        let widget = popupRecuse.container.getWidget('cardapio') ||
            popupRecuse.container.getWidget('widgetCardProp');
        if (widget.isValid()) {
            let row = widget.parentRow || widget.currentRow;
            let filter = this.buildFilter(row);
            filter.P_DSOBSERVACAO = popupRecuse.currentRow.DSOBSERVACAO;
            filter.P_IDMOTIVO = "RECUSADO";
            Tco01200CardapioDaPropostaService.requireInfoOrRecuse(filter).then(response => {
                this.afterSuccess(widget, filter);
            }, error => {
                this.afterError(widget, error);
            });
        }
    };

    /**
     * Função responsável por abrir o popup de notificações e configurá-lo de
     * acordo com a ação de Solicitar Informação.
     * 
     * @params {Args} args - objeto args
     **/

    this.openPopupRequireInfo = function(args) {
        let action = args.owner.name;
        let widget = args.owner.widget;
        let appliedFilter = getFilterInfo(widget);
        let currentRow = setCurrentRowByWid(args);

        if (currentRow.NRCARDAPIOTRAB) {
            if (Tco01200_ActionsValidator.verifyPermission(action, currentRow)) {
                let popupRequireInfo = args.owner.widget.container.getWidget('widPopup');
                popupRequireInfo.label = ScreenService.i18n("Solicitar Informações");
                popupRequireInfo.getAction('CONFIRM').events[0].code = "Tco01207_controller_geral.requireInfo(args);";
                popupRequireInfo.currentRow = {};
                ScreenService.openPopup(popupRequireInfo);
            }
        } else {
            ScreenService.showMessage('Selecione um serviço que possua cardápio.');
        }
    };

    /**
     * Função que chama a rota responsável por acionar a procedure 
     * de solicitar informações.
     * 
     * Na função de erro os tipos possíveis de mensagens retornadas são tratados.
     * 
     * @params {Args} args - objeto args
     **/

    this.requireInfo = function(args) {
        let popupRequireInfo = args.owner.widget;
        let widget = popupRequireInfo.container.getWidget('cardapio') ||
            popupRequireInfo.container.getWidget('widgetCardProp');
        if (widget.isValid()) {
            let row = widget.parentRow || widget.currentRow;
            let filter = this.buildFilter(row);
            filter.P_DSOBSERVACAO = popupRequireInfo.currentRow.DSOBSERVACAO;
            filter.P_IDMOTIVO = "DUVIDA";
            Tco01200CardapioDaPropostaService.requireInfoOrRecuse(filter).then(response => {
                this.afterSuccess(widget, filter);
            }, error => {
                this.afterError(widget, error);
            });
        }
    };

    /**
     * Função responsável por buildar o filtro tanto da função de finish quanto
     * da requireInfo e na de recuse.
     * Os parâmetros que não estão preenchidos são preenchidos no backend.
     * 
     * @params {Object} row - row com os parametros do filtro.
     **/

    this.buildFilter = function(row) {
        let filter = {
            "P_NRSERVICOPROPCOM": row.NRSERVICOPROPCOM,
            "P_NRSERVICOPROPOSTA": row.NRSERVICOPROPOSTA,
            "P_NRDETSERVPROPCOMPLAN": row.NRDETSERVPROPCOMPLAN,
            "P_NRDETALHESERVPROPCOM": row.NRDETALHESERVPROPCOM,
            "P_NRCARDAPIOTRAB": row.NRCARDAPIOTRAB,
            "P_CDOPERADOR": row.CDRESPONSAVELPLAN,
            "P_CDFILIALLOGADA": getLocalVar("CDFILIAL"),
            "P_IDORIGEM": "PLANEJAMENTO",
            "P_IDDESTINO": "GN",
            "P_IDMOTIVO": "",
            "P_DSOBSERVACAO": "",
            "P_TXHISTORICO": "",
            "P_NRORG": row.NRORG
        };
        return filter;
    };

    /**
     * Responsável por tratar os casos de sucesso das funções que
     * geram notificações (finish, require, recuse).
     * 
     * @params {Object} widget - widget cardapio ou widgetCardProp.
     * @params {Object} filter - parâmetros para montar o filtro na buildRowToNotification.
     **/

    this.afterSuccess = function(widget, filter) {
        let row = buildRowToNotification(filter);
        ScreenService.successNotification("Atualizado com Sucesso!");
        ScreenService.closePopup(true);
        widget.activate();
        TcoEmailService.enviaEmail(row);
        widget.reload();
    };

    /**
     * Monta os parâmetros necessários para executar a função de enviar email
     * para a equipe de gerentes de negocio cadastrada.
     * 
     * @params {Object} currentRow - row com os parametros do filtro.
     **/

    var buildRowToNotification = function(currentRow) {
        let nrdetalhe = Util.isEmptyOrBlank(currentRow.P_NRDETSERVPROPCOMPLAN) ? currentRow.P_NRDETALHESERVPROPCOM : currentRow.P_NRDETSERVPROPCOMPLAN;
        return {
            "NRDETALHESERVPROPCOM": nrdetalhe,
            "MOTIVO": currentRow.P_IDMOTIVO
        };
    };

    /**
     * Responsável por tratar os casos de erro das funções que
     * geram notificações (finish, require, recuse).
     * 
     * @params {Object} widget - widget cardapio ou widgetCardProp.
     * @params {Object} error - erro retornado do backend.
     **/

    this.afterError = function(widget, error) {
        if (!Util.isEmptyOrBlank(error.dataset.msgError)) {
            TcoUtil.buildMessagesToShow(error.dataset.msgError[0].MSGCODE).then(function(msgError) {
                ScreenService.showMessage(msgError, 'error');
            });
        } else if (!Util.isEmptyOrBlank(error.dataset.listError)) {
            TcoUtil.buildMessagesToShow(error.dataset.listError).then(function(listError) {
                General.showErrorMessages(widget, listError, ScreenService.i18n("Erro"));
            });
        }
    };

    /**
     * Função responsável por abrir o popup da action Efetivar.
     * Antes deve ser validado se existe algum serviço selecionado.
     * 
     **/

    this.openPopupEffect = function(args) {
        let action = args.owner.name;
        let widget = args.owner.widget;
        let appliedFilter = getFilterInfo(widget);
        let currentRow = setCurrentRowByWid(args);

        if (currentRow.NRCARDAPIOTRAB) {
            if (Tco01200_ActionsValidator.verifyPermission(action, currentRow)) {
                let widEffect = templateManager.container.getWidget('effect');
                widEffect.newRow();
                widEffect.currentRow = currentRow;
                setDataSourceFilterServEffect(widEffect.getField('NMSERVICO'));
                ScreenService.openPopup(widEffect);
            }
        } else {
            ScreenService.showMessage('Selecione um serviço que possua cardápio.');
        }
    };

    /**
     * Função responsável por acionar a procedure de efetivar.
     * No .then são tratados os possíveis erros.
     * 
     * @params {Object} - args - objeto args
     */

    this.confirmEffect = function(args) {
        let widget = args.owner.widget;
        let widgetCardProp = args.owner.widget.container.getWidget('widgetCardProp');
        let widgetPlanilaCardapio = args.owner.widget.container.getWidget('cardapio');
        if (widget.isValid()) {
            widget.currentRow.P_DTINICARD = '01/' + widget.currentRow.DTINICARD;
            Tco01200CardapioDaPropostaService.effect(widget.currentRow).then(response => {
                if (response.dataset.SUCCESS) {
                    ScreenService.closePopup(true);
                    if (typeof widgetCardProp !== 'undefined' && widgetCardProp != null) {
                        widgetCardProp.reload();
                        widgetCardProp.activate();
                        ScreenService.successNotification('Cardápio efetivado com sucesso!');
                    } else {
                        var stripeData = window.history.state.stripe;
                        stripeData.IDSERVICOPROPCOM = "EFETIVADO";
                        this.buildStripe(stripeData);
                        widgetPlanilaCardapio.reload();
                        widgetPlanilaCardapio.activate();
                        ScreenService.successNotification('Cardápio efetivado com sucesso!');
                    }
                } else if (response.dataset.msgError) {
                    MessageService.showMessageByCode(response.dataset.msgError[0].MSGCODE);
                } else if (response.dataset.ERROR) {
                    TcoUtil.buildMessagesToShow(response.dataset.ERROR).then(function(mensagens) {
                        General.showErrorMessages(widget, mensagens, ScreenService.i18n("Erro"));
                    });
                }
            }, error => {
                if (error.data.dataset.msgError) {
                    MessageService.showMessageByCode(error.data.dataset.msgError[0].MSGCODE);
                } else if (error.error) {
                    ScreenService.showMessage(ScreenService.i18n(error.error, {}));
                }
            });
        }
    };

    /**
     * Dicionário do campo IDSERVICOPROPCOM.
     * 
     * @params {String} code - value do IDSERVICOPROPCOM.
     **/

    this.dicionary = function(code) {
        var dicionary = {
            "NAOSOLICITADO": "Não Solicitado",
            "AGUARDANDOPLANEJAMENTO": "Aguardando Planejamento",
            "EMANDAMENTO": "Em Andamento",
            "RECUSADO": "Recusado",
            "AGUARDANDOGN": "Aguardando GN",
            "EMANDAMENTO_RETORNOGN": "Em andamento (Retorno GN)",
            "FINALIZADOPLANEJAMENTO": "Finalizado Planejamento",
            "CONCLUIDO": "Concluido",
            "APROVADO": "Aprovado",
            "EFETIVADO": "Efetivado"
        };
        return dicionary[code];
    };

    /**
     * Monta o stripe da tela de elaboração de cardápio.
     * 
     * @params {Object} data - Dados do stripe.
     **/

    this.buildStripe = function(data) {
        var widgetStripe = templateManager.container.getWidget("widgetStripe");
        data.IDSERVICOPROPCOM = this.dicionary(data.IDSERVICOPROPCOM);
        widgetStripe.currentRow = data;
    };

    /**
     * Abre widget de serviços complementares.
     * No .then são tratados os possíveis erros.
     */
    this.openServCompl = function() {
        let widget = templateManager.container.getWidget('widgetCardProp');
        let params = Util.isEmptyOrBlank(widget) ? window.history.state.currentRow.params : widget.currentRow;

        Tco01200CardapioDaPropostaService.getServCompl(params).then((response) => {

            var servComplWidget = templateManager.container.getWidget('servComplWidget');

            if (response.dataset.servComplData) {
                servComplWidget.dataSource.data = response.dataset.servComplData;
            } else if (response.dataset.msgError) {
                MessageService.showMessageByCode(response.dataset.msgError[0].MSGCODE);
            }
            ScreenService.openPopup(servComplWidget);

        }).catch((error) => {
            if (error.data.dataset.msgError) {
                MessageService.showMessageByCode(error.data.dataset.msgError[0].MSGCODE);
            } else if (error.error) {
                ScreenService.showMessage(ScreenService.i18n(error.error, {}));
            }
        });
    };

    /**
     * Função responsável por trazer a lista de erros do serviço complementar,
     * caso ele possua.
     * 
     * Ao abrir a popup dos erros é necessário dar um redraw para corrigir uma
     * falha do framework, que não estava atualizando os field do grid.
     * 
     * @params {Object} - args - objeto args
     */

    this.getErrorServComp = function(args) {
        let widget = args.owner;
        //let widErroServComp = templateManager.container.getWidget('widErroServComp');
        if (widget.currentRow.IDERRO === 'S') {
            Tco01200CardapioDaPropostaService.getErrorServComp(widget.currentRow).then(response => {

                TcoUtil.buildMessagesToShow(response.dataset.SUCCESS).then(function(mensagens) {
                    General.showErrorMessages(widget, mensagens, ScreenService.i18n("Erros"));
                });
                //ScreenService.openPopup(widErroServComp);
            }).catch((error) => {
                if (error.error) {
                    ScreenService.showMessage(ScreenService.i18n(error.error, {}));
                }
            });
        }
    };

    /**
     * Função responsável por atualizar o grid dos serviços da proposta
     * e manter o escopo anterior.
     * 
     * Caso houvesse algum serviço selecionado, ele será mantido.
     * 
     * Função chamada no onEnter. Util quando alterações também referentes
     * a essa tela são feitas na tela de elaboração do cardápio.
     * 
     * @params {Args} args - objeto args
     **/

    this.refreshInScope = function(widget) {
        let nrpropostacomercial = widget.currentRow.NRSERVICOPROPCOM ? angular.copy(widget.currentRow.NRSERVICOPROPCOM) : "";
        if (nrpropostacomercial) {
            widget.reload().then(f => {
                let currentRow = widget.dataSource.data.filter(row => {
                    return row.NRSERVICOPROPCOM === nrpropostacomercial;
                })[0];
                widget.setCurrentRow(currentRow);
            });
        }
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01207_controller_geral', Tco01207_controller_geral);
});

// FILE: ./assets/js/controllers/Tco01300_lista_precos_controller.js
/* global angular, Configuration*/
var Tco01300_lista_precos_controller = function(ScreenService, General, TcoUtil, templateManager, Tco01300_lista_precos_service, MessageService) {

    this.abreFiltro = function(args) {
        var widgetMain = args.owner;
        widgetMain.getField("DTREFERENCIA_FILTRO").defaultValue = {
            start: moment().format("DD/MM/YYYY"),
            end: moment().format("DD/MM/YYYY")
        };
        TcoUtil.startFilter(widgetMain);
    };

    this.corrigeFiltro = function(args) {
        var widgetMain = args.owner;
        var filterRow = args.e.preference.FILTER_ROW;

        var fixedFilter = [
            {
                name: "DTREFERENCIA",
                operator: "BETWEEN",
                value: [filterRow.DTREFERENCIA_FILTRO.start, filterRow.DTREFERENCIA_FILTRO.end]
            }
        ];

        if (typeof filterRow.NRLISTAPRECOTCO !== "undefined" && filterRow.NRLISTAPRECOTCO !== "") {
            fixedFilter.push({
                name: "NRLISTAPRECOTCO",
                operator: "LIKE_I",
                value: "%" + filterRow.NRLISTAPRECOTCO + "%"
            });
        }

        if (typeof filterRow.CDREGIAO !== "undefined" && filterRow.CDREGIAO.length !== 0) {
            fixedFilter.push({
                name: "CDREGIAO",
                operator: "IN",
                value: filterRow.CDREGIAO
            });
        }

        widgetMain.dataSourceFilter = fixedFilter;
        widgetMain.reload().then(function() {
            ScreenService.closePopup();
            widgetMain.activate();
        });
        args.e.preventDefault();
    };

    this.abreProdutosListaPreco = function(args) {
        var widgetMain = args.owner;
        var widgetTabProdutos = args.owner.container.getWidget("widgetTabProdutos");
        var widgetProdutos = args.owner.container.getWidget("widgetProdutos");
        TcoUtil.maskProduto("widgetProdutos");

        var filter = [
            {
                "name": "NRLISTAPRECOTCO",
                "operator": "=",
                "value": widgetMain.currentRow.NRLISTAPRECOTCO
            }
        ];

        widgetProdutos.dataSourceFilter = filter;

        widgetProdutos.reload().then(() => ScreenService.openSwipe(widgetTabProdutos));
    };

    this.abreAtualizaListaPreco = function(args) {
        var popupAtualizaListaPreco = args.owner.widget.container.getWidget("popupAtualizaListaPreco");
        popupAtualizaListaPreco.getField("DTREFERENCIA").value(moment().format("DD/MM/YYYY"));
        popupAtualizaListaPreco.getField("TODAS_REGIOES").value("N");
        popupAtualizaListaPreco.getField("TODAS_REGIOES").checked = false;
        popupAtualizaListaPreco.getField("CDREGIAO").readOnly = false;
        popupAtualizaListaPreco.getField("CDREGIAO").validations = {
            required: true
        };
        ScreenService.openPopup(popupAtualizaListaPreco);
    };

    this.onChangeAllRegions = function(args) {
        General.toggleReadOnlySelect(args.owner.field, args.owner.field.widget.getField('CDREGIAO'));
        templateManager.updateTemplate();
    };

    this.closePopupAtualizacao = function() {
        ScreenService.closePopup();
    };

    this.confirmaAtualizacaoListaPrecos = function(args) {
        var widgetMain = args.owner.widget.container.getWidget("widgetMain");
        var popupAtualizaListaPreco = args.owner.widget;
        if (popupAtualizaListaPreco.isValid()) {
            Tco01300_lista_precos_service.atualizaListaPrecos(popupAtualizaListaPreco.currentRow).then(
                (success) => {
                    if (success.dataset.regioes_atualizadas.length < 1) {
                        MessageService.showMessageByCode("TCO01300_LISTA_DE_PRECOS_VAZIA");
                    } else {
                        MessageService.successNotificationByCode("LISTA_PRECO_ATUALIZADA");
                    }
                    widgetMain.reload().then(() => {
                        ScreenService.closePopup();
                        widgetMain.activate();
                    });
                },
                (fail) => ScreenService.showMessage(ScreenService.i18n(fail.data.error))
            );
        }
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01300_lista_precos_controller', Tco01300_lista_precos_controller);
});

// FILE: ./assets/js/controllers/Tco01400_custo_pela_media_controller.js
/* global angular, Configuration*/
var Tco01400_custo_pela_media_controller = function(ScreenService, templateManager, Tco01400_custo_pela_media_service, MessageService) {

    this.abreNovosProdutos = function(args) {
        var widgetProdutos = args.owner.widget.container.getWidget("widgetProdutos");

        widgetProdutos.reload().then(() => ScreenService.openPopup(widgetProdutos));
    };

    this.selecionaProduto = function(args) {
        var widgetProdutos = args.owner;
        var widgetMain = args.owner.container.getWidget("widgetMain");

        Tco01400_custo_pela_media_service.adicionaNovoProduto(widgetProdutos.currentRow).then((success) => {
            if (typeof success.dataset.validacoes !== "undefined") {
                MessageService.showMessageByCode(success.dataset.validacoes.message, widgetProdutos.currentRow);
            } else {
                ScreenService.notificationMessage(ScreenService.i18n("Produto salvo com sucesso."), 'success', 3000);
                widgetMain.reload().then(() => ScreenService.closePopup());
            }
        });
    };

    this.excluirProdutosSelecionados = function(args) {
        var widgetMain = args.owner.widget;

        ScreenService.confirmMessage(ScreenService.i18n('Remover Produto(s)?'), 'question', function(success) {
            Tco01400_custo_pela_media_service.excluiProdutos(widgetMain.getCheckedRows()).then(() => {
                ScreenService.showMessage(ScreenService.i18n("Produto(s) excluído(s) com sucesso."), null, "checked");
                widgetMain.reload().then(() => {
                    ScreenService.closePopup();
                    widgetMain.dataSource.clearCheckedRows();
                });
            });
        });
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01400_custo_pela_media_controller', Tco01400_custo_pela_media_controller);
});

// FILE: ./assets/js/controllers/Tco01500_fornecedor_prioritario_produto_controller.js
/* global angular, Configuration*/
var Tco01500_fornecedor_prioritario_produto_controller = function(ScreenService, General, TcoUtil, templateManager, maskEngine, ZhUtil, Tco01500_fornecedor_prioritario_produto_service, MessageService) {

    this.abreFiltro = function(args) {
        var widgetMain = args.owner;
        TcoUtil.startFilter(widgetMain);
    };

    this.aplicaFiltroStripe = function(args) {
        var stripeWidget = args.owner.container.getWidget("stripeWidget");

        stripeWidget.currentRow.DSREGIAO = args.e.preference.FILTER_ROW.CDREGIAO + " - " + args.e.preference.FILTER_ROW.NMREGIAO;
        stripeWidget.currentRow.CDREGIAO = args.e.preference.FILTER_ROW.CDREGIAO;
        stripeWidget.currentRow.NMREGIAO = args.e.preference.FILTER_ROW.NMREGIAO;
    };

    this.novoProdutoFornecedor = function(args) {
        var widgetProdutos = args.owner.widget.container.getWidget("widgetProdutos");
        var stripeWidget = args.owner.widget.container.getWidget("stripeWidget");

        if (typeof stripeWidget.currentRow.CDREGIAO !== "undefined" && stripeWidget.currentRow.CDREGIAO !== null && stripeWidget.currentRow.CDREGIAO !== "") {
            widgetProdutos.newRow();
            widgetProdutos.currentRow.CDREGIAO = stripeWidget.currentRow.CDREGIAO;
            widgetProdutos.edit();
            ScreenService.openSwipe(widgetProdutos);
        } else {
            MessageService.showMessageByCode("TCO01500_REGIAO_NAO_SELECIONADA");
        }
    };

    this.salvaNovoProdutoFornecedor = function(args) {
        var widgetProdutos = args.owner.widget;
        var widgetMain = args.owner.widget.container.getWidget("widgetMain");

        if (widgetProdutos.isValid()) {
            Tco01500_fornecedor_prioritario_produto_service.salvaProdutoFornecedor(widgetProdutos.currentRow).then((success) => {
                if (typeof success.dataset.validacoes !== "undefined") {
                    MessageService.showMessageByCode(success.dataset.validacoes.message, success.dataset.validacoes.params);
                } else {
                    ScreenService.notificationMessage(ScreenService.i18n("Produto salvo com sucesso."), 'success', 3000);
                    widgetMain.reload().then(() => ScreenService.closeSwipe());
                }
            });
        }

    };

    this.excluirProdutosFornecedorSelecionados = function(args) {
        var widgetMain = args.owner.widget;

        ScreenService.confirmMessage(ScreenService.i18n('Remover Produto(s)?'), 'question', function(success) {
            Tco01500_fornecedor_prioritario_produto_service.excluiProdutosFornecedor(widgetMain.getCheckedRows()).then(() => {
                ScreenService.showMessage(ScreenService.i18n("Produto(s) excluído(s) com sucesso."), null, "checked");
                widgetMain.reload().then(() => {
                    ScreenService.closePopup();
                    widgetMain.dataSource.clearCheckedRows();
                });
            });
        });
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01500_fornecedor_prioritario_produto_controller', Tco01500_fornecedor_prioritario_produto_controller);
});

// FILE: ./assets/js/controllers/Tco01600_previsao_consumo_controller.js
/* global angular, Configuration*/
var Tco01600_previsao_consumo_controller = function(ScreenService, General, TcoUtil, templateManager, Tco01600_previsao_consumo_service, MessageService) {

    this.inicializaWidget = function(args) {
        if (!window.history.state) {
            window.history.back();
        } else {
            var widgetMain = args.owner.getWidget("widgetMain");
            widgetMain.elaboracaoCardapioBaseParams = window.history.state.elaboracaoCardapioBaseParams;

            aplicaFiltroFieldsDia(widgetMain);
            aplicaFiltroFieldSubgrupoProduto(widgetMain);
            aplicaDataFieldNivel(widgetMain);

            widgetMain.newRow();
            widgetMain.currentRow.REPORT = "TCO01600";
            widgetMain.currentRow.TIPONIVEL = "0";
            widgetMain.currentRow.VRPEACRESCIMO = 0;
            widgetMain.currentRow.IDARRENDONDAMENTO = "N";
            widgetMain.currentRow.IDORDENACAO = "C";
            widgetMain.currentRow.DESCON_NMSUBGRPROD = "S";
            Object.assign(widgetMain.currentRow, widgetMain.elaboracaoCardapioBaseParams);
        }
    };

    var aplicaFiltroFieldsDia = function(widgetMain) {
        var filter = [
            {
                name: "NRCARDAPIOBASE",
                operator: "=",
                value: widgetMain.elaboracaoCardapioBaseParams.NRCARDAPIOBASE
            }
        ];

        widgetMain.getField("DSSEMANA_INICIAL").dataSourceFilter = filter.slice();
        widgetMain.getField("DSSEMANA_FINAL").dataSourceFilter = filter.slice();
        widgetMain.getField("DSSEMANA_INICIAL").reload();
        widgetMain.getField("DSSEMANA_FINAL").reload();
    };

    var aplicaFiltroFieldSubgrupoProduto = function(widgetMain) {
        var filter = [
            {
                name: "CDFILIAL",
                operator: "=",
                value: getLocalVar("CDFILIAL")
            }
        ];
        widgetMain.getField("NMSUBGRPROD").dataSourceFilter = filter;
        widgetMain.getField("NMSUBGRPROD").readOnly = true;
        widgetMain.getField("NMSUBGRPROD").validations = {};
        widgetMain.getField("DESCON_NMSUBGRPROD").value('S');
        widgetMain.getField("NMSUBGRPROD").reload();
    };

    var aplicaDataFieldNivel = function(widgetMain) {
        var nivelMascara = General.getLevelByMask("CDPICTPROD", 1);

        var data = [
            {
                name: ScreenService.i18n("Nível 0"),
                value: "0"
            }
        ];

        for (var posicao = 0; posicao < nivelMascara.length - 2; posicao++) {
            data.push({
                name: ScreenService.i18n("Nível " + nivelMascara[posicao].nivel),
                value: nivelMascara[posicao].nivel
            });
        }

        widgetMain.getField("TIPONIVEL").dataSource.data = data;
    };

    this.goBack = function() {
        window.history.back();
    };

    this.doNothing = function(args) {
        console.log(args.owner.field);
    };

    this.montaLabelsSemanaDia = function(field) {
        field.dataSource.data.forEach(function(row) {
            if (!row.mounted) {
                row.DSSEMANA = row.NRSEMANA + "º " + ScreenService.i18n(row.DSSEMANA.slice(1, row.DSSEMANA.length));
                row.mounted = true;
            }
        });
    };

    this.alteraFieldSubgrupo = function(args) {
        General.toggleReadOnlySelect(args.owner.field, args.owner.field.widget.getField('NMSUBGRPROD'));
    };

    this.confirmarRelatorio = function(args) {
        var widgetMain = args.owner.widget;
        if (widgetMain.isValid()) {
            if (validaPeriodoProd(widgetMain) && validaPeriodoDia(widgetMain)) {
                TcoUtil.openReportPopup(widgetMain, relatorioPrevisaoConsumo, [widgetMain]);
            }
        }
    };

    var relatorioPrevisaoConsumo = function(widgetMain) {
        widgetMain.currentRow.CDARVPRODINI = widgetMain.getField("NMPRODUTOINI").value() == "" ? "0" : widgetMain.currentRow.CDARVPRODINI;
        widgetMain.currentRow.CDARVPRODFIN = widgetMain.getField("NMPRODUTOFIN").value() == "" ? "zzzzzzzzzz" : widgetMain.currentRow.CDARVPRODFIN;
        widgetMain.currentRow.VRPEACRESCIMO = (!widgetMain.currentRow.VRPEACRESCIMO || (widgetMain.currentRow.VRPEACRESCIMO == "")) ? 0 : widgetMain.currentRow.VRPEACRESCIMO;

        Tco01600_previsao_consumo_service.relatorioPrevisaoConsumo(widgetMain.currentRow).then((response) => {
            if (response.dataset && response.dataset && response.dataset.ERROS) {
                TcoUtil.buildMessagesToShow(response.dataset.ERROS).then(function(mensagens) {
                    General.showErrorMessages(widgetMain, mensagens, ScreenService.i18n("Erro"));
                });
            } else {
                if (response && response.messages) {
                    var report = response.messages.pop();
                    var reportOpened = window.open(report.message);
                    if (!reportOpened) {
                        ScreenService.showMessage(ScreenService.i18n('Bloqueador de Pop-up está ativado!'));
                    }
                }
                if (response && response.dataset && response.dataset.RESPONSE) {
                    widgetMain.reportRow.P_NRLISTA = response.dataset.RESPONSE.NRLISTA;
                    widgetMain.hasList = true;
                }
            }
        });
    };

    var validaPeriodoProd = function(widget) {
        var valido = false;

        if (widget.getField("NMPRODUTOINI").value() != "" && widget.getField("NMPRODUTOFIN").value() == "") {
            MessageService.showMessageByCode("TCO01600_PRODUTO_FINAL_NAO_PREENCHIDO");
            valido = false;
        } else if (widget.getField("NMPRODUTOFIN").value() != "" && widget.getField("NMPRODUTOINI").value() == "") {
            MessageService.showMessageByCode("TCO01600_PRODUTO_INICIAL_NAO_PREENCHIDO");
            valido = false;
        } else {
            if (widget.currentRow.CDPRODUTOINI > widget.currentRow.CDPRODUTOFIN) {
                MessageService.showMessageByCode("TCO01600_PERIODO_PROD_INVALIDO");
                valido = false;
            } else {
                valido = true;
            }
        }

        return valido;
    };

    var validaPeriodoDia = function(widget) {
        var valido = false;

        if (widget.currentRow.NRSEMANA_INICIAL > widget.currentRow.NRSEMANA_FINAL) {
            MessageService.showMessageByCode("TCO01600_PERIODO_DIA_INVALIDO");
            valido = false;
        } else {
            if (widget.currentRow.NRDIASEMANA_INICIAL > widget.currentRow.NRDIASEMANA_FINAL) {
                MessageService.showMessageByCode("TCO01600_PERIODO_DIA_INVALIDO");
                valido = false;
            } else {
                valido = true;
            }
        }

        return valido;
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01600_previsao_consumo_controller', Tco01600_previsao_consumo_controller);
});

// FILE: ./assets/js/controllers/Tco01700_tipo_preparo_controller.js
/* global angular, Configuration*/
var Tco01700_tipo_preparo_controller = function(ScreenService, General, templateManager, Tco01700_tipo_preparo_service, MessageService) {

    this.activeMainWidget = function(args) {
        var widgetMain = args.owner;
        widgetMain.activate();
    };

    this.novoTipoPreparo = function(args) {
        var widgetMain = args.owner.widget;
        widgetMain.newRow();
        this.habilitaPaginaPratos(false);
    };

    this.editarTipoPreparo = function(args) {
        var widgetMain = args.owner.widget;
        widgetMain.backupRow = angular.copy(widgetMain.currentRow);
        widgetMain.edit();
        this.habilitaPaginaPratos(false);
    };

    this.salvarTipoPreparo = function(args) {
        var widgetMain = args.owner.widget;
        if (widgetMain.isValid()) {
            Tco01700_tipo_preparo_service.salvarTipoPreparo(widgetMain.currentRow).then((success) => {
                widgetMain.reload().then((success) => {
                    ScreenService.closeSwipe();
                    this.habilitaPaginaPratos(true);
                });
            });
        }
    };

    this.removerTipoPreparo = function(args) {
        var widgetMain = args.owner.widget;

        ScreenService.confirmMessage(ScreenService.i18n('Remover Tipo de Preparo?'), 'question', function(success) {
            Tco01700_tipo_preparo_service.deleteTipoPreparo(widgetMain.currentRow).then((success) => {
                if (typeof success.dataset.TIPOPREPAROUTILIZADO !== 'undefined' && success.dataset.TIPOPREPAROUTILIZADO.length > 0) {
                    buildMessages(success.dataset.TIPOPREPAROUTILIZADO).then(function(mensagens) {
                        General.showErrorMessages(widgetMain, mensagens, ScreenService.i18n("Tipo de Preparo em uso."));
                    });
                } else {
                    ScreenService.showMessage(success.notifications[0].message, null, "checked").then(() => {
                        ScreenService.closeSwipe();
                        widgetMain.reload();
                    });
                }
            });
        });
    };

    var buildMessages = function(data) {
        var mensagens = [];
        var msgParams = [];
        data.forEach(function(row) {
            //mensagens.push(ScreenService.i18n(row.MESSAGECODE, row));
            msgParams.push(row);
            mensagens.push(row.MESSAGECODE);
        });

        return MessageService.getMessagesByCode(mensagens).then(function(mensagensTraduzidas) {
            for (let i = 0; i < mensagensTraduzidas.length; i++) {
                mensagensTraduzidas[i] = ScreenService.i18n(mensagensTraduzidas[i], msgParams[i]);
            }
            return mensagensTraduzidas;
        });
    };

    this.backDetail = function(args) {
        ScreenService.closeSwipe();
        this.habilitaPaginaPratos(true);
    };

    this.closeRelPratos = function(args) {
        var widgetMain = templateManager.container.getWidget('widgetMain');
        ScreenService.closePopup();
        widgetMain.activate();

    };

    this.closeCustomPopup = function(args) {
        ScreenService.closePopup();
    };

    this.openRelacionaPratos = function(args) {
        var widgetPratoTab = args.owner.widget.container.getWidget('widgetPratoTab');
        widgetPratoTab.dataSource.unSelectAll();
        this.resetPratoVinculado(args);
        widgetPratoTab.reload().then((success) => {
            ScreenService.openPopup(widgetPratoTab);
            widgetPratoTab.activate();
        });
    };

    this.onChangePratoVinculado = function(args) {
        var widgetPratoTab = templateManager.container.getWidget('widgetPratoTab');
        let semcomvinculo = widgetPratoTab.getAction('SEMCOMVINCULO');
        widgetPratoTab.dataSourceFilter.forEach((condicao) => {
            if (condicao.name === 'P_TIPOPREPVAZIO') {
                if (condicao.value === 'N') {
                    condicao.value = 'S';
                    semcomvinculo.label = ScreenService.i18n("Todos");
                } else {
                    condicao.value = 'N';
                    semcomvinculo.label = ScreenService.i18n("A vincular");
                }
            }
        });

        widgetPratoTab.reload();
    };

    this.resetPratoVinculado = function(args) {
        var widgetPratoTab = templateManager.container.getWidget('widgetPratoTab');
        let semcomvinculo = widgetPratoTab.getAction('SEMCOMVINCULO');
        widgetPratoTab.dataSourceFilter.forEach((condicao) => {
            if (condicao.name === 'P_TIPOPREPVAZIO')
                condicao.value = 'N';
        });
        semcomvinculo.label = ScreenService.i18n("A vincular");
    };

    this.vincularSelecaoPratos = function(args) {
        var widgetTipoPreparo = args.owner.widget.container.getWidget('widgetTipoPreparo');
        widgetTipoPreparo.reload().then(() => {
            ScreenService.openPopup(widgetTipoPreparo);
        });
    };

    this.selectTipoPreparo = function(args) {
        var widgetTipoPreparo = args.owner;

        //previnir abertura das tabs
        args.e.preventDefault();
        args.e.stopPropagation();

        var widgetPratoTab = args.owner.container.getWidget('widgetPratoTab');

        widgetTipoPreparo.currentRow.PRATOSSELECIONADOS = widgetPratoTab.dataSource.getCheckedRows();
        Tco01700_tipo_preparo_service.vinculaTipoPreparo(widgetTipoPreparo.currentRow).then(() => {
            ScreenService.successNotification(ScreenService.i18n("Tipo de preparo vinculado com sucesso."));
            widgetPratoTab.reload().then(() => {
                widgetPratoTab.dataSource.unSelectAll();
                ScreenService.closePopup();
            });
        });
    };

    this.desvincularSelecaoPratos = function(args) {
        var widgetPratoTab = args.owner.widget.container.getWidget('widgetPratoTab');
        var row = {};

        row.PRATOSSELECIONADOS = widgetPratoTab.dataSource.getCheckedRows();
        ScreenService.confirmMessage(ScreenService.i18n("Desvincular Tipo de Preparo?"), 'question', function(success) {
            Tco01700_tipo_preparo_service.desvinculaTipoPreparo(row).then(() => {
                ScreenService.successNotification(ScreenService.i18n("Tipo de preparo desvinculado com sucesso."));
                widgetPratoTab.reload().then(() => widgetPratoTab.dataSource.unSelectAll());
            });
        });
    };

    this.abreAdicionarPratos = function(args) {
        var widgetAdicionarPratos = templateManager.container.getWidget('widgetAdicionarPratos');
        this.resetPratoVinculadoAdicao(args);

        widgetAdicionarPratos.dataSource.unSelectAll();
        widgetAdicionarPratos.reload().then((success) => {
            ScreenService.openPopup(widgetAdicionarPratos);
            widgetAdicionarPratos.activate();
        });
    };

    this.adicionarPratos = function(args) {
        var widgetMain = templateManager.container.getWidget('widgetMain');
        var widgetPratoTipoPreparo = templateManager.container.getWidget('widgetPratoTipoPreparo');
        //previnir abertura das tabs
        args.e.preventDefault();
        args.e.stopPropagation();

        var widgetAdicionarPratos = templateManager.container.getWidget('widgetAdicionarPratos');
        let row = angular.copy(widgetMain.currentRow);
        row.PRATOSSELECIONADOS = widgetAdicionarPratos.dataSource.getCheckedRows();
        let existe_preparo = row.PRATOSSELECIONADOS.filter(function(r) {
            if (r.NRTIPOPREPAROTCO)
                return true;
            return false;
        });

        let salvar = function() {
            Tco01700_tipo_preparo_service.vinculaTipoPreparo(row).then(() => {
                ScreenService.successNotification(ScreenService.i18n("Tipo de preparo vinculado com sucesso."));
                widgetPratoTipoPreparo.reload().then(() => {
                    widgetPratoTipoPreparo.dataSource.unSelectAll();
                    ScreenService.closePopup();
                    widgetPratoTipoPreparo.activate();
                });
            });
        };

        if (existe_preparo.length > 0) {
            ScreenService.confirmMessage(ScreenService.i18n("O preparo dos pratos selecionados será sobrescrito. Confirma operação?"), 'question', salvar);
        } else {
            salvar();
        }
    };

    this.removerPratos = function(args) {
        var widgetPratoTipoPreparo = templateManager.container.getWidget('widgetPratoTipoPreparo');
        var row = {};

        row.PRATOSSELECIONADOS = widgetPratoTipoPreparo.dataSource.getCheckedRows();

        ScreenService.confirmMessage(ScreenService.i18n("Confirma remoção dos pratos?"), 'question', function(success) {
            Tco01700_tipo_preparo_service.desvinculaTipoPreparo(row).then(() => {
                ScreenService.successNotification(ScreenService.i18n("Pratos removidos com sucesso."));
                widgetPratoTipoPreparo.reload().then(() => widgetPratoTipoPreparo.dataSource.unSelectAll());
            });
        });
    };
    this.cancelaAdicionarPratos = function(args) {
        ScreenService.closePopup();
        var widgetPratoTipoPreparo = templateManager.container.getWidget('widgetPratoTipoPreparo');
        widgetPratoTipoPreparo.activate();
    };
    this.filtraPratos = function(args) {
        var widgetMain = templateManager.container.getWidget('widgetMain');
        var widgetPratoTipoPreparo = templateManager.container.getWidget('widgetPratoTipoPreparo');

        widgetPratoTipoPreparo.dataSource.unSelectAll();

        var widgetAdicionarPratos = templateManager.container.getWidget('widgetAdicionarPratos');

        let filter = [
            {
                "name": "P_TIPOPREPVAZIO",
                "operator": "=",
                "value": "N"
            },
            {
                "operator": "ORDER_BY",
                "name": [
                    {
                        "name": "CDPRATO",
                        "direction": "ASC"
                    }
                ]
            },
            {
                "name": "NRTIPOPREPAROTCO",
                "operator": "=",
                "value": widgetMain.currentRow.NRTIPOPREPAROTCO
            }
        ];

        widgetPratoTipoPreparo.dataSourceFilter = filter;

        filter = [
            {
                "name": "P_TIPOPREPVAZIO",
                "operator": "=",
                "value": "N"
            },
            {
                "operator": "ORDER_BY",
                "name": [
                    {
                        "name": "CDPRATO",
                        "direction": "ASC"
                    }
                ]
            },
            {
                "name": "P_NRTIPOPREPAROTCO",
                "operator": "=",
                "value": widgetMain.currentRow.NRTIPOPREPAROTCO
            }
        ];

        widgetAdicionarPratos.dataSourceFilter = filter;

        widgetPratoTipoPreparo.reload();
    };

    this.habilitaPaginaPratos = function(habilita) {
        let widgetPratoTipoPreparo = templateManager.container.getWidget('widgetPratoTipoPreparo');
        widgetPratoTipoPreparo.setEnabled(habilita);
    };

    this.onChangePratoVinculadoAdicao = function(args) {
        var widgetAdicionarPratos = templateManager.container.getWidget('widgetAdicionarPratos');
        let semcomvinculo = widgetAdicionarPratos.getAction('SEMCOMVINCULO_ADD');
        widgetAdicionarPratos.dataSourceFilter.forEach((condicao) => {
            if (condicao.name === 'P_TIPOPREPVAZIO') {
                if (condicao.value === 'N') {
                    condicao.value = 'S';
                    semcomvinculo.label = ScreenService.i18n("Todos");
                } else {
                    condicao.value = 'N';
                    semcomvinculo.label = ScreenService.i18n("A vincular");
                }
            }
        });

        widgetAdicionarPratos.reload();
    };
    this.resetPratoVinculadoAdicao = function(args) {
        var widgetAdicionarPratos = templateManager.container.getWidget('widgetAdicionarPratos');
        let semcomvinculo = widgetAdicionarPratos.getAction('SEMCOMVINCULO_ADD');
        widgetAdicionarPratos.dataSourceFilter.forEach((condicao) => {
            if (condicao.name === 'P_TIPOPREPVAZIO')
                condicao.value = 'N';
        });
        semcomvinculo.label = ScreenService.i18n("A vincular");

        widgetAdicionarPratos.reload();
    };
};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01700_tipo_preparo_controller', Tco01700_tipo_preparo_controller);
});

// FILE: ./assets/js/controllers/Tco01800_fator_correcao_controller.js
/* global getLocalVar, Configuration*/
var Tco01800_fator_correcao_controller = function(ScreenService, General, TcoUtil, templateManager, ZHPromise, Tco01800_fator_correcao_service, MessageService) {

    this.novoFatorCorrecao = function(args) {
        var widgetNovoFatorCorrecao = args.owner.widget.container.getWidget('widgetNovoFatorCorrecao');

        ScreenService.openSwipe(widgetNovoFatorCorrecao).then(() => {
            widgetNovoFatorCorrecao.newRow();
            widgetNovoFatorCorrecao.edit();
            widgetNovoFatorCorrecao.getField('NMPRODUTO').readOnly = true;
            widgetNovoFatorCorrecao.currentRow.VRDEGELO = "0";
            widgetNovoFatorCorrecao.currentRow.VRLIMPEZA = "0";
            widgetNovoFatorCorrecao.currentRow.VRCOCCAO = "0";
        });
    };

    this.defineFilterProduto = function(args) {
        var widgetNovoFatorCorrecao = args.owner.field.widget.container.getWidget('widgetNovoFatorCorrecao');
        var fieldProduto = args.owner.field.widget.getField('NMPRODUTO');

        if (widgetNovoFatorCorrecao.currentRow.NRGRUPOPRATO != null && widgetNovoFatorCorrecao.currentRow.NRGRUPOPRATO != "" && widgetNovoFatorCorrecao.currentRow.NRTIPOPREPAROTCO != null && widgetNovoFatorCorrecao.currentRow.NRTIPOPREPAROTCO != "") {
            widgetNovoFatorCorrecao.currentRow.NMPRODUTO = "";
            widgetNovoFatorCorrecao.currentRow.CDPRODUTO = "";
            widgetNovoFatorCorrecao.currentRow.CDARVPROD = "";
            fieldProduto.readOnly = false;
        } else {
            widgetNovoFatorCorrecao.currentRow.NMPRODUTO = "";
            widgetNovoFatorCorrecao.currentRow.CDPRODUTO = "";
            widgetNovoFatorCorrecao.currentRow.CDARVPROD = "";
            fieldProduto.readOnly = true;
        }

    };

    this.backDetail = function(args) {
        ScreenService.closeSwipe();
    };

    this.salvarTabFatorCorrecao = function(args) {
        var widgetNovoFatorCorrecao = args.owner.widget;
        var widgetMain = args.owner.widget.container.getWidget('widgetMain');
        if (widgetNovoFatorCorrecao.isValid() && validaValores(widgetNovoFatorCorrecao.currentRow)) {
            Tco01800_fator_correcao_service.salvaNovaTabFatorCorrecao(widgetNovoFatorCorrecao.currentRow).then((success) => {
                if (typeof success.dataset.tabfatorcorrecaoexistente != 'undefined' && success.dataset.tabfatorcorrecaoexistente.length > 0) {
                    MessageService.showMessageByCode("TCO01800_FATO_CORRECAO_EXISTENTE", success.dataset.tabfatorcorrecaoexistente[0]);
                } else {
                    widgetMain.reload().then(() => {
                        ScreenService.closeSwipe();
                        widgetMain.activate();
                    });
                    ScreenService.successNotification(ScreenService.i18n("Fator de Correção incluído com sucesso."));
                }
            });
        }
    };

    this.salvaEditTabFatCorrecao = function(args) {
        var widgetMain = args.owner;
        var validacao = true;
        widgetMain.dataSource.data.forEach((row) => {
            if (!validaValores(row)) {
                validacao = false;
                var posicao = widgetMain.dataSource.data.findIndex((rowData) => {
                    return rowData.NRGRUPOPRATO == row.NRGRUPOPRATO && rowData.NRTIPOPREPAROTCO == row.NRTIPOPREPAROTCO && rowData.CDPRODUTO == row.CDPRODUTO;
                });
                widgetMain.setCurrentRow(widgetMain.dataSource.data[posicao < 0 ? 0 : posicao]);
                setTimeout(function() {
                    $(".zh-new-grid-" + widgetMain.name).find(".grid-vertical-scroll-control").scrollTop(widgetMain.currentRow.__index * 36);
                }, 200);
            }
        });

        if (validacao) {
            Tco01800_fator_correcao_service.salvaEdicaoTabFatorCorrecao(widgetMain.dataSource.data).then(() => {
                widgetMain.reload().then(() => {
                    if (widgetMain.mode == 'inline_edit') {
                        widgetMain.list();
                    }
                });
                ScreenService.successNotification(ScreenService.i18n("Fator de Correção alterado com sucesso."));
            });
        }
    };

    var validaValores = function(row) {
        var valido = false;

        if (!row.__is_new) {
            row.VRDEGELO = (row.VRDEGELO.toString()).replace(".", ",");
            row.VRLIMPEZA = (row.VRLIMPEZA.toString()).replace(".", ",");
            row.VRCOCCAO = (row.VRCOCCAO.toString()).replace(".", ",");
        }

        if (row.changed) {
            let vrdegelo = parseFloat((row.VRDEGELO.replace(".", "")).replace(",", "."));
            let vrlimpeza = parseFloat((row.VRLIMPEZA.replace(".", "")).replace(",", "."));
            let vrcoccao = parseFloat((row.VRCOCCAO.replace(".", "")).replace(",", "."));

            let limpezaDegelo = (vrdegelo + vrlimpeza) <= 100;
            let coccao = (vrcoccao) <= 100;

            if (!limpezaDegelo) {
                if (!row.__is_new) {
                    if (typeof row.CDPRODUTO != "undefined" && row.CDPRODUTO != null && row.CDPRODUTO != "") {
                        MessageService.showMessageByCode("TCO01800_VALIDACAO_VALORES_LIMPEZA_DEGELO_DETALHE_PRODUTO", row);
                    } else {
                        MessageService.showMessageByCode("TCO01800_VALIDACAO_VALORES_LIMPEZA_DEGELO_DETALHE", row);
                    }
                } else {
                    MessageService.showMessageByCode("TCO01800_VALIDACAO_VALORES_LIMPEZA_DEGELO", row);
                }
            }

            if (!coccao) {
                if (!row.__is_new) {
                    if (typeof row.CDPRODUTO != "undefined" && row.CDPRODUTO != null && row.CDPRODUTO != "") {
                        MessageService.showMessageByCode("TCO01800_VALIDACAO_VALORES_COCCAO_DETALHE_PRODUTO", row);
                    } else {
                        MessageService.showMessageByCode("TCO01800_VALIDACAO_VALORES_COCCAO_DETALHE", row);
                    }
                } else {
                    MessageService.showMessageByCode("TCO01800_VALIDACAO_VALORES_COCCAO", row);
                }
            }

            valido = limpezaDegelo && coccao;
        } else {
            valido = true;
        }

        return valido;
    };

    this.redefineValores = function(args) {
        var widgetMain = args.owner;
        if (widgetMain.mode == 'inline_edit') {
            widgetMain.list();
            args.owner.rollbackInlineEdit();
        }
    };

    this.removeTabFatorCorrecao = function(args) {
        var widgetMain = args.owner.widget;
        var tabelasSelecionadas = widgetMain.dataSource.getCheckedRows();

        ScreenService.confirmMessage(ScreenService.i18n('Deseja excluir o regitro?'), 'question', function(success) {
            Tco01800_fator_correcao_service.excluiTabFatorCorrecao(tabelasSelecionadas).then(() => {
                widgetMain.reload().then(() => {
                    widgetMain.dataSource.clearCheckedRows();
                });
                ScreenService.successNotification(ScreenService.i18n("Fator(es) de Correção excluído(s) com sucesso."));
            });
        });
    };

    this.desabilitaCheckboxInlineEdit = function(args) {
        var widgetMain = args.owner;

        if (widgetMain.mode === "inline_edit") {
            args.e.preventDefault();
        }
    };

    this.onInlineEdit = function(args) {
        var widgetMain = args.owner.field.widget;

        widgetMain.dataSource.clearCheckedRows();
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('Tco01800_fator_correcao_controller', Tco01800_fator_correcao_controller);
});
